@@ -1,5 +1,5 @@
 /* 
- * $Id: db_row.h 1827 2007-03-12 15:22:53Z bogdan_iancu $ 
+ * $Id: db_row.h 2196 2007-05-11 08:01:24Z henningw $ 
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -39,5 +39,9 @@ typedef struct db_row {
 #define ROW_VALUES(rw) ((rw)->values)
 #define ROW_N(rw)      ((rw)->n)
 
+/*
+ * Release memory used by row
+ */
+int db_free_row(db_row_t* _r);
 
 #endif /* DB_ROW_H */
