@@ -1,5 +1,5 @@
 /* 
- * $Id: io_wait.h 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: io_wait.h 2770 2007-09-14 15:27:38Z bogdan_iancu $
  * 
  * Copyright (C) 2005 iptelorg GmbH
  *
@@ -221,16 +221,15 @@ static inline int kq_ev_change(io_wait_h* h, int fd, int filter, int flag,
 
 	if (h->kq_nchanges>=h->kq_changes_size){
 		/* changes array full ! */
-		LOG(L_WARN, "WARNING: kq_ev_change: kqueue changes array full"
-					" trying to flush...\n");
+		LM_WARN("kqueue changes array full trying to flush...\n");
 		tspec.tv_sec=0;
 		tspec.tv_nsec=0;
 again:
 		n=kevent(h->kq_fd, h->kq_changes, h->kq_nchanges, 0, 0, &tspec);
 		if (n==-1){
 			if (errno==EINTR) goto again;
-			LOG(L_ERR, "ERROR: io_watch_add: kevent flush changes "
-						" failed: %s [%d]\n", strerror(errno), errno);
+			LM_ERR("kevent flush changes failed: %s [%d]\n", 
+				strerror(errno), errno);
 			return -1;
 		}
 		h->kq_nchanges=0; /* changes array is empty */
@@ -268,12 +267,12 @@ inline static int io_watch_add(	io_wait_h* h,
 	do{ \
 			flags=fcntl(fd, F_GETFL); \
 			if (flags==-1){ \
-				LOG(L_ERR, "ERROR: io_watch_add: fnctl: GETFL failed:" \
+				LM_ERR("fnctl: GETFL failed:" \
 						" %s [%d]\n", strerror(errno), errno); \
 				goto error; \
 			} \
 			if (fcntl(fd, F_SETFL, flags|(f))==-1){ \
-				LOG(L_ERR, "ERROR: io_watch_add: fnctl: SETFL" \
+				LM_ERR("fnctl: SETFL" \
 							" failed: %s [%d]\n", strerror(errno), errno); \
 				goto error; \
 			} \
@@ -300,28 +299,28 @@ inline static int io_watch_add(	io_wait_h* h,
 #endif
 	e=0;
 	if (fd==-1){
-		LOG(L_CRIT, "BUG: io_watch_add: fd is -1!\n");
+		LM_CRIT("fd is -1!\n");
 		goto error;
 	}
 	/* check if not too big */
 	if (h->fd_no>=h->max_fd_no){
-		LOG(L_CRIT, "ERROR: io_watch_add: maximum fd number exceeded:"
+		LM_CRIT("maximum fd number exceeded:"
 				" %d/%d\n", h->fd_no, h->max_fd_no);
 		goto error;
 	}
-	DBG("DBG: io_watch_add(%p, %d, %d, %p), fd_no=%d\n",
+	LM_DBG("io_watch_add(%p, %d, %d, %p), fd_no=%d\n",
 			h, fd, type, data, h->fd_no);
 	/*  hash sanity check */
 	e=get_fd_map(h, fd);
 	if (e && (e->type!=0 /*F_NONE*/)){
-		LOG(L_ERR, "ERROR: io_watch_add: trying to overwrite entry %d"
+		LM_ERR("trying to overwrite entry %d"
 				" in the hash(%d, %d, %p) with (%d, %d, %p)\n",
 				fd, e->fd, e->type, e->data, fd, type, data);
 		goto error;
 	}
 	
 	if ((e=hash_fd_map(h, fd, type, data))==0){
-		LOG(L_ERR, "ERROR: io_watch_add: failed to hash the fd %d\n", fd);
+		LM_ERR("failed to hash the fd %d\n", fd);
 		goto error;
 	}
 	switch(h->poll_method){ /* faster then pointer to functions */
@@ -345,19 +344,19 @@ inline static int io_watch_add(	io_wait_h* h,
 			 */
 			/* set async & signal */
 			if (fcntl(fd, F_SETOWN, my_pid())==-1){
-				LOG(L_ERR, "ERROR: io_watch_add: fnctl: SETOWN"
+				LM_ERR("fnctl: SETOWN"
 				" failed: %s [%d]\n", strerror(errno), errno);
 				goto error;
 			}
 			if (fcntl(fd, F_SETSIG, h->signo)==-1){
-				LOG(L_ERR, "ERROR: io_watch_add: fnctl: SETSIG"
+				LM_ERR("fnctl: SETSIG"
 					" failed: %s [%d]\n", strerror(errno), errno);
 				goto error;
 			}
 			/* set both non-blocking and async */
 			set_fd_flags(O_ASYNC| O_NONBLOCK);
 #ifdef EXTRA_DEBUG
-			DBG("io_watch_add: sigio_rt on f %d, signal %d to pid %d\n",
+			LM_DBG("sigio_rt on f %d, signal %d to pid %d\n",
 					fd,  h->signo, my_pid());
 #endif
 			/* empty socket receive buffer, if buffer is already full
@@ -378,7 +377,7 @@ again1:
 			n=epoll_ctl(h->epfd, EPOLL_CTL_ADD, fd, &ep_event);
 			if (n==-1){
 				if (errno==EAGAIN) goto again1;
-				LOG(L_ERR, "ERROR: io_watch_add: epoll_ctl failed: %s [%d]\n",
+				LM_ERR("epoll_ctl failed: %s [%d]\n",
 					strerror(errno), errno);
 				goto error;
 			}
@@ -391,7 +390,7 @@ again2:
 			n=epoll_ctl(h->epfd, EPOLL_CTL_ADD, fd, &ep_event);
 			if (n==-1){
 				if (errno==EAGAIN) goto again2;
-				LOG(L_ERR, "ERROR: io_watch_add: epoll_ctl failed: %s [%d]\n",
+				LM_ERR("epoll_ctl failed: %s [%d]\n",
 					strerror(errno), errno);
 				goto error;
 			}
@@ -413,7 +412,7 @@ again2:
 again_devpoll:
 			if (write(h->dpoll_fd, &pfd, sizeof(pfd))==-1){
 				if (errno==EAGAIN) goto again_devpoll;
-				LOG(L_ERR, "ERROR: io_watch_add: /dev/poll write failed:"
+				LM_ERR("/dev/poll write failed:"
 							"%s [%d]\n", strerror(errno), errno);
 				goto error;
 			}
@@ -421,7 +420,7 @@ again_devpoll:
 #endif
 			
 		default:
-			LOG(L_CRIT, "BUG: io_watch_add: no support for poll method "
+			LM_CRIT("no support for poll method "
 					" %s (%d)\n", poll_method_str[h->poll_method],
 					h->poll_method);
 			goto error;
@@ -438,7 +437,7 @@ check_io_again:
 		while( ((n=poll(&pf, 1, 0))>0) && (handle_io(e, idx)>0));
 		if (n==-1){
 			if (errno==EINTR) goto check_io_again;
-			LOG(L_ERR, "ERROR: io_watch_add: check_io poll: %s [%d]\n",
+			LM_ERR("check_io poll: %s [%d]\n",
 						strerror(errno), errno);
 		}
 	}
@@ -494,21 +493,19 @@ inline static int io_watch_del(io_wait_h* h, int fd, int idx, int flags)
 #endif
 	
 	if ((fd<0) || (fd>=h->max_fd_no)){
-		LOG(L_CRIT, "BUG: io_watch_del: invalid fd %d, not in [0, %d) \n",
-						fd, h->fd_no);
+		LM_CRIT("invalid fd %d, not in [0, %d) \n", fd, h->fd_no);
 		goto error;
 	}
-	DBG("DBG: io_watch_del (%p, %d, %d, 0x%x) fd_no=%d called\n",
+	LM_DBG("io_watch_del (%p, %d, %d, 0x%x) fd_no=%d called\n",
 			h, fd, idx, flags, h->fd_no);
 	e=get_fd_map(h, fd);
 	/* more sanity checks */
 	if (e==0){
-		LOG(L_CRIT, "BUG: io_watch_del: no corresponding hash entry for %d\n",
-					fd);
+		LM_CRIT("no corresponding hash entry for %d\n", fd);
 		goto error;
 	}
 	if (e->type==0 /*F_NONE*/){
-		LOG(L_ERR, "ERROR: io_watch_del: trying to delete already erased"
+		LM_ERR("trying to delete already erased"
 				" entry %d in the hash(%d, %d, %p) )\n",
 				fd, e->fd, e->type, e->data);
 		goto error;
@@ -542,12 +539,12 @@ inline static int io_watch_del(io_wait_h* h, int fd, int idx, int flags)
 				/* reset ASYNC */
 				fd_flags=fcntl(fd, F_GETFL); 
 				if (fd_flags==-1){ 
-					LOG(L_ERR, "ERROR: io_watch_del: fnctl: GETFL failed:" 
+					LM_ERR("fnctl: GETFL failed:" 
 							" %s [%d]\n", strerror(errno), errno); 
 					goto error; 
 				} 
 				if (fcntl(fd, F_SETFL, fd_flags&(~O_ASYNC))==-1){ 
-					LOG(L_ERR, "ERROR: io_watch_del: fnctl: SETFL" 
+					LM_ERR("fnctl: SETFL" 
 								" failed: %s [%d]\n", strerror(errno), errno); 
 					goto error; 
 				} 
@@ -566,7 +563,7 @@ inline static int io_watch_del(io_wait_h* h, int fd, int idx, int flags)
 #endif
 				n=epoll_ctl(h->epfd, EPOLL_CTL_DEL, fd, &ep_event);
 				if (n==-1){
-					LOG(L_ERR, "ERROR: io_watch_del: removing fd from epoll "
+					LM_ERR("removing fd from epoll "
 							"list failed: %s [%d]\n", strerror(errno), errno);
 					goto error;
 				}
@@ -593,17 +590,15 @@ inline static int io_watch_del(io_wait_h* h, int fd, int idx, int flags)
 again_devpoll:
 				if (write(h->dpoll_fd, &pfd, sizeof(pfd))==-1){
 					if (errno==EINTR) goto again_devpoll;
-					LOG(L_ERR, "ERROR: io_watch_del: removing fd from "
-								"/dev/poll failed: %s [%d]\n", 
-								strerror(errno), errno);
+					LM_ERR("removing fd from /dev/poll failed: "
+						"%s [%d]\n", strerror(errno), errno);
 					goto error;
 				}
 				break;
 #endif
 		default:
-			LOG(L_CRIT, "BUG: io_watch_del: no support for poll method "
-					" %s (%d)\n", poll_method_str[h->poll_method], 
-					h->poll_method);
+			LM_CRIT("no support for poll method %s (%d)\n",
+				poll_method_str[h->poll_method], h->poll_method);
 			goto error;
 	}
 	h->fd_no--;
@@ -631,8 +626,7 @@ again:
 		if (n==-1){
 			if (errno==EINTR) goto again; /* signal, ignore it */
 			else{
-				LOG(L_ERR, "ERROR:io_wait_loop_poll: poll: %s [%d]\n",
-						strerror(errno), errno);
+				LM_ERR("poll: %s [%d]\n", strerror(errno), errno);
 				goto error;
 			}
 		}
@@ -642,8 +636,7 @@ again:
 				/* sanity checks */
 				if ((h->fd_array[r].fd >= h->max_fd_no)||
 						(h->fd_array[r].fd < 0)){
-					LOG(L_CRIT, "BUG: io_wait_loop_poll: bad fd %d "
-							"(no in the 0 - %d range)\n",
+					LM_CRIT("bad fd %d (no in the 0 - %d range)\n",
 							h->fd_array[r].fd, h->max_fd_no);
 					/* try to continue anyway */
 					h->fd_array[r].events=0; /* clear the events */
@@ -675,8 +668,7 @@ again:
 		ret=n=select(h->max_fd_select+1, &sel_set, 0, 0, &timeout);
 		if (n<0){
 			if (errno==EINTR) goto again; /* just a signal */
-			LOG(L_ERR, "ERROR: io_wait_loop_select: select: %s [%d]\n",
-					strerror(errno), errno);
+			LM_ERR("select: %s [%d]\n", strerror(errno), errno);
 			n=0;
 			/* continue */
 		}
@@ -704,8 +696,7 @@ again:
 		if (n==-1){
 			if (errno==EINTR) goto again; /* signal, ignore it */
 			else{
-				LOG(L_ERR, "ERROR:io_wait_loop_epoll: "
-						"epoll_wait(%d, %p, %d, %d): %s [%d]\n", 
+				LM_ERR("epoll_wait(%d, %p, %d, %d): %s [%d]\n", 
 						h->epfd, h->ep_array, h->fd_no, t*1000,
 						strerror(errno), errno);
 				goto error;
@@ -714,7 +705,7 @@ again:
 #if 0
 		if (n>1){
 			for(r=0; r<n; r++){
-				LOG(L_ERR, "WARNING: ep_array[%d]= %x, %p\n",
+				LM_ERR("ep_array[%d]= %x, %p\n",
 						r, h->ep_array[r].events, h->ep_array[r].data.ptr);
 			}
 		}
@@ -724,9 +715,8 @@ again:
 				while((handle_io((struct fd_map*)h->ep_array[r].data.ptr,-1)>0)
 					&& repeat);
 			}else{
-				LOG(L_ERR, "ERROR:io_wait_loop_epoll: unexpected event %x"
-							" on %d/%d, data=%p\n", h->ep_array[r].events,
-							r+1, n, h->ep_array[r].data.ptr);
+				LM_ERR("unexpected event %x on %d/%d, data=%p\n", 
+					h->ep_array[r].events, r+1, n, h->ep_array[r].data.ptr);
 			}
 		}
 error:
@@ -750,15 +740,14 @@ again:
 		if (n==-1){
 			if (errno==EINTR) goto again; /* signal, ignore it */
 			else{
-				LOG(L_ERR, "ERROR: io_wait_loop_kqueue: kevent:"
-						" %s [%d]\n", strerror(errno), errno);
+				LM_ERR("kevent: %s [%d]\n", strerror(errno), errno);
 				goto error;
 			}
 		}
 		h->kq_nchanges=0; /* reset changes array */
 		for (r=0; r<n; r++){
 #ifdef EXTRA_DEBUG
-			DBG("DBG: kqueue: event %d/%d: fd=%d, udata=%lx, flags=0x%x\n",
+			LM_DBG("event %d/%d: fd=%d, udata=%lx, flags=0x%x\n",
 					r, n, h->kq_array[r].ident, (long)h->kq_array[r].udata,
 					h->kq_array[r].flags);
 #endif
@@ -767,8 +756,7 @@ again:
 				   trying to remove an already closed fd: race between
 				   adding smething to the changes array, close() and
 				   applying the changes */
-				LOG(L_INFO, "INFO: io_wait_loop_kqueue: kevent error on "
-							"fd %u: %s [%ld]\n",
+				LM_INFO("kevent error on fd %u: %s [%ld]\n",
 							(unsigned int)h->kq_array[r].ident,
 							strerror(h->kq_array[r].data),
 							(long)h->kq_array[r].data);
@@ -800,8 +788,7 @@ inline static int io_wait_loop_sigio_rt(io_wait_h* h, int t)
 	ts.tv_sec=t;
 	ts.tv_nsec=0;
 	if (!sigismember(&h->sset, h->signo) || !sigismember(&h->sset, SIGIO)){
-		LOG(L_CRIT, "BUG: io_wait_loop_sigio_rt: the signal mask"
-				" is not properly set!\n");
+		LM_CRIT("the signal mask is not properly set!\n");
 		goto error;
 	}
 
@@ -813,8 +800,7 @@ again:
 			ret=0;
 			goto end;
 		}else{
-			LOG(L_ERR, "ERROR: io_wait_loop_sigio_rt: sigtimed_wait"
-					" %s [%d]\n", strerror(errno), errno);
+			LM_ERR("sigtimed_wait %s [%d]\n", strerror(errno), errno);
 			goto error;
 		}
 	}
@@ -838,8 +824,7 @@ again:
 		}
 		if (siginfo.si_code==SI_SIGIO){
 			/* old style, we don't know the event (linux 2.2.?) */
-			LOG(L_WARN, "WARNING: io_wait_loop_sigio_rt: old style sigio"
-					" interface\n");
+			LM_WARN("old style sigio interface\n");
 			fm=get_fd_map(h, sigio_fd);
 			/* we can have queued signals generated by fds not watched
 			 * any more, or by fds in transition, to a child => ignore them*/
@@ -847,7 +832,7 @@ again:
 				handle_io(fm, -1);
 		}else{
 #ifdef EXTRA_DEBUG
-			DBG("io_wait_loop_sigio_rt: siginfo: signal=%d (%d),"
+			LM_DBG("siginfo: signal=%d (%d),"
 					" si_code=%d, si_band=0x%x,"
 					" si_fd=%d\n",
 					siginfo.si_signo, n, siginfo.si_code, 
@@ -865,28 +850,26 @@ again:
 				if (fm->type)
 					handle_io(fm, -1);
 				else
-					LOG(L_ERR, "WARNING: io_wait_loop_sigio_rt: ignoring event"
+					LM_ERR("ignoring event"
 							" %x on fd %d (fm->fd=%d, fm->data=%p)\n",
 							sigio_band, sigio_fd, fm->fd, fm->data);
 			}else{
-				LOG(L_ERR, "ERROR: io_wait_loop_sigio_rt: unexpected event"
-							" on fd %d: %x\n", sigio_fd, sigio_band);
+				LM_ERR("unexpected event on fd %d: %x\n", sigio_fd, sigio_band);
 			}
 		}
 	}else{
 		/* signal queue overflow 
 		 * TODO: increase signal queue size: 2.4x /proc/.., 2.6x -rlimits */
-		LOG(L_WARN, "WARNING: io_wait_loop_sigio_rt: signal queue overflowed"
-					"- falling back to poll\n");
+		LM_WARN("signal queue overflowed- falling back to poll\n");
 		/* clear real-time signal queue
 		 * both SIG_IGN and SIG_DFL are needed , it doesn't work
 		 * only with SIG_DFL  */
 		if (signal(h->signo, SIG_IGN)==SIG_ERR){
-			LOG(L_CRIT, "BUG: do_poll: couldn't reset signal to IGN\n");
+			LM_CRIT("couldn't reset signal to IGN\n");
 		}
 		
 		if (signal(h->signo, SIG_DFL)==SIG_ERR){
-			LOG(L_CRIT, "BUG: do_poll: couldn't reset signal to DFL\n");
+			LM_CRIT("couldn't reset signal to DFL\n");
 		}
 		/* falling back to normal poll */
 		ret=io_wait_loop_poll(h, -1, 1);
@@ -915,15 +898,13 @@ again:
 		if (n==-1){
 			if (errno==EINTR) goto again; /* signal, ignore it */
 			else{
-				LOG(L_ERR, "ERROR:io_wait_loop_devpoll: ioctl: %s [%d]\n",
-						strerror(errno), errno);
+				LM_ERR("ioctl: %s [%d]\n", strerror(errno), errno);
 				goto error;
 			}
 		}
 		for (r=0; r< n; r++){
 			if (h->fd_array[r].revents & (POLLNVAL|POLLERR)){
-				LOG(L_ERR, "ERROR: io_wait_loop_devpoll: pollinval returned"
-							" for fd %d, revents=%x\n",
+				LM_ERR("pollinval returned for fd %d, revents=%x\n",
 							h->fd_array[r].fd, h->fd_array[r].revents);
 			}
 			/* POLLIN|POLLHUP just go through */
