@@ -1,5 +1,5 @@
 /*
- * $Id: fmt.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: fmt.c 2785 2007-09-17 17:07:01Z bogdan_iancu $
  *
  * Copyright (C) 2006 Voice Sistem SRL
  *
@@ -37,7 +37,7 @@ int mi_fmt_init( unsigned int size )
 {
 	mi_fmt_buf = (char*)pkg_malloc(size);
 	if (mi_fmt_buf==NULL) {
-		LOG(L_ERR,"ERROR:mi:mi_fmt_init: no more pkg mem\n");
+		LM_ERR("no more pkg mem\n");
 		return -1;
 	}
 	mi_fmt_buf_len = size;
