@@ -1,5 +1,5 @@
 /*
- * $Id: api.c 2793 2007-09-20 07:47:17Z bogdan_iancu $
+ * $Id: api.c 4180 2008-05-14 10:05:57Z bogdan_iancu $
  *
  * Digest Authentication Module
  *
@@ -206,8 +206,17 @@ auth_result_t pre_auth(struct sip_msg* _m, str* _realm, hdr_types_t _hftype,
 		return ERROR;
 	}
 
+	if (mark_authorized_cred(_m, *_h) < 0) {
+		LM_ERR("failed to mark parsed credentials\n");
+		if (send_resp(_m, 500, &auth_500_err, 0, 0) == -1) {
+			LM_ERR("failed to send 500 reply\n");
+		}
+		return ERROR;
+	}
+
 	if (check_nonce(&c->digest.nonce, &secret) != 0) {
 		LM_DBG("invalid nonce value received\n");
+		c->stale = 1;
 		return STALE_NONCE;
 	}
 
@@ -242,14 +251,6 @@ auth_result_t post_auth(struct sip_msg* _m, struct hdr_field* _h)
 		}
 	}
 
-	if (mark_authorized_cred(_m, _h) < 0) {
-		LM_ERR("failed to mark parsed credentials\n");
-		if (send_resp(_m, 500, &auth_400_err, 0, 0) == -1) {
-			LM_ERR("failed to send 500 reply\n");
-		}
-		res = ERROR;
-	}
-
 	return res;
 }
 
