@@ -1,5 +1,5 @@
 /*
- * $Id: authorize.c 3202 2007-11-23 14:23:11Z henningw $
+ * $Id: authorize.c 4002 2008-04-10 10:14:25Z bogdan_iancu $
  *
  * Digest Authentication - Database support
  *
@@ -258,7 +258,7 @@ static inline int authorize(struct sip_msg* _m, pv_elem_t* _realm,
 	}
 
 	auth_dbf.free_result(auth_db_handle, result);
-	return AUTH_ERROR;
+	return INVALID_PASSWORD;
 }
 
 
