@@ -1,5 +1,5 @@
 /*
- * $Id: avpops.c 1901 2007-03-23 14:46:44Z miconda $
+ * $Id: avpops.c 2845 2007-10-04 11:21:22Z miconda $
  *
  * Copyright (C) 2004-2006 Voice Sistem SRL
  *
@@ -67,7 +67,6 @@ static int fixup_db_store_avp(void** param, int param_no);
 static int fixup_db_query_avp(void** param, int param_no);
 static int fixup_delete_avp(void** param, int param_no);
 static int fixup_copy_avp(void** param, int param_no);
-static int fixup_printf(void** param, int param_no);
 static int fixup_pushto_avp(void** param, int param_no);
 static int fixup_check_avp(void** param, int param_no);
 static int fixup_op_avp(void** param, int param_no);
@@ -82,7 +81,6 @@ static int w_dbquery1_avps(struct sip_msg* msg, char* query, char* param);
 static int w_dbquery2_avps(struct sip_msg* msg, char* query, char* dest);
 static int w_delete_avps(struct sip_msg* msg, char* param, char *foo);
 static int w_copy_avps(struct sip_msg* msg, char* param, char *check);
-static int w_printf(struct sip_msg* msg, char* dest, char *format);
 static int w_pushto_avps(struct sip_msg* msg, char* destination, char *param);
 static int w_check_avps(struct sip_msg* msg, char* param, char *check);
 static int w_op_avps(struct sip_msg* msg, char* param, char *op);
@@ -93,35 +91,33 @@ static int w_is_avp_set(struct sip_msg* msg, char* param, char *foo);
  * Exported functions
  */
 static cmd_export_t cmds[] = {
-	{"avp_print",  w_print_avps, 0, 0,
+	{"avp_print",  w_print_avps, 0, 0, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_db_load",   w_dbload_avps,  2, fixup_db_load_avp,
+	{"avp_db_load",   w_dbload_avps,  2, fixup_db_load_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_db_delete", w_dbdelete_avps, 2, fixup_db_delete_avp,
+	{"avp_db_delete", w_dbdelete_avps, 2, fixup_db_delete_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_db_store",  w_dbstore_avps,  2, fixup_db_store_avp,
+	{"avp_db_store",  w_dbstore_avps,  2, fixup_db_store_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_db_query",  w_dbquery1_avps, 1, fixup_db_query_avp,
+	{"avp_db_query",  w_dbquery1_avps, 1, fixup_db_query_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_db_query",  w_dbquery2_avps, 2, fixup_db_query_avp,
+	{"avp_db_query",  w_dbquery2_avps, 2, fixup_db_query_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_delete", w_delete_avps, 1, fixup_delete_avp,
+	{"avp_delete", w_delete_avps, 1, fixup_delete_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_copy",   w_copy_avps,  2,  fixup_copy_avp,
+	{"avp_copy",   w_copy_avps,  2,  fixup_copy_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_printf", w_printf,  2, fixup_printf,
+	{"avp_pushto", w_pushto_avps, 2, fixup_pushto_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_pushto", w_pushto_avps, 2, fixup_pushto_avp,
+	{"avp_check",  w_check_avps, 2, fixup_check_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_check",  w_check_avps, 2, fixup_check_avp,
+	{"avp_op",     w_op_avps, 2, fixup_op_avp, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_op",     w_op_avps, 2, fixup_op_avp,
+	{"avp_subst",  w_subst,   2, fixup_subst, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"avp_subst",  w_subst,   2, fixup_subst,
+	{"is_avp_set", w_is_avp_set, 1, fixup_is_avp_set, 0,
 					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{"is_avp_set", w_is_avp_set, 1, fixup_is_avp_set,
-					REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE},
-	{0, 0, 0, 0, 0}
+	{0, 0, 0, 0, 0, 0}
 };
 
 
@@ -152,6 +148,7 @@ struct module_exports exports = {
 	0,          /* exported statistics */
 	0,          /* exported MI functions */
 	0,          /* exported pseudo-variables */
+	0,          /* extra processes */
 	avpops_init,/* Module initialization function */
 	(response_function) 0,
 	(destroy_function) 0,
@@ -161,7 +158,7 @@ struct module_exports exports = {
 
 static int avpops_init(void)
 {
-	LOG(L_INFO,"AVPops - initializing\n");
+	LM_INFO("initializing...\n");
 
 	/* if DB_URL defined -> bind to a DB module */
 	if (DB_URL!=0)
@@ -169,8 +166,7 @@ static int avpops_init(void)
 		/* check AVP_TABLE param */
 		if (DB_TABLE==0)
 		{
-			LOG(L_CRIT,"ERROR:avpops_init: \"AVP_DB\" present but "
-				"\"AVP_TABLE\" found empty\n");
+			LM_CRIT("\"AVP_DB\" present but \"AVP_TABLE\" found empty\n");
 			goto error;
 		}
 		/* bind to the DB module */
@@ -205,30 +201,29 @@ static int fixup_db_avp(void** param, int param_no, int allow_scheme)
 	struct db_param  *dbp;
 	int flags;
 	int flags0;
-	char *s;
+	str s;
 	char *p;
 
 	flags=0;
 	flags0=0;
 	if (DB_URL==0)
 	{
-		LOG(L_ERR,"ERROR:avpops:fixup_db_avp: you have to config a db url "
-			"for using avp_db_xxx functions\n");
+		LM_ERR("you have to config a db url for using avp_db_xxx functions\n");
 		return E_UNSPEC;
 	}
 
-	s = (char*)*param;
+	s.s = (char*)*param;
 	if (param_no==1)
 	{
 		/* prepare the fis_param structure */
 		sp = (struct fis_param*)pkg_malloc(sizeof(struct fis_param));
 		if (sp==0) {
-			LOG(L_ERR,"ERROR:avpops:fixup_db_avp: no more pkg mem!\n");
+			LM_ERR("no more pkg mem!\n");
 			return E_OUT_OF_MEM;
 		}
 		memset( sp, 0, sizeof(struct fis_param));
 
-		if ( (p=strchr(s,'/'))!=0)
+		if ( (p=strchr(s.s,'/'))!=0)
 		{
 			*(p++) = 0;
 			/* check for extra flags/params */
@@ -241,35 +236,35 @@ static int fixup_db_avp(void** param, int param_no, int allow_scheme)
 			} else if (!strcasecmp("uuid",p)) {
 				flags|=AVPOPS_FLAG_UUID0;
 			} else {
-				LOG(L_ERR,"ERROR:avpops:fixup_db_avp: unknow flag "
+				LM_ERR("unknow flag "
 					"<%s>\n",p);
 				return E_UNSPEC;
 			}
 		}
-		if (*s!='$')
+		if (*s.s!='$')
 		{
 			/* is a constant string -> use it as uuid*/
 			sp->opd = ((flags==0)?AVPOPS_FLAG_UUID0:flags)|AVPOPS_VAL_STR;
-			sp->sval.p.val.s = (char*)pkg_malloc(strlen(s)+1);
-			if (sp->sval.p.val.s==0) {
-				LOG(L_ERR,"ERROR:avpops:fixup_db_avp: no more pkg mem!!\n");
+			sp->u.s.s = (char*)pkg_malloc(strlen(s.s)+1);
+			if (sp->u.s.s==0) {
+				LM_ERR("no more pkg mem!!\n");
 				return E_OUT_OF_MEM;
 			}
-			sp->sval.p.val.len = strlen(s);
-			strcpy(sp->sval.p.val.s,s);
+			sp->u.s.len = strlen(s.s);
+			strcpy(sp->u.s.s, s.s);
 		} else {
 			/* is a variable $xxxxx */
-			p = xl_parse_spec(s, &sp->sval,
-					XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
-			if (p==0 || sp->sval.type==XL_NULL || sp->sval.type==XL_EMPTY)
+			s.len = strlen(s.s);
+			p = pv_parse_spec(&s, &sp->u.sval);
+			if (p==0 || sp->u.sval.type==PVT_NULL || sp->u.sval.type==PVT_EMPTY)
 			{
-				LOG(L_ERR,"ERROR:avops:fixup_db_avp: bad param 1; "
+				LM_ERR("bad param 1; "
 					"expected : $pseudo-variable or int/str value\n");
 				return E_UNSPEC;
 			}
 			
-			if(sp->sval.type==XL_RURI || sp->sval.type==XL_FROM
-					|| sp->sval.type==XL_TO || sp->sval.type==XL_OURI)
+			if(sp->u.sval.type==PVT_RURI || sp->u.sval.type==PVT_FROM
+					|| sp->u.sval.type==PVT_TO || sp->u.sval.type==PVT_OURI)
 			{
 				sp->opd = ((flags==0)?AVPOPS_FLAG_URI0:flags)|AVPOPS_VAL_PVAR;
 			} else {
@@ -282,13 +277,13 @@ static int fixup_db_avp(void** param, int param_no, int allow_scheme)
 		dbp = (struct db_param*)pkg_malloc(sizeof(struct db_param));
 		if (dbp==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_db_avp: no more pkg mem!!!\n");
+			LM_ERR("no more pkg mem!!!\n");
 			return E_OUT_OF_MEM;
 		}
 		memset( dbp, 0, sizeof(struct db_param));
-		if ( parse_avp_db( s, dbp, allow_scheme)!=0 )
+		if ( parse_avp_db( s.s, dbp, allow_scheme)!=0 )
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_db_avp: parse failed\n");
+			LM_ERR("parse failed\n");
 			return E_UNSPEC;
 		}
 		*param=(void*)dbp;
@@ -316,49 +311,46 @@ static int fixup_db_store_avp(void** param, int param_no)
 
 static int fixup_db_query_avp(void** param, int param_no)
 {
-	xl_elem_t *model = NULL;
-	itemname_list_t *anlist = NULL;
-	char *s;
+	pv_elem_t *model = NULL;
+	pvname_list_t *anlist = NULL;
+	str s;
 
 	if (DB_URL==0)
 	{
-		LOG(L_ERR,"ERROR:avpops:fixup_db_query_avp: you have to config a"
-				" db url for using avp_db_query function\n");
+		LM_ERR("you have to config a db url for using avp_db_query function\n");
 		return E_UNSPEC;
 	}
 
-	s = (char*)(*param);
+	s.s = (char*)(*param);
 	if (param_no==1)
 	{
-		if(s==NULL)
+		if(s.s==NULL)
 		{
-			LOG(L_ERR, "ERROR:avpops:fixup_db_query_avp: null format in P%d\n",
+			LM_ERR("null format in P%d\n",
 					param_no);
 			return E_UNSPEC;
 		}
-		if(xl_parse_format(s, &model, XL_DISABLE_COLORS|XL_THROW_ERROR)<0)
+		s.len = strlen(s.s);
+		if(pv_parse_format(&s, &model)<0)
 		{
-			LOG(L_ERR,
-				"ERROR:avpops:fixup_db_query_avp: wrong format[%s]\n",
-				s);
+			LM_ERR("wrong format[%s]\n", s.s);
 			return E_UNSPEC;
 		}
 			
 		*param = (void*)model;
 		return 0;
 	} else if(param_no==2) {
-		if(s==NULL)
+		if(s.s==NULL)
 		{
-			LOG(L_ERR, "ERROR:avpops:fixup_db_query_avp: null format in P%d\n",
-					param_no);
+			LM_ERR("null format in P%d\n", param_no);
 			return E_UNSPEC;
 		}
-		anlist = parse_itemname_list(s, XL_AVP);
+				s.len = strlen(s.s);
+
+		anlist = parse_pvname_list(&s, PVT_AVP);
 		if(anlist==NULL)
 		{
-			LOG(L_ERR,
-				"ERROR:avpops:fixup_db_query_avp: bad format in P%d [%s]\n",
-				param_no, s);
+			LM_ERR("bad format in P%d [%s]\n", param_no, s.s);
 			return E_UNSPEC;
 		}
 		*param = (void*)anlist;
@@ -386,33 +378,30 @@ static int fixup_delete_avp(void** param, int param_no)
 		if(*s=='$')
 		{
 			/* is variable */
-			ap = avpops_parse_pvar(s, 
-				XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
+			ap = avpops_parse_pvar(s);
 			if (ap==0)
 			{
-				LOG(L_ERR,"ERROR:avpops:fixup_delete_avp: unable to get"
+				LM_ERR("unable to get"
 					" pseudo-variable in param \n");
 				return E_UNSPEC;
 			}
-			if (ap->sval.type!=XL_AVP)
+			if (ap->u.sval.type!=PVT_AVP)
 			{
-				LOG(L_ERR,"ERROR:avops:fixup_delete_avp: bad param; "
-					"expected : $avp(name)\n");
+				LM_ERR("bad param; expected : $avp(name)\n");
 				return E_UNSPEC;
 			}
 			ap->opd|=AVPOPS_VAL_PVAR;
+			ap->type = AVPOPS_VAL_PVAR;
 		} else {
 			if(strlen(s)<1)
 			{
-				LOG(L_ERR,"ERROR:avops:parse_avp_db: bad param - "
-					"expected : $avp(name), *, s or i value\n");
+				LM_ERR("bad param - expected : $avp(name), *, s or i value\n");
 				return E_UNSPEC;
 			}
 			ap = (struct fis_param*)pkg_malloc(sizeof(struct fis_param));
 			if (ap==0)
 			{
-				LOG(L_ERR,"ERROR:avpops:fixup_delete_avp: no more "
-					"pkg mem\n");
+				LM_ERR(" no more pkg mem\n");
 				return E_OUT_OF_MEM;
 			}
 			memset(ap, 0, sizeof(struct fis_param));
@@ -428,8 +417,7 @@ static int fixup_delete_avp(void** param, int param_no)
 					ap->opd = AVPOPS_VAL_NONE;
 				break;
 				default:
-					LOG(L_ERR,"ERROR:avpops:fixup_delete_avp: bad param - "
-						"expected : *, s or i AVP flag\n");
+					LM_ERR(" bad param - expected : *, s or i AVP flag\n");
 					pkg_free(ap);
 					return E_UNSPEC;
 			}
@@ -441,13 +429,13 @@ static int fixup_delete_avp(void** param, int param_no)
 				s0.len = strlen(s0.s);
 				if(str2int(&s0, &flags)!=0)
 				{
-					LOG(L_ERR,
-					"ERROR:avpops:fixup_delete_avp: error - bad avp flags\n");
+					LM_ERR("bad avp flags\n");
 					pkg_free(ap);
 					return E_UNSPEC;
 				}
 			}
-			ap->sval.flags |= flags<<24;
+			ap->type = AVPOPS_VAL_INT;
+			ap->u.n = flags<<8;
 		}
 
 		/* flags */
@@ -460,8 +448,7 @@ static int fixup_delete_avp(void** param, int param_no)
 					ap->ops|=AVPOPS_FLAG_ALL;
 					break;
 				default:
-					LOG(L_ERR,"ERROR:avpops:fixup_delete_avp: bad flag "
-						"<%c>\n",*p);
+					LM_ERR(" bad flag <%c>\n",*p);
 					if(ap!=NULL)
 						pkg_free(ap);
 					return E_UNSPEC;
@@ -494,20 +481,17 @@ static int fixup_copy_avp(void** param, int param_no)
 			*(p++)=0;
 	}
 
-	ap = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
+	ap = avpops_parse_pvar(s);
 	if (ap==0)
 	{
-		LOG(L_ERR,"ERROR:avpops:fixup_copy_avp: unable to get"
-				" pseudo-variable in P%d\n", param_no);
+		LM_ERR("unable to get pseudo-variable in P%d\n", param_no);
 		return E_OUT_OF_MEM;
 	}
 
 	/* attr name is mandatory */
-	if (ap->sval.type!=XL_AVP)
+	if (ap->u.sval.type!=PVT_AVP)
 	{
-		LOG(L_ERR,"ERROR:avpops:fixup_copy_avp: you must specify "
-			"only AVP as parameter\n");
+		LM_ERR("you must specify only AVP as parameter\n");
 		return E_UNSPEC;
 	}
 
@@ -534,8 +518,7 @@ static int fixup_copy_avp(void** param, int param_no)
 					ap->ops|=AVPOPS_FLAG_CASTS;
 					break;
 				default:
-					LOG(L_ERR,"ERROR:avpops:fixup_copy_avp: bad flag "
-						"<%c>\n",*p);
+					LM_ERR("bad flag <%c>\n",*p);
 					return E_UNSPEC;
 			}
 		}
@@ -558,25 +541,22 @@ static int fixup_pushto_avp(void** param, int param_no)
 	{
 		if ( *s!='$')
 		{
-			LOG(L_ERR,"ERROR:avops:fixup_pushto_avp: bad param 1; expected : "
-				"$ru $du ...\n");
+			LM_ERR("bad param 1; expected : $ru $du ...\n");
 			return E_UNSPEC;
 		}
 		/* compose the param structure */
 
 		if ( (p=strchr(s,'/'))!=0 )
 			*(p++)=0;
-		ap = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
+		ap = avpops_parse_pvar(s);
 		if (ap==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_pushto_avp: unable to get"
-					" pseudo-variable in param 1\n");
+			LM_ERR("unable to get pseudo-variable in param 1\n");
 			return E_OUT_OF_MEM;
 		}
 
-		switch(ap->sval.type) {
-			case XL_RURI:
+		switch(ap->u.sval.type) {
+			case PVT_RURI:
 				ap->opd = AVPOPS_VAL_NONE|AVPOPS_USE_RURI;
 				if ( p && !(
 					(!strcasecmp("username",p)
@@ -584,65 +564,57 @@ static int fixup_pushto_avp(void** param, int param_no)
 					(!strcasecmp("domain",p)
 							&& (ap->opd|=AVPOPS_FLAG_DOMAIN0)) ))
 				{
-					LOG(L_ERR,"ERROR:avpops:fixup_pushto_avp: unknown "
-						" ruri flag \"%s\"!\n",p);
+					LM_ERR("unknown ruri flag \"%s\"!\n",p);
 					return E_UNSPEC;
 				}
 			break;
-			case XL_DSTURI:
+			case PVT_DSTURI:
 				if ( p!=0 )
 				{
-					LOG(L_ERR,"ERROR:avpops:fixup_pushto_avp: unknown "
-						" duri flag \"%s\"!\n",p);
+					LM_ERR("unknown duri flag \"%s\"!\n",p);
 					return E_UNSPEC;
 				}
 				ap->opd = AVPOPS_VAL_NONE|AVPOPS_USE_DURI;
 			break;
-			case XL_HDR:
+			case PVT_HDR:
 				/* what's the hdr destination ? request or reply? */
-				LOG(L_ERR,"ERROR:avpops:fixup_pushto_avp: push to header "
-						" is obsoleted - use append_hf() or append_to_reply()"
-						" from textops module!\n");
+				LM_ERR("push to header  is obsoleted - use append_hf() "
+						"or append_to_reply() from textops module!\n");
 				return E_UNSPEC;
 			break;
-			case XL_BRANCH:
+			case PVT_BRANCH:
 				if ( p!=0 )
 				{
-					LOG(L_ERR,"ERROR:avpops:fixup_pushto_avp: unknown "
-						" branch flag \"%s\"!\n",p);
+					LM_ERR("unknown branch flag \"%s\"!\n",p);
 					return E_UNSPEC;
 				}
 				ap->opd = AVPOPS_VAL_NONE|AVPOPS_USE_BRANCH;
 			break;
 			default:
-				LOG(L_ERR,"ERROR:avpops:fixup_pushto_avp: unsupported "
-					" destination \"%s\"; expected $ru,$du,$br\n",s);
+				LM_ERR("unsupported destination \"%s\"; "
+						"expected $ru,$du,$br\n",s);
 				return E_UNSPEC;
 		}
 	} else if (param_no==2) {
 		/* attribute name*/
 		if ( *s!='$')
 		{
-			LOG(L_ERR,"ERROR:avops:fixup_pushto_avp: bad param 1; expected : "
-				"$pseudo-variable ...\n");
+			LM_ERR("bad param 1; expected :$pseudo-variable ...\n");
 			return E_UNSPEC;
 		}
 		/* compose the param structure */
 
 		if ( (p=strchr(s,'/'))!=0 )
 			*(p++)=0;
-		ap = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_COLORS);
+		ap = avpops_parse_pvar(s);
 		if (ap==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_pudhto_avp: unable to get"
-					" pseudo-variable in param 2\n");
+			LM_ERR("unable to get pseudo-variable in param 2\n");
 			return E_OUT_OF_MEM;
 		}
-		if (ap->sval.type==XL_NULL)
+		if (ap->u.sval.type==PVT_NULL)
 		{
-			LOG(L_ERR,"ERROR:avops:fixup_pushto_avp: bad param 2; "
-				"expected : $pseudo-variable ...\n");
+			LM_ERR("bad param 2; expected : $pseudo-variable ...\n");
 			pkg_free(ap);
 			return E_UNSPEC;
 		}
@@ -657,8 +629,7 @@ static int fixup_pushto_avp(void** param, int param_no)
 					ap->ops|=AVPOPS_FLAG_ALL;
 					break;
 				default:
-					LOG(L_ERR,"ERROR:avpops:fixup_pushto_avp: bad flag "
-						"<%c>\n",*p);
+					LM_ERR("bad flag <%c>\n",*p);
 					pkg_free(ap);
 					return E_UNSPEC;
 			}
@@ -680,26 +651,22 @@ static int fixup_check_avp(void** param, int param_no)
 
 	if (param_no==1)
 	{
-		ap = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
+		ap = avpops_parse_pvar(s);
 		if (ap==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_check_avp: unable to get"
-					" pseudo-variable in P1\n");
+			LM_ERR(" unable to get pseudo-variable in P1\n");
 			return E_OUT_OF_MEM;
 		}
 		/* attr name is mandatory */
-		if (ap->sval.type==XL_NULL)
+		if (ap->u.sval.type==PVT_NULL)
 		{
-			LOG(L_ERR,
-				"ERROR:avpops:fixup_check_avp: NULL pseudo-variable in P1\n");
+			LM_ERR("null pseudo-variable in P1\n");
 			return E_UNSPEC;
 		}
 	} else if (param_no==2) {
 		if ( (ap=parse_check_value(s))==0 )
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_check_avp: failed to parse "
-				"checked value \n");
+			LM_ERR(" failed to parse checked value \n");
 			return E_UNSPEC;
 		}
 		/* if REGEXP op -> compile the expresion */
@@ -707,36 +674,32 @@ static int fixup_check_avp(void** param, int param_no)
 		{
 			if ( (ap->opd&AVPOPS_VAL_STR)==0 )
 			{
-				LOG(L_ERR,"ERROR:avpops:fixup_check_avp: regexp operation "
-					"requires string value\n");
+				LM_ERR(" regexp operation requires string value\n");
 				return E_UNSPEC;
 			}
 			re = pkg_malloc(sizeof(regex_t));
 			if (re==0)
 			{
-				LOG(L_ERR,"ERROR:avpops:fixup_check_avp: no more pkg mem\n");
+				LM_ERR(" no more pkg mem\n");
 				return E_OUT_OF_MEM;
 			}
-			DBG("DEBUG:avpops:fixup_check_avp: compiling regexp <%.*s>\n",
-				ap->sval.p.val.len, ap->sval.p.val.s);
-			if (regcomp(re, ap->sval.p.val.s,
+			LM_DBG("compiling regexp <%.*s>\n", ap->u.s.len, ap->u.s.s);
+			if (regcomp(re, ap->u.s.s,
 						REG_EXTENDED|REG_ICASE|REG_NEWLINE))
 			{
 				pkg_free(re);
-				LOG(L_ERR,"ERROR:avpops:fixip_check_avp: bad re <%.*s>\n",
-					ap->sval.p.val.len, ap->sval.p.val.s);
+				LM_ERR("bad re <%.*s>\n", ap->u.s.len, ap->u.s.s);
 				return E_BAD_RE;
 			}
 			/* free the string and link the regexp */
 			// pkg_free(ap->sval.p.s);
-			ap->sval.p.val.s = (char*)re;
+			ap->u.s.s = (char*)re;
 		} else if (ap->ops&AVPOPS_OP_FM) {
 			if ( !( ap->opd&AVPOPS_VAL_PVAR ||
 			(!(ap->opd&AVPOPS_VAL_PVAR) && ap->opd&AVPOPS_VAL_STR) ) )
 			{
-				LOG(L_ERR,"ERROR:avpops:fixup_check_avp: fast_match operation "
-					"requires string value or avp name/alias (%d/%d)\n",
-					ap->opd, ap->ops);
+				LM_ERR(" fast_match operation requires string value or "
+						"avp name/alias (%d/%d)\n",	ap->opd, ap->ops);
 				return E_UNSPEC;
 			}
 		}
@@ -746,54 +709,6 @@ static int fixup_check_avp(void** param, int param_no)
 	return 0;
 }
 
-static int fixup_printf(void** param, int param_no)
-{
-	struct fis_param *ap;
-	xl_elem_t *model;
-	char *s;
-
-	s = (char*)(*param);
-	if (param_no==1) {
-		/* attribute name / alias */
-		ap = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
-		if (ap==0)
-		{
-			LOG(L_ERR,"ERROR:avpops:fixup_printf_avp: unable to get"
-					" pseudo-variable in param\n");
-			return E_OUT_OF_MEM;
-		}
-
-		if (ap->sval.type!=XL_AVP)
-		{
-			LOG(L_ERR,"ERROR:avpops:fixup_printf: bad avp name"
-				" <%s>\n", (char*)*param);
-			return E_UNSPEC;
-		}
-		*param=(void*)ap;
-	} else if (param_no==2) {
-		if(*param)
-		{
-			if(xl_parse_format((char*)(*param), &model, XL_DISABLE_COLORS)<0)
-			{
-				LOG(L_ERR, "ERROR:avpops:fixup_printf: wrong format[%s]\n",
-					(char*)(*param));
-				return E_UNSPEC;
-			}
-			
-			*param = (void*)model;
-			return 0;
-		}
-		else
-		{
-			LOG(L_ERR, "ERROR:avpops:fixup_printf: null format\n");
-			return E_UNSPEC;
-		}
-	}
-
-	return 0;
-}
-
 static int fixup_subst(void** param, int param_no)
 {
 	struct subst_expr* se;
@@ -810,7 +725,7 @@ static int fixup_subst(void** param, int param_no)
 		av = (struct fis_param**)pkg_malloc(2*sizeof(struct fis_param*));
 		if(av==NULL)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_subst: no more memory\n");
+			LM_ERR("no more pkg memory\n");
 			return E_UNSPEC;			
 		}
 		memset(av, 0, 2*sizeof(struct fis_param*));
@@ -818,26 +733,22 @@ static int fixup_subst(void** param, int param_no)
 		/* avp src / avp dst /flags */
 		if ( (p=strchr(s,'/'))!=0 )
 			*(p++)=0;
-		ap = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
+		ap = avpops_parse_pvar(s);
 		if (ap==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_subst: unable to get"
-					" pseudo-variable in param 2 [%s]\n", s);
+			LM_ERR("unable to get pseudo-variable in param 2 [%s]\n", s);
 			return E_OUT_OF_MEM;
 		}
-		if (ap->sval.type!=XL_AVP)
+		if (ap->u.sval.type!=PVT_AVP)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_subst: bad attribute name"
-				" <%s>\n", (char*)*param);
+			LM_ERR("bad attribute name <%s>\n", (char*)*param);
 			pkg_free(av);
 			return E_UNSPEC;
 		}
 		/* attr name is mandatory */
 		if (ap->opd&AVPOPS_VAL_NONE)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_subst: you must specify "
-				"a name for the AVP\n");
+			LM_ERR("you must specify a name for the AVP\n");
 			return E_UNSPEC;
 		}
 		av[0] = ap;
@@ -849,33 +760,29 @@ static int fixup_subst(void** param, int param_no)
 		
 		/* dst || flags */
 		s = p;
-		if(*s==ITEM_MARKER)
+		if(*s==PV_MARKER)
 		{
 			if ( (p=strchr(s,'/'))!=0 )
 				*(p++)=0;
 			if(p==0 || (p!=0 && p-s>1))
 			{
-				ap = avpops_parse_pvar(s, 
-					XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
+				ap = avpops_parse_pvar(s);
 				if (ap==0)
 				{
-					LOG(L_ERR,"ERROR:avpops:fixup_subst: unable to get"
-						" pseudo-variable in param 2 [%s]\n",s);
+					LM_ERR("unable to get pseudo-variable in param 2 [%s]\n",s);
 					return E_OUT_OF_MEM;
 				}
 			
-				if (ap->sval.type!=XL_AVP)
+				if (ap->u.sval.type!=PVT_AVP)
 				{
-					LOG(L_ERR,"ERROR:avpops:fixup_subst: bad attribute name"
-						" <%s>!\n", s);
+					LM_ERR("bad attribute name <%s>!\n", s);
 					pkg_free(av);
 					return E_UNSPEC;
 				}
 				/* attr name is mandatory */
 				if (ap->opd&AVPOPS_VAL_NONE)
 				{
-					LOG(L_ERR,"ERROR:avpops:fixup_subst: you must specify "
-						"a name for the AVP!\n");
+					LM_ERR("you must specify a name for the AVP!\n");
 					return E_UNSPEC;
 				}
 				av[1] = ap;
@@ -900,20 +807,18 @@ static int fixup_subst(void** param, int param_no)
 					av[0]->ops|=AVPOPS_FLAG_DELETE;
 					break;
 				default:
-					LOG(L_ERR,"ERROR:avpops:fixup_subst: bad flag "
-						"<%c>\n",*p);
+					LM_ERR("bad flag <%c>\n",*p);
 					return E_UNSPEC;
 			}
 		}
 		*param=(void*)av;
 	} else if (param_no==2) {
-		DBG("%s:fixup_subst: fixing %s\n", exports.name, (char*)(*param));
+		LM_DBG("%s fixing %s\n", exports.name, (char*)(*param));
 		subst.s=*param;
 		subst.len=strlen(*param);
 		se=subst_parser(&subst);
 		if (se==0){
-			LOG(L_ERR, "ERROR:%s:fixup_subst: bad subst re %s\n",exports.name, 
-					(char*)*param);
+			LM_ERR("%s: bad subst re %s\n",exports.name, (char*)*param);
 			return E_BAD_RE;
 		}
 		/* don't free string -- needed for specifiers */
@@ -940,7 +845,7 @@ static int fixup_op_avp(void** param, int param_no)
 		av = (struct fis_param**)pkg_malloc(2*sizeof(struct fis_param*));
 		if(av==NULL)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_op_avp: no more memory\n");
+			LM_ERR("no more pkg memory\n");
 			return E_UNSPEC;			
 		}
 		memset(av, 0, 2*sizeof(struct fis_param*));
@@ -948,18 +853,15 @@ static int fixup_op_avp(void** param, int param_no)
 		if ( (p=strchr(s,'/'))!=0 )
 			*(p++)=0;
 
-		av[0] = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
+		av[0] = avpops_parse_pvar(s);
 		if (av[0]==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_op_avp: unable to get"
-					" pseudo-variable in param 1\n");
+			LM_ERR("unable to get pseudo-variable in param 1\n");
 			return E_OUT_OF_MEM;
 		}
-		if (av[0]->sval.type!=XL_AVP)
+		if (av[0]->u.sval.type!=PVT_AVP)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_op_avp: bad attribute name"
-				" <%s>\n", (char*)*param);
+			LM_ERR("bad attribute name <%s>\n", (char*)*param);
 			pkg_free(av);
 			return E_UNSPEC;
 		}
@@ -970,18 +872,15 @@ static int fixup_op_avp(void** param, int param_no)
 		}
 		
 		s = p;
-		ap = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS);
+		ap = avpops_parse_pvar(s);
 		if (ap==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_op_avp: unable to get"
-					" pseudo-variable in param 1 (2)\n");
+			LM_ERR("unable to get pseudo-variable in param 1 (2)\n");
 			return E_OUT_OF_MEM;
 		}
-		if (ap->sval.type!=XL_AVP)
+		if (ap->u.sval.type!=PVT_AVP)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_op_avp: bad attribute name"
-				"/alias <%s>!\n", s);
+			LM_ERR("bad attribute name/alias <%s>!\n", s);
 			pkg_free(av);
 			return E_UNSPEC;
 		}
@@ -991,15 +890,13 @@ static int fixup_op_avp(void** param, int param_no)
 	} else if (param_no==2) {
 		if ( (ap=parse_op_value(s))==0 )
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_op_avp: failed to parse "
-				"the value \n");
+			LM_ERR("failed to parse the value \n");
 			return E_UNSPEC;
 		}
 		/* only integer values or avps */
 		if ( (ap->opd&AVPOPS_VAL_STR)!=0 && (ap->opd&AVPOPS_VAL_PVAR)==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_op_avp: operations "
-				"requires integer values\n");
+			LM_ERR("operations requires integer values\n");
 			return E_UNSPEC;
 		}
 		*param=(void*)ap;
@@ -1020,19 +917,16 @@ static int fixup_is_avp_set(void** param, int param_no)
 		if ( (p=strchr(s,'/'))!=0 )
 			*(p++)=0;
 		
-		ap = avpops_parse_pvar(s, 
-			XL_THROW_ERROR|XL_DISABLE_COLORS);
+		ap = avpops_parse_pvar(s);
 		if (ap==0)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_is_avp_set: unable to get"
-					" pseudo-variable in param\n");
+			LM_ERR("unable to get pseudo-variable in param\n");
 			return E_OUT_OF_MEM;
 		}
 		
-		if (ap->sval.type!=XL_AVP)
+		if (ap->u.sval.type!=PVT_AVP)
 		{
-			LOG(L_ERR,"ERROR:avpops:fixup_is_avp_set: bad attribute name"
-					" <%s>\n", (char*)*param);
+			LM_ERR("bad attribute name <%s>\n", (char*)*param);
 			return E_UNSPEC;
 		}
 		if(p==0 || *p=='\0')
@@ -1050,8 +944,7 @@ static int fixup_is_avp_set(void** param, int param_no)
 				case 'N':
 					if(ap->ops&AVPOPS_FLAG_CASTS)
 					{
-						LOG(L_ERR,"ERROR:avpops:fixup_is_avp_set: invalid flag"
-							" combination <%c> and 's|S'\n",*p);
+						LM_ERR("invalid flag combination <%c> and 's|S'\n",*p);
 						return E_UNSPEC;
 					}
 					ap->ops|=AVPOPS_FLAG_CASTN;
@@ -1060,15 +953,13 @@ static int fixup_is_avp_set(void** param, int param_no)
 				case 'S':
 					if(ap->ops&AVPOPS_FLAG_CASTN)
 					{
-						LOG(L_ERR,"ERROR:avpops:fixup_is_avp_set: invalid flag"
-							" combination <%c> and 'n|N'\n",*p);
+						LM_ERR("invalid flag combination <%c> and 'n|N'\n",*p);
 						return E_UNSPEC;
 					}
 					ap->ops|=AVPOPS_FLAG_CASTS;
 					break;
 				default:
-					LOG(L_ERR,"ERROR:avpops:fixup_is_avp_set: bad flag "
-						"<%c>\n",*p);
+					LM_ERR("bad flag <%c>\n",*p);
 					return E_UNSPEC;
 			}
 		}
@@ -1099,12 +990,12 @@ static int w_dbstore_avps(struct sip_msg* msg, char* source, char* param)
 
 static int w_dbquery1_avps(struct sip_msg* msg, char* query, char* param)
 {
-	return ops_dbquery_avps ( msg, (xl_elem_t*)query, 0);
+	return ops_dbquery_avps ( msg, (pv_elem_t*)query, 0);
 }
 
 static int w_dbquery2_avps(struct sip_msg* msg, char* query, char* dest)
 {
-	return ops_dbquery_avps ( msg, (xl_elem_t*)query, (itemname_list_t*)dest);
+	return ops_dbquery_avps ( msg, (pv_elem_t*)query, (pvname_list_t*)dest);
 }
 
 static int w_delete_avps(struct sip_msg* msg, char* param, char* foo)
@@ -1118,11 +1009,6 @@ static int w_copy_avps(struct sip_msg* msg, char* name1, char *name2)
 								(struct fis_param*)name2);
 }
 
-static int w_printf(struct sip_msg* msg, char* dest, char *format)
-{
-	return ops_printf(msg, (struct fis_param*)dest, (xl_elem_t*)format);
-}
-
 static int w_pushto_avps(struct sip_msg* msg, char* destination, char *param)
 {
 	return ops_pushto_avp ( msg, (struct fis_param*)destination,
