@@ -1,5 +1,5 @@
 /*
- * $Id: cpl.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: cpl.c 3129 2007-11-13 22:03:48Z bogdan_iancu $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -49,7 +49,7 @@
 #include "../../ut.h"
 #include "../../dprint.h"
 #include "../../data_lump_rpl.h"
-#include "../../items.h"
+#include "../../pvar.h"
 #include "../../parser/parse_uri.h"
 #include "../../parser/parse_from.h"
 #include "../../parser/parse_content.h"
@@ -75,7 +75,6 @@ static char *DB_URL        = 0;  /* database url */
 static char *DB_TABLE      = 0;  /* */
 static char *dtd_file      = 0;  /* name of the DTD file for CPL parser */
 static char *lookup_domain = 0;
-static pid_t aux_process   = 0;  /* pid of the private aux. process */
 static char *timer_avp     = 0;  /* name of variable timer AVP */
 
 
@@ -107,9 +106,19 @@ static int w_process_register_norpl(struct sip_msg* msg, char* str,char* str2);
 static int cpl_process_register(struct sip_msg* msg, int no_rpl);
 static int fixup_cpl_run_script(void** param, int param_no);
 static int cpl_init(void);
-static int mi_child_init();
+static int mi_child_init(void);
 static int cpl_child_init(int rank);
 static int cpl_exit(void);
+static void cpl_process(int rank);
+
+
+/*
+ * Exported processes
+ */
+static proc_export_t cpl_procs[] = {
+	{"CPL Aux",  0,  0,  cpl_process, 1 },
+	{0,0,0,0,0}
+};
 
 
 /*
@@ -117,12 +126,12 @@ static int cpl_exit(void);
  */
 static cmd_export_t cmds[] = {
 	{"cpl_run_script",            cpl_invoke_script,        2,
-			fixup_cpl_run_script, REQUEST_ROUTE},
+			fixup_cpl_run_script, 0, REQUEST_ROUTE},
 	{"cpl_process_register",      w_process_register,       0,
-			0,                    REQUEST_ROUTE},
+			0, 0,                    REQUEST_ROUTE},
 	{"cpl_process_register_norpl",w_process_register_norpl, 0,
-			0,                    REQUEST_ROUTE},
-	{0, 0, 0, 0, 0}
+			0, 0,                    REQUEST_ROUTE},
+	{0, 0, 0, 0, 0, 0}
 };
 
 
@@ -157,7 +166,7 @@ static mi_export_t mi_cmds[] = {
 	{ "LOAD_CPL",   mi_cpl_load,     0,  0,  mi_child_init },
 	{ "REMOVE_CPL", mi_cpl_remove,   0,  0,  0             },
 	{ "GET_CPL",    mi_cpl_get,      0,  0,  0             },
-	{ 0, 0, 0, 0}
+	{ 0, 0, 0, 0, 0}
 };
 
 
@@ -171,6 +180,7 @@ struct module_exports exports = {
 	0,        /* exported statistics */
 	mi_cmds,  /* exported MI functions */
 	0,        /* exported pseudo-variables */
+	cpl_procs,/* extra processes */
 	cpl_init, /* Module initialization function */
 	(response_function) 0,
 	(destroy_function) cpl_exit,
@@ -189,8 +199,7 @@ static int fixup_cpl_run_script(void** param, int param_no)
 		else if (!strcasecmp( "outgoing", *param))
 			flag = CPL_RUN_OUTGOING;
 		else {
-			LOG(L_ERR,"ERROR:fixup_cpl_run_script: script directive \"%s\""
-				" unknown!\n",(char*)*param);
+			LM_ERR("script directive \"%s\" unknown!\n",(char*)*param);
 			return E_UNSPEC;
 		}
 		pkg_free(*param);
@@ -204,8 +213,7 @@ static int fixup_cpl_run_script(void** param, int param_no)
 		} else if ( !strcasecmp("force_stateful", *param) ) {
 			flag = CPL_FORCE_STATEFUL;
 		} else {
-			LOG(L_ERR,"ERROR:fixup_cpl_run_script: flag \"%s\" (second param)"
-				" unknown!\n",(char*)*param);
+			LM_ERR("flag \"%s\" (second param) unknown!\n",(char*)*param);
 			return E_UNSPEC;
 		}
 		pkg_free(*param);
@@ -222,26 +230,25 @@ static int cpl_init(void)
 	struct stat   stat_t;
 	char *ptr;
 	int val;
-	xl_spec_t avp_spec;
+	pv_spec_t avp_spec;
 	unsigned short avp_type;
+	str stmp;
 
-	LOG(L_INFO,"CPL - initializing\n");
+	LM_INFO("initializing...\n");
 
 	/* check the module params */
 	if (DB_URL==0) {
-		LOG(L_CRIT,"ERROR:cpl_init: mandatory parameter \"DB_URL\" "
-			"found empty\n");
+		LM_CRIT("mandatory parameter \"DB_URL\" found empty\n");
 		goto error;
 	}
 
 	if (DB_TABLE==0) {
-		LOG(L_CRIT,"ERROR:cpl_init: mandatory parameter \"DB_TABLE\" "
-			"found empty\n");
+		LM_CRIT("mandatory parameter \"DB_TABLE\" found empty\n");
 		goto error;
 	}
 
 	if (cpl_env.proxy_recurse>MAX_PROXY_RECURSE) {
-		LOG(L_CRIT,"ERROR:cpl_init: value of proxy_recurse param (%d) exceeds "
+		LM_CRIT("value of proxy_recurse param (%d) exceeds "
 			"the maximum safety value (%d)\n",
 			cpl_env.proxy_recurse,MAX_PROXY_RECURSE);
 		goto error;
@@ -249,85 +256,78 @@ static int cpl_init(void)
 
 	/* fix the timer_avp name */
 	if (timer_avp && *timer_avp) {
-		if (xl_parse_spec(timer_avp, &avp_spec,
-					XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS)==0
-				|| avp_spec.type!=XL_AVP) {
-			LOG(L_ERR, "ERROR:cpl_init: malformed or non AVP %s "
-				"AVP definition\n", timer_avp);
+		stmp.s = timer_avp; stmp.len = strlen(stmp.s);
+		if (pv_parse_spec(&stmp, &avp_spec)==0
+				|| avp_spec.type!=PVT_AVP) {
+			LM_ERR("malformed or non AVP %s AVP definition\n", timer_avp);
 			return -1;
 		}
 
-		if(xl_get_avp_name(0, &avp_spec, &cpl_env.timer_avp, &avp_type)!=0)
+		if(pv_get_avp_name(0, &(avp_spec.pvp), &cpl_env.timer_avp,
+							&avp_type)!=0)
 		{
-			LOG(L_ERR, "ERROR:cpl_init: [%s]- invalid "
-				"AVP definition\n", timer_avp);
+			LM_ERR("[%s]- invalid AVP definition\n", timer_avp);
 			return -1;
 		}
 		cpl_env.timer_avp_type = avp_type;
 	}
 
 	if (dtd_file==0) {
-		LOG(L_CRIT,"ERROR:cpl_init: mandatory parameter \"cpl_dtd_file\" "
-			"found empty\n");
+		LM_CRIT("mandatory parameter \"cpl_dtd_file\" found empty\n");
 		goto error;
 	} else {
 		/* check if the dtd file exists */
 		if (stat( dtd_file, &stat_t)==-1) {
-			LOG(L_ERR,"ERROR:cpl_init: checking file \"%s\" status failed;"
-				" stat returned %s\n",dtd_file,strerror(errno));
+			LM_ERR("checking file \"%s\" status failed; stat returned %s\n",
+					dtd_file,strerror(errno));
 			goto error;
 		}
 		if ( !S_ISREG( stat_t.st_mode ) ) {
-			LOG(L_ERR,"ERROR:cpl_init: dir \"%s\" is not a regular file!\n",
-				dtd_file);
+			LM_ERR("dir \"%s\" is not a regular file!\n", dtd_file);
 			goto error;
 		}
 		if (access( dtd_file, R_OK )==-1) {
-			LOG(L_ERR,"ERROR:cpl_init: checking file \"%s\" for permissions "
+			LM_ERR("checking file \"%s\" for permissions "
 				"failed; access returned %s\n",dtd_file,strerror(errno));
 			goto error;
 		}
 	}
 
 	if (cpl_env.log_dir==0) {
-		LOG(L_INFO,"INFO:cpl_init: log_dir param found void -> logging "
-			" disabled!\n");
+		LM_INFO("log_dir param found void -> logging disabled!\n");
 	} else {
 		if ( strlen(cpl_env.log_dir)>MAX_LOG_DIR_SIZE ) {
-			LOG(L_ERR,"ERROR:cpl_init: dir \"%s\" has a too long name :-(!\n",
-				cpl_env.log_dir);
+			LM_ERR("dir \"%s\" has a too long name :-(!\n",	cpl_env.log_dir);
 			goto error;
 		}
 		/* check if the dir exists */
 		if (stat( cpl_env.log_dir, &stat_t)==-1) {
-			LOG(L_ERR,"ERROR:cpl_init: checking dir \"%s\" status failed;"
+			LM_ERR("checking dir \"%s\" status failed;"
 				" stat returned %s\n",cpl_env.log_dir,strerror(errno));
 			goto error;
 		}
 		if ( !S_ISDIR( stat_t.st_mode ) ) {
-			LOG(L_ERR,"ERROR:cpl_init: dir \"%s\" is not a directory!\n",
-				cpl_env.log_dir);
+			LM_ERR("dir \"%s\" is not a directory!\n", cpl_env.log_dir);
 			goto error;
 		}
 		if (access( cpl_env.log_dir, R_OK|W_OK )==-1) {
-			LOG(L_ERR,"ERROR:cpl_init: checking dir \"%s\" for permissions "
-				"failed; access returned %s\n",
-				cpl_env.log_dir, strerror(errno));
+			LM_ERR("checking dir \"%s\" for permissions failed; access "
+					"returned %s\n", cpl_env.log_dir, strerror(errno));
 			goto error;
 		}
 	}
 
 	/* bind to the mysql module */
-	if (cpl_db_bind(DB_URL)<0) goto error;
+	if (cpl_db_bind(DB_URL,DB_TABLE)<0) goto error;
 
 	/* load TM API */
 	if (load_tm_api(&cpl_fct.tmb)!=0) {
-		LOG(L_ERR, "ERROR:cpl-c:mod_init: can't load TM API\n");
+		LM_ERR("can't load TM API\n");
 		goto error;
 	}
 	/* load SL API */
 	if (load_sl_api(&cpl_fct.slb)!=0) {
-		LOG(L_ERR, "ERROR:cpl-c:mod_init: can't load SL API\n");
+		LM_ERR("can't load SL API\n");
 		goto error;
 	}
 
@@ -336,46 +336,43 @@ static int cpl_init(void)
 		/* import all usrloc functions */
 		bind_usrloc = (bind_usrloc_t)find_export("ul_bind_usrloc", 1, 0);
 		if (!bind_usrloc) {
-			LOG(L_ERR, "ERROR:cpl_c:cpl_init: Can't bind usrloc\n");
+			LM_ERR("can't bind usrloc\n");
 			goto error;
 		}
 		if (bind_usrloc( &(cpl_fct.ulb) ) < 0) {
-			LOG(L_ERR, "ERROR:cpl_c:cpl_init: importing usrloc failed\n");
+			LM_ERR("importing usrloc failed\n");
 			goto error;
 		}
 		/* convert lookup_domain from char* to udomain_t* pointer */
 		if (cpl_fct.ulb.register_udomain( lookup_domain, &cpl_env.lu_domain)
 		< 0) {
-			LOG(L_ERR, "ERROR:cpl_c:cpl_init: Error while registering domain "
-				"<%s>\n",lookup_domain);
+			LM_ERR("failed to register domain <%s>\n",lookup_domain);
 			goto error;
 		}
 	} else {
-		LOG(L_NOTICE,"NOTICE:cpl_init: no lookup_domain given -> disable "
-			" lookup node\n");
+		LM_NOTICE("no lookup_domain given -> disable lookup node\n");
 	}
 
 	/* build a pipe for sending commands to aux process */
 	if ( pipe( cpl_env.cmd_pipe )==-1 ) {
-		LOG(L_CRIT,"ERROR:cpl_init: cannot create command pipe: %s!\n",
-			strerror(errno) );
+		LM_CRIT("cannot create command pipe: %s!\n", strerror(errno) );
 		goto error;
 	}
 	/* set the writing non blocking */
 	if ( (val=fcntl(cpl_env.cmd_pipe[1], F_GETFL, 0))<0 ) {
-		LOG(L_ERR,"ERROR:cpl_init: getting flags from pipe[1] failed: fcntl "
-			"said %s!\n",strerror(errno));
+		LM_ERR("getting flags from pipe[1] failed: fcntl said %s!\n",
+				strerror(errno));
 		goto error;
 	}
 	if ( fcntl(cpl_env.cmd_pipe[1], F_SETFL, val|O_NONBLOCK) ) {
-		LOG(L_ERR,"ERROR:cpl_init: setting flags to pipe[1] failed: fcntl "
-			"said %s!\n",strerror(errno));
+		LM_ERR("setting flags to pipe[1] failed: fcntl said %s!\n",
+				strerror(errno));
 		goto error;
 	}
 
 	/* init the CPL parser */
 	if (init_CPL_parser( dtd_file )!=1 ) {
-		LOG(L_ERR,"ERROR:cpl_init: init_CPL_parser failed!\n");
+		LM_ERR("init_CPL_parser failed!\n");
 		goto error;
 	}
 
@@ -383,7 +380,7 @@ static int cpl_init(void)
 	ptr = getenv("TZ");
 	cpl_env.orig_tz.len = 3/*"TZ="*/ + (ptr?(strlen(ptr)+1):0);
 	if ( (cpl_env.orig_tz.s=shm_malloc( cpl_env.orig_tz.len ))==0 ) {
-		LOG(L_ERR,"ERROR:cpl_init: no more shm mem. for saving TZ!\n");
+		LM_ERR("no more shm mem. for saving TZ!\n");
 		goto error;
 	}
 	memcpy(cpl_env.orig_tz.s,"TZ=",3);
@@ -406,68 +403,29 @@ error:
 
 static int cpl_child_init(int rank)
 {
-	pid_t pid;
-
-	/* don't do anything for non-worker process */
-	if (rank<1)
-		return 0;
-
-	/* only child 1 will fork the aux process */
-	if (rank==1) {
-		pid = fork();
-		if (pid==-1) {
-			LOG(L_CRIT,"ERROR:cpl_child_init(%d): cannot fork: %s!\n",
-				rank, strerror(errno));
-			goto error;
-		} else if (pid==0) {
-			/* I'm the child */
-			cpl_aux_process( cpl_env.cmd_pipe[0], cpl_env.log_dir);
-		} else {
-			LOG(L_INFO,"INFO:cpl_child_init(%d): I just gave birth to a child!"
-				" I'm a PARENT!!\n",rank);
-			/* I'm the parent -> remember the pid */
-			aux_process = pid;
-		}
-	}
-
 	return cpl_db_init(DB_URL, DB_TABLE);
-error:
-	return -1;
 }
 
 
-static int mi_child_init()
+static int mi_child_init(void)
 {
 	return cpl_db_init(DB_URL, DB_TABLE);
 }
 
 
+static void cpl_process(int rank)
+{
+	cpl_aux_process( cpl_env.cmd_pipe[0], cpl_env.log_dir);
+	exit(-1);
+}
+
+
 static int cpl_exit(void)
 {
 	/* free the TZ orig */
 	if (cpl_env.orig_tz.s)
 		shm_free(cpl_env.orig_tz.s);
 
-	/* if still running, stop the aux process */
-	if (!aux_process) {
-		LOG(L_INFO,"INFO:cpl_c:cpl_exit: aux process hasn't been created -> "
-			"nothing to kill :-(\n");
-	} else {
-		/* kill the auxiliary process */
-		if (kill( aux_process, SIGKILL)!=0) {
-			if (errno==ESRCH) {
-				LOG(L_INFO,"INFO:cpl_c:cpl_exit: seems that my child is "
-					"already dead! :-((\n");
-			} else {
-				LOG(L_ERR,"ERROR:cpl_c:cpl_exit: killing the aux. process "
-					"failed! kill said: %s\n",strerror(errno));
-				return -1;
-			}
-		} else {
-			LOG(L_INFO,"INFO:cl_c:cpl_exit: I have blood on my hands!! I just"
-				" killed my own child!");
-		}
-	}
 	return 0;
 }
 
@@ -497,7 +455,7 @@ static inline int build_user_AOR(str *username, str *domain, str *uh, int sip)
 
 	uh->s = (char*)shm_malloc( uh->len + 1 );
 	if (!uh->s) {
-		LOG(L_ERR,"ERROR:cpl-c:build_userhost: no more shm memory.\n");
+		LM_ERR("no more shm memory.\n");
 		return -1;
 	}
 
@@ -525,8 +483,7 @@ static inline int build_user_AOR(str *username, str *domain, str *uh, int sip)
 
 	/* sanity check */
 	if (p-uh->s!=uh->len+1) {
-		LOG(L_CRIT,"BUG:cpl-c:build_userhost: buffer overflow l=%d,w=%ld\n",
-			uh->len,(long)(p-uh->s));
+		LM_CRIT("buffer overflow l=%d,w=%ld\n", uh->len,(long)(p-uh->s));
 		return -1;
 	}
 	return 0;
@@ -539,21 +496,20 @@ static inline int get_dest_user(struct sip_msg *msg, str *username, str *domain)
 	struct sip_uri uri;
 
 	/*  get the user_name from new_uri/RURI/To */
-	DBG("DEBUG:cpl-c:get_dest_user: trying to get user from new_uri\n");
+	LM_DBG("trying to get user from new_uri\n");
 	if ( !msg->new_uri.s || parse_uri( msg->new_uri.s,msg->new_uri.len,&uri)<0
 	|| !uri.user.len )
 	{
-		DBG("DEBUG:cpl-c:get_dest_user: trying to get user from R_uri\n");
+		LM_DBG("trying to get user from R_uri\n");
 		if ( parse_uri( msg->first_line.u.request.uri.s,
 		msg->first_line.u.request.uri.len ,&uri)==-1 || !uri.user.len )
 		{
-			DBG("DEBUG:cpl-c:get_dest_user: trying to get user from To\n");
+			LM_DBG("trying to get user from To\n");
 			if ( (!msg->to&&((parse_headers(msg,HDR_TO_F,0)==-1)||!msg->to))||
 			parse_uri( get_to(msg)->uri.s, get_to(msg)->uri.len, &uri)<0
 			|| !uri.user.len)
 			{
-				LOG(L_ERR,"ERROR:cpl-c:get_dest_user: unable to extract user"
-					" name from RURI or To header!\n");
+				LM_ERR("unable to extract user name from RURI or To header!\n");
 				return -1;
 			}
 		}
@@ -572,17 +528,15 @@ static inline int get_orig_user(struct sip_msg *msg, str *username, str *domain)
 	
 	/* if it's outgoing -> get the user_name from From */
 	/* parsing from header */
-	DBG("DEBUG:cpl-c:get_orig_user: trying to get user from From\n");
+	LM_DBG("trying to get user from From\n");
 	if ( parse_from_header( msg )==-1 ) {
-		LOG(L_ERR,"ERROR:cpl-c:get_orig_user: unable to extract URI "
-			"from FROM header\n");
+		LM_ERR("unable to extract URI from FROM header\n");
 		return -1;
 	}
 	from = (struct to_body*)msg->from->parsed;
 	/* parse the extracted uri from From */
 	if (parse_uri( from->uri.s, from->uri.len, &uri)||!uri.user.len) {
-		LOG(L_ERR,"ERROR:cpl-c:get_orig_user: unable to extract user name "
-			"from URI (From header)\n");
+		LM_ERR("unable to extract user name from URI (From header)\n");
 		return -1;
 	}
 	*username = uri.user;
@@ -636,7 +590,9 @@ static int cpl_invoke_script(struct sip_msg* msg, char* str1, char* str2)
 	/* for OUTGOING we need also the destination user for init. with him
 	 * the location set */
 	if ( ((unsigned long)str1)&CPL_RUN_OUTGOING ) {
-		/* build user initial location */
+		/* build user initial location -> get the destination user name */
+		if (get_dest_user( msg, &username, &domain)==-1)
+			goto error2;
 		if (build_user_AOR( &username, &domain, &loc, 1)!=0 )
 			goto error2;
 		if (add_location( &(cpl_intr->loc_set), &loc, 0, 10, 0/*no dup*/)==-1)
@@ -707,8 +663,7 @@ static inline int do_script_action(struct sip_msg *msg, int action)
 	/* content-length (if present) */
 	if ( !msg->content_length &&
 	((parse_headers(msg,HDR_CONTENTLENGTH_F,0)==-1)||!msg->content_length)) {
-		LOG(L_ERR,"ERROR:cpl-c:do_script_action: no Content-Length "
-			"hdr found!\n");
+		LM_ERR("no Content-Length hdr found!\n");
 		goto error;
 	}
 	body.len = get_content_length( msg );
@@ -722,15 +677,13 @@ static inline int do_script_action(struct sip_msg *msg, int action)
 		case STORE_SCRIPT :
 			/* check the len -> it must not be 0 */
 			if (body.len==0) {
-				LOG(L_ERR,"ERROR:cpl-c:do_script_action: 0 content-len found "
-					"for store\n");
+				LM_ERR("0 content-len found for store\n");
 				goto error_1;
 			}
 			/* get the message's body */
 			body.s = get_body( msg );
 			if (body.s==0) {
-				LOG(L_ERR,"ERROR:cpl-c:do_script_action: cannot extract "
-					"body from msg!\n");
+				LM_ERR("cannot extract body from msg!\n");
 				goto error_1;
 			}
 			/* now compile the script and place it into database */
@@ -750,8 +703,7 @@ static inline int do_script_action(struct sip_msg *msg, int action)
 		case REMOVE_SCRIPT:
 			/* check the len -> it must be 0 */
 			if (body.len!=0) {
-				LOG(L_ERR,"ERROR:cpl-c:do_script_action: non-0 content-len "
-					"found for remove\n");
+				LM_ERR("non-0 content-len found for remove\n");
 				goto error_1;
 			}
 			/* remove the script for the user */
@@ -790,7 +742,7 @@ static inline int do_script_download(struct sip_msg *msg)
 	/* add a lump with content-type hdr */
 	if (add_lump_rpl( msg, CONTENT_TYPE_HDR, CONTENT_TYPE_HDR_LEN,
 	LUMP_RPL_HDR)==0) {
-		LOG(L_ERR,"ERROR:cpl-c:do_script_download: cannot build hdr lump\n");
+		LM_ERR("cannot build hdr lump\n");
 		cpl_err = &intern_err;
 		goto error;
 	}
@@ -798,8 +750,7 @@ static inline int do_script_download(struct sip_msg *msg)
 	if (script.s!=0) {
 		/* user has a script -> add a body lump */
 		if ( add_lump_rpl( msg, script.s, script.len, LUMP_RPL_BODY)==0) {
-			LOG(L_ERR,"ERROR:cpl-c:do_script_download: cannot build "
-				"body lump\n");
+			LM_ERR("cannot build body lump\n");
 			cpl_err = &intern_err;
 			goto error;
 		}
@@ -848,16 +799,14 @@ static int cpl_process_register(struct sip_msg* msg, int no_rpl)
 		goto error;
 
 	/* check the mime type */
-	DBG("DEBUG:cpl_process_register: Content-Type mime found %u, %u\n",
+	LM_DBG("Content-Type mime found %u, %u\n",
 		mime>>16,mime&0x00ff);
 	if ( mime && mime==(TYPE_APPLICATION<<16)+SUBTYPE_CPLXML ) {
 		/* can be an upload or remove -> check for the content-purpose and
 		 * content-action headers */
-		DBG("DEBUG:cpl_process_register: carrying CPL -> look at "
-			"Content-Disposition\n");
+		LM_DBG("carrying CPL -> look at Content-Disposition\n");
 		if (parse_content_disposition( msg )!=0) {
-			LOG(L_ERR,"ERROR:cpl_process_register: Content-Disposition missing "
-				"or corrupted\n");
+			LM_ERR("Content-Disposition missing or corrupted\n");
 			goto error;
 		}
 		disp = get_content_disposition(msg);
@@ -865,7 +814,7 @@ static int cpl_process_register(struct sip_msg* msg, int no_rpl)
 		/* check if the type of disposition is SCRIPT */
 		if (disp->type.len!=CPL_SCRIPT_LEN ||
 		strncasecmp(disp->type.s,CPL_SCRIPT,CPL_SCRIPT_LEN) ) {
-			LOG(L_ERR,"ERROR:cpl_process_register: bogus message - Content-Type"
+			LM_ERR("bogus message - Content-Type"
 				"says CPL_SCRIPT, but Content-Disposition something else\n");
 			goto error;
 		}
@@ -876,7 +825,7 @@ static int cpl_process_register(struct sip_msg* msg, int no_rpl)
 				break;
 		}
 		if (param==0) {
-			LOG(L_ERR,"ERROR:cpl_process_register: bogus message - "
+			LM_ERR("bogus message - "
 				"Content-Disposition has no action param\n");
 			goto error;
 		}
@@ -894,7 +843,7 @@ static int cpl_process_register(struct sip_msg* msg, int no_rpl)
 			if (do_script_action( msg, REMOVE_SCRIPT)==-1)
 				goto error;
 		} else {
-			LOG(L_ERR,"ERROR:cpl_process_register: unknown action <%.*s>\n",
+			LM_ERR("unknown action <%.*s>\n",
 				param->body.len,param->body.s);
 			goto error;
 		}
@@ -919,7 +868,7 @@ static int cpl_process_register(struct sip_msg* msg, int no_rpl)
 
 	/* looks if the REGISTER accepts cpl-xml or * */
 	while (*mimes) {
-		DBG("DEBUG: accept mime found %u, %u\n",
+		LM_DBG("accept mime found %u, %u\n",
 			(*mimes)>>16,(*mimes)&0x00ff);
 		if (*mimes==(TYPE_ALL<<16)+SUBTYPE_ALL ||
 		*mimes==(TYPE_APPLICATION<<16)+SUBTYPE_CPLXML )
