@@ -1,5 +1,5 @@
 /*
- * $Id: cpl_time.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: cpl_time.c 3184 2007-11-21 17:40:14Z bogdan_iancu $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -104,7 +104,7 @@ static inline char* trim(char* _s)
 #define ac_get_wday_yr(t) (int)((t)->tm_yday/7)
 #define ac_get_wday_mr(t) (int)(((t)->tm_mday-1)/7)
 
-ac_tm_p ac_tm_new()
+ac_tm_p ac_tm_new(void)
 {
 	ac_tm_p _atp = NULL;
 	_atp = (ac_tm_p)pkg_malloc(sizeof(ac_tm_t));
@@ -133,18 +133,9 @@ int ac_tm_fill(ac_tm_p _atp, struct tm* _tm)
 	_atp->yweek = ac_get_yweek(_tm);
 	_atp->ywday = ac_get_wday_yr(_tm);
 	_atp->mwday = ac_get_wday_mr(_tm);
-	DBG("---> fill = %s\n",asctime(&(_atp->t)) );
 	return 0;
 }
 
-int ac_tm_set(ac_tm_p _atp, struct tm* _tm)
-{
-	if(!_atp || !_tm)
-		return -1;
-	_atp->time = mktime(_tm);
-	return ac_tm_fill(_atp, _tm);
-}
-
 int ac_tm_set_time(ac_tm_p _atp, time_t _t)
 {
 	if(!_atp)
@@ -192,7 +183,7 @@ int ac_get_yweek(struct tm* _tm)
 	return week;
 }
 
-int ac_get_wkst()
+int ac_get_wkst(void)
 {
 #ifdef USE_YWEEK_U
 	return 0;
@@ -315,7 +306,7 @@ int ac_print(ac_tm_p _atp)
 
 #define _D(c) ((c) -'0')
 
-tr_byxxx_p tr_byxxx_new()
+tr_byxxx_p tr_byxxx_new(void)
 {
 	tr_byxxx_p _bxp = NULL;
 	_bxp = (tr_byxxx_p)pkg_malloc(sizeof(tr_byxxx_t));
@@ -359,7 +350,7 @@ int tr_byxxx_free(tr_byxxx_p _bxp)
 	return 0;
 }
 
-tmrec_p tmrec_new()
+tmrec_p tmrec_new(void)
 {
 	tmrec_p _trp = NULL;
 	_trp = (tmrec_p)pkg_malloc(sizeof(tmrec_t));
