@@ -90,7 +90,7 @@ static db_func_t dialog_dbf;
 		if (VAL_NULL((_values)+ (_index))) { \
 			if (_not_null) {\
 				if (_unref) unref_dlg(dlg,1);\
-				continue; \
+				goto next_dialog; \
 			} else { \
 				(_res).s = 0; \
 				(_res).len = 0; \
@@ -141,7 +141,7 @@ int init_dlg_db( char *db_url, int dlg_hash_size , int db_update_period)
 		return -1;
 	} else if (ver != DLG_TABLE_VERSION) {
 		LM_ERR("Invalid table version (found %d , required %d)\n"
-			"(use openser_mysql.sh reinstall)\n",
+			"(use openserdbctl reinit)\n",
 			ver, DLG_TABLE_VERSION );
 		return -1;
 	}
@@ -350,6 +350,8 @@ static int load_dialog_info_from_db(int dlg_hash_size)
 	
 		dlg->lifetime = 0;
 		dlg->flags = 0;
+next_dialog:
+		;
 	}
 
 end:
