@@ -1,5 +1,5 @@
 /*
- * $Id: dlg_handlers.c 3815 2008-03-01 12:41:16Z osas $
+ * $Id: dlg_handlers.c 4150 2008-05-10 02:50:03Z osas $
  *
  * Copyright (C) 2006 Voice System SRL
  *
@@ -537,18 +537,24 @@ void dlg_onroute(struct sip_msg* req, str *route_params, void *param)
 
 			dlg = lookup_dlg( h_entry, h_id);
 			if (dlg==0) {
-				LM_WARN("unable to find dialog for %.*s\n",
+				LM_WARN("unable to find dialog for %.*s "
+					"with route param '%.*s'\n",
 					req->first_line.u.request.method.len,
-					req->first_line.u.request.method.s);
+					req->first_line.u.request.method.s,
+					val.len,val.s);
 				return;
 			}
 
 			if (pre_match_parse( req, &callid, &ftag, &ttag)<0)
 				return;
 			if (match_dialog( dlg, &callid, &ftag, &ttag, &dir )==0) {
-				LM_WARN("tight matching failed for %.*s\n",
+				LM_WARN("tight matching failed for %.*s "
+					"with clid '%.*s' and tags '%.*s' '%.*s' "
+					"and direction %d\n",
 					req->first_line.u.request.method.len,
-					req->first_line.u.request.method.s);
+					req->first_line.u.request.method.s,
+					callid.len,callid.s,
+					ftag.len,ftag.s,ttag.len,ttag.s,dir);
 				return;
 			}
 		}
@@ -654,7 +660,10 @@ void dlg_ontimeout( struct dlg_tl *tl)
 	next_state_dlg( dlg, DLG_EVENT_REQBYE, &old_state, &new_state, &unref);
 
 	if (new_state==DLG_STATE_DELETED && old_state!=DLG_STATE_DELETED) {
-		LM_DBG("dlg %p timeout at %d\n", dlg, tl->timeout);
+		LM_WARN("timeout for dlg with CallID '%.*s' and tags '%.*s' '%.*s'\n",
+			dlg->callid.len, dlg->callid.s,
+			dlg->tag[DLG_CALLER_LEG].len, dlg->tag[DLG_CALLER_LEG].s,
+			dlg->tag[DLG_CALLEE_LEG].len, dlg->tag[DLG_CALLEE_LEG].s);
 
 		/* dialog timeout */
 		run_dlg_callbacks( DLGCB_EXPIRED, dlg, 0);
