@@ -1,6 +1,6 @@
 /*
  *
- * $Id: exec.h 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: exec.h 2725 2007-09-09 20:55:56Z miconda $
  *
  *
  * Copyright (C) 2001-2003 FhG Fokus
@@ -25,11 +25,11 @@
 #ifndef _EXEC_H
 #define _EXEC_H
 
-#include "../../items.h"
+#include "../../pvar.h"
 
 int exec_str(struct sip_msg *msg, char *cmd, char *param, int param_len);
 int exec_msg(struct sip_msg *msg, char *cmd );
-int exec_avp(struct sip_msg *msg, char *cmd, itemname_list_p avpl);
+int exec_avp(struct sip_msg *msg, char *cmd, pvname_list_p avpl);
 
 #endif
 
