@@ -1,5 +1,5 @@
 /*
- * $Id: xjab_load.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: xjab_load.c 2793 2007-09-20 07:47:17Z bogdan_iancu $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -23,20 +23,18 @@
 
 #include "xjab_load.h"
 
-#define LOAD_ERROR "ERROR:XJAB:xjab_bind: module function "
-
 int load_xjab(struct xjab_binds *xjb)
 {
 	if(!( xjb->register_watcher=(pa_register_watcher_f)
 			find_export("jab_register_watcher", XJ_NO_SCRIPT_F, 0)) ) 
 	{
-		LOG(L_ERR, LOAD_ERROR "'jab_register_watcher' not found!\n");
+		LM_ERR("'jab_register_watcher' not found!\n");
 		return -1;
 	}
 	if(!( xjb->unregister_watcher=(pa_unregister_watcher_f)
 			find_export("jab_unregister_watcher", XJ_NO_SCRIPT_F, 0)) ) 
 	{
-		LOG(L_ERR, LOAD_ERROR "'jab_unregister_watcher' not found!\n");
+		LM_ERR("'jab_unregister_watcher' not found!\n");
 		return -1;
 	}
 	return 1;
