@@ -1,5 +1,5 @@
 /*
- * $Id: lcr_mod.c 3561 2008-01-22 09:39:45Z juhe $
+ * $Id: lcr_mod.c 4104 2008-05-03 12:56:14Z juhe $
  *
  * Least Cost Routing module (also implements sequential forking)
  *
@@ -1514,6 +1514,7 @@ int next_gw(struct sip_msg* _m, char* _s1, char* _s2)
 	at = at + (strip_char - gw_uri_val.s.s);
 	strip = strtol(strip_char + 1, &endptr, 10);
 	if (endptr != at_char) {
+	    pkg_free(new_ruri.s);
 	    LM_ERR("Non-digit char between | and @ chars in gw URI <%.*s>\n",
 		   gw_uri_val.s.len, gw_uri_val.s.s);
 	    return -1;
@@ -1584,6 +1585,7 @@ int next_gw(struct sip_msg* _m, char* _s1, char* _s2)
 	at = at + (strip_char - gw_uri_val.s.s);
 	strip = strtol(strip_char + 1, &endptr, 10);
 	if (endptr != at_char) {
+	    pkg_free(new_ruri.s);
 	    LM_ERR("Non-digit char between | and @ chars in gw URI <%.*s>\n",
 		   gw_uri_val.s.len, gw_uri_val.s.s);
 	    return -1;
@@ -2158,8 +2160,7 @@ static int fixstringloadgws(void **param, int param_count)
 	}
 	if(model->spec.getf==NULL) {
 	    if(param_count==1) {
-		if(str2int(&s, (unsigned int*)&model->spec.pvp.pvn.u.isname.name.n)!=0
-			   || model->spec.pvp.pvn.u.isname.name.n<100) {
+		if(str2int(&s, (unsigned int*)&model->spec.pvp.pvn.u.isname.name.n)!=0) {
 		    LM_ERR("Wrong value <%s> for param <%d>!\n",
 			   s.s, param_count);
 		    return -1;
