@@ -1,6 +1,6 @@
-/* $Id: mediaproxy.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+/* $Id: mediaproxy.c 3058 2007-11-08 12:50:02Z dan_pascu $
  *
- * Copyright (C) 2004 Dan Pascu
+ * Copyright (C) 2004-2007 Dan Pascu
  *
  * This file is part of openser, a free SIP server.
  *
@@ -28,7 +28,7 @@
 #include "../../sr_module.h"
 #include "../../dprint.h"
 #include "../../str.h"
-#include "../../usr_avp.h"
+#include "../../pvar.h"
 #include "../../error.h"
 #include "../../data_lump.h"
 #include "../../forward.h"
@@ -42,6 +42,7 @@
 #include "../../msg_translator.h"
 #include "../registrar/sip_msg.h"
 #include "../usrloc/usrloc.h"
+#include "../../mod_fix.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -60,7 +61,8 @@
 
 MODULE_VERSION
 
-#define SIGNALING_IP_AVP_NAME  "s:signaling_ip"
+#define SIGNALING_IP_AVP_SPEC  "$avp(s:signaling_ip)"
+#define DOMAIN_AVP_SPEC        "$avp(s:mediaproxy_domain)"
 
 
 // Although `AF_LOCAL' is mandated by POSIX.1g, `AF_UNIX' is portable to
@@ -124,14 +126,14 @@ typedef struct {
     long timestamp;    // for checking if it was modified
 
     regex_t **clients; // the asymmetric clients regular expressions
-    int size;          // size of array above
-    int count;         // how many clients are in array above
+    int size;          // size of the array above
+    int count;         // how many clients are in the array above
 } AsymmetricClients;
 
 typedef struct AVP_Param {
-    str name;
-    int type;
-    int_str avp;
+    str spec;
+    int_str name;
+    unsigned short type;
 } AVP_Param;
 
 
@@ -142,7 +144,6 @@ static int UseMediaProxy(struct sip_msg *msg, char *str1, char *str2);
 static int EndMediaSession(struct sip_msg *msg, char *str1, char *str2);
 
 static int mod_init(void);
-static int fixstring2int(void **param, int param_count);
 
 static Bool testPrivateContact(struct sip_msg* msg);
 static Bool testSourceAddress(struct sip_msg* msg);
@@ -155,7 +156,10 @@ static char *mediaproxySocket = "/var/run/proxydispatcher.sock";
 static int natpingInterval = 60; // 60 seconds
 
 /* The AVP where the caller signaling IP is stored (if defined) */
-static AVP_Param signaling_ip_avp = {{SIGNALING_IP_AVP_NAME, sizeof(SIGNALING_IP_AVP_NAME)-1}, 0, {0}};
+static AVP_Param signaling_ip_avp = {str_init(SIGNALING_IP_AVP_SPEC), {0}, 0};
+
+/* The AVP where the application-defined mediaproxy domain is stored */
+static AVP_Param domain_avp = {str_init(DOMAIN_AVP_SPEC), {0}, 0};
 
 static usrloc_api_t userLocation;
 
@@ -194,15 +198,11 @@ NatTest natTests[] = {
 };
 
 static cmd_export_t commands[] = {
-	{"fix_contact",       FixContact,      0, 0,
-		REQUEST_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
-	{"use_media_proxy",   UseMediaProxy,   0, 0,
-		REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE},
-	{"end_media_session", EndMediaSession, 0, 0,
-		REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE},
-	{"client_nat_test",   ClientNatTest,   1, fixstring2int,
-		REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE},
-	{0, 0, 0, 0, 0}
+    {"fix_contact",       FixContact,      0, 0, 0, REQUEST_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
+    {"use_media_proxy",   UseMediaProxy,   0, 0, 0, REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE},
+    {"end_media_session", EndMediaSession, 0, 0, 0, REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE},
+    {"client_nat_test",   ClientNatTest,   1, fixup_str2int, 0, REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE},
+    {0, 0, 0, 0, 0, 0}
 };
 
 static param_export_t parameters[] = {
@@ -210,29 +210,29 @@ static param_export_t parameters[] = {
     {"sip_asymmetrics",   STR_PARAM, &(sipAsymmetrics.file)},
     {"rtp_asymmetrics",   STR_PARAM, &(rtpAsymmetrics.file)},
     {"natping_interval",  INT_PARAM, &natpingInterval},
-    {"signaling_ip_avp",  STR_PARAM, &(signaling_ip_avp.name.s)},
+    {"signaling_ip_avp",  STR_PARAM, &(signaling_ip_avp.spec.s)},
+    {"domain_avp",        STR_PARAM, &(domain_avp.spec.s)},
     {0, 0, 0}
 };
 
 struct module_exports exports = {
-    "mediaproxy", // module name
-	DEFAULT_DLFLAGS, /* dlopen flags */
-    commands,     // module exported functions
-    parameters,   // module exported parameters
-    NULL,         // exported statistics
-    NULL,         /* exported MI functions */
-	NULL,         /* exported pseudo-variables */
-    mod_init,     // module init (before any kid is created. kids will inherit)
-    NULL,         // reply processing
-    NULL,         // destroy function
-    NULL          // child_init
+    "mediaproxy",    // module name
+    DEFAULT_DLFLAGS, // dlopen flags
+    commands,        // exported functions
+    parameters,      // exported parameters
+    NULL,            // exported statistics
+    NULL,            // exported MI functions
+    NULL,            // exported pseudo-variables
+    NULL,            // extra processes
+    mod_init,        // module init function (before fork. kids will inherit)
+    NULL,            // reply processing function
+    NULL,            // destroy function
+    NULL             // child init function
 };
 
 
 
-/* Helper functions */
-
-// Functions dealing with strings
+// String processing functions
 
 /*
  * strfind() finds the start of the first occurrence of the substring needle
@@ -350,7 +350,7 @@ encodeQuopri(str buf)
 
     result = pkg_malloc(buf.len*3+1);
     if (!result) {
-        LOG(L_ERR, "error: mediaproxy/encodeQuopri(): out of memory\n");
+        LM_ERR("out of memory\n");
         return NULL;
     }
 
@@ -449,9 +449,8 @@ getStrTokens(str *string, str *tokens, int limit)
 }
 
 
-/* Functions to extract the info we need from the SIP/SDP message */
+// Functions to extract the info we need from the SIP/SDP message
 
-/* Extract Call-ID value. */
 static Bool
 getCallId(struct sip_msg* msg, str *cid)
 {
@@ -478,8 +477,8 @@ getSignalingIP(struct sip_msg* msg)
     int_str value;
 
     if (!search_first_avp(signaling_ip_avp.type | AVP_VAL_STR,
-                          signaling_ip_avp.avp, &value, NULL) ||
-        !value.s.s || value.s.len==0) {
+                          signaling_ip_avp.name, &value, NULL) ||
+        value.s.s==NULL || value.s.len==0) {
 
         value.s.s = ip_addr2a(&msg->rcv.src_ip);
         value.s.len = strlen(value.s.s);
@@ -488,6 +487,20 @@ getSignalingIP(struct sip_msg* msg)
     return value.s;
 }
 
+/* Get the application-defined mediaproxy domain if defined */
+static str
+getMediaproxyDomain(struct sip_msg* msg)
+{
+    int_str value;
+
+    if (!search_first_avp(domain_avp.type | AVP_VAL_STR,
+                          domain_avp.name, &value, NULL) || value.s.s==NULL) {
+        value.s.len = 0;
+    }
+
+    return value.s;
+}
+
 /* Get caller domain */
 static str
 getFromDomain(struct sip_msg* msg)
@@ -498,14 +511,14 @@ getFromDomain(struct sip_msg* msg)
     str uri;
 
     if (parse_from_header(msg) < 0) {
-        LOG(L_ERR, "error: mediaproxy/getFromDomain(): error parsing `From' header\n");
+        LM_ERR("cannot parse the From header\n");
         return notfound;
     }
 
     uri = get_from(msg)->uri;
 
     if (parse_uri(uri.s, uri.len, &puri) < 0) {
-        LOG(L_ERR, "error: mediaproxy/getFromDomain(): error parsing `From' URI\n");
+        LM_ERR("cannot parse the From URI\n");
         return notfound;
     } else if (puri.host.len == 0) {
         return notfound;
@@ -514,8 +527,7 @@ getFromDomain(struct sip_msg* msg)
     return puri.host;
 }
 
-/* Get destination domain */
-// This function only works when called for a request
+/* Get destination domain (only works for requests) */
 static str
 getDestinationDomain(struct sip_msg* msg)
 {
@@ -523,7 +535,7 @@ getDestinationDomain(struct sip_msg* msg)
     static str notfound = {buf, 7};  // use the constant string directly!
 
     if (parse_sip_msg_uri(msg) < 0) {
-        LOG(L_ERR, "error: mediaproxy/getDestinationDomain(): error parsing destination URI\n");
+        LM_ERR("cannot parse the destination URI\n");
         return notfound;
     } else if (msg->parsed_uri.host.len==0) {
         return notfound;
@@ -533,7 +545,6 @@ getDestinationDomain(struct sip_msg* msg)
 }
 
 
-/* Get From tag */
 static str
 getFromAddress(struct sip_msg *msg)
 {
@@ -542,8 +553,8 @@ getFromAddress(struct sip_msg *msg)
     str uri;
     char *ptr;
 
-    if (parse_from_header(msg)<0) {
-        LOG(L_ERR, "error: mediaproxy/getFromAddress(): error parsing From: field\n");
+    if (parse_from_header(msg) < 0) {
+        LM_ERR("cannot parse the From header\n");
         return notfound;
     }
 
@@ -565,7 +576,6 @@ getFromAddress(struct sip_msg *msg)
 }
 
 
-/* Get To tag */
 static str
 getToAddress(struct sip_msg *msg)
 {
@@ -575,7 +585,7 @@ getToAddress(struct sip_msg *msg)
     char *ptr;
 
     if (!msg->to) {
-        LOG(L_ERR, "error: mediaproxy/getToAddress(): missing To: field\n");
+        LM_ERR("missing To header\n");
         return notfound;
     }
 
@@ -597,7 +607,6 @@ getToAddress(struct sip_msg *msg)
 }
 
 
-/* Get From tag */
 static str
 getFromTag(struct sip_msg *msg)
 {
@@ -605,8 +614,8 @@ getFromTag(struct sip_msg *msg)
     static str notfound = {buf, 0}; // use the constant string directly!
     str tag;
 
-    if (parse_from_header(msg)<0) {
-        LOG(L_ERR, "error: mediaproxy/getFromTag(): error parsing From: field\n");
+    if (parse_from_header(msg) < 0) {
+        LM_ERR("cannot parse the From header\n");
         return notfound;
     }
 
@@ -619,7 +628,6 @@ getFromTag(struct sip_msg *msg)
 }
 
 
-/* Get To tag */
 static str
 getToTag(struct sip_msg *msg)
 {
@@ -628,7 +636,7 @@ getToTag(struct sip_msg *msg)
     str tag;
 
     if (!msg->to) {
-        LOG(L_ERR, "error: mediaproxy/getToTag(): missing To: field\n");
+        LM_ERR("missing To header\n");
         return notfound;
     }
 
@@ -641,7 +649,6 @@ getToTag(struct sip_msg *msg)
 }
 
 
-/* Extract User-Agent */
 static str
 getUserAgent(struct sip_msg* msg)
 {
@@ -655,11 +662,8 @@ getUserAgent(struct sip_msg* msg)
         return msg->user_agent->body;
     }
 
-    // If we can't find user-agent, look after the Server: field
-
-    // This is a temporary hack. Normally it should be extracted by ser
-    // (either as the Server field, or if User-Agent is missing in place
-    // of the User-Agent field)
+    // If we can't find user-agent, look after the `Server' header
+    // This is a temporary hack. Normally it should be extracted by openser.
 
     block.s   = msg->buf;
     block.len = msg->len;
@@ -678,7 +682,7 @@ getUserAgent(struct sip_msg* msg)
     return server;
 }
 
-// Get URI from the Contact: field.
+
 static Bool
 getContactURI(struct sip_msg* msg, struct sip_uri *uri, contact_t** _c)
 {
@@ -687,7 +691,7 @@ getContactURI(struct sip_msg* msg, struct sip_uri *uri, contact_t** _c)
         return False;
 
     if (!msg->contact->parsed && parse_contact(msg->contact) < 0) {
-        LOG(L_ERR, "error: mediaproxy/getContactURI(): cannot parse Contact header\n");
+        LM_ERR("cannot parse the Contact header\n");
         return False;
     }
 
@@ -698,7 +702,7 @@ getContactURI(struct sip_msg* msg, struct sip_uri *uri, contact_t** _c)
     }
 
     if (parse_uri((*_c)->uri.s, (*_c)->uri.len, uri) < 0 || uri->host.len <= 0) {
-        LOG(L_ERR, "error: mediaproxy/getContactURI(): cannot parse Contact URI\n");
+        LM_ERR("cannot parse the Contact URI\n");
         return False;
     }
 
@@ -714,8 +718,8 @@ checkContentType(struct sip_msg *msg)
     str type;
 
     if (!msg->content_type) {
-        LOG(L_WARN, "warning: mediaproxy/checkContentType(): Content-Type "
-            "header missing! Let's assume the content is text/plain ;-)\n");
+        LM_WARN("the Content-Type header is missing! Assume the content type "
+                "is text/plain ;-)\n");
         return True;
     }
 
@@ -723,14 +727,12 @@ checkContentType(struct sip_msg *msg)
     trim(&type);
 
     if (strncasecmp(type.s, "application/sdp", 15) != 0) {
-        LOG(L_ERR, "error: mediaproxy/checkContentType(): invalid Content-Type "
-            "for SDP message\n");
+        LM_ERR("invalid Content-Type for SDP: %.*s\n", type.len, type.s);
         return False;
     }
 
     if (!(isspace((int)type.s[15]) || type.s[15] == ';' || type.s[15] == 0)) {
-        LOG(L_ERR,"error: mediaproxy/checkContentType(): invalid character "
-            "after Content-Type!\n");
+        LM_ERR("invalid character after Content-Type: `%c'\n", type.s[15]);
         return False;
     }
 
@@ -748,7 +750,7 @@ getSDPMessage(struct sip_msg *msg, str *sdp)
 {
     sdp->s = get_body(msg);
     if (sdp->s==NULL) {
-        LOG(L_ERR, "error: mediaproxy/getSDPMessage(): cannot get body from message\n");
+        LM_ERR("cannot get the SDP body\n");
         return -1;
     }
 
@@ -757,7 +759,7 @@ getSDPMessage(struct sip_msg *msg, str *sdp)
         return -2;
 
     if (!checkContentType(msg)) {
-        LOG(L_ERR, "error: mediaproxy/getSDPMessage(): content type is not `application/sdp'\n");
+        LM_ERR("content type is not `application/sdp'\n");
         return -1;
     }
 
@@ -788,8 +790,7 @@ getMediaIPFromBlock(str *block, str *mediaip)
     count = getStrTokens(&zone, tokens, 3);
 
     if (count != 3) {
-        LOG(L_ERR, "error: mediaproxy/getMediaIPFromBlock(): invalid `c=' "
-            "line in SDP body\n");
+        LM_ERR("invalid `c=' line in SDP body\n");
         return -1;
     }
 
@@ -800,7 +801,6 @@ getMediaIPFromBlock(str *block, str *mediaip)
 }
 
 
-// Get the session-level media IP defined by the SDP message
 static Bool
 getSessionLevelMediaIP(str *sdp, str *mediaip)
 {
@@ -817,8 +817,8 @@ getSessionLevelMediaIP(str *sdp, str *mediaip)
     }
 
     if (getMediaIPFromBlock(&block, mediaip) == -1) {
-        LOG(L_ERR, "error: mediaproxy/getSessionLevelMediaIP(): parse error "
-            "while getting session-level media IP from SDP body\n");
+        LM_ERR("parse error while getting session-level media IP from "
+               "the SDP body\n");
         return False;
     }
 
@@ -828,7 +828,6 @@ getSessionLevelMediaIP(str *sdp, str *mediaip)
 }
 
 
-// will get all media streams
 static int
 getMediaStreams(str *sdp, str *sessionIP, StreamInfo *streams, int limit)
 {
@@ -849,8 +848,7 @@ getMediaStreams(str *sdp, str *sessionIP, StreamInfo *streams, int limit)
         count = getStrTokens(&zone, tokens, 2);
 
         if (count != 2) {
-            LOG(L_ERR, "error: mediaproxy/getMediaStreams(): invalid `m=' "
-                "line in SDP body\n");
+            LM_ERR("invalid `m=' line in the SDP body\n");
             return -1;
         }
 
@@ -872,14 +870,13 @@ getMediaStreams(str *sdp, str *sessionIP, StreamInfo *streams, int limit)
 
         result = getMediaIPFromBlock(&block, &(streams[i].ip));
         if (result == -1) {
-            LOG(L_ERR, "error: mediaproxy/getMediaStreams(): parse error in "
-                "getting the contact IP for the media stream nr. %d\n", i+1);
+            LM_ERR("parse error while getting the contact IP for the "
+                   "media stream number %d\n", i+1);
             return -1;
         } else if (result == 0) {
             if (sessionIP->s == NULL) {
-                LOG(L_ERR, "error: mediaproxy/getMediaStreams(): media stream "
-                    "doesn't define a contact IP and the session-level IP "
-                    "is missing\n");
+                LM_ERR("media stream number %d doesn't define a contact IP "
+                       "and the session-level IP is missing\n", i+1);
                 return -1;
             }
             streams[i].ip = *sessionIP;
@@ -906,13 +903,13 @@ replaceElement(struct sip_msg *msg, str *oldElem, str *newElem)
 
     buf = pkg_malloc(newElem->len);
     if (!buf) {
-        LOG(L_ERR, "error: mediaproxy/replaceElement(): out of memory\n");
+        LM_ERR("out of memory\n");
         return False;
     }
 
     anchor = del_lump(msg, oldElem->s - msg->buf, oldElem->len, 0);
     if (!anchor) {
-        LOG(L_ERR, "error: mediaproxy/replaceElement(): failed to delete old element\n");
+        LM_ERR("failed to delete old element\n");
         pkg_free(buf);
         return False;
     }
@@ -920,7 +917,7 @@ replaceElement(struct sip_msg *msg, str *oldElem, str *newElem)
     memcpy(buf, newElem->s, newElem->len);
 
     if (insert_new_lump_after(anchor, buf, newElem->len, 0)==0) {
-        LOG(L_ERR, "error: mediaproxy/replaceElement(): failed to insert new element\n");
+        LM_ERR("failed to insert new element\n");
         pkg_free(buf);
         return False;
     }
@@ -989,12 +986,12 @@ sendMediaproxyCommand(char *command)
 
     smpSocket = socket(AF_LOCAL, SOCK_STREAM, 0);
     if (smpSocket < 0) {
-        LOG(L_ERR, "error: mediaproxy/sendMediaproxyCommand(): can't create socket\n");
+        LM_ERR("failed to create socket\n");
         return NULL;
     }
     if (connect(smpSocket, (struct sockaddr*)&addr, sizeof(addr)) < 0) {
         close(smpSocket);
-        LOG(L_ERR, "error: mediaproxy/sendMediaproxyCommand(): can't connect to MediaProxy\n");
+        LM_ERR("failed to connect to MediaProxy\n");
         return NULL;
     }
 
@@ -1002,7 +999,7 @@ sendMediaproxyCommand(char *command)
 
     if (len <= 0) {
         close(smpSocket);
-        LOG(L_ERR, "error: mediaproxy/sendMediaproxyCommand(): can't send command to MediaProxy\n");
+        LM_ERR("failed to send command to MediaProxy\n");
         return NULL;
     }
 
@@ -1011,7 +1008,7 @@ sendMediaproxyCommand(char *command)
     close(smpSocket);
 
     if (len < 0) {
-        LOG(L_ERR, "error: mediaproxy/sendMediaproxyCommand(): can't read reply from MediaProxy\n");
+        LM_ERR("failed to read reply from MediaProxy\n");
         return NULL;
     }
 
@@ -1077,8 +1074,6 @@ checkAsymmetricFile(AsymmetricClients *aptr)
     if (statbuf.st_mtime <= aptr->timestamp)
         return; // not changed
 
-    // now we have work to do
-
     which = (aptr == &sipAsymmetrics ? "SIP" : "RTP");
 
     if (!aptr->clients) {
@@ -1089,10 +1084,9 @@ checkAsymmetricFile(AsymmetricClients *aptr)
         size = 32;
         aptr->clients = (regex_t**)pkg_malloc(size*sizeof(regex_t**));
         if (!aptr->clients) {
-            LOG(L_WARN, "warning: mediaproxy/checkAsymmetricFile() cannot "
-                "allocate memory for the %s asymmetric client list. "
-                "%s asymmetric clients will not be handled properly.\n",
-                which, which);
+            LM_WARN("cannot allocate memory for the %s asymmetric client list."
+                    " %s asymmetric clients will not be handled properly.\n",
+                    which, which);
             return; // ignore as it is not fatal
         }
         aptr->size  = size;
@@ -1125,10 +1119,9 @@ checkAsymmetricFile(AsymmetricClients *aptr)
 
         re = (regex_t*)pkg_malloc(sizeof(regex_t));
         if (!re) {
-            LOG(L_WARN, "warning: mediaproxy/checkAsymmetricFile(): "
-                "cannot allocate memory for all the %s asymmetric "
-                "clients listed in file. Some of them will not be "
-                "handled properly.\n", which);
+            LM_WARN("cannot allocate memory for all the %s asymmetric "
+                    "clients listed in file. Some of them will not be "
+                    "handled properly.\n", which);
             break;
         }
         code = regcomp(re, line.s, REG_EXTENDED|REG_ICASE|REG_NOSUB);
@@ -1138,10 +1131,9 @@ checkAsymmetricFile(AsymmetricClients *aptr)
                 regs = aptr->clients;
                 regs = (regex_t**)pkg_realloc(regs, size*sizeof(regex_t**));
                 if (!regs) {
-                    LOG(L_WARN, "warning: mediaproxy/checkAsymmetricFile(): "
-                        "cannot allocate memory for all the %s asymmetric "
-                        "clients listed in file. Some of them will not be "
-                        "handled properly.\n", which);
+                    LM_WARN("cannot allocate memory for all the %s asymmetric "
+                            "clients listed in file. Some of them will not be "
+                            "handled properly.\n", which);
                     break;
                 }
                 aptr->clients = regs;
@@ -1151,18 +1143,18 @@ checkAsymmetricFile(AsymmetricClients *aptr)
             aptr->count++;
         } else {
             regerror(code, re, errbuf, 256);
-            LOG(L_WARN, "warning: mediaproxy/checkAsymmetricFile(): cannot "
-                "compile line %d of the %s asymmetric clients file into a "
-                "regular expression (will be ignored): %s", i, which, errbuf);
+            LM_WARN("cannot compile line %d of the %s asymmetric clients file "
+                    "into a regular expression (will be ignored): %s",
+                    i, which, errbuf);
             pkg_free(re);
         }
     }
 
     aptr->timestamp = statbuf.st_mtime;
 
-    LOG(L_INFO, "info: mediaproxy: %sloaded %s asymmetric clients file "
-        "containing %d entr%s.\n", firstTime ? "" : "re",
-        which, aptr->count, aptr->count==1 ? "y" : "ies");
+    LM_INFO("%sloaded %s asymmetric clients file containing %d entr%s.\n",
+            firstTime ? "" : "re", which, aptr->count,
+            aptr->count==1 ? "y" : "ies");
 
 }
 
@@ -1213,8 +1205,7 @@ isSIPAsymmetric(str userAgent)
         } else if (code != REG_NOMATCH) {
             char errbuf[256];
             regerror(code, sipAsymmetrics.clients[i], errbuf, 256);
-            LOG(L_WARN, "warning: mediaproxy/isSIPAsymmetric() failed to "
-                "match regexp: %s\n", errbuf);
+            LM_WARN("failed to match regexp: %s\n", errbuf);
         }
     }
 
@@ -1246,8 +1237,7 @@ isRTPAsymmetric(str userAgent)
         } else if (code != REG_NOMATCH) {
             char errbuf[256];
             regerror(code, rtpAsymmetrics.clients[i], errbuf, 256);
-            LOG(L_WARN, "warning: mediaproxy/isRTPAsymmetric() failed to "
-                "match regexp: %s\n", errbuf);
+            LM_WARN("failed to match regexp: %s\n", errbuf);
         }
     }
 
@@ -1259,7 +1249,7 @@ isRTPAsymmetric(str userAgent)
 
 // NAT tests
 
-/* tests if address of signaling is different from address in 1st Via field */
+/* Test if address of signaling is different from address in 1st Via field */
 static Bool
 testSourceAddress(struct sip_msg* msg)
 {
@@ -1278,7 +1268,7 @@ testSourceAddress(struct sip_msg* msg)
     return (diffIP || diffPort);
 }
 
-/* tests if Contact field contains a private IP address as defined in RFC1918 */
+/* Test if Contact field contains a private IP address as defined in RFC1918 */
 static Bool
 testPrivateContact(struct sip_msg* msg)
 {
@@ -1291,7 +1281,7 @@ testPrivateContact(struct sip_msg* msg)
     return isPrivateAddress(&(uri.host));
 }
 
-/* tests if top Via field contains a private IP address as defined in RFC1918 */
+/* Test if top Via field contains a private IP address as defined in RFC1918 */
 static Bool
 testPrivateVia(struct sip_msg* msg)
 {
@@ -1327,13 +1317,13 @@ EndMediaSession(struct sip_msg* msg, char* str1, char* str2)
     str callId;
 
     if (!getCallId(msg, &callId)) {
-        LOG(L_ERR, "error: end_media_session(): can't get Call-Id\n");
+        LM_ERR("can't get Call-Id\n");
         return -1;
     }
 
     command = pkg_malloc(callId.len + 20);
     if (command == NULL) {
-        LOG(L_ERR, "error: end_media_session(): out of memory\n");
+        LM_ERR("out of memory\n");
         return -1;
     }
 
@@ -1350,7 +1340,7 @@ static int
 UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
 {
     str sdp, sessionIP, signalingIP, callId, userAgent, tokens[64];
-    str fromDomain, toDomain, fromAddr, toAddr, fromTag, toTag;
+    str fromDomain, toDomain, fromAddr, toAddr, fromTag, toTag, domain;
     char *ptr, *command, *result, *agent, *fromType, *toType, *info;
     int streamCount, i, port, count, portCount, cmdlen, infolen, status;
     StreamInfo streams[64], stream;
@@ -1365,7 +1355,7 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
     }
 
     if (!getCallId(msg, &callId)) {
-        LOG(L_ERR, "error: use_media_proxy(): can't get Call-Id\n");
+        LM_ERR("can't get Call-Id\n");
         return -1;
     }
 
@@ -1375,21 +1365,18 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
         return status;
 
     if (!getSessionLevelMediaIP(&sdp, &sessionIP)) {
-        LOG(L_ERR, "error: use_media_proxy(): error parsing the SDP message\n");
+        LM_ERR("failed to parse the SDP message\n");
         return -1;
     }
 
     streamCount = getMediaStreams(&sdp, &sessionIP, streams, 64);
     if (streamCount == -1) {
-        LOG(L_ERR, "error: use_media_proxy(): can't extract media streams "
-            "from the SDP message\n");
+        LM_ERR("can't extract media streams from the SDP message\n");
         return -1;
     }
 
-    if (streamCount == 0) {
-        // there are no media streams. we have nothing to do.
-        return 1;
-    }
+    if (streamCount == 0)
+        return 1; // there are no media streams. we have nothing to do.
 
     fromDomain = getFromDomain(msg);
     fromType   = (isFromLocal(msg, NULL, NULL)>0) ? "local" : "remote";
@@ -1408,8 +1395,10 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
     }
 
     signalingIP = getSignalingIP(msg);
+    domain = getMediaproxyDomain(msg);
 
-    infolen = fromAddr.len + toAddr.len + fromTag.len + toTag.len + 64;
+    infolen = fromAddr.len + toAddr.len + fromTag.len + toTag.len +
+        domain.len + 64;
 
     cmdlen = callId.len + signalingIP.len + fromDomain.len + toDomain.len +
         userAgent.len*3 + infolen + 128;
@@ -1421,7 +1410,7 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
 
     command = pkg_malloc(cmdlen);
     if (!command) {
-        LOG(L_ERR, "error: use_media_proxy(): out of memory\n");
+        LM_ERR("out of memory\n");
         return -1;
     }
 
@@ -1441,14 +1430,14 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
 
     agent = encodeQuopri(userAgent);
     if (!agent) {
-        LOG(L_ERR, "error: use_media_proxy(): out of memory\n");
+        LM_ERR("out of memory\n");
         pkg_free(command);
         return -1;
     }
 
     info = pkg_malloc(infolen);
     if (!info) {
-        LOG(L_ERR, "error: use_media_proxy(): out of memory\n");
+        LM_ERR("out of memory\n");
         pkg_free(command);
         pkg_free(agent);
         return -1;
@@ -1457,6 +1446,11 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
     sprintf(info, "from:%.*s,to:%.*s,fromtag:%.*s,totag:%.*s",
             fromAddr.len, fromAddr.s, toAddr.len, toAddr.s,
             fromTag.len, fromTag.s, toTag.len, toTag.s);
+
+    if (domain.len) {
+        strcat(info, ",domain:");
+        strncat(info, domain.s, domain.len);
+    }
     if (isRTPAsymmetric(userAgent)) {
         strcat(info, ",asymmetric");
     }
@@ -1478,23 +1472,22 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
     count = getTokens(result, tokens, sizeof(tokens)/sizeof(str));
 
     if (count == 0) {
-        LOG(L_ERR, "error: use_media_proxy(): empty response from mediaproxy\n");
+        LM_ERR("empty response from mediaproxy\n");
         return -1;
     } else if (count<streamCount+1) {
         if (request) {
-            LOG(L_ERR, "error: use_media_proxy(): insufficient ports returned "
-                "from mediaproxy: got %d, expected %d\n", count-1, streamCount);
+            LM_ERR("insufficient ports returned from mediaproxy: got %d, "
+                   "expected %d\n", count-1, streamCount);
             return -1;
         } else {
-            LOG(L_WARN, "warning: use_media_proxy(): broken client. Called UA "
-                "added extra media stream(s) in the OK reply\n");
+            LM_WARN("broken client. Called UA added extra media stream(s) "
+                    "in the OK reply\n");
         }
     }
 
     if (sessionIP.s && !isAnyAddress(sessionIP)) {
         if (!replaceElement(msg, &sessionIP, &tokens[0])) {
-            LOG(L_ERR, "error: use_media_proxy(): failed to replace "
-                "session-level media IP in SDP body\n");
+            LM_ERR("failed to replace session-level media IP in the SDP body\n");
             return -1;
         }
     }
@@ -1505,24 +1498,22 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
         // check. is this really necessary?
         port = strtoint(&tokens[i+1]);
         if (port <= 0 || port > 65535) {
-            LOG(L_ERR, "error: use_media_proxy(): invalid port returned "
-                "by mediaproxy: %.*s\n", tokens[i+1].len, tokens[i+1].s);
+            LM_ERR("invalid port returned by mediaproxy: %.*s\n",
+                   tokens[i+1].len, tokens[i+1].s);
             //return -1;
             continue;
         }
 
         if (streams[i].port.len!=1 || streams[i].port.s[0]!='0') {
             if (!replaceElement(msg, &(streams[i].port), &tokens[i+1])) {
-                LOG(L_ERR, "error: use_media_proxy(): failed to replace "
-                    "port in media stream nr. %d\n", i+1);
+                LM_ERR("failed to replace port in media stream number %d\n", i+1);
                 return -1;
             }
         }
 
         if (streams[i].localIP && !isAnyAddress(streams[i].ip)) {
             if (!replaceElement(msg, &(streams[i].ip), &tokens[0])) {
-                LOG(L_ERR, "error: use_media_proxy(): failed to replace "
-                    "IP address in media stream nr. %d\n", i+1);
+                LM_ERR("failed to replace IP address in media stream number %d\n", i+1);
                 return -1;
             }
         }
@@ -1538,44 +1529,61 @@ static int
 mod_init(void)
 {
     bind_usrloc_t ul_bind_usrloc;
+    pv_spec_t avp_spec;
 
     // initialize the signaling_ip_avp structure
-    if (signaling_ip_avp.name.s==NULL || *(signaling_ip_avp.name.s)==0) {
-        LOG(L_ERR, "error: mediaproxy/mod_init: missing/empty signaling_ip_avp parameter. using default.\n");
-        signaling_ip_avp.name.s = SIGNALING_IP_AVP_NAME;
+    if (signaling_ip_avp.spec.s==NULL || *(signaling_ip_avp.spec.s)==0) {
+        LM_WARN("missing/empty signaling_ip_avp parameter. will use default.\n");
+        signaling_ip_avp.spec.s = SIGNALING_IP_AVP_SPEC;
     }
-    signaling_ip_avp.name.len = strlen(signaling_ip_avp.name.s);
-    if (parse_avp_spec(&(signaling_ip_avp.name), &(signaling_ip_avp.type), &(signaling_ip_avp.avp)) < 0) {
-        LOG(L_CRIT, "error: mediaproxy/mod_init: invalid signaling_ip_avp specification `%s'\n", signaling_ip_avp.name.s);
+    signaling_ip_avp.spec.len = strlen(signaling_ip_avp.spec.s);
+    if (pv_parse_spec(&(signaling_ip_avp.spec), &avp_spec)==0 || avp_spec.type!=PVT_AVP) {
+        LM_CRIT("invalid AVP specification for signaling_ip_avp: `%s'\n", signaling_ip_avp.spec.s);
+        return -1;
+    }
+    if (pv_get_avp_name(0, &(avp_spec.pvp), &(signaling_ip_avp.name), &(signaling_ip_avp.type))!=0) {
+        LM_CRIT("invalid AVP specification for signaling_ip_avp: `%s'\n", signaling_ip_avp.spec.s);
+        return -1;
+    }
+
+    // initialize the domain_avp structure
+    if (domain_avp.spec.s==NULL || *(domain_avp.spec.s)==0) {
+        LM_WARN("missing/empty domain_avp parameter. will use default.\n");
+        domain_avp.spec.s = DOMAIN_AVP_SPEC;
+    }
+    domain_avp.spec.len = strlen(domain_avp.spec.s);
+    if (pv_parse_spec(&(domain_avp.spec), &avp_spec)==0 || avp_spec.type!=PVT_AVP) {
+        LM_CRIT("invalid AVP specification for domain_avp: `%s'\n", domain_avp.spec.s);
+        return -1;
+    }
+    if (pv_get_avp_name(0, &(avp_spec.pvp), &(domain_avp.name), &(domain_avp.type))!=0) {
+        LM_CRIT("invalid AVP specification for domain_avp: `%s'\n", domain_avp.spec.s);
         return -1;
     }
 
     isFromLocal = (CheckLocalPartyProc)find_export("is_from_local", 0, 0);
     isDestinationLocal = (CheckLocalPartyProc)find_export("is_uri_host_local", 0, 0);
     if (!isFromLocal || !isDestinationLocal) {
-        LOG(L_ERR, "error: mediaproxy/mod_init(): can't find is_from_local "
-            "and/or is_uri_host_local functions. Check if domain.so is loaded\n");
+        LM_CRIT("can't find the is_from_local and/or is_uri_host_local "
+                "functions. Check if domain.so is loaded\n");
         return -1;
     }
 
     if (natpingInterval > 0) {
         ul_bind_usrloc = (bind_usrloc_t)find_export("ul_bind_usrloc", 1, 0);
         if (!ul_bind_usrloc) {
-            LOG(L_ERR, "error: mediaproxy/mod_init(): can't find the usrloc "
-                "module. Check if usrloc.so is loaded.\n");
+            LM_CRIT("can't find the usrloc module. Check if usrloc.so is loaded.\n");
             return -1;
         }
 
         if (ul_bind_usrloc(&userLocation) < 0) {
-            LOG(L_ERR, "error: mediaproxy/mod_init(): can't access the usrloc "
-                "module.\n");
+            LM_CRIT("can't access the usrloc module.\n");
             return -1;
         }
 
         if (userLocation.nat_flag==0) {
-            LOG(L_ERR, "ERROR:mediaproxy:mod_init: Bad config - "
-                "nat ping enabled, but not nat bflag set in usrloc "
-                "module\n");
+            LM_CRIT("bad config - nat ping enabled, but no nat bflag set in "
+                    "the usrloc module\n");
             return -1;
         }
 
@@ -1590,29 +1598,3 @@ mod_init(void)
 
     return 0;
 }
-
-
-// convert string parameter to integer for functions that expect an integer
-static int
-fixstring2int(void **param, int param_count)
-{
-    unsigned long number;
-    int err;
-
-    if (param_count == 1) {
-        number = str2s(*param, strlen(*param), &err);
-        if (err == 0) {
-            pkg_free(*param);
-            *param = (void*)number;
-            return 0;
-        } else {
-            LOG(L_ERR, "error: mediaproxy/fixstring2int(): bad number `%s'\n",
-                (char*)(*param));
-            return E_CFG;
-        }
-    }
-
-    return 0;
-}
-
-
