@@ -1,5 +1,5 @@
 /*
- * $Id: xr_server.h 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: xr_server.h 2870 2007-10-05 14:19:51Z bogdan_iancu $
  *
  * Copyright (C) 2006 Voice Sistem SRL
  *
@@ -22,6 +22,7 @@
  * History:
  * ---------
  *  2006-11-30  first version (lavinia)
+ *  2007-10-05  support for libxmlrpc-c3 version 1.x.x added (dragos)
  */
 
 
@@ -32,13 +33,21 @@
 #define XMLRPC_WANT_INTERNAL_DECLARATIONS
 #include <xmlrpc.h>
 
+#ifdef XMLRPC_OLD_VERSION
+
 xmlrpc_value * default_method ( xmlrpc_env * env, char * host,
-		char * methodName, xmlrpc_value * paramArray, void * serverInfo );
+		 char * methodName, xmlrpc_value * paramArray, void * serverInfo );
 
-int set_default_method ( xmlrpc_env * env );
+#else
 
-int init_async_lock();
+xmlrpc_value * default_method ( xmlrpc_env * env, const char * host,
+		const char * methodName, xmlrpc_value * paramArray, void * serverInfo );
 
-void destroy_async_lock();
+#endif
+int set_default_method ( xmlrpc_env * env , xmlrpc_registry * registry);
+
+int init_async_lock(void);
+
+void destroy_async_lock(void);
 
 #endif /* _XR_SERVER_H_ */
