@@ -1,5 +1,5 @@
 /* 
- * $Id: db_con.c 1827 2007-03-12 15:22:53Z bogdan_iancu $ 
+ * $Id: db_con.c 2733 2007-09-10 15:25:12Z henningw $ 
  *
  * Database connection related functions
  *
@@ -36,7 +36,7 @@
 int db_mysql_use_table(db_con_t* _h, const char* _t)
 {
 	if ((!_h) || (!_t)) {
-		LOG(L_ERR, "use_table: Invalid parameter value\n");
+		LM_ERR("invalid parameter value\n");
 		return -1;
 	}
 
