@@ -1,5 +1,5 @@
 /* 
- * $Id: res.h 1827 2007-03-12 15:22:53Z bogdan_iancu $ 
+ * $Id: res.h 2871 2007-10-05 14:32:16Z henningw $ 
  *
  * MySQL module result related functions
  *
@@ -29,12 +29,6 @@
 #include "../../db/db_con.h"
 
 
-/*
- * Create a new result structure and initialize it
- */
-db_res_t* db_mysql_new_result(void);
-
-
 /*
  * Fill the structure with data from database
  */
@@ -46,10 +40,6 @@ int db_mysql_convert_result(db_con_t* _h, db_res_t* _r);
  */
 int db_mysql_free_dbresult(db_res_t* _r);
 
-/*
- * Release memory used by rows in a result structure
- */
-int db_mysql_free_rows(db_res_t* _r);
 
 int db_mysql_get_columns(db_con_t* _h, db_res_t* _r);
 
