@@ -43,6 +43,9 @@
 extern char* _osp_device_ip;
 extern char* _osp_device_port;
 extern int _osp_max_dests;
+extern int _osp_redir_uri;
+extern int_str _osp_snid_avpname;
+extern unsigned short _osp_snid_avptype;
 extern OSPTPROVHANDLE _osp_provider;
 extern auth_api_t osp_auth;
 
@@ -52,26 +55,26 @@ const int OSP_MAIN_ROUTE = 1;
 const int OSP_BRANCH_ROUTE = 0;
 const str OSP_CALLING_NAME = {"_osp_calling_translated_", 24};
 
-static int ospLoadRoutes(struct sip_msg* msg, OSPTTRANHANDLE transaction, int destcount, char* source, char* sourcedev, time_t authtime);
-static int ospPrepareDestination(struct sip_msg* msg, int isfirst, int type);
+static int ospLoadRoutes(OSPTTRANHANDLE transaction, int destcount, char* source, char* sourcedev, char* origcalled, time_t authtime);
+static int ospPrepareDestination(struct sip_msg* msg, int isfirst, int type, int format);
 static int ospSetRpid(struct sip_msg* msg, osp_dest* dest);
 
 /*
  * Get routes from AuthRsp
- * param msg SIP message
  * param transaction Transaction handle
  * param destcount Expected destination count
  * param source Source IP
  * param sourcedev Source device IP
+ * param origcalled Original called number
  * param authtime Request authorization time
- * return MODULE_RETURNCODE_TRUE success, MODULE_RETURNCODE_FALSE failure
+ * return 0 success, -1 failure
  */
 static int ospLoadRoutes(
-    struct sip_msg* msg, 
     OSPTTRANHANDLE transaction, 
     int destcount, 
     char* source, 
     char* sourcedev, 
+    char* origcalled, 
     time_t authtime)
 {
     int count;
@@ -80,20 +83,19 @@ static int ospLoadRoutes(
     osp_dest dests[OSP_DEF_DESTS];
     OSPE_DEST_PROT protocol;
     OSPE_DEST_OSP_ENABLED enabled;
-    int result = MODULE_RETURNCODE_TRUE;
+    int result = 0;
     
-    LOG(L_DBG, "osp: ospLoadRoutes\n");
-
     for (count = 0; count < destcount; count++) {
         /* This is necessary becuase we will save destinations in reverse order */
         dest = ospInitDestination(&dests[count]);
 
         if (dest == NULL) {
-            result = MODULE_RETURNCODE_FALSE;
+            result = -1;
             break;
         }
 
-        dest->destinationCount = count+1;
+        dest->destinationCount = count + 1;
+        strncpy(dest->origcalled, origcalled, sizeof(dest->origcalled) - 1);
 
         if (count == 0) {
             errorcode = OSPPTransactionGetFirstDestination(
@@ -136,20 +138,19 @@ static int ospLoadRoutes(
                 dest->token);
         }
         
-        if (errorcode != 0) {
-            LOG(L_ERR, 
-                "osp: ERROR: failed to load routes (%d) expected '%d' current '%d'\n", 
+        if (errorcode != OSPC_ERR_NO_ERROR) {
+            LM_ERR("failed to load routes (%d) expected '%d' current '%d'\n", 
                 errorcode, 
                 destcount, 
                 count);
-            result = MODULE_RETURNCODE_FALSE;
+            result = -1;
             break;
         }
 
         errorcode = OSPPTransactionGetDestProtocol(transaction, &protocol);
-        if (errorcode != 0) {
+        if (errorcode != OSPC_ERR_NO_ERROR) {
             /* This does not mean an ERROR. The OSP server may not support OSP 2.1.1 */
-            LOG(L_DBG, "osp: cannot get dest protocol (%d)\n", errorcode);
+            LM_DBG("cannot get dest protocol (%d)\n", errorcode);
             protocol = OSPE_DEST_PROT_SIP;
         }
         switch (protocol) {
@@ -167,9 +168,9 @@ static int ospLoadRoutes(
         }
 
         errorcode = OSPPTransactionIsDestOSPEnabled(transaction, &enabled);
-        if (errorcode != 0) {
+        if (errorcode != OSPC_ERR_NO_ERROR) {
             /* This does not mean an ERROR. The OSP server may not support OSP 2.1.1 */
-            LOG(L_DBG, "osp: cannot get dest OSP version (%d)\n", errorcode);
+            LM_DBG("cannot get dest OSP version (%d)\n", errorcode);
         } else if (enabled == OSPE_OSP_FALSE) {
             /* Destination device does not support OSP. Do not send token to it */
             dest->token[0] = '\0';
@@ -177,20 +178,19 @@ static int ospLoadRoutes(
         }
 
         errorcode = OSPPTransactionGetDestNetworkId(transaction, dest->networkid);
-        if (errorcode != 0) {
+        if (errorcode != OSPC_ERR_NO_ERROR) {
             /* This does not mean an ERROR. The OSP server may not support OSP 2.1.1 */
-            LOG(L_DBG, "osp: cannot get dest network ID (%d)\n", errorcode);
+            LM_DBG("cannot get dest network ID (%d)\n", errorcode);
             dest->networkid[0] = '\0';
         }
 
-        strcpy(dest->source, source);
-        strcpy(dest->srcdev, sourcedev);
+        strncpy(dest->source, source, sizeof(dest->source) - 1);
+        strncpy(dest->srcdev, sourcedev, sizeof(dest->srcdev) - 1);
         dest->type = OSPC_SOURCE;
-        dest->tid = ospGetTransactionId(transaction);
+        dest->transid = ospGetTransactionId(transaction);
         dest->authtime = authtime;
 
-        LOG(L_INFO,
-            "osp: get destination '%d': "
+        LM_INFO("get destination '%d': "
             "valid after '%s' "
             "valid until '%s' "
             "time limit '%i' seconds "
@@ -220,7 +220,7 @@ static int ospLoadRoutes(
      * when we start searching avps the destinations
      * will be in order 
      */
-    if (result == MODULE_RETURNCODE_TRUE) {
+    if (result == 0) {
         for(count = destcount -1; count >= 0; count--) {
             ospSaveOrigDestination(&dests[count]);
         }
@@ -243,9 +243,12 @@ int ospRequestRouting(
 {
     int errorcode;
     time_t authtime;
-    char tmp[OSP_STRBUF_SIZE];
     char source[OSP_E164BUF_SIZE];
     char sourcedev[OSP_STRBUF_SIZE];
+    char src[OSP_STRBUF_SIZE];
+    struct usr_avp* snidavp = NULL;
+    int_str snidval;
+    char snid[OSP_STRBUF_SIZE];
     char destination[OSP_E164BUF_SIZE];
     unsigned int callidnumber = 1;
     OSPTCALLID* callids[callidnumber];
@@ -256,43 +259,50 @@ int ospRequestRouting(
     OSPTTRANHANDLE transaction = -1;
     int result = MODULE_RETURNCODE_FALSE;
 
-    LOG(L_DBG, "osp: ospRequestRouting\n");
-
     authtime = time(NULL);
 
     destcount = _osp_max_dests;
 
-    if ((errorcode = OSPPTransactionNew(_osp_provider, &transaction)) != 0) {
-        LOG(L_ERR, "osp: ERROR: failed to create new OSP transaction (%d)\n", errorcode);
+    if ((errorcode = OSPPTransactionNew(_osp_provider, &transaction)) != OSPC_ERR_NO_ERROR) {
+        LM_ERR("failed to create new OSP transaction (%d)\n", errorcode);
     } else if ((ospGetRpidUserpart(msg, source, sizeof(source)) != 0) &&
         (ospGetFromUserpart(msg, source, sizeof(source)) != 0)) 
     {
-        LOG(L_ERR, "osp: ERROR: failed to extract calling number\n");
+        LM_ERR("failed to extract calling number\n");
     } else if ((ospGetUriUserpart(msg, destination, sizeof(destination)) != 0) &&
         (ospGetToUserpart(msg, destination, sizeof(destination)) != 0)) 
     {
-        LOG(L_ERR, "osp: ERROR: failed to extract called number\n");
+        LM_ERR("failed to extract called number\n");
     } else if (ospGetCallId(msg, &(callids[0])) != 0) {
-        LOG(L_ERR, "osp: ERROR: failed to extract call id\n");
+        LM_ERR("failed to extract call id\n");
     } else if (ospGetSourceAddress(msg, sourcedev, sizeof(sourcedev)) != 0) {
-        LOG(L_ERR, "osp: ERROR: failed to extract source address\n");
+        LM_ERR("failed to extract source address\n");
     } else {
-        ospConvertAddress(sourcedev, tmp, sizeof(tmp));
+        ospConvertAddress(sourcedev, src, sizeof(src));
 
-        LOG(L_INFO,
-            "osp: request auth and routing for: "
-            "transaction '%i' "
+        if ((_osp_snid_avpname.n != 0) &&
+            ((snidavp = search_first_avp(_osp_snid_avptype, _osp_snid_avpname, &snidval, 0)) != NULL) &&
+            (snidavp->flags & AVP_VAL_STR) && (snidval.s.s && snidval.s.len)) 
+        {
+            snprintf(snid, sizeof(snid), "%.*s", snidval.s.len, snidval.s.s);
+            OSPPTransactionSetNetworkIds(transaction, snid, "");
+        } else {
+            snid[0] = '\0';
+        }
+
+        LM_INFO("request auth and routing for: "
             "source '%s' "
             "source_port '%s' "
             "source_dev '%s' "
+            "source_networkid '%s' "
             "e164_source '%s' "
             "e164_dest '%s' "
             "call_id '%.*s' "
             "dest_count '%i'\n",
-            transaction,
             _osp_device_ip,
             _osp_device_port,
-            tmp,                        /* sourcedev in "[x.x.x.x]" or host.domain format */
+            src,                        /* sourcedev in "[x.x.x.x]" or host.domain format */
+            snid,
             source,
             destination,
             callids[0]->ospmCallIdLen,
@@ -300,18 +310,11 @@ int ospRequestRouting(
             destcount
         );    
 
-/* Do not use network ID as port */
-#if 0
-        if (strlen(_osp_device_port) > 0) {
-            OSPPTransactionSetNetworkIds(transaction, _osp_device_port, "");
-        }
-#endif
-
         /* try to request authorization */
         errorcode = OSPPTransactionRequestAuthorisation(
             transaction,       /* transaction handle */
             _osp_device_ip,    /* from the configuration file */
-            tmp,               /* source of call, protocol specific, in OSP format */
+            src,               /* source of call, protocol specific, in OSP format */
             source,            /* calling number in nodotted e164 notation */
             OSPC_E164,         /* calling number format */
             destination,       /* called number */
@@ -324,30 +327,34 @@ int ospRequestRouting(
             &logsize,          /* size allocated for detaillog (next param) 0=no log */
             detaillog);        /* memory location for detaillog to be stored */
 
-        if ((errorcode == 0) && (destcount > 0)) {
-            LOG(L_INFO, 
-                "osp: there are '%d' OSP routes, call_id '%.*s' transaction_id '%lld'\n",
+        if ((errorcode == OSPC_ERR_NO_ERROR) &&
+            (ospLoadRoutes(transaction, destcount, _osp_device_ip, sourcedev, destination, authtime) == 0))
+        {
+            LM_INFO("there are '%d' OSP routes, call_id '%.*s'\n",
                 destcount,
-                callids[0]->ospmCallIdLen, 
-                callids[0]->ospmCallIdVal,
-                ospGetTransactionId(transaction));
-            /* Will record a unique cookie in the on-branch section */
-            result = ospLoadRoutes(msg, transaction, destcount, _osp_device_ip, sourcedev, authtime);
-        } else if ((errorcode == 0) && (destcount == 0)) {
-            LOG(L_INFO, 
-                "osp: there is 0 osp routes, call_id '%.*s' transaction_id '%lld'\n",
                 callids[0]->ospmCallIdLen,
-                callids[0]->ospmCallIdVal,
-                ospGetTransactionId(transaction));
-            /* Must do manually since callback does not work for this case. Do not know why. */
-            ospRecordEvent(0, 503);
+                callids[0]->ospmCallIdVal);
+            result = MODULE_RETURNCODE_TRUE;
         } else {
-            LOG(L_ERR, 
-                "osp: ERROR: failed to request auth and routing (%i), call_id '%.*s' transaction_id '%lld'\n",
+            LM_ERR("failed to request auth and routing (%i), call_id '%.*s\n",
                 errorcode,
                 callids[0]->ospmCallIdLen,
-                callids[0]->ospmCallIdVal,
-                ospGetTransactionId(transaction));
+                callids[0]->ospmCallIdVal);
+            switch (errorcode) {
+                case OSPC_ERR_TRAN_ROUTE_BLOCKED:
+                    result = -403;
+                    break;
+                case OSPC_ERR_TRAN_ROUTE_NOT_FOUND:
+                    result = -404;
+                    break;
+                case OSPC_ERR_NO_ERROR:
+                    /* AuthRsp ok but ospLoadRoutes fails */
+                    result = -500;
+                    break;
+                default:
+                    result = MODULE_RETURNCODE_FALSE;
+                    break;
+            }
         }
     }
 
@@ -374,8 +381,6 @@ int ospCheckRoute(
     char* ignore1, 
     char* ignore2)
 {
-    LOG(L_DBG, "osp: ospCheckRoute\n");
-
     if (ospCheckOrigDestination() == 0) {
         return MODULE_RETURNCODE_TRUE;
     } else {
@@ -399,12 +404,10 @@ static int ospSetRpid(
     char buffer[OSP_STRBUF_SIZE];
     int result = -1;
 
-    LOG(L_DBG, "osp: ospSetRpid\n");
-
     if ((ospGetRpidUserpart(msg, calling, sizeof(calling)) != 0) &&
         (ospGetFromUserpart(msg, calling, sizeof(calling)) !=0))
     {
-        LOG(L_ERR, "osp: ERROR: failed to extract calling number\n");
+        LM_ERR("failed to extract calling number\n");
         return result;
     } 
 
@@ -412,12 +415,13 @@ static int ospSetRpid(
         /* Do nothing for this case */ 
         result = 1;
     } else if ((osp_auth.rpid_avp.s.s == NULL) || (osp_auth.rpid_avp.s.len == 0)) {
-        LOG(L_WARN, "osp: WARN: rpid_avp is not foune, cannot set rpid avp\n");
+        LM_WARN("rpid_avp is not foune, cannot set rpid avp\n");
         result = -1;
     } else {
         if (dest->source[0] == '[') {
             /* Strip "[]" */
-            strncpy(source, &dest->source[1], sizeof(source));
+            memset(source, 0, sizeof(source));
+            strncpy(source, &dest->source[1], sizeof(source) - 1);
             source[strlen(source) - 1] = '\0';
         }
     
@@ -452,23 +456,18 @@ int ospCheckTranslation(
     char* ignore1, 
     char* ignore2)
 {
-    struct usr_avp* callingavp = NULL;
     int_str callingval;
     int result = MODULE_RETURNCODE_FALSE;
 
-    LOG(L_DBG, "osp: ospCheckTranslation\n");
-
-    callingavp = search_first_avp(AVP_NAME_STR, (int_str)OSP_CALLING_NAME, NULL, 0);
-    if (callingavp != NULL) {
-        get_avp_val(callingavp, &callingval);
+    if (search_first_avp(AVP_NAME_STR, (int_str)OSP_CALLING_NAME, &callingval, 0) != NULL) {
         if (callingval.n == 0) {
-            LOG(L_DBG, "osp: the calling number does not been translated\n");
+            LM_DBG("the calling number does not been translated\n");
         } else {
-            LOG(L_DBG, "osp: the calling number is translated\n");
+            LM_DBG("the calling number is translated\n");
             result = MODULE_RETURNCODE_TRUE;
         }
     } else {
-        LOG(L_ERR, "osp: ERROR: there is not calling translation avp\n");
+        LM_ERR("there is not calling translation avp\n");
     }
 
     return result;
@@ -479,40 +478,40 @@ int ospCheckTranslation(
  * param msg SIP message
  * param isfirst Is first destination
  * param type Main or branch route block
+ * param format URI format
  * return MODULE_RETURNCODE_TRUE success MODULE_RETURNCODE_FALSE failure
  */
 static int ospPrepareDestination(
     struct sip_msg* msg, 
     int isfirst,
-    int type)
+    int type,
+    int format)
 {
     int_str val;
     int res;
     str newuri = {NULL, 0};
     int result = MODULE_RETURNCODE_FALSE;
 
-    LOG(L_DBG, "osp: ospPrepareDestination\n");
-
     osp_dest* dest = ospGetNextOrigDestination();
 
     if (dest != NULL) {
-        ospRebuildDestionationUri(&newuri, dest->called, dest->host, "");
+        ospRebuildDestionationUri(&newuri, dest->called, dest->host, "", format);
 
-        LOG(L_INFO, 
-            "osp: prepare route to URI '%.*s' for call_id '%.*s' transaction_id '%lld'\n",
+        LM_INFO("prepare route to URI '%.*s' for call_id '%.*s' transaction_id '%llu'\n",
             newuri.len,
             newuri.s,
             dest->callidsize,
             dest->callid,
-            dest->tid);
+            dest->transid);
 
         if (type == OSP_MAIN_ROUTE) {
             if (isfirst == OSP_FIRST_ROUTE) {
                 rewrite_uri(msg, &newuri);
             } else {
-                append_branch(msg, &newuri, NULL, NULL, 0, 0, NULL);
+                append_branch(msg, &newuri, NULL, NULL, Q_UNSPECIFIED, 0, NULL);
             }
             /* Do not add route specific OSP information */
+            result = MODULE_RETURNCODE_TRUE;
         } else if (type == OSP_BRANCH_ROUTE) {
             /* For branch route, add route specific OSP information */
 
@@ -523,7 +522,7 @@ static int ospPrepareDestination(
             ospAddOspHeader(msg, dest->token, dest->tokensize);
 
             /* Add branch-specific OSP Cookie */
-            ospRecordOrigTransaction(msg, dest->tid, dest->srcdev, dest->calling, dest->called, dest->authtime, dest->destinationCount);
+            ospRecordOrigTransaction(msg, dest->transid, dest->srcdev, dest->calling, dest->called, dest->authtime, dest->destinationCount);
 
             /* Add rpid avp for calling number translation */
             res = ospSetRpid(msg, dest);
@@ -534,7 +533,7 @@ static int ospPrepareDestination(
                     add_avp(AVP_NAME_STR, (int_str)OSP_CALLING_NAME, val);
                     break;
                 default:
-                    LOG(L_DBG, "osp: cannot set rpid avp\n");
+                    LM_DBG("cannot set rpid avp\n");
                     /* Just like without calling translation */
                 case 1:
                     /* Calling number does not been translated */
@@ -545,10 +544,10 @@ static int ospPrepareDestination(
 
             result = MODULE_RETURNCODE_TRUE;
         } else {
-            LOG(L_ERR, "osp: ERROR: unsupported route block type\n");
+            LM_ERR("unsupported route block type\n");
         }
     } else {
-        LOG(L_DBG, "osp: there is no more routes\n");
+        LM_DBG("there is no more routes\n");
         ospReportOrigSetupUsage();
     }
 
@@ -575,10 +574,8 @@ int ospPrepareRoute(
 {
     int result = MODULE_RETURNCODE_TRUE;
 
-    LOG(L_DBG, "osp: ospPrepareRoute\n");
-
     /* The first parameter will be ignored */
-    result = ospPrepareDestination(msg, OSP_FIRST_ROUTE, OSP_BRANCH_ROUTE);
+    result = ospPrepareDestination(msg, OSP_FIRST_ROUTE, OSP_BRANCH_ROUTE, 0);
 
     return result;
 }
@@ -598,11 +595,9 @@ int ospPrepareAllRoutes(
 {
     int result = MODULE_RETURNCODE_TRUE;
 
-    LOG(L_DBG, "osp: ospPrepareAllRoutes\n");
-
-    for(result = ospPrepareDestination(msg, OSP_FIRST_ROUTE, OSP_MAIN_ROUTE);
+    for(result = ospPrepareDestination(msg, OSP_FIRST_ROUTE, OSP_MAIN_ROUTE, _osp_redir_uri);
         result == MODULE_RETURNCODE_TRUE;
-        result = ospPrepareDestination(msg, OSP_NEXT_ROUTE, OSP_MAIN_ROUTE))
+        result = ospPrepareDestination(msg, OSP_NEXT_ROUTE, OSP_MAIN_ROUTE, _osp_redir_uri))
     {
     }
 
