@@ -32,6 +32,7 @@
  *  2006-03-13  RR functions are loaded via API function (bogdan)
  */
 
+#include <osp/osp.h>
 #include "../rr/api.h"
 #include "../../usr_avp.h"
 #include "usage.h"
@@ -45,29 +46,40 @@
 #define OSP_RELEASE_ORIG    0
 #define OSP_RELEASE_TERM    1
 
+/* The up case tags for the destinations may corrupt OSP cookies */
+#define OSP_COOKIE_TRANSID      't'
+#define OSP_COOKIE_TRANSIDUP    'T'
+#define OSP_COOKIE_SRCIP        's'
+#define OSP_COOKIE_SRCIPUP      'S'
+#define OSP_COOKIE_AUTHTIME     'a'
+#define OSP_COOKIE_AUTHTIMEUP   'A'
+#define OSP_COOKIE_DSTCOUNT     'c'
+#define OSP_COOKIE_DSTCOUNTUP   'C'
+
 extern char* _osp_device_ip;
 extern OSPTPROVHANDLE _osp_provider;
 extern str OSP_ORIGDEST_NAME;
 extern struct rr_binds osp_rr;
 
-static void ospRecordTransaction(struct sip_msg* msg, unsigned long long tranid, char* uac, char* from, char* to, time_t authtime, int isorig, unsigned destinationCount);
+static void ospRecordTransaction(struct sip_msg* msg, unsigned long long transid, char* uac, char* from, char* to, time_t authtime, int isorig, unsigned destinationCount);
 static int ospBuildUsageFromDestination(OSPTTRANHANDLE transaction, osp_dest* dest, int lastcode);
 static int ospReportUsageFromDestination(OSPTTRANHANDLE transaction, osp_dest* dest);
-static int ospReportUsageFromCookie(struct sip_msg* msg, char* cooky, OSPTCALLID* callid, int release, int isorig);
+static int ospReportUsageFromCookie(struct sip_msg* msg, char* cooky, OSPTCALLID* callid, int release, OSPE_MSG_ROLETYPES type);
 
 /*
  * Create OSP cookie and insert it into Record-Route header
  * param msg SIP message
- * param tansaction Transaction handle
+ * param tansid Transaction ID
  * param uac Source IP
  * param from
  * param to
  * param authtime Request authorization time
  * param isorig Originate / Terminate
+ * param destinationCount Destination count
  */
 static void ospRecordTransaction(
     struct sip_msg* msg, 
-    unsigned long long tranid, 
+    unsigned long long transid,
     char* uac, 
     char* from, 
     char* to, 
@@ -78,13 +90,8 @@ static void ospRecordTransaction(
     str cookie;
     char buffer[OSP_STRBUF_SIZE];
 
-    LOG(L_DBG, "osp: ospRecordTransaction\n");
-
     if (osp_rr.add_rr_param == 0) {
-        LOG(L_WARN,
-            "osp: WARN: add_rr_param function is not found, "
-            "cannot record information about the "
-            "OSP transaction\n");
+        LM_WARN("add_rr_param function is not found, cannot record information about the OSP transaction\n");
         return;
     }
 
@@ -94,44 +101,52 @@ static void ospRecordTransaction(
         cookie.len = snprintf(
             buffer,
             sizeof(buffer),
-            ";%s=t%llu_s%s_T%d_c%d",
+            ";%s=%c%llu_%c%s_%c%d_%c%d",
             OSP_ORIG_COOKIE,
-            tranid,
+            OSP_COOKIE_TRANSID,
+            transid,
+            OSP_COOKIE_SRCIP,
             uac,
+            OSP_COOKIE_AUTHTIME,
             (unsigned int)authtime,
+            OSP_COOKIE_DSTCOUNT,
             destinationCount);
     } else {
         cookie.len = snprintf(
             buffer,
             sizeof(buffer),
-            ";%s=t%llu_s%s_T%d",
+            ";%s=%c%llu_%c%s_%c%d",
             OSP_TERM_COOKIE,
-            tranid,
+            OSP_COOKIE_TRANSID,
+            transid,
+            OSP_COOKIE_SRCIP,
             uac,
+            OSP_COOKIE_AUTHTIME,
             (unsigned int)authtime);
     }
 
     if (cookie.len < 0) {
-        LOG(L_ERR, "osp: ERROR: failed to create OSP cookie\n");
+        LM_ERR("failed to create OSP cookie\n");
         return;
     }
 
-    LOG(L_DBG, "osp: adding RR parameter '%s'\n", buffer);
+    LM_DBG("adding RR parameter '%s'\n", buffer);
     osp_rr.add_rr_param(msg, &cookie);
 }
 
 /*
  * Create OSP originate cookie and insert it into Record-Route header
  * param msg SIP message
- * param tansaction Transaction handle
+ * param tansid Transaction ID
  * param uac Source IP
  * param from
  * param to
  * param authtime Request authorization time
+ * param destinationCount Destination count
  */
 void ospRecordOrigTransaction(
     struct sip_msg* msg, 
-    unsigned long long tranid, 
+    unsigned long long transid, 
     char* uac, 
     char* from, 
     char* to, 
@@ -140,15 +155,13 @@ void ospRecordOrigTransaction(
 {
     int isorig = 1;
 
-    LOG(L_DBG, "osp: ospRecordOrigTransaction\n");
-
-    ospRecordTransaction(msg, tranid, uac, from, to, authtime, isorig, destinationCount);
+    ospRecordTransaction(msg, transid, uac, from, to, authtime, isorig, destinationCount);
 }
 
 /*
  * Create OSP terminate cookie and insert it into Record-Route header
  * param msg SIP message
- * param tansaction Transaction handle
+ * param tansid Transaction ID
  * param uac Source IP
  * param from
  * param to
@@ -156,7 +169,7 @@ void ospRecordOrigTransaction(
  */
 void ospRecordTermTransaction(
     struct sip_msg* msg, 
-    unsigned long long tranid, 
+    unsigned long long transid, 
     char* uac, 
     char* from, 
     char* to, 
@@ -165,9 +178,7 @@ void ospRecordTermTransaction(
     int isorig = 0;
     unsigned destinationCount = 0; /* N/A */
 
-    LOG(L_DBG, "osp: ospRecordTermTransaction\n");
-
-    ospRecordTransaction(msg, tranid, uac, from, to, authtime, isorig, destinationCount);
+    ospRecordTransaction(msg, transid, uac, from, to, authtime, isorig, destinationCount);
 }
 
 /*
@@ -176,7 +187,7 @@ void ospRecordTermTransaction(
  * param cookie OSP cookie
  * param callid Call ID
  * param release Who releases the call first. 0 orig, 1 term
- * param isorig Originate / Terminate
+ * param type Usage type
  * return
  */
 static int ospReportUsageFromCookie(
@@ -184,15 +195,15 @@ static int ospReportUsageFromCookie(
     char* cookie, 
     OSPTCALLID* callid, 
     int release,
-    int isorig)
+    OSPE_MSG_ROLETYPES type)
 {
     char* tmp;
     char* token;
     char tag;
     char* value;
-    unsigned long long transactionid = 0;
-    char* uac = "";
-    time_t authtime = -1;
+    unsigned long long transid = 0;
+    time_t authtime = 0;
+    unsigned destinationCount = 0;
     time_t endtime = time(NULL);
     char firstvia[OSP_STRBUF_SIZE];
     char from[OSP_STRBUF_SIZE];
@@ -200,8 +211,8 @@ static int ospReportUsageFromCookie(
     char nexthop[OSP_STRBUF_SIZE];
     char* calling;
     char* called;
+    char* originator = NULL;
     char* terminator;
-    unsigned issource;
     char* source;
     char srcbuf[OSP_STRBUF_SIZE];
     char* destination;
@@ -210,34 +221,37 @@ static int ospReportUsageFromCookie(
     char devbuf[OSP_STRBUF_SIZE];
     OSPTTRANHANDLE transaction = -1;
     int errorcode;
-    unsigned destinationCount = 0;
 
-    LOG(L_DBG, "osp: ospReportUsageFromCookie\n");
+    LM_DBG("'%s' type '%d'\n", cookie, type);
+    if (cookie != NULL) {
+        for (token = strtok_r(cookie, "_", &tmp);
+            token;
+            token = strtok_r(NULL, "_", &tmp))
+        {
+            tag = *token;
+            value= token + 1;
 
-    LOG(L_DBG, "osp: '%s' isorig '%d'\n", cookie, isorig);
-    for (token = strtok_r(cookie, "_", &tmp);
-        token;
-        token = strtok_r(NULL, "_", &tmp))
-    {
-        tag = *token;
-        value= token + 1;
-
-        switch (tag) {
-            case 't':
-                transactionid = atoll(value);
-                break;
-            case 'T':
-                authtime = atoi(value);
-                break;
-            case 's':
-                uac = value;
-                break;
-            case 'c':
-                destinationCount = (unsigned)atoi(value);
-                break;
-            default:
-                LOG(L_ERR, "osp: ERROR: unexpected tag '%c' / value '%s'\n", tag, value);
-                break;
+            switch (tag) {
+                case OSP_COOKIE_TRANSID:
+                case OSP_COOKIE_TRANSIDUP:
+                    transid = atoll(value);
+                    break;
+                case OSP_COOKIE_AUTHTIME:
+                case OSP_COOKIE_AUTHTIMEUP:
+                    authtime = atoi(value);
+                    break;
+                case OSP_COOKIE_SRCIP:
+                case OSP_COOKIE_SRCIPUP:
+                    originator = value;
+                    break;
+                case OSP_COOKIE_DSTCOUNT:
+                case OSP_COOKIE_DSTCOUNTUP:
+                    destinationCount = (unsigned)atoi(value);
+                    break;
+                default:
+                    LM_ERR("unexpected tag '%c' / value '%s'\n", tag, value);
+                    break;
+            }
         }
     }
 
@@ -246,30 +260,34 @@ static int ospReportUsageFromCookie(
     ospGetToUserpart(msg, to, sizeof(to));
     ospGetNextHop(msg, nexthop, sizeof(nexthop));
 
-    LOG(L_DBG, "osp: first via '%s' from '%s' to '%s' next hop '%s'\n",
+    LM_DBG("first via '%s' from '%s' to '%s' next hop '%s'\n",
         firstvia,
         from,
         to,
         nexthop);
 
     if (release == OSP_RELEASE_ORIG) {
-        LOG(L_DBG,
-            "osp: orig '%s' released the call, call_id '%.*s' transaction_id '%lld'\n",
+        LM_DBG("orig '%s' released the call, call_id '%.*s' transaction_id '%llu'\n",
             firstvia,
             callid->ospmCallIdLen,
             callid->ospmCallIdVal,
-            transactionid);
+            transid);
+        if (originator == NULL) {
+            originator = firstvia;
+        }
         calling = from;
         called = to;
         terminator = nexthop;
     } else {
         release = OSP_RELEASE_TERM;
-        LOG(L_DBG,
-            "osp: term '%s' released the call, call_id '%.*s' transaction_id '%lld'\n",
+        LM_DBG("term '%s' released the call, call_id '%.*s' transaction_id '%llu'\n",
             firstvia,
             callid->ospmCallIdLen,
             callid->ospmCallIdVal,
-            transactionid);
+            transid);
+        if (originator == NULL) {
+            originator = nexthop;
+        }
         calling = to;
         called = from;
         terminator = firstvia;
@@ -277,27 +295,31 @@ static int ospReportUsageFromCookie(
 
     errorcode = OSPPTransactionNew(_osp_provider, &transaction);
 
-    LOG(L_DBG, "osp: created transaction handle '%d' (%d)\n", transaction, errorcode);
+    LM_DBG("created transaction handle '%d' (%d)\n", transaction, errorcode);
 
-    if (isorig == 1) {
-        issource = OSPC_SOURCE;
-        source = _osp_device_ip;
-        ospConvertAddress(terminator, dstbuf, sizeof(dstbuf));
-        destination = dstbuf;
-        ospConvertAddress(uac, devbuf, sizeof(devbuf));
-        srcdev = devbuf;
-    } else {
-        issource = OSPC_DESTINATION;
-        ospConvertAddress(uac, srcbuf, sizeof(srcbuf));
-        source = srcbuf;
-        destination = _osp_device_ip;
-        srcdev = "";
+    switch (type) {
+        case OSPC_DESTINATION:
+            ospConvertAddress(originator, srcbuf, sizeof(srcbuf));
+            source = srcbuf;
+            destination = _osp_device_ip;
+            srcdev = "";
+            break;
+        case OSPC_SOURCE:
+        case OSPC_OTHER:
+        case OSPC_UNDEFINED_ROLE:
+        default:
+            source = _osp_device_ip;
+            ospConvertAddress(terminator, dstbuf, sizeof(dstbuf));
+            destination = dstbuf;
+            ospConvertAddress(originator, devbuf, sizeof(devbuf));
+            srcdev = devbuf;
+            break;
     }
 
     errorcode = OSPPTransactionBuildUsageFromScratch(
         transaction,
-        transactionid,
-        issource,
+        transid,
+        type,
         source,
         destination,
         srcdev,
@@ -312,9 +334,9 @@ static int ospReportUsageFromCookie(
         NULL,
         NULL);
 
-    LOG(L_DBG, "osp: built usage handle '%d' (%d)\n", transaction, errorcode);
+    LM_DBG("built usage handle '%d' (%d)\n", transaction, errorcode);
 
-    if (errorcode == 0 && destinationCount > 0) {
+    if ((errorcode == OSPC_ERR_NO_ERROR) && (destinationCount > 0)) {
         errorcode = OSPPTransactionSetDestinationCount(
             transaction,
             destinationCount);
@@ -322,8 +344,8 @@ static int ospReportUsageFromCookie(
 
     ospReportUsageWrapper(
         transaction,
-        10016,
-        endtime - authtime,
+        cookie == NULL ? 9016 : 10016,
+        cookie == NULL ? 0 : endtime - authtime,
         authtime,
         endtime,
         0,0,
@@ -349,53 +371,55 @@ int ospReportUsage(
     char* tmp;
     char* token;
     char parameters[OSP_HEADERBUF_SIZE];
-    int isorig;
     OSPTCALLID* callid = NULL;
     int result = MODULE_RETURNCODE_FALSE;
 
-    LOG(L_DBG, "osp: ospReportUsage\n");
-
     ospGetCallId(msg, &callid);
 
-    if (callid != NULL && ospGetRouteParameters(msg, parameters, sizeof(parameters)) == 0) {
+    if (callid != NULL) {
         /* Who releases the call first, 0 orig, 1 term */
-        if (sscanf(whorelease, "%d", &release) != 1) {
+        if (sscanf(whorelease, "%d", &release) != 1 || (release != OSP_RELEASE_ORIG && release != OSP_RELEASE_TERM)) {
             release = OSP_RELEASE_ORIG;
         }
-        LOG(L_DBG, "osp: who releases the call first '%d'\n", release);
+        LM_DBG("who releases the call first '%d'\n", release);
 
-        for (token = strtok_r(parameters, ";", &tmp);
-             token;
-             token = strtok_r(NULL, ";", &tmp))
-        {
-            if (strncmp(token, OSP_ORIG_COOKIE, strlen(OSP_ORIG_COOKIE)) == 0) {
-                LOG(L_INFO,
-                    "osp: report originate duration usage for call_id '%.*s'\n",
-                    callid->ospmCallIdLen,
-                    callid->ospmCallIdVal);
-                isorig = 1;
-                ospReportUsageFromCookie(msg, token + strlen(OSP_ORIG_COOKIE) + 1, callid, release, isorig);
-                result = MODULE_RETURNCODE_TRUE;
-            } else if (strncmp(token, OSP_TERM_COOKIE, strlen(OSP_TERM_COOKIE)) == 0) {
-                LOG(L_INFO,
-                    "osp: report terminate duration usage for call_id '%.*s'\n",
-                    callid->ospmCallIdLen,
-                    callid->ospmCallIdVal);
-                isorig = 0;
-                ospReportUsageFromCookie(msg, token + strlen(OSP_TERM_COOKIE) + 1, callid, release, isorig);
-                result = MODULE_RETURNCODE_TRUE;
-            } else {
-                LOG(L_DBG, "osp: ignoring parameter '%s'\n", token);
+        if (ospGetRouteParameters(msg, parameters, sizeof(parameters)) == 0) {
+            for (token = strtok_r(parameters, ";", &tmp);
+                 token;
+                 token = strtok_r(NULL, ";", &tmp))
+            {
+                if (strncmp(token, OSP_ORIG_COOKIE, strlen(OSP_ORIG_COOKIE)) == 0) {
+                    LM_INFO("report orig duration for call_id '%.*s'\n",
+                        callid->ospmCallIdLen,
+                        callid->ospmCallIdVal);
+                    ospReportUsageFromCookie(msg, token + strlen(OSP_ORIG_COOKIE) + 1, callid, release, OSPC_SOURCE);
+                    result = MODULE_RETURNCODE_TRUE;
+                } else if (strncmp(token, OSP_TERM_COOKIE, strlen(OSP_TERM_COOKIE)) == 0) {
+                    LM_INFO("report term duration for call_id '%.*s'\n",
+                        callid->ospmCallIdLen,
+                        callid->ospmCallIdVal);
+                    ospReportUsageFromCookie(msg, token + strlen(OSP_TERM_COOKIE) + 1, callid, release, OSPC_DESTINATION);
+                    result = MODULE_RETURNCODE_TRUE;
+                } else {
+                    LM_DBG("ignoring parameter '%s'\n", token);
+                }
             }
         }
-    }
 
-    if (callid != NULL) {
+        if (result == MODULE_RETURNCODE_FALSE) {
+            LM_DBG("without orig or term OSP information\n");
+            LM_INFO("report other duration for call_id '%.*s'\n",
+               callid->ospmCallIdLen,
+               callid->ospmCallIdVal);
+            ospReportUsageFromCookie(msg, NULL, callid, release, OSPC_SOURCE);
+            result = MODULE_RETURNCODE_TRUE;
+        }
+
         OSPPCallIdDelete(&callid);
     }
 
     if (result == MODULE_RETURNCODE_FALSE) {
-        LOG(L_DBG, "osp: without OSP orig or term usage information\n");
+        LM_ERR("failed to report usage\n");
     }
 
     return result;
@@ -418,10 +442,6 @@ static int ospBuildUsageFromDestination(
     char* source;
     char* srcdev;
 
-    dest->reported = 1;
-
-    LOG(L_DBG, "osp: ospBuildUsageFromDestination\n");
-
     if (dest->type == OSPC_SOURCE) {
         ospConvertAddress(dest->srcdev, addr, sizeof(addr));
         source = dest->source;
@@ -434,7 +454,7 @@ static int ospBuildUsageFromDestination(
 
     errorcode = OSPPTransactionBuildUsageFromScratch(
         transaction,
-        dest->tid,
+        dest->transid,
         dest->type,
         source,
         dest->host,
@@ -442,7 +462,7 @@ static int ospBuildUsageFromDestination(
         dest->destdev,
         dest->calling,
         OSPC_E164,
-        dest->called,
+        dest->origcalled,       /* Report original called number */
         OSPC_E164,
         dest->callidsize,
         dest->callid,
@@ -489,9 +509,7 @@ void ospReportOrigSetupUsage(void)
     int_str destval;
     OSPTTRANHANDLE transaction = -1;
     int lastcode = 0;
-    int errorcode = 0;
-
-    LOG(L_DBG, "osp: ospReportOrigSetupUsage\n");
+    int errorcode;
 
     errorcode = OSPPTransactionNew(_osp_provider, &transaction);
 
@@ -506,11 +524,13 @@ void ospReportOrigSetupUsage(void)
 
         if (dest->used == 1) {
             if (dest->reported == 1) {
-                LOG(L_DBG, "osp: orig setup already reported\n");
+                LM_DBG("orig setup already reported\n");
                 break;
+            } else {
+                dest->reported = 1;
             }
 
-            LOG(L_DBG, "osp: iterating through used destination\n");
+            LM_DBG("iterating through used destination\n");
 
             ospDumpDestination(dest);
 
@@ -520,17 +540,16 @@ void ospReportOrigSetupUsage(void)
 
             lastcode = dest->lastcode;
         } else {
-            LOG(L_DBG, "osp: destination has not been used, breaking out\n");
+            LM_DBG("destination has not been used, breaking out\n");
             break;
         }
     }
 
     if (lastused) {
-        LOG(L_INFO,
-            "osp: report orig setup for call_id '%.*s' transaction_id '%lld'\n",
+        LM_INFO("report orig setup for call_id '%.*s' transaction_id '%llu'\n",
             lastused->callidsize,
             lastused->callid,
-            lastused->tid);
+            lastused->transid);
         errorcode = ospReportUsageFromDestination(transaction, lastused);
     } else {
         /* If a Toolkit transaction handle was created, but we did not find
@@ -548,24 +567,22 @@ void ospReportTermSetupUsage(void)
 {
     osp_dest* dest = NULL;
     OSPTTRANHANDLE transaction = -1;
-    int errorcode = 0;
-
-    LOG(L_DBG, "osp: ospReportTermSetupUsage\n");
+    int errorcode;
 
     if ((dest = ospGetTermDestination())) {
         if (dest->reported == 0) {
-            LOG(L_INFO,
-                "osp: report term setup for call_id '%.*s' transaction_id '%lld'\n",
+            dest->reported = 1;
+            LM_INFO("report term setup for call_id '%.*s' transaction_id '%llu'\n",
                 dest->callidsize,
                 dest->callid,
-                dest->tid);
+                dest->transid);
             errorcode = OSPPTransactionNew(_osp_provider, &transaction);
             errorcode = ospBuildUsageFromDestination(transaction, dest, 0);
             errorcode = ospReportUsageFromDestination(transaction, dest);
         } else {
-            LOG(L_DBG, "osp: term setup already reported\n");
+            LM_DBG("term setup already reported\n");
         }
     } else {
-        LOG(L_ERR, "osp: ERROR: without term setup to report\n");
+        LM_ERR("without term setup to report\n");
     }
 }
