@@ -1,5 +1,5 @@
 /**
- * $Id: pdt.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: pdt.c 3732 2008-02-18 20:48:49Z anomarme $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -49,6 +49,7 @@
 #include "../../parser/parse_uri.h"
 #include "../../timer.h"
 #include "../../ut.h"
+#include "../../locking.h"
 #include "../../action.h"
 #include "../../parser/parse_from.h"
 #include "domains.h"
@@ -59,13 +60,11 @@ MODULE_VERSION
 
 #define NR_KEYS			3
 
-int hs_two_pow = 2;
+int hs_two_pow = 4;
 
 /** structures containing prefix-domain pairs */
-hash_list_t *_dhash = NULL; 
-pdt_tree_t *_ptree = NULL; 
-
-time_t last_sync;
+hash_list_t **_dhash = NULL; 
+pdt_tree_t **_ptree = NULL; 
 
 /** database connection */
 static db_con_t *db_con = NULL;
@@ -73,7 +72,7 @@ static db_func_t pdt_dbf;
 
 
 /** parameters */
-static char *db_url = "mysql://root@127.0.0.1/pdt";
+static char *db_url = DEFAULT_DB_URL;
 char *db_table = "pdt";
 char *sdomain_column = "sdomain";
 char *prefix_column  = "prefix";
@@ -81,33 +80,37 @@ char *domain_column  = "domain";
 
 /** pstn prefix */
 str prefix = {"", 0};
-int sync_time = 600;
-int clean_time = 900;
+/* List of allowed chars for a prefix*/
+str pdt_char_list = {"0123456789", 10};
+
+/* lock, ref counter and flag used for reloading the date */
+static gen_lock_t *pdt_lock = 0;
+static volatile int pdt_tree_refcnt = 0;
+static volatile int pdt_reload_flag = 0;
 
 static int  w_prefix2domain(struct sip_msg* msg, char* str1, char* str2);
 static int  w_prefix2domain_1(struct sip_msg* msg, char* mode, char* str2);
 static int  w_prefix2domain_2(struct sip_msg* msg, char* mode, char* sd_en);
 static int  mod_init(void);
 static void mod_destroy(void);
-static int  child_init();
+static int  child_init(void);
 static int  mod_child_init(int r);
 
 static int prefix2domain(struct sip_msg*, int mode, int sd_en);
 
+static struct mi_root* pdt_mi_reload(struct mi_root*, void* param);
 static struct mi_root* pdt_mi_add(struct mi_root*, void* param);
 static struct mi_root* pdt_mi_delete(struct mi_root*, void* param);
 static struct mi_root* pdt_mi_list(struct mi_root*, void* param);
 
 int update_new_uri(struct sip_msg *msg, int plen, str *d, int mode);
 int pdt_load_db();
-int pdt_sync_cache();
-void pdt_clean_cache(unsigned int ticks, void *param);
 
 static cmd_export_t cmds[]={
-	{"prefix2domain", w_prefix2domain,   0, 0, REQUEST_ROUTE|FAILURE_ROUTE},
-	{"prefix2domain", w_prefix2domain_1, 1, 0, REQUEST_ROUTE|FAILURE_ROUTE},
-	{"prefix2domain", w_prefix2domain_2, 2, 0, REQUEST_ROUTE|FAILURE_ROUTE},
-	{0, 0, 0, 0, 0}
+	{"prefix2domain", w_prefix2domain,   0, 0, 0, REQUEST_ROUTE|FAILURE_ROUTE},
+	{"prefix2domain", w_prefix2domain_1, 1, 0, 0, REQUEST_ROUTE|FAILURE_ROUTE},
+	{"prefix2domain", w_prefix2domain_2, 2, 0, 0, REQUEST_ROUTE|FAILURE_ROUTE},
+	{0, 0, 0, 0, 0, 0}
 };
 
 static param_export_t params[]={
@@ -117,14 +120,14 @@ static param_export_t params[]={
 	{"prefix_column",  STR_PARAM, &prefix_column},
 	{"domain_column",  STR_PARAM, &domain_column},
 	{"prefix",         STR_PARAM, &prefix.s},
+	{"char_list",      STR_PARAM, &pdt_char_list.s},
 	{"hsize_2pow",     INT_PARAM, &hs_two_pow},
-	{"sync_time",      INT_PARAM, &sync_time},
-	{"clean_time",     INT_PARAM, &clean_time},
 	{0, 0, 0}
 };
 
 static mi_export_t mi_cmds[] = {
 	{ "pdt_add",     pdt_mi_add,     0,  0,  child_init },
+	{ "pdt_reload",  pdt_mi_reload,  0,  0,  0 },
 	{ "pdt_delete",  pdt_mi_delete,  0,  0,  0 },
 	{ "pdt_list",    pdt_mi_list,    0,  0,  0 },
 	{ 0, 0, 0, 0, 0}
@@ -139,6 +142,7 @@ struct module_exports exports = {
 	0,
 	mi_cmds,        /* exported MI functions */
 	0,              /* exported pseudo-variables */
+	0,              /* extra processes */
 	mod_init,       /* module initialization function */
 	0,              /* response function */
 	mod_destroy,    /* destroy function */
@@ -152,27 +156,34 @@ struct module_exports exports = {
  */
 static int mod_init(void)
 {
-	DBG("PDT: initializing...\n");
+	LM_INFO("initializing...\n");
 
 	if(hs_two_pow<0)
 	{
-		LOG(L_ERR, "PDT:mod_init: hash_size_two_pow must be"
-					" positive and less than %d\n", MAX_HSIZE_TWO_POW);
+		LM_ERR("hash_size_two_pow must be positive and less than %d\n",
+				MAX_HSIZE_TWO_POW);
 		return -1;
 	}
 
 	prefix.len = strlen(prefix.s);
+	pdt_char_list.len = strlen(pdt_char_list.s);
+	if(pdt_char_list.len<=0)
+	{
+		LM_ERR("invalid pdt char list\n");
+		return -1;
+	}
+	LM_INFO("pdt_char_list=%s \n",pdt_char_list.s);
 
 	/* binding to mysql module */
 	if(bind_dbmod(db_url, &pdt_dbf))
 	{
-		LOG(L_ERR, "PDT:mod_init: Database module not found\n");
+		LM_ERR("database module not found\n");
 		return -1;
 	}
 
 	if (!DB_CAPABILITY(pdt_dbf, DB_CAP_ALL))
 	{
-		LOG(L_ERR, "PDT: mod_init: Database module does not "
+		LM_ERR("database module does not "
 		    "implement all functions needed by the module\n");
 		return -1;
 	}
@@ -181,54 +192,71 @@ static int mod_init(void)
 	db_con = pdt_dbf.init(db_url);
 	if(db_con==NULL)
 	{
-		LOG(L_ERR,
-			"PDT: mod_init: Error while connecting to database\n");        
+		LM_ERR("failed to connect to the database\n");        
 		return -1;
 	}
 	
 	if (pdt_dbf.use_table(db_con, db_table) < 0)
 	{
-		LOG(L_ERR, "PDT: mod_init: Error in use_table\n");
+		LM_ERR("failed to use_table\n");
 		goto error1;
 	}
-	DBG("PDT: mod_init: Database connection opened successfully\n");
+	LM_DBG("database connection opened successfully\n");
 	
-	/* init the hash and tree in share memory */
-	if( (_dhash = init_hash_list(hs_two_pow)) == NULL)
-	{
-		LOG(L_ERR, "PDT:mod_init: domain hash could not be allocated\n");	
+	if ( (pdt_lock=lock_alloc())==0) {
+		LM_CRIT("failed to alloc lock\n");
+		goto error1;
+	}
+	if (lock_init(pdt_lock)==0 ) {
+		LM_CRIT("failed to init lock\n");
 		goto error1;
 	}
 	
+	/* pdt hash and tree pointers in shm */
+	_dhash = (hash_list_t**)shm_malloc( sizeof(hash_list_t*) );
+	if (_dhash==0) {
+		LM_ERR("out of shm mem for dhash\n");
+		goto error1;
+	}
+	*_dhash=0;
+	
+	_ptree = (pdt_tree_t**)shm_malloc( sizeof(pdt_tree_t*) );
+	if (_ptree==0) {
+		LM_ERR("out of shm mem for pdtree\n");
+		goto error1;
+	}
+	*_ptree=0;
 
 	/* loading all information from database */
 	if(pdt_load_db()!=0)
 	{
-		LOG(L_ERR, "PDT:mod_init: cannot load info from database\n");	
-		goto error2;
+		LM_ERR("cannot load info from database\n");	
+		goto error1;
 	}
 		
 	pdt_dbf.close(db_con);
 	db_con = 0;
 
-	pdt_print_tree(_ptree);
-	DBG("PDT:mod_init: -------------------\n");
-	pdt_print_hash_list(_dhash);
-
-	last_sync = time(NULL);
-
-	register_timer(pdt_clean_cache, 0, clean_time);
+#if 0
+	pdt_print_tree(*_ptree);
+	pdt_print_hash_list(*_dhash);
+#endif
 
 	/* success code */
 	return 0;
 
-error2:
-	if(_dhash!=NULL)
-	{
-		free_hash_list(_dhash);
-		_dhash = 0;
-	}
 error1:
+	if (pdt_lock)
+	{
+		lock_destroy( pdt_lock );
+		lock_dealloc( pdt_lock );
+		pdt_lock = 0;
+	}
+	if(_dhash!=0)
+		shm_free(_dhash);
+	if(_ptree!=0)
+		shm_free(_ptree);
+
 	if(db_con!=NULL)
 	{
 		pdt_dbf.close(db_con);
@@ -238,18 +266,18 @@ error1:
 }
 
 
-static int child_init()
+static int child_init(void)
 {
 	db_con = pdt_dbf.init(db_url);
 	if(db_con==NULL)
 	{
-		LOG(L_ERR,"ERROR:PDT:child_init: failed to connect to database\n");
+		LM_ERR("failed to connect to database\n");
 		return -1;
 	}
 
 	if (pdt_dbf.use_table(db_con, db_table) < 0)
 	{
-		LOG(L_ERR, "ERROR:PDT:child_init: use_table failed\n");
+		LM_ERR("use_table failed\n");
 		return -1;
 	}
 	return 0;
@@ -259,33 +287,10 @@ static int child_init()
 /* each child get a new connection to the database */
 static int mod_child_init(int r)
 {
-	if(r>0)
-	{
-		if(_dhash==NULL)
-		{
-			LOG(L_ERR,"ERROR:PDT:mod_child_init #%d: no domain hash\n", r);
-			return -1;
-		}
-	
-		lock_get(&_dhash->hl_lock);
-		_dhash->workers++;
-		lock_release(&_dhash->hl_lock);
-	} else {
-		if(_ptree!=NULL)
-		{
-			pdt_free_tree(_ptree);
-			_ptree = 0;
-		}
-	}
-
 	if ( child_init()!=0 )
 		return -1;
 
-	if(sync_time<=0)
-		sync_time = 300;
-	sync_time += r%60;
-
-	DBG("PDT:mod_child_init #%d: Database connection opened successfully\n",r);
+	LM_DBG("#%d: database connection opened successfully\n",r);
 
 	return 0;
 }
@@ -293,13 +298,29 @@ static int mod_child_init(int r)
 
 static void mod_destroy(void)
 {
-	DBG("PDT: mod_destroy : Cleaning up\n");
+	LM_DBG("cleaning up\n");
 	if (_dhash!=NULL)
-		free_hash_list(_dhash);
+	{
+		if (*_dhash!=NULL)
+			free_hash_list(*_dhash);
+		shm_free(_dhash);
+	}
 	if (_ptree!=NULL)
-		pdt_free_tree(_ptree);
+	{
+		if (*_ptree!=NULL)
+			pdt_free_tree(*_ptree);
+		shm_free(_ptree);
+	}
 	if (db_con!=NULL && pdt_dbf.close!=NULL)
 		pdt_dbf.close(db_con);
+		/* destroy lock */
+	if (pdt_lock)
+	{
+		lock_destroy( pdt_lock );
+		lock_dealloc( pdt_lock );
+		pdt_lock = 0;
+	}
+
 }
 
 
@@ -321,8 +342,9 @@ static int w_prefix2domain_2(struct sip_msg* msg, char* mode, char* sd_en)
 {
 	int tmp=0;
 	
-	if((sd_en==NULL) || ((sd_en!=NULL) && (*sd_en!='0') && (*sd_en!='1') && (*sd_en!='2')))
-			return -1;
+	if((sd_en==NULL) || ((sd_en!=NULL) && (*sd_en!='0') && (*sd_en!='1')
+				&& (*sd_en!='2')))
+		return -1;
 	
     if (*sd_en=='1')
 		tmp = 1;
@@ -341,13 +363,12 @@ static int w_prefix2domain_2(struct sip_msg* msg, char* mode, char* sd_en)
 static int prefix2domain(struct sip_msg* msg, int mode, int sd_en)
 {
 	str *d, p, all={"*",1};
-	time_t crt_time;
 	int plen;
 	struct sip_uri uri;
 	
 	if(msg==NULL)
 	{
-		LOG(L_ERR,"PDT:prefix2domain: weird error\n");
+		LM_ERR("received null msg\n");
 		return -1;
 	}
 	
@@ -355,14 +376,14 @@ static int prefix2domain(struct sip_msg* msg, int mode, int sd_en)
 	if(msg->parsed_uri_ok==0)
 		if(parse_sip_msg_uri(msg)<0)
 		{
-			LOG(L_ERR,"PDT:prefix2domain: ERROR while parsing the R-URI\n");
+			LM_ERR("failed to parse the R-URI\n");
 			return -1;
 		}
 
     /* if the user part begin with the prefix for PSTN users, extract the code*/
 	if (msg->parsed_uri.user.len<=0)
 	{
-		DBG("PDT:prefix2domain: user part of the message is empty\n");
+		LM_DBG("user part of the message is empty\n");
 		return -1;
 	}   
     
@@ -370,12 +391,12 @@ static int prefix2domain(struct sip_msg* msg, int mode, int sd_en)
 	{
 		if (msg->parsed_uri.user.len<=prefix.len)
 		{
-			DBG("PDT:prefix2domain: user part is less than prefix\n");
+			LM_DBG("user part is less than prefix\n");
 			return -1;
 		}   
 		if(strncasecmp(prefix.s, msg->parsed_uri.user.s, prefix.len)!=0)
 		{
-			DBG("PDT:prefix2domain: PSTN prefix did not matched\n");
+			LM_DBG("PSTN prefix did not matched\n");
 			return -1;
 		}
 	}   
@@ -383,104 +404,102 @@ static int prefix2domain(struct sip_msg* msg, int mode, int sd_en)
 	if(prefix.len>0 && prefix.len < msg->parsed_uri.user.len
 			&& strncasecmp(prefix.s, msg->parsed_uri.user.s, prefix.len)!=0)
 	{
-		DBG("PDT:prefix2domain: PSTN prefix did not matched\n");
-		return 1;
+		LM_DBG("PSTN prefix did not matched\n");
+		return -1;
 			
 	}
 
 	p.s   = msg->parsed_uri.user.s + prefix.len;
 	p.len = msg->parsed_uri.user.len - prefix.len;
 
-	pdt_print_tree(_ptree);
-
-	/* check if need for sync */
-	crt_time = time(NULL);
-	if(last_sync + sync_time < crt_time)
-	{
-		last_sync = crt_time;
-		if(pdt_sync_cache())
-		{
-			/* keep reporting but continue */
-			LOG(L_ERR, "PDT:prefix2domain: cannot update the cache\n");
-			/* return -1; */
-		}
+again:
+	lock_get( pdt_lock );
+	if (pdt_reload_flag) {
+		lock_release( pdt_lock );
+		sleep_us(5);
+		goto again;
 	}
+	pdt_tree_refcnt++;
+	lock_release( pdt_lock );
 
 	if(sd_en==2)
 	{	
 		/* take the domain from  FROM uri as sdomain */
-		if(parse_from_header(msg)<0 ||  msg->from == NULL || get_from(msg)==NULL)
+		if(parse_from_header(msg)<0 ||  msg->from == NULL 
+				|| get_from(msg)==NULL)
 		{
-			LOG(L_ERR,
-				"prefix_to_domain: ERROR cannot parse FROM header\n");
-			return -1;
+			LM_ERR("cannot parse FROM header\n");
+			goto error;
 		}	
 		
 		memset(&uri, 0, sizeof(struct sip_uri));
 		if (parse_uri(get_from(msg)->uri.s, get_from(msg)->uri.len , &uri)<0)
 		{
-			LOG(L_ERR,"prefix_to_domain: failed to parse From uri\n");
-			return -1;
+			LM_ERR("failed to parse From uri\n");
+			goto error;
 		}
 	
 		/* find the domain that corresponds to this prefix */
 		plen = 0;
-		if((d=pdt_get_domain(_ptree, &uri.host, &p, &plen))==NULL)
+		if((d=pdt_get_domain(*_ptree, &uri.host, &p, &plen))==NULL)
 		{
 			plen = 0;
-			if((d=pdt_get_domain(_ptree, &all, &p, &plen))==NULL)
+			if((d=pdt_get_domain(*_ptree, &all, &p, &plen))==NULL)
 			{
-				LOG(L_INFO, "PDT:prefix2domain: no prefix found in [%.*s]\n",
-					p.len, p.s);
-				return -1;
+				LM_INFO("no prefix found in [%.*s]\n", p.len, p.s);
+				goto error;
 			}
 		}
-	}
-	else if(sd_en==1)
-	{	
+	} else if(sd_en==1) {	
 		/* take the domain from  FROM uri as sdomain */
-		if(parse_from_header(msg)<0 ||  msg->from == NULL || get_from(msg)==NULL)
+		if(parse_from_header(msg)<0 ||  msg->from == NULL
+				|| get_from(msg)==NULL)
 		{
-			LOG(L_ERR,
-				"prefix_to_domain: ERROR cannot parse FROM header\n");
-			return -1;
+			LM_ERR("ERROR cannot parse FROM header\n");
+			goto error;
 		}	
 		
 		memset(&uri, 0, sizeof(struct sip_uri));
 		if (parse_uri(get_from(msg)->uri.s, get_from(msg)->uri.len , &uri)<0)
 		{
-			LOG(L_ERR,"prefix_to_domain: failed to parse From uri\n");
-			return -1;
+			LM_ERR("failed to parse From uri\n");
+			goto error;
 		}
 	
 		/* find the domain that corresponds to this prefix */
 		plen = 0;
-		if((d=pdt_get_domain(_ptree, &uri.host, &p, &plen))==NULL)
+		if((d=pdt_get_domain(*_ptree, &uri.host, &p, &plen))==NULL)
 		{
-			LOG(L_INFO, "PDT:prefix2domain: no prefix found in [%.*s]\n",
-				p.len, p.s);
-			return -1;
+			LM_INFO("no prefix found in [%.*s]\n", p.len, p.s);
+			goto error;
 		}
-	}
-	else
-	{
+	} else {
 		/* find the domain that corresponds to this prefix */
 		plen = 0;
-		if((d=pdt_get_domain(_ptree, &all, &p, &plen))==NULL)
+		if((d=pdt_get_domain(*_ptree, &all, &p, &plen))==NULL)
 		{
-			LOG(L_INFO, "PDT:prefix2domain: no prefix found in [%.*s]\n",
-				p.len, p.s);
-			return -1;
+			LM_INFO("no prefix found in [%.*s]\n", p.len, p.s);
+			goto error;
 		}
 	}
 	
 	/* update the new uri */
 	if(update_new_uri(msg, plen, d, mode)<0)
 	{
-		LOG(L_ERR, "PDT:prefix2domain: new_uri cannot be updated\n");
-		return -1;
+		LM_ERR("new_uri cannot be updated\n");
+		goto error;
 	}
+
+	lock_get( pdt_lock );
+	pdt_tree_refcnt--;
+	lock_release( pdt_lock );
 	return 1;
+
+error:
+	lock_get( pdt_lock );
+	pdt_tree_refcnt--;
+	lock_release( pdt_lock );
+	return -1;
 }
 
 /* change the uri according to translation of the prefix */
@@ -489,7 +508,7 @@ int update_new_uri(struct sip_msg *msg, int plen, str *d, int mode)
 	struct action act;
 	if(msg==NULL || d==NULL)
 	{
-		LOG(L_ERR, "PDT:update_new_uri: bad parameters\n");
+		LM_ERR("bad parameters\n");
 		return -1;
 	}
 	
@@ -505,7 +524,7 @@ int update_new_uri(struct sip_msg *msg, int plen, str *d, int mode)
 
 		if (do_action(&act, msg) < 0)
 		{
-			LOG(L_ERR, "PDT:update_new_uri:Error removing prefix\n");
+			LM_ERR("failed to remove prefix\n");
 			return -1;
 		}
 	}
@@ -517,220 +536,155 @@ int update_new_uri(struct sip_msg *msg, int plen, str *d, int mode)
 
 	if (do_action(&act, msg) < 0)
 	{
-		LOG(L_ERR, "PDT:update_new_uri:Error changing domain\n");
+		LM_ERR("failed to change domain\n");
 		return -1;
 	}
 
-	DBG("PDT: update_new_uri: len=%d uri=%.*s\n", msg->new_uri.len, 
+	LM_DBG("len=%d uri=%.*s\n", msg->new_uri.len, 
 			msg->new_uri.len, msg->new_uri.s);
 	
 	return 0;
 }
 
-int pdt_load_db()
+int pdt_load_db(void)
 {
 	db_key_t db_cols[3] = {sdomain_column, prefix_column, domain_column};
 	str p, d, sdomain;
 	db_res_t* db_res = NULL;
-	int i;
+	int i, ret;
+	hash_list_t *_dhash_new = NULL; 
+	pdt_tree_t *_ptree_new = NULL; 
+	hash_list_t *old_hash = NULL; 
+	pdt_tree_t *old_tree = NULL; 
 	
 	
 	if(db_con==NULL)
 	{
-		LOG(L_ERR, "PDT:pdt_load_db: no db connection\n");
+		LM_ERR("no db connection\n");
 		return -1;
 	}
 		
 	if (pdt_dbf.use_table(db_con, db_table) < 0)
 	{
-		LOG(L_ERR, "PDT:pdt_load_db: Error in use_table\n");
+		LM_ERR("failed to use_table\n");
 		return -1;
 	}
 	
-	if(pdt_dbf.query(db_con, NULL, NULL, NULL, db_cols,
-				0, 3, sdomain_column, &db_res)==0)
+	if((ret=pdt_dbf.query(db_con, NULL, NULL, NULL, db_cols,
+				0, 3, sdomain_column, &db_res))!=0
+			|| RES_ROW_N(db_res)<=0 )
 	{
-		for(i=0; i<RES_ROW_N(db_res); i++)
-		{
-			/* check for NULL values ?!?! */
-			sdomain.s = (char*)(RES_ROWS(db_res)[i].values[0].val.string_val);
-			sdomain.len = strlen(sdomain.s);
+		pdt_dbf.free_result(db_con, db_res);
+		if( ret==0)
+			return 0;
+		else
+			return -1;
+	}
 
-			p.s = (char*)(RES_ROWS(db_res)[i].values[1].val.string_val);
-			p.len = strlen(p.s);
+	/* init the hash and tree in share memory */
+	if( (_dhash_new = init_hash_list(hs_two_pow)) == NULL)
+	{
+		LM_ERR("domain hash could not be allocated\n");	
+		goto error;
+	}
+	
+	for(i=0; i<RES_ROW_N(db_res); i++)
+	{
+		/* check for NULL values ?!?! */
+		sdomain.s = (char*)(RES_ROWS(db_res)[i].values[0].val.string_val);
+		sdomain.len = strlen(sdomain.s);
+
+		p.s = (char*)(RES_ROWS(db_res)[i].values[1].val.string_val);
+		p.len = strlen(p.s);
 			
-			d.s = (char*)(RES_ROWS(db_res)[i].values[2].val.string_val);
-			d.len = strlen(d.s);
+		d.s = (char*)(RES_ROWS(db_res)[i].values[2].val.string_val);
+		d.len = strlen(d.s);
 
-			if(p.s==NULL || d.s==NULL || sdomain.s==NULL ||
-					p.len<=0 || d.len<=0 || sdomain.len<=0)
-			{
-				LOG(L_ERR, "PDT:pdt_load_db: Error - bad values in db\n");
-				continue;
-			}
+		if(p.s==NULL || d.s==NULL || sdomain.s==NULL ||
+				p.len<=0 || d.len<=0 || sdomain.len<=0)
+		{
+			LM_ERR("Error - bad values in db\n");
+			continue;
+		}
 		
-			if(pdt_check_pd(_dhash, &sdomain, &p, &d)==1)
-			{
-				LOG(L_ERR,
-				"PDT:pdt_load_db:sdomain [%.*s]: prefix [%.*s] or domain <%.*s> duplicated\n",
-					sdomain.len, sdomain.s, p.len, p.s, d.len, d.s);
-				continue;
-			}
-
-			if(pdt_add_to_tree(&_ptree, &sdomain, &p, &d)<0)
-			{
-				LOG(L_ERR, "PDT:pdt_load_db: Error adding info to tree\n");
-				goto error;
-			}
-			
-			if(pdt_add_to_hash(_dhash, &sdomain, &p, &d, 0)!=0)
-			{
-				LOG(L_ERR, "PDT:pdt_load_db: Error adding info to hash\n");
-				goto error;
-			}
- 		}
-	}
-	
-	pdt_dbf.free_result(db_con, db_res);
-	return 0;
-
-error:
-	pdt_dbf.free_result(db_con, db_res);
-	return -1;
-}
-
-int pdt_sync_cache()
-{
-	pd_op_t *ito;
-	hash_t *it;
-	pdt_tree_t *itree;
-	
-	DBG("PDT:pdt_sync_cache: ...\n");
-
-	if(_dhash==NULL || _ptree==NULL)
-	{
-		LOG(L_ERR, "PDT:pdt_sync_cache: strange situation\n");
-		return -1;
-	}
-	
-	lock_get(&_dhash->hl_lock);
-	it = _dhash->hash;
-	while(it != NULL)
-	{
-		itree = pdt_get_tree(_ptree, &it->sdomain);
-		if(itree!=NULL && itree->idsync >= it->max_id)
+		if(pdt_check_pd(_dhash_new, &sdomain, &p, &d)==1)
 		{
-			it = it->next;
+			LM_ERR("sdomain [%.*s]: prefix [%.*s] or domain <%.*s> "
+				"duplicated\n", sdomain.len, sdomain.s, p.len, p.s, d.len, d.s);
 			continue;
 		}
 
-		ito = it->diff;
-		while(ito!=NULL && itree->idsync >= ito->id)
-			ito = ito->n;
-		
-		while(ito!=NULL && itree->idsync<ito->id)
+		if(pdt_add_to_tree(&_ptree_new, &sdomain, &p, &d)<0)
 		{
-			switch(ito->op)
-			{
-				case PDT_ADD:
-					LOG(L_ERR,
-						"PDT:pdt_sync_cache: add (%d) [%.*s-%.*s => %.*s]\n",
-						ito->id, it->sdomain.len, it->sdomain.s,
-						ito->cell->prefix.len, ito->cell->prefix.s,
-						ito->cell->domain.len, ito->cell->domain.s);
-					if(pdt_add_to_tree(&_ptree, &it->sdomain, &ito->cell->prefix,
-								&ito->cell->domain)<0)
-					{
-						LOG(L_ERR, "PDT:pdt_sync_cache: Error to insert into tree\n");
-						break;
-					}
-					break;
-				case PDT_DELETE:
-					if(itree==NULL)
-					{
-						LOG(L_ERR,
-							"PDT:pdt_sync_cache: Error to remove from tree, tree does not exist\n");
-						goto error;
-					}
-					LOG(L_ERR,
-						"PDT:pdt_sync_cache: adel (%d) [%.*s-%.*s]\n",
-						ito->id, it->sdomain.len, it->sdomain.s,
-						ito->cell->prefix.len, ito->cell->prefix.s);
-					if(pdt_remove_prefix_from_tree(itree, &it->sdomain, &ito->cell->prefix)!=0)
-					{
-						LOG(L_ERR,
-							"PDT:pdt_sync_cache: Error to remove from tree\n");
-						break;
-					}
-					break;
-				default:
-					LOG(L_ERR, "PDT:pdt_sync_cache: unknown operation %d (%d)\n",
-							ito->op, ito->id);
-			}
-			ito->count++;
-			ito = ito->n;
+			LM_ERR("Error adding info to tree\n");
+			goto error;
 		}
-		if(it->diff!=NULL)
-			itree->idsync = it->diff->id;
-		it = it->next;
+			
+		if(pdt_add_to_hash(_dhash_new, &sdomain, &p, &d)!=0)
+		{
+			LM_ERR("Error adding info to hash\n");
+			goto error;
+		}
+ 	}
+	
+	pdt_dbf.free_result(db_con, db_res);
+
+
+	/* block all readers */
+	lock_get( pdt_lock );
+	pdt_reload_flag = 1;
+	lock_release( pdt_lock );
+
+	while (pdt_tree_refcnt) {
+		sleep_us(10);
 	}
 
+	old_tree = *_ptree;
+	*_ptree = _ptree_new;
+	old_hash = *_dhash;
+	*_dhash = _dhash_new;
+
+	pdt_reload_flag = 0;
+
+	/* free old data */
+	if (old_hash!=NULL)
+		free_hash_list(old_hash);
+	if (old_tree!=NULL)
+		pdt_free_tree(old_tree);
 
-	lock_release(&_dhash->hl_lock);
 	return 0;
+
 error:
-	lock_release(&_dhash->hl_lock);
+	pdt_dbf.free_result(db_con, db_res);
+	if (_dhash_new!=NULL)
+		free_hash_list(_dhash_new);
+	if (_ptree_new!=NULL)
+		pdt_free_tree(_ptree_new);
 	return -1;
 }
 
-void pdt_clean_cache(unsigned int ticks, void *param)
+/**************************** MI ***************************/
+
+/**
+ * "pdt_reload" syntax :
+ * \n
+ */
+static struct mi_root* pdt_mi_reload(struct mi_root *cmd_tree, void *param)
 {
-	pd_op_t *ito, *tmp;
-	hash_t *it;
-
-	/* DBG("PDT:pdt_clean_cache: ...\n"); */
-	
-	if(_dhash==NULL)
+	/* re-loading all information from database */
+	if(pdt_load_db()!=0)
 	{
-		LOG(L_ERR, "PDT:pdt_clean_cache: strange situation\n");
-		return;
+		LM_ERR("cannot re-load info from database\n");	
+		goto error;
 	}
 	
-	lock_get(&_dhash->hl_lock);
+	return init_mi_tree( 200, MI_OK_S, MI_OK_LEN);
 
-	it = _dhash->hash;
-	while(it!=NULL)
-	{
-		ito = it->diff;
-		while(ito!=NULL)
-		{
-			if(ito->count >= _dhash->workers)
-			{
-				DBG("PDT:pdt_clean_cache: cleaning sdomain<%.*s> op[%d]=%d...\n",
-					it->sdomain.len, it->sdomain.s, ito->id, ito->op);
-				free_cell(ito->cell);
-				if(ito->p!=NULL)
-					(ito->p)->n = ito->n;
-				else
-					it->diff = ito->n;
-				if(ito->n!=NULL)
-					(ito->n)->p = ito->p;
-				tmp = ito;
-				ito = ito->n;
-				shm_free(tmp);
-			} 
-			else
-				ito = ito->n;
-		}
-		it = it->next;	
-	}
-	lock_release(&_dhash->hl_lock);
-	return;
+error:
+	return init_mi_tree( 500, "Failed to reload",16);
 }
 
 
-/**************************** MI ***************************/
-
 
 /**
  * "pdt_add" syntax :
@@ -749,7 +703,7 @@ struct mi_root* pdt_mi_add(struct mi_root* cmd_tree, void* param)
 
 	if(_dhash==NULL)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_add: strange situation\n");
+		LM_ERR("strange situation\n");
 		return init_mi_tree( 500, MI_INTERNAL_ERR_S, MI_INTERNAL_ERR_LEN);
 	}
 
@@ -773,7 +727,7 @@ struct mi_root* pdt_mi_add(struct mi_root* cmd_tree, void* param)
 	sp= node->value;
 	if(sp.s== NULL || sp.len==0)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_add: could not read prefix\n");
+		LM_ERR("could not read prefix\n");
 		return init_mi_tree( 404, "prefix not found", 16);
 	}
 
@@ -782,7 +736,7 @@ struct mi_root* pdt_mi_add(struct mi_root* cmd_tree, void* param)
 
 	while(i< sp.len)
 	{
-		if(sp.s[i] < '0' || sp.s[i] > '9')
+		if(strpos(pdt_char_list.s,sp.s[i]) < 0) 
 			return init_mi_tree( 400, "bad prefix", 10);
 		i++;
 	}
@@ -795,7 +749,7 @@ struct mi_root* pdt_mi_add(struct mi_root* cmd_tree, void* param)
 	sd= node->value;
 	if(sd.s== NULL || sd.len==0)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_add: could not read domain\n");
+		LM_ERR("could not read domain\n");
 		return init_mi_tree( 400, "domain not found", 16);
 	}
 
@@ -803,11 +757,11 @@ struct mi_root* pdt_mi_add(struct mi_root* cmd_tree, void* param)
 		 return init_mi_tree( 400, "empty param", 11);
 
 	
-	if(pdt_check_pd(_dhash, &sdomain, &sp, &sd)==1)
+	if(pdt_check_pd(*_dhash, &sdomain, &sp, &sd)==1)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_add: (sdomain,prefix,domain) exists\n");
+		LM_ERR("(sdomain,prefix,domain) exists\n");
 		return init_mi_tree( 400,
-			"(sdomain,prefix,domain) exists already", 38);
+				"(sdomain,prefix,domain) exists already", 38);
 	}
 	db_vals[0].type = DB_STR;
 	db_vals[0].nul = 0;
@@ -827,13 +781,14 @@ struct mi_root* pdt_mi_add(struct mi_root* cmd_tree, void* param)
 	/* insert a new domain into database */
 	if(pdt_dbf.insert(db_con, db_keys, db_vals, NR_KEYS)<0)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_add: error storing new prefix/domain\n");
+		LM_ERR("failed to store new prefix/domain\n");
 		return init_mi_tree( 500,"Cannot store prefix/domain", 26);
 	}
-	
-	if(pdt_add_to_hash(_dhash, &sdomain, &sp, &sd, 1)!=0)
+
+	/* re-loading all information from database */
+	if(pdt_load_db()!=0)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_add: could not add to cache\n");
+		LM_ERR("cannot re-load info from database\n");	
 		goto error;
 	}
 	
@@ -844,7 +799,7 @@ struct mi_root* pdt_mi_add(struct mi_root* cmd_tree, void* param)
 	
 error:
 	if(pdt_dbf.delete(db_con, db_keys, db_ops, db_vals, NR_KEYS)<0)
-		LOG(L_ERR,"PDT:pdt_mi_add: database/cache are inconsistent\n");
+		LM_ERR("database/cache are inconsistent\n");
 	return init_mi_tree( 500, "could not add to cache", 23 );
 
 error1:
@@ -860,7 +815,6 @@ error1:
 struct mi_root* pdt_mi_delete(struct mi_root* cmd_tree, void* param)
 {
 	str sd, sdomain;
-	int ret;
 	struct mi_node* node= NULL;
 	db_key_t db_keys[2] = {sdomain_column, domain_column};
 	db_val_t db_vals[2];
@@ -868,7 +822,7 @@ struct mi_root* pdt_mi_delete(struct mi_root* cmd_tree, void* param)
 
 	if(_dhash==NULL)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_delete: strange situation\n");
+		LM_ERR("strange situation\n");
 		return init_mi_tree( 500, MI_INTERNAL_ERR_S, MI_INTERNAL_ERR_LEN);
 	}
 
@@ -892,7 +846,7 @@ struct mi_root* pdt_mi_delete(struct mi_root* cmd_tree, void* param)
 	sd= node->value;
 	if(sd.s== NULL || sd.len==0)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_delete: could not read domain\n");
+		LM_ERR("could not read domain\n");
 		return init_mi_tree(404, "domain not found", 16);
 	}
 
@@ -900,20 +854,6 @@ struct mi_root* pdt_mi_delete(struct mi_root* cmd_tree, void* param)
 		 return init_mi_tree( 400, "empty param", 11);
 
 
-	if((ret = pdt_remove_from_hash_list(_dhash, &sdomain, &sd))<0)
-	{
-		DBG("PDT:pdt_mi_delete: error encountered when deleting domain\n");
-		return 0;
-	}
-
-	if(ret==1)
-	{
-		DBG("PDT:pdt_mi_delete: prefix for sdomain [%.*s]domain [%.*s] "
-			"not found\n", sdomain.len, sdomain.s, sd.len, sd.s);
-		return init_mi_tree( 404, "domain not found", 16);
-	}
-	
-
 	db_vals[0].type = DB_STR;
 	db_vals[0].nul = 0;
 	db_vals[0].val.str_val.s = sdomain.s;
@@ -926,11 +866,17 @@ struct mi_root* pdt_mi_delete(struct mi_root* cmd_tree, void* param)
 
 	if(pdt_dbf.delete(db_con, db_keys, db_ops, db_vals, 2)<0)
 	{
-		LOG(L_ERR,"PDT:pdt_mi_delete: database/cache are inconsistent\n");
+		LM_ERR("database/cache are inconsistent\n");
 		return init_mi_tree( 500, "database/cache are inconsistent", 31 );
 	} 
+	/* re-loading all information from database */
+	if(pdt_load_db()!=0)
+	{
+		LM_ERR("cannot re-load info from database\n");	
+		return init_mi_tree( 500, "cannot reload", 13 );
+	}
 
-	DBG("PDT:pdt_mi_delete: prefix for sdomain [%.*s] domain [%.*s] "
+	LM_DBG("prefix for sdomain [%.*s] domain [%.*s] "
 			"removed\n", sdomain.len, sdomain.s, sd.len, sd.s);
 	return init_mi_tree( 200, MI_OK_S, MI_OK_LEN);
 error:
@@ -967,10 +913,9 @@ struct mi_root* pdt_mi_list(struct mi_root* cmd_tree, void* param)
 	struct mi_node* node = NULL;
 	struct mi_attr* attr= NULL;
 
-	DBG("PDT:pdt_mi_list ...\n");
 	if(_dhash==NULL)
 	{
-		LOG(L_ERR, "PDT:pdt_mi_list: empty domain list\n");
+		LM_ERR("empty domain list\n");
 		return init_mi_tree( 500, MI_INTERNAL_ERR_S, MI_INTERNAL_ERR_LEN);
 	}
 
@@ -1001,10 +946,9 @@ struct mi_root* pdt_mi_list(struct mi_root* cmd_tree, void* param)
 			else {
 				while(sp.s!=NULL && i!=sp.len)
 				{
-					if(sp.s[i] < '0' || sp.s[i] > '9')
+					if(strpos(pdt_char_list.s,sp.s[i]) < 0)
 					{
-						LOG(L_ERR, "PDT:pdt_mi_list: bad prefix [%.*s]\n",
-							sp.len, sp.s);
+						LM_ERR("bad prefix [%.*s]\n", sp.len, sp.s);
 						return init_mi_tree( 400, "bad prefix", 10);
 					}
 					i++;
@@ -1025,10 +969,13 @@ struct mi_root* pdt_mi_list(struct mi_root* cmd_tree, void* param)
 	rpl_tree = init_mi_tree( 200, MI_OK_S, MI_OK_LEN );
 	if(rpl_tree == NULL)
 		return 0;
+	if(*_dhash==0)
+		return rpl_tree;
+
 	rpl = &rpl_tree->node;
 
-	lock_get(&_dhash->hl_lock);
-	h = _dhash->hash;
+	lock_get(&(*_dhash)->hl_lock);
+	h = (*_dhash)->hash;
 
 	while(h!=NULL)
 	{
@@ -1042,10 +989,10 @@ struct mi_root* pdt_mi_list(struct mi_root* cmd_tree, void* param)
 				while(it!=NULL)
 				{
 					if((sp.s==NULL && sd.s==NULL)
-						||(sp.s==NULL && (sd.s!=NULL && it->domain.len>=sd.len &&
-							strncasecmp(it->domain.s, sd.s, sd.len)==0)) 
-						|| ( sd.s==NULL && (sp.s!=NULL && it->prefix.len>=sp.len &&
-							strncmp(it->prefix.s, sp.s, sp.len)==0))
+						||(sp.s==NULL && (sd.s!=NULL && it->domain.len>=sd.len
+							&& strncasecmp(it->domain.s, sd.s, sd.len)==0)) 
+						|| (sd.s==NULL && (sp.s!=NULL && it->prefix.len>=sp.len
+							&& strncmp(it->prefix.s, sp.s, sp.len)==0))
 						|| ((sp.s!=NULL && it->prefix.len>=sp.len &&
 							strncmp(it->prefix.s, sp.s, sp.len)==0)
 						&& (sd.s!=NULL && it->domain.len>=sd.len &&
@@ -1074,15 +1021,15 @@ struct mi_root* pdt_mi_list(struct mi_root* cmd_tree, void* param)
 				}
 			}
 		}
-			h = h->next;
+		h = h->next;
 	}
 
-	lock_release(&_dhash->hl_lock);
+	lock_release(&(*_dhash)->hl_lock);
 	
 	return rpl_tree;
 
 error:
-	lock_release(&_dhash->hl_lock);
+	lock_release(&(*_dhash)->hl_lock);
 	free_mi_tree(rpl_tree);
 	return 0;
 }
