@@ -1,5 +1,5 @@
 /*
- * $Id: perl.h 2164 2007-05-08 07:30:16Z bastian $
+ * $Id: perl.h 2153 2007-05-04 21:19:20Z bastian $
  *
  * Perl module for OpenSER
  *
@@ -44,4 +44,7 @@ extern PerlInterpreter *my_perl;
 
 extern struct sl_binds slb;
 
+#define PERLCLASS_MESSAGE	"OpenSER::Message"
+#define PERLCLASS_URI		"OpenSER::URI"
+
 #endif /* PERL_MOD_H */
