@@ -36,6 +36,7 @@
 #define MI_ADDRESS_DUMP "address_dump"
 #define MI_SUBNET_DUMP "subnet_dump"
 
+#define MI_ALLOW_URI "allow_uri"
 
 struct mi_root* mi_trusted_reload(struct mi_root *cmd, void *param);
 
@@ -47,4 +48,6 @@ struct mi_root* mi_address_dump(struct mi_root *cmd, void *param);
 
 struct mi_root* mi_subnet_dump(struct mi_root *cmd_tree, void *param);
 
+struct mi_root* mi_allow_uri(struct mi_root *cmd, void *param);
+
 #endif
