@@ -1,5 +1,5 @@
 /*
- * $Id: db_res.c 2871 2007-10-05 14:32:16Z henningw $
+ * $Id: db_res.c 4193 2008-05-15 11:52:40Z henningw $
  *
  * POSTGRES module, portions of this code were templated using
  * the mysql module, thus it's similarity.
@@ -57,7 +57,7 @@
  */
 int pg_convert_result(db_con_t* _con, db_res_t* _res)
 {
-
+        int rows;
 #ifdef PARANOID
         if (!_con)  {
                 LM_ERR("db_con_t parameter cannot be NULL\n");
@@ -75,7 +75,14 @@ int pg_convert_result(db_con_t* _con, db_res_t* _res)
                 return -2;
         }
 
-        if (pg_convert_rows(_con, _res, 0, PQntuples(CON_RESULT(_con))) < 0) {
+        rows = PQntuples(CON_RESULT(_con));
+        if (rows == 0) {
+            LM_DBG("no rows in the query\n");
+            return 0;
+        }
+
+
+        if (pg_convert_rows(_con, _res, 0, rows) < 0) {
                LM_ERR("failed to convert rows\n");
                 pg_free_columns(_res);
                 return -3;
@@ -199,10 +206,10 @@ int pg_get_columns(db_con_t* _con, db_res_t* _res)
 			case CHAROID:
 			case VARCHAROID:
 			case BPCHAROID:
-			case TEXTOID:
 				RES_TYPES(_res)[col] = DB_STRING;
 			break;
 
+			case TEXTOID:
 			case BYTEAOID:
 				RES_TYPES(_res)[col] = DB_BLOB;
 			break;
@@ -243,11 +250,6 @@ int pg_convert_rows(db_con_t* _con, db_res_t* _res, int row_start, int row_count
 	}
 #endif
 
-	if (row_count == 0) {
-		LM_ERR("no rows requested from the query\n");
-		return 0;
-	}
-
 	if (!RES_NUM_ROWS(_res)) {
 		LM_DBG("no rows returned from the query\n");
 		return 0;
