@@ -1,5 +1,5 @@
 /*
- * $Id: dbase.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: dbase.c 3151 2007-11-15 22:04:03Z bogdan_iancu $
  *
  * POSTGRES module, portions of this code were templated using
  * the mysql module, thus it's similarity.
@@ -70,7 +70,8 @@
 #include "../../dprint.h"
 #include "../../mem/mem.h"
 #include "../../db/db.h"
-#include "db_utils.h"
+#include "../../db/db_ut.h"
+#include "../../db/db_res.h"
 #include "defs.h"
 #include "dbase.h"
 #include "pg_con.h"
@@ -80,8 +81,6 @@ long getpid();
 static char _s[SQL_BUF_LEN];
 
 static int submit_query(db_con_t* _con, const char* _s);
-//static int connect_db(db_con_t* _con);
-//static int disconnect_db(db_con_t* _con);
 static int free_query(db_con_t* _con);
 
 /*
@@ -93,12 +92,12 @@ int pg_use_table(db_con_t* _con, const char* _t)
 
 #ifdef PARANOID
 	if (!_con) {
-		LOG(L_ERR, "PG[pg_use_table]: db_con_t parameter cannot be NULL\n");
+		LM_ERR("db_con_t parameter cannot be NULL\n");
 		return -1;
 	}
 
 	if (!_t) {
-		LOG(L_ERR, "PG[pg_use_table]: _t parameter cannot be NULL\n");
+		LM_ERR("_t parameter cannot be NULL\n");
 		return -1;
 	}
 
@@ -108,164 +107,6 @@ int pg_use_table(db_con_t* _con, const char* _t)
 	return 0;
 }
 
-#if 0
-/*
-** connect_db	Connect to a database
-**
-**	Arguments :
-**		db_con_t *	as previously supplied by pg_init()
-**
-**	Returns :
-**		0 upon success
-**		negative number upon failure
-**
-**	Notes :
-**		If currently connected, a disconnect is done first
-**		if this process did the connection, otherwise the
-**		disconnect is not done before the new connect.
-**		This is important, as the process that owns the connection
-**		should clean up after itself.
-*/
-
-static int connect_db(db_con_t* _con)
-{
-	char* user, *password, *host, *port, *database;
-	char urlbuf[SQLURL_LEN];
-
-#ifdef PARANOID
-	if(! _con)
-	{
-		LOG(L_ERR, "PG[connect_db]: db_con_t parameter cannot be NULL\n");
-		return(-1);
-	}
-#endif
-
-	if(CON_CONNECTED(_con))
-	{
-		DLOG("connect_db", "disconnect first!");
-		disconnect_db(_con);
-	}
-	
-	if (!CON_SQLURL(_con)) {
-		PLOG("connect_db","FATAL ERROR: no sql url!");
-		return(-1);
-	}
-
-	/*
-	** CON_CONNECTED(_con) is now 0, set by disconnect_db()
-	*/
-
-	/*
-	** Note :
-	** Make a scratch pad copy of given SQL URL in urlbuf and work on it.
-	** all memory allocated to this connection is rooted
-	** from this.
-	** This is an important concept.
-	** as long as you always allocate memory using the function:
-	** mem = aug_alloc(size, CON_SQLURL(_con)) or
-	** where size is the amount of memory, then in the future
-	** when CON_SQLURL(_con) is freed (in the function disconnect_db())
-	** all other memory allocated in this manner is freed.
-	** this will keep memory leaks from happening.
-	*/
-
-	snprintf(urlbuf,SQLURL_LEN,"%s",CON_SQLURL(_con));
-
-	/*
-	** get the connection parameters parsed from the db_url string
-	** it looks like: postgres://username:userpass@dbhost:dbport/dbname
-	** username/userpass : name and password for the database
-	** dbhost :            the host name or ip address hosting the database
-	** dbport :            the port to connect to database on
-	** dbname :            the name of the database
-	*/
-	if(parse_sql_url(urlbuf, &user, &password, &host, &port, &database) < 0)
-	{
-		char buf[SQL_BUF_LEN];
-		snprintf(buf, SQL_BUF_LEN, "Error while parsing %s", CON_SQLURL(_con));
-		PLOG("connect_db", buf);
-
-		return -3;
-	}
-
-	/*
-	** finally, actually connect to the database
-	*/
-	CON_CONNECTION(_con) =
-		PQsetdbLogin(host,port,NULL,NULL,database,user, password);
-
-	if(CON_CONNECTION(_con) == 0
-	    || PQstatus(CON_CONNECTION(_con)) != CONNECTION_OK)
-	{
-		PLOG("connect_db", PQerrorMessage(CON_CONNECTION(_con)));
-		PQfinish(CON_CONNECTION(_con));
-		return -4;
-	}
-
-	CON_PID(_con) = getpid();
-
-	/*
-	** all is well, database was connected, we can now submit_query's
-	*/
-	CON_CONNECTED(_con) = 1;
-	return 0;
-}
-
-
-/*
-** disconnect_db	Disconnect a database
-**
-**	Arguments :
-**		db_con_t *	as previously supplied by pg_init()
-**
-**	Returns :
-**		0 upon success
-**		negative number upon failure
-**
-**	Notes :
-**		All memory associated with CON_SQLURL is freed.
-**		
-*/
-
-static int disconnect_db(db_con_t* _con)
-{
-
-#ifdef PARANOID
-	if(! _con)
-	{
-		LOG(L_ERR, "PG[disconnect_db]: db_con_t parameter cannot be NULL\n");
-		return(-1);
-	}
-#endif
-
-	/*
-	** ignore if there is no current connection
-	*/
-	if(CON_CONNECTED(_con) != 1)
-	{
-		DLOG("disconnect_db", "not connected, ignored!\n");
-		return 0;
-	}
-
-	/*
-	** make sure we are trying to close a connection that was opened
-	** by our process ID
-	*/
-	if(CON_PID(_con) == getpid())
-	{
-		PQfinish(CON_CONNECTION(_con));
-		CON_CONNECTED(_con) = 0;
-	}
-	else
-	{
-		DLOG("disconnect_db",
-			"attempt to release connection not owned, ignored!\n");
-	}
-
-	return 0;
-}
-#endif
-
 
 /*
 ** pg_init	initialize database for future queries
@@ -290,7 +131,7 @@ db_con_t *pg_init(const char* _url)
 
 	if (strlen(_url)>(SQLURL_LEN-1)) 
 	{
-		LOG(L_ERR, "PG[pg_init]: ERROR sql url too long\n");
+		LM_ERR("ERROR sql url too long\n");
 		return 0;
 	}
 
@@ -299,15 +140,17 @@ db_con_t *pg_init(const char* _url)
 	*/
 	_res = (db_con_t*)pkg_malloc(con_size);
 	if (!_res) {
-		LOG(L_ERR, "PG[pg_init]: Failed trying to allocate %d bytes for database connection\n", con_size);
+		LM_ERR("no more pkg memory for database connection(%i bytes)\n",
+				con_size);
 		return 0;
 	}
-	LOG(L_DBG, "PG[pg_init]: %p=pkg_malloc(%d) for database connection\n", (db_con_t*)_res, con_size);
+	LM_DBG("%p=pkg_malloc(%d) for database connection\n", 
+			(db_con_t*)_res, con_size);
 	memset(_res, 0, con_size);
 
 	id = new_db_id(_url);
 	if (!id) {
-		LOG(L_ERR, "PG[pg_init]: Error: Cannot parse URL '%s'\n", _url);
+		LM_ERR("cannot parse URL '%s'\n", _url);
 		goto err;
 	}
 
@@ -317,13 +160,13 @@ db_con_t *pg_init(const char* _url)
 		/* 
 		 * The LOG below exposes the username/password of the database connection
 		 * by default, it is commented.
-		 * LOG(L_DBG, "PG[pg_init]: Connection '%s' not found in pool\n", _url);
+		 * LM_DBG("connection '%s' not found in pool\n", _url);
 		 *
 		 */
-		LOG(L_DBG, "PG[pg_init]: Connection %p not found in pool\n", id);
+		LM_DBG("connection %p not found in pool\n", id);
 		_con = pg_new_conn(id);
 		if (!_con) {
-			LOG(L_ERR, "PG[pg_init]: Error: pg_new_con failed to add connection to pool\n");
+			LM_ERR("pg_new_con failed to add connection to pool\n");
 			goto err;
 		}
 		pool_insert((struct pool_con*)_con);
@@ -331,10 +174,10 @@ db_con_t *pg_init(const char* _url)
 		/* 
 		 * The LOG below exposes the username/password of the database connection
 		 * by default, it is commented.
-		 * LOG(L_DBG, "PG[pg_init]: Connection '%s' found in pool\n", _url);
+		 * LM_DBG("connection '%s' found in pool\n", _url);
 		 *
 		 */
-		LOG(L_DBG, "PG[pg_init]: Connection %p found in pool\n", id);
+		LM_DBG("connection %p found in pool\n", id);
 	}
 
 	_res->tail = (unsigned long)_con;
@@ -343,14 +186,14 @@ db_con_t *pg_init(const char* _url)
 err:
 	if (id) free_db_id(id);
 	if (_res) {
-		LOG(L_ERR, "PG[pg_init]: Error: Cleaning up: %p=pkg_free()\n", _res);
+		LM_ERR("cleaning up: %p=pkg_free()\n", _res);
 		pkg_free(_res);
 	}
 	return 0;
 }
 
 
-/**
+/*
 ** pg_close	last function to call when db is no longer needed
 **
 **	Arguments :
@@ -373,7 +216,7 @@ void pg_close(db_con_t* _con)
 		pg_free_conn((struct pg_con*)con);
 	}
 	
-	LOG(L_DBG, "PG[pg_close]: %p=pkg_free() _con\n", _con);
+	LM_DBG("%p=pkg_free() _con\n", _con);
 	pkg_free(_con);
 }
 
@@ -394,7 +237,7 @@ static int submit_query(db_con_t* _con, const char* _s)
 #ifdef PARANOID
 	if(! _con)
 	{
-		LOG(L_ERR, "PG[submit_query]: db_con_t parameter cannot be NULL\n");
+		LM_ERR("db_con_t parameter cannot be NULL\n");
 		return(-1);
 	}
 #endif
@@ -406,7 +249,7 @@ static int submit_query(db_con_t* _con, const char* _s)
 		case CONNECTION_OK: 
 			break;
 		case CONNECTION_BAD:
-			LOG(L_DBG, "PG[submit_query]: connection reset\n");
+			LM_DBG("connection reset\n");
 			PQreset(CON_CONNECTION(_con));
 			break;
 		case CONNECTION_STARTED:
@@ -417,7 +260,8 @@ static int submit_query(db_con_t* _con, const char* _s)
 		case CONNECTION_SSL_STARTUP:
 		case CONNECTION_NEEDED:
 		default:
-                	LOG(L_ERR, "PG[submit_query]: %p PQstatus(%s) invalid: %s\n", _con, PQerrorMessage(CON_CONNECTION(_con)), _s);
+                	LM_ERR("%p PQstatus(%s) invalid: %s\n", _con,
+							PQerrorMessage(CON_CONNECTION(_con)), _s);
 			return -1;
 	}
 
@@ -434,16 +278,17 @@ static int submit_query(db_con_t* _con, const char* _s)
 	*/
 
         if (PQsendQuery(CON_CONNECTION(_con), _s)) {
-                LOG(L_DBG, "PG[submit_query]: %p PQsendQuery(%s)\n", _con, _s);
+                LM_DBG("%p PQsendQuery(%s)\n", _con, _s);
         } else {
-                LOG(L_ERR, "PG[submit_query]: %p PQsendQuery Error: %s Query: %s\n", _con, PQerrorMessage(CON_CONNECTION(_con)), _s);
+                LM_ERR("%p PQsendQuery Error: %s Query: %s\n", _con,
+						PQerrorMessage(CON_CONNECTION(_con)), _s);
 		return -1;
         }
 	return 0;
 
 }
 
-/**
+/*
  *
  * pg_fetch_result: Gets a partial result set.
  *
@@ -457,27 +302,30 @@ int pg_fetch_result(db_con_t* _con, db_res_t** _res, int nrows)
 
 #ifdef PARANOID
 	if (!_con) {
-                LOG(L_ERR, "PG[fetch_result]: db_con_t parameter cannot be NULL\n");
+                LM_ERR("db_con_t parameter cannot be NULL\n");
                 return -1;
 	}
 	if (!_res) {
-                LOG(L_ERR, "PG[fetch_result]: db_res_t parameter cannot be NULL\n");
+                LM_ERR("db_res_t parameter cannot be NULL\n");
                 return -1;
 	}
 	if (nrows < 0) {
-                LOG(L_ERR, "PG[fetch_result]: nrows parameter cannot be less than zero\n");
+                LM_ERR("nrows parameter cannot be less than zero\n");
                 return -1;
 	}
 #endif
 
 	/* exit if the fetch count is zero */
 	if (nrows == 0) {
+		if (*_res)
+			pg_free_result(*_res);
+        	*_res = 0;
 		return 0;
 	}
 
 	if (*_res == NULL) {
 		/* Allocate a new result structure */
-		*_res = pg_new_result();
+		*_res = db_new_result();
 
 		/* Get the result of the previous query */
 		while (1) {
@@ -488,7 +336,8 @@ int pg_fetch_result(db_con_t* _con, db_res_t** _res, int nrows)
 			}
 		}
 		pqresult = PQresultStatus(CON_RESULT(_con));
-		LOG(L_DBG, "PG[fetch_result]: %p PQresultStatus(%s) PQgetResult(%p)\n", _con, PQresStatus(pqresult), CON_RESULT(_con));
+		LM_DBG("%p PQresultStatus(%s) PQgetResult(%p)\n", _con,
+				PQresStatus(pqresult), CON_RESULT(_con));
 
         	switch(pqresult) {
                 	case PGRES_COMMAND_OK:
@@ -497,7 +346,7 @@ int pg_fetch_result(db_con_t* _con, db_res_t** _res, int nrows)
                 	case PGRES_TUPLES_OK:
                         	/* Successful completion of a command returning data (such as a SELECT or SHOW). */
 				if (pg_get_columns(_con, *_res) < 0) {
-					LOG(L_ERR, "PG[fetch_result]: Error while getting column names\n");
+					LM_ERR("failed to get column names\n");
 					return -2;
         			}
                         	break;
@@ -507,9 +356,10 @@ int pg_fetch_result(db_con_t* _con, db_res_t** _res, int nrows)
                 	case PGRES_BAD_RESPONSE:
                 	case PGRES_NONFATAL_ERROR:
                 	case PGRES_FATAL_ERROR:
-        			LOG(L_WARN, "PG[fetch_result]: %p Warning: probable invalid query\n", _con);
+        				LM_WARN("%p - probable invalid query\n", _con);
                 	default:
-        			LOG(L_WARN, "PG[fetch_result]: %p Warning: PQresultStatus(%s)\n", _con, PQresStatus(pqresult));
+        				LM_WARN("%p - PQresultStatus(%s)\n",
+							_con, PQresStatus(pqresult));
        				if (*_res) 
 					pg_free_result(*_res);
         			*_res = 0;
@@ -537,13 +387,13 @@ int pg_fetch_result(db_con_t* _con, db_res_t** _res, int nrows)
 
         RES_ROW_N(*_res) = rows;
 
-	LOG(L_DBG, "PG[fetch_result]: Converting row %d of %d count %d\n", RES_LAST_ROW(*_res), RES_NUM_ROWS(*_res), RES_ROW_N(*_res));
+	LM_DBG("converting row %d of %d count %d\n", RES_LAST_ROW(*_res),
+			RES_NUM_ROWS(*_res), RES_ROW_N(*_res));
 
         if (pg_convert_rows(_con, *_res, RES_LAST_ROW(*_res), RES_ROW_N(*_res)) < 0) {
-                LOG(L_ERR, "PG[fetch_result]: Error while converting rows\n");
-                pg_free_columns(*_res);
-       		if (*_res) 
-			pg_free_result(*_res);
+                LM_ERR("failed to convert rows\n");
+		if (*_res) 
+                	pg_free_result(*_res);
         	*_res = 0;
 		return -3;
         }
@@ -554,7 +404,7 @@ int pg_fetch_result(db_con_t* _con, db_res_t** _res, int nrows)
         return 0;
 }
 
-/**
+/*
 ** free_query	clear the db channel and clear any old query result status
 **
 **	Arguments :
@@ -569,7 +419,7 @@ static int free_query(db_con_t* _con)
 {
 	if(CON_RESULT(_con))
 	{
-		LOG(L_DBG, "PG[free_query]: PQclear(%p) result set\n", CON_RESULT(_con));
+		LM_DBG("PQclear(%p) result set\n", CON_RESULT(_con));
 		PQclear(CON_RESULT(_con));
 		CON_RESULT(_con) = 0;
 	}
@@ -648,7 +498,7 @@ static int begin_transaction(db_con_t * _con, char *_s)
 				** related to the parent process forking,
 				** or being forked.
 				*/
-				LOG(L_ERR,"PG[begin_transaction]: corrupt connection\n");
+				LM_ERR("corrupt connection\n");
 				CON_CONNECTED(_con) = 0;
 			}
 			else
@@ -657,14 +507,14 @@ static int begin_transaction(db_con_t * _con, char *_s)
 				** this is the normal way out.
 				** the transaction ran fine.
 				*/
-				LOG(L_DBG, "PG[begin_transaction]: %p PQclear(%p) result set\n", _con, mr);
+				LM_DBG("%p PQclear(%p) result set\n", _con, mr);
 				PQclear(mr);
 				return(0);
 			}
 		}
 		else
 		{
-			LOG(L_DBG, "PG[begin_transaction]: called before pg_init\n");
+			LM_DBG("called before pg_init\n");
 		}
 
 		/*
@@ -680,14 +530,14 @@ static int begin_transaction(db_con_t * _con, char *_s)
 			*/
 			char buf[SQL_BUF_LEN];
 			snprintf(buf, SQL_BUF_LEN, "no connection, FATAL %d!", rv);
-			LOG(L_DBG, "PG[begin_transaction]: %s\n", buf);
+			LM_DBG("%s\n", buf);
 			return(rv);
 		}
-		LOG(L_DBG, "PG[begin_transaction]: successfully reconnected\n");
+		LM_DBG("successfully reconnected\n");
 	}
 	else
 	{
-		LOG(L_DBG, "PG[begin_transaction]: must call pg_init first!\n");
+		LM_DBG("must call pg_init first!\n");
 		return(-1);
 	}
 
@@ -702,13 +552,13 @@ static int begin_transaction(db_con_t * _con, char *_s)
 		char buf[SQL_BUF_LEN];
 		snprintf(buf, SQL_BUF_LEN, "FATAL %s, '%s'!\n",
 			PQerrorMessage(CON_CONNECTION(_con)), _s);
-		LOG(L_DBG, "PG[begin_transaction]: %s\n", buf);
+		LM_DBG("%s\n", buf);
 		return(-1);
 	}
 
-	LOG(L_DBG, "PG[begin_transaction]: db channel reset successful\n");
+	LM_DBG("db channel reset successful\n");
 
-	LOG(L_DBG, "PG[begin_transaction]: %p PQclear(%p) result set\n", _con, mr);
+	LM_DBG("%p PQclear(%p) result set\n", _con, mr);
 	PQclear(mr);
 	return(0);
 }
@@ -731,10 +581,10 @@ static int commit_transaction(db_con_t * _con)
 	mr = PQexec(CON_CONNECTION(_con), "COMMIT");
 	if(!mr || PQresultStatus(mr) != PGRES_COMMAND_OK)
 	{
-		LOG(L_ERR, "PG[commit_transaction]: error");
+		LM_ERR("error");
 		return -1;
 	}
-	LOG(L_DBG, "PG[commit_transaction]: %p PQclear(%p) result set\n", _con, mr);
+	LM_DBG("%p PQclear(%p) result set\n", _con, mr);
 	PQclear(mr);
 	return(0);
 }
@@ -757,108 +607,16 @@ static int rollback_transaction(db_con_t * _con)
 	mr = PQexec(CON_CONNECTION(_con), "ROLLBACK");
 	if(!mr || PQresultStatus(mr) != PGRES_COMMAND_OK)
 	{
-		LOG(L_ERR, "PG[rollback_transaction]: error");
+		LM_ERR("error");
 		return -1;
 	}
-	LOG(L_DBG, "PG[rollback_transaction]: %p PQclear(%p) result set\n", _con, mr);
+	LM_DBG("%p PQclear(%p) result set\n", _con, mr);
 	PQclear(mr);
 	return(0);
 }
 
 #endif
 
-/*
- * Print list of columns separated by comma
- */
-static int print_columns(char* _b, int _l, db_key_t* _c, int _n)
-{
-	int i;
-	int res = 0;
-	for(i = 0; i < _n; i++) {
-		if (i == (_n - 1)) {
-			res += snprintf(_b + res, _l - res, "%s ", _c[i]);
-		} else {
-			res += snprintf(_b + res, _l - res, "%s,", _c[i]);
-		}
-	}
-	return res;
-}
-
-
-/*
- * Print list of values separated by comma
- */
-static int print_values(db_con_t* _con, char* _b, int _l, db_val_t* _v, int _n)
-{
-	int i, res = 0, l;
-
-	for(i = 0; i < _n; i++) {
-		l = _l - res;
-/*		LOG(L_ERR, "%d sizes l = _l - res %d = %d - %d\n", i, l,_l,res);
-*/
-		if (val2str(_con, _v + i, _b + res, &l) < 0) {
-			LOG(L_ERR, "PG[print_values]: Error converting value to string\n");
-			return 0;
-		}
-		res += l;
-		if (i != (_n - 1)) {
-			*(_b + res) = ',';
-			res++;
-		}
-	}
-	return res;
-}
-
-
-/*
- * Print where clause of SQL statement
- */
-static int print_where(db_con_t* _con, char* _b, int _l, db_key_t* _k,
-	db_op_t* _o, db_val_t* _v, int _n)
-{
-	int i;
-	int res = 0;
-	int l;
-	for(i = 0; i < _n; i++) {
-		if (_o) {
-			res += snprintf(_b + res, _l - res, "%s%s",
-				_k[i], _o[i]);
-		} else {
-			res += snprintf(_b + res, _l - res, "%s=", _k[i]);
-		}
-		l = _l - res;
-		val2str(_con, &(_v[i]), _b + res, &l);
-		res += l;
-		if (i != (_n - 1)) {
-			res += snprintf(_b + res, _l - res, " AND ");
-		}
-	}
-	return res;
-}
-
-
-/*
- * Print set clause of update SQL statement
- */
-static int print_set(db_con_t* _con, char* _b, int _l, db_key_t* _k,
-	db_val_t* _v, int _n)
-{
-	int i;
-	int res = 0;
-	int l;
-	for(i = 0; i < _n; i++) {
-		res += snprintf(_b + res, _l - res, "%s=", _k[i]);
-		l = _l - res;
-		val2str(_con, &(_v[i]), _b + res, &l);
-		res += l;
-		if (i != (_n - 1)) {
-			if ((_l - res) >= 1) {
-				*(_b + res++) = ',';
-			}
-		}
-	}
-	return res;
-}
 
 
 /*
@@ -882,26 +640,26 @@ int pg_query(db_con_t* _con, db_key_t* _k, db_op_t* _op,
 			"select * from %s ", CON_TABLE(_con));
 	} else {
 		off = snprintf(_s, SQL_BUF_LEN, "select ");
-		off += print_columns(_s + off, SQL_BUF_LEN - off, _c, _nc);
+		off += db_print_columns(_s + off, SQL_BUF_LEN - off, _c, _nc);
 		off += snprintf(_s + off, SQL_BUF_LEN - off,
 			"from %s ", CON_TABLE(_con));
 	}
 	if (_n) {
 		off += snprintf(_s + off, SQL_BUF_LEN - off, "where ");
-		off += print_where(_con, _s + off, SQL_BUF_LEN - off,
-			_k, _op, _v, _n);
+		off += db_print_where(_con, _s + off, SQL_BUF_LEN - off,
+			_k, _op, _v, _n, val2str);
 	}
 	if (_o) {
 		off += snprintf(_s + off, SQL_BUF_LEN - off,
-			"order by %s", _o);
+			" order by %s", _o);
 	}
 
-	LOG (L_DBG, "PG[pg_query]: %p %p %s\n", _con, _r, _s);
+	LM_DBG("%p %p %s\n", _con, _r, _s);
 
         if(_r) {
 		/* if(begin_transaction(_con, _s)) return(-1); */
 		if (submit_query(_con, _s) < 0) {
-			LOG(L_ERR, "PG[pg_query]: Error while submitting query\n");
+			LM_ERR("failed to submit query\n");
 			/* rollback_transaction(_con); */
 			return -2;
 		}
@@ -910,7 +668,7 @@ int pg_query(db_con_t* _con, db_key_t* _k, db_op_t* _op,
 		return(rv);
 	} else {
 		if (submit_query(_con, _s) < 0) {
-			LOG(L_ERR, "PG[pg_query]: Error while submitting query\n");
+			LM_ERR("failed to submit query\n");
 			return -2;
 		}
 	}
@@ -926,12 +684,12 @@ int pg_raw_query(db_con_t* _con, char* _s, db_res_t** _r)
 {
 	int rv;
 	
-	LOG (L_DBG, "PG[pg_raw_query]: %p %p %s\n", _con, _r, _s);
+	LM_DBG("%p %p %s\n", _con, _r, _s);
 
         if(_r) {
                 /* if(begin_transaction(_con, _s)) return(-1); */
                 if (submit_query(_con, _s) < 0) {
-                        LOG(L_ERR, "PG[pg_raw_query]: Error while submitting query\n");
+						LM_ERR("failed to submit query\n");
                         /* rollback_transaction(_con); */
                         return -2;
                 }
@@ -940,7 +698,7 @@ int pg_raw_query(db_con_t* _con, char* _s, db_res_t** _r)
                 return(rv);
         } else {
                 if (submit_query(_con, _s) < 0) {
-                        LOG(L_ERR, "PG[pg_raw_query]: Error while submitting query\n");
+						LM_ERR("failed to submit query\n");
                         return -2;
                 }
         }
@@ -979,7 +737,7 @@ int pg_get_result(db_con_t* _con, db_res_t** _r)
         ExecStatusType pqresult;
 	int rc = 0;
 
-	*_r = pg_new_result();
+	*_r = db_new_result();
 
 	while (1) {
 		if ((res = PQgetResult(CON_CONNECTION(_con)))) {
@@ -991,7 +749,8 @@ int pg_get_result(db_con_t* _con, db_res_t** _r)
 
         pqresult = PQresultStatus(CON_RESULT(_con));
 	
-	LOG(L_DBG, "PG[get_result]: %p PQresultStatus(%s) PQgetResult(%p)\n", _con, PQresStatus(pqresult), CON_RESULT(_con));
+	LM_DBG("%p PQresultStatus(%s) PQgetResult(%p)\n", _con,
+			PQresStatus(pqresult), CON_RESULT(_con));
 
         switch(pqresult) {
                 case PGRES_COMMAND_OK:
@@ -1001,7 +760,8 @@ int pg_get_result(db_con_t* _con, db_res_t** _r)
                 case PGRES_TUPLES_OK:
                         /* Successful completion of a command returning data (such as a SELECT or SHOW). */
                         if (pg_convert_result(_con, *_r) < 0) {
-                                LOG(L_ERR, "PG[get_result]: %p Error returned from convert_result()\n", _con);
+                                LM_ERR("%p Error returned from"
+										"convert_result()\n", _con);
        				if (*_r) pg_free_result(*_r);
         			*_r = 0;
                                 rc = -4;
@@ -1014,10 +774,11 @@ int pg_get_result(db_con_t* _con, db_res_t** _r)
                 case PGRES_BAD_RESPONSE:
                 case PGRES_NONFATAL_ERROR:
                 case PGRES_FATAL_ERROR:
-        		LOG(L_WARN, "PG[get_result]: %p Warning: Probable invalid query\n", _con);
+        		LM_WARN("%p Probable invalid query\n", _con);
                 default:
-        		LOG(L_WARN, "PG[get_result]: %p Warning: %s\n", _con, PQresStatus(pqresult));
-        		LOG(L_WARN, "PG[get_result]: %p Warning: %s\n", _con, PQresultErrorMessage(CON_RESULT(_con)));
+        		LM_WARN("%p: %s\n", _con, PQresStatus(pqresult));
+        		LM_WARN("%p: %s\n", _con,
+						PQresultErrorMessage(CON_RESULT(_con)));
        			if (*_r) pg_free_result(*_r);
         		*_r = 0;
 			rc = (int)pqresult;
@@ -1042,23 +803,23 @@ int pg_insert(db_con_t* _con, db_key_t* _k, db_val_t* _v, int _n)
 	int rv = 0;
 
 	off = snprintf(_s, SQL_BUF_LEN, "insert into %s (", CON_TABLE(_con));
-	off += print_columns(_s + off, SQL_BUF_LEN - off, _k, _n);
+	off += db_print_columns(_s + off, SQL_BUF_LEN - off, _k, _n);
 	off += snprintf(_s + off, SQL_BUF_LEN - off, ") values (");
-	off += print_values(_con, _s + off, SQL_BUF_LEN - off, _v, _n);
+	off += db_print_values(_con, _s + off, SQL_BUF_LEN - off, _v, _n, val2str);
 	*(_s + off++) = ')';
 	*(_s + off) = '\0';
 
-	LOG(L_DBG, "PG[insert]: %p %s\n", _con, _s);
+	LM_DBG("%p %s\n", _con, _s);
 
 	/* if(begin_transaction(_con, _s)) return(-1); */
 	if (submit_query(_con, _s) < 0) {
-		LOG(L_ERR, "PG[insert]: Error while inserting\n");
+		LM_ERR("failed to insert\n");
 		/* rollback_transaction(_con); */
 		return -2;
 	}
 	rv = pg_get_result(_con,&_r);	
 	if (rv != 0) {
-		LOG(L_WARN, "PG[insert]: Warning: %p Query: %s\n", _con, _s);
+		LM_WARN("%p Query: %s\n", _con, _s);
 	}
 	if (_r) 
 		pg_free_result(_r);
@@ -1084,21 +845,21 @@ int pg_delete(db_con_t* _con, db_key_t* _k, db_op_t* _o, db_val_t* _v, int _n)
 	off = snprintf(_s, SQL_BUF_LEN, "delete from %s", CON_TABLE(_con));
 	if (_n) {
 		off += snprintf(_s + off, SQL_BUF_LEN - off, " where ");
-		off += print_where(_con, _s + off, SQL_BUF_LEN - off, _k,
-			_o, _v, _n);
+		off += db_print_where(_con, _s + off, SQL_BUF_LEN - off, _k,
+			_o, _v, _n, val2str);
 	}
 
-	LOG(L_DBG, "pg_delete: %p %s\n", _con, _s);
+	LM_DBG("%p %s\n", _con, _s);
 
 	/* if(begin_transaction(_con, _s)) return(-1); */
 	if (submit_query(_con, _s) < 0) {
-		LOG(L_ERR, "PG[delete]: Error while deleting\n");
+		LM_ERR("failed to delete\n");
 		/* rollback_transaction(_con); */
 		return -2;
 	}
 	rv = pg_get_result(_con,&_r);	
 	if (rv != 0) {
-		LOG(L_WARN, "PG[delete]: Warning: %p Query: %s\n", _con, _s);
+		LM_WARN("%p Query: %s\n", _con, _s);
 	}
 	if (_r) 
 		pg_free_result(_r);
@@ -1126,25 +887,25 @@ int pg_update(db_con_t* _con, db_key_t* _k, db_op_t* _o, db_val_t* _v,
 	int rv = 0;
 
 	off = snprintf(_s, SQL_BUF_LEN, "update %s set ", CON_TABLE(_con));
-	off += print_set(_con, _s + off, SQL_BUF_LEN - off, _uk, _uv, _un);
+	off += db_print_set(_con, _s + off, SQL_BUF_LEN - off, _uk, _uv, _un, val2str);
 	if (_n) {
 		off += snprintf(_s + off, SQL_BUF_LEN - off, " where ");
-		off += print_where(_con, _s + off, SQL_BUF_LEN - off, _k,
-			_o, _v, _n);
+		off += db_print_where(_con, _s + off, SQL_BUF_LEN - off, _k,
+			_o, _v, _n, val2str);
 		*(_s + off) = '\0';
 	}
 
-	LOG(L_DBG, "PG[update]: %p %s\n", _con, _s);
+	LM_DBG("%p %s\n", _con, _s);
 
 	/* if(begin_transaction(_con, _s)) return(-1); */
 	if (submit_query(_con, _s) < 0) {
-		LOG(L_ERR, "PG[update]: Error while updating\n");
+		LM_ERR("failed to update\n");
 		/* rollback_transaction(_con); */
 		return -2;
 	}
 	rv = pg_get_result(_con,&_r);	
 	if (rv != 0) {
-		LOG(L_WARN, "PG[update]: Warning: %p Query: %s\n", _con, _s);
+		LM_WARN("%p Query: %s\n", _con, _s);
 	}
 	if (_r) 
 		pg_free_result(_r);
