@@ -1,5 +1,5 @@
 /*
- * $Id: notify.c 3843 2008-03-04 15:24:27Z anca_vamanu $
+ * $Id: notify.c 4189 2008-05-15 10:38:46Z anca_vamanu $
  *
  * presence module- presence server implementation
  *
@@ -158,15 +158,7 @@ int build_str_hdr(subs_t* subs, int is_body, str** hdr)
 	strcpy(str_hdr->s+str_hdr->len, status);
 	str_hdr->len+= strlen(status);
 	
-	if(subs->expires <= 0)
-	{
-		expires_t = 0;
-		subs->status= TERMINATED_STATUS;
-		subs->reason.s= "timeout";
-		subs->reason.len= 7;
-	}
-	else
-		expires_t= subs->expires;
+	expires_t= subs->expires;
 	
 	if(subs->status== TERMINATED_STATUS)
 	{
@@ -409,7 +401,8 @@ str* get_wi_notify_body(subs_t* subs, subs_t* watcher_subs)
 		}
 	}
 
-	hash_code= core_hash(&subs->pres_uri, &subs->event->name, shtable_size);
+	hash_code= core_hash(&subs->pres_uri, &subs->event->wipeer->name,
+            shtable_size);
 	lock_get(&subs_htable[hash_code].lock);
 
 	s= subs_htable[hash_code].entries;
@@ -1378,8 +1371,9 @@ int send_notify_request(subs_t* subs, subs_t * watcher_subs,
 	
 jump_over_body:
 
-	if(subs->expires== 0)
+	if(subs->expires<= 0)
 	{
+        subs->expires = 0;
 		subs->status= TERMINATED_STATUS;
 		subs->reason.s= "timeout";
 		subs->reason.len= 7;
@@ -1483,7 +1477,7 @@ error:
 int notify(subs_t* subs, subs_t * watcher_subs,str* n_body,int force_null_body)
 {
 	/* update first in hash table and the send Notify */
-	if(subs->expires!= 0)
+	if(subs->expires!= 0 && subs->status != TERMINATED_STATUS)
 	{
 		unsigned int hash_code;
 		hash_code= core_hash(&subs->pres_uri, &subs->event->name, shtable_size);
@@ -1506,8 +1500,11 @@ int notify(subs_t* subs, subs_t * watcher_subs,str* n_body,int force_null_body)
 			}
 		}
 	}
+    if(subs->reason.s && subs->status== ACTIVE_STATUS &&
+         subs->reason.len== 12 && strncmp(subs->reason.s, "polite-block", 12)== 0)
+	    force_null_body = 1;
 
-	if(send_notify_request(subs, watcher_subs, n_body, force_null_body)< 0)
+    if(send_notify_request(subs, watcher_subs, n_body, force_null_body)< 0)
 	{
 		LM_ERR("sending Notify not successful\n");
 		return -1;
