@@ -1,5 +1,5 @@
 /*
- * $Id: presence.c 3843 2008-03-04 15:24:27Z anca_vamanu $
+ * $Id: presence.c 4182 2008-05-14 10:11:36Z anca_vamanu $
  *
  * presence module - presence server implementation
  *
@@ -582,6 +582,51 @@ error:
 	return 0;
 }
 
+int pres_db_delete_status(subs_t* s)
+{
+    int n_query_cols= 0;
+    db_key_t query_cols[5];
+    db_val_t query_vals[5];
+
+    if (pa_dbf.use_table(pa_db, active_watchers_table) < 0) 
+    {
+        LM_ERR("sql use table failed\n");
+        return -1;
+    }
+
+    query_cols[n_query_cols]= "event";
+    query_vals[n_query_cols].nul= 0;
+    query_vals[n_query_cols].type= DB_STR;
+    query_vals[n_query_cols].val.str_val= s->event->name ;
+    n_query_cols++;
+
+    query_cols[n_query_cols]= "presentity_uri";
+    query_vals[n_query_cols].nul= 0;
+    query_vals[n_query_cols].type= DB_STR;
+    query_vals[n_query_cols].val.str_val= s->pres_uri;
+    n_query_cols++;
+
+    query_cols[n_query_cols]= "watcher_username";
+    query_vals[n_query_cols].nul= 0;
+    query_vals[n_query_cols].type= DB_STR;
+    query_vals[n_query_cols].val.str_val= s->from_user;
+    n_query_cols++;
+
+    query_cols[n_query_cols]= "watcher_domain";
+    query_vals[n_query_cols].nul= 0;
+    query_vals[n_query_cols].type= DB_STR;
+    query_vals[n_query_cols].val.str_val= s->from_domain;
+    n_query_cols++;
+
+    if(pa_dbf.delete(pa_db, query_cols, 0, query_vals, n_query_cols)< 0)
+    {
+        LM_ERR("sql delete failed\n");
+        return -1;
+    }
+    return 0;
+
+}
+
 int update_watchers_status(str pres_uri, pres_ev_t* ev, str* rules_doc)
 {
 	subs_t subs;
@@ -757,7 +802,17 @@ int update_watchers_status(str pres_uri, pres_ev_t* ev, str* rules_doc)
 			LM_ERR( "sending Notify request\n");
 			goto done;
 		}
-		s= s->next;
+        /* delete terminated dialogs from database */
+        if(s->status== TERMINATED_STATUS)
+        {
+            if(pres_db_delete_status(s)<0)
+            {
+                err_ret= -1;
+                LM_ERR("failed to delete terminated dialog from database\n");
+                goto done;
+            }
+        }
+        s= s->next;
 	}
 	
 	free_subs_list(subs_array, PKG_MEM_TYPE, 0);
@@ -804,17 +859,17 @@ int update_pw_dialogs(subs_t* subs, unsigned int hash_code, subs_t** subs_array)
 			(*subs_array)= cs;
 			if(s->status== TERMINATED_STATUS)
 			{
-				cs->expires= 0;
 				ps->next= s->next;
-				shm_free(s);
+				shm_free(s->contact.s);
+                shm_free(s);
 			}
 			else
-				ps= ps->next;
+				ps= s;
 
 			printf_subs(cs);
 		}
 		else
-			ps= ps->next;
+			ps= s;
 	}
 	LM_DBG("found %d matching dialogs\n", i);
 
