@@ -1,5 +1,5 @@
 /*
- * $Id: subscribe.c 3503 2008-01-06 11:39:14Z anca_vamanu $
+ * $Id: subscribe.c 3950 2008-03-25 15:02:50Z anca_vamanu $
  *
  * presence module - presence server implementation
  *
@@ -538,18 +538,21 @@ int handle_subscribe(struct sip_msg* msg, char* str1, char* str2)
 
 	/* getting presentity uri from Request-URI if initial subscribe - or else from database*/
 	if(to_tag_gen)
-	{
-		if( parse_sip_msg_uri(msg)< 0)
+	{	
+		if(parsed_event->parsed!= EVENT_DIALOG)
 		{
-			LM_ERR("failed to parse R-URI\n");
-			return -1;
-		}
+			if( parse_sip_msg_uri(msg)< 0)
+			{
+				LM_ERR("failed to parse R-URI\n");
+				return -1;
+			}
 
-		if(uandd_to_uri(msg->parsed_uri.user, msg->parsed_uri.host,
-					&subs.pres_uri)< 0)
-		{
-			LM_ERR("failed to construct uri from user and domain\n");
-			goto error;
+			if(uandd_to_uri(msg->parsed_uri.user, msg->parsed_uri.host,
+						&subs.pres_uri)< 0)
+			{
+				LM_ERR("failed to construct uri from user and domain\n");
+				goto error;
+			}
 		}
 	}
 	else
@@ -649,7 +652,7 @@ int handle_subscribe(struct sip_msg* msg, char* str1, char* str2)
 	if(reason.s)
 		pkg_free(reason.s);
 	
-	if(subs.pres_uri.s)
+	if(parsed_event->parsed!= EVENT_DIALOG && subs.pres_uri.s)
 		pkg_free(subs.pres_uri.s);
 	
 	if((!server_address.s) || (server_address.len== 0))
@@ -681,7 +684,7 @@ error:
 		}
 	}
 
-	if(subs.pres_uri.s)
+	if(parsed_event->parsed!= EVENT_DIALOG &&subs.pres_uri.s)
 		pkg_free(subs.pres_uri.s);
 	
 	if(subs.auth_rules_doc)
@@ -813,6 +816,10 @@ int extract_sdialog_info(subs_t* subs,struct sip_msg* msg, int mexp, int* to_tag
 		subs->from_domain = uri.host;
 	}
 
+	if(subs->event->evp->parsed== EVENT_DIALOG)
+	{
+		subs->pres_uri= pfrom->uri;
+	}
 
 	/*generate to_tag if the message does not have a to_tag*/
 	if (pto->tag_value.s==NULL || pto->tag_value.len==0 )
@@ -958,7 +965,7 @@ int get_stored_info(struct sip_msg* msg, subs_t* subs, int* reply_code,
 	{
 		lock_get(&subs_htable[i].lock);
 		s= search_shtable(subs_htable, subs->callid,subs->to_tag,subs->from_tag, i);
-		if(s)
+		if(s && s->event->evp->parsed!= EVENT_DIALOG)
 		{
 			pres_uri.s= (char*)pkg_malloc(s->pres_uri.len* sizeof(char));
 			if(pres_uri.s== NULL)
@@ -987,7 +994,10 @@ int get_stored_info(struct sip_msg* msg, subs_t* subs, int* reply_code,
 found_rec:
 	
 	LM_DBG("Record found in hash_table\n");
-	subs->pres_uri= pres_uri;
+	
+	if(s->event->evp->parsed!= EVENT_DIALOG)
+		subs->pres_uri= pres_uri;
+	
 	subs->status= s->status;
 	if(s->reason.s && s->reason.len)
 	{	
@@ -1183,17 +1193,20 @@ int get_database_info(struct sip_msg* msg, subs_t* subs, int* reply_code, str* r
 
 	subs->local_cseq= row_vals[local_cseq_col].val.int_val;
 	
-	pres_uri.s= (char*)row_vals[pres_uri_col].val.string_val;
-	pres_uri.len= strlen(pres_uri.s);
-	subs->pres_uri.s= (char*)pkg_malloc(pres_uri.len* sizeof(char));
-	if(subs->pres_uri.s== NULL)
+	if(subs->event->evp->parsed!= EVENT_DIALOG)
 	{
-		if(subs->reason.s)
-			pkg_free(subs->reason.s);
-		ERR_MEM(PKG_MEM_STR);
+		pres_uri.s= (char*)row_vals[pres_uri_col].val.string_val;
+		pres_uri.len= strlen(pres_uri.s);
+		subs->pres_uri.s= (char*)pkg_malloc(pres_uri.len* sizeof(char));
+		if(subs->pres_uri.s== NULL)
+		{	
+			if(subs->reason.s)
+				pkg_free(subs->reason.s);
+			ERR_MEM(PKG_MEM_STR);
+		}
+		memcpy(subs->pres_uri.s, pres_uri.s, pres_uri.len);
+		subs->pres_uri.len= pres_uri.len;
 	}
-	memcpy(subs->pres_uri.s, pres_uri.s, pres_uri.len);
-	subs->pres_uri.len= pres_uri.len;
 
 	record_route.s= (char*)row_vals[record_route_col].val.string_val;
 	if(record_route.s)
@@ -1208,7 +1221,6 @@ int get_database_info(struct sip_msg* msg, subs_t* subs, int* reply_code, str* r
 		subs->record_route.len= record_route.len;
 	}
 
-
 	pa_dbf.free_result(pa_db, result);
 	result= NULL;
 
