@@ -1,5 +1,5 @@
 /*
- * $Id: pua_bind.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: pua_bind.c 2701 2007-09-03 13:47:19Z anca_vamanu $
  *
  * pua module - presence user agent module
  *
@@ -25,31 +25,22 @@
 
 #include "pua_bind.h"
 #include "../../dprint.h"
-#include "../../sr_module.h"
-
+#include "hash.h"
 
 int bind_pua(pua_api_t* api)
 {
 	if (!api) {
-		LOG(L_ERR, "PUA:bind_pua: Invalid parameter value\n");
+		LM_ERR("Invalid parameter value\n");
 		return -1;
 	}
 
-	api->send_publish = (send_publish_t )find_export
-		("send_publish", 1, 0);
-	if (api->send_publish == 0)
-	{
-		LOG(L_ERR, "PUA:bind_pua: Can't bind send_publish\n");
-		return -1;
-	}
+	api->send_publish   =  send_publish; 
+	api->send_subscribe =  send_subscribe;
+	api->register_puacb =  register_puacb;
+	api->is_dialog      =  is_dialog;
+	api->get_record_id  =  get_record_id;
+	api->add_event      =  add_pua_event;
 
-	api->send_subscribe = ( send_subscribe_t)find_export
-		("send_subscribe", 1, 0);
-	if (api->send_subscribe == 0)
-	{
-		LOG(L_ERR, "PUA:bind_pua: Can't bind send_subscribe\n");
-		return -1;
-	}
 	return 0;
 }
 
