@@ -1,5 +1,5 @@
 /*
- * $Id: pua_bind.h 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: pua_bind.h 2542 2007-07-30 09:22:56Z anca_vamanu $
  *
  * pua module - presence user agent module
  *
@@ -27,10 +27,16 @@
 #define PUA_API_H
 #include "send_subscribe.h"
 #include "send_publish.h"
+#include "pua_callback.h"
+#include "hash.h"
 
 typedef struct pua_api {
 	send_subscribe_t send_subscribe;
 	send_publish_t send_publish;
+	register_puacb_t register_puacb;
+	query_dialog_t is_dialog;
+	get_record_id_t get_record_id;
+	add_pua_event_t add_event;
 } pua_api_t;
 
 typedef int (*bind_pua_t)(pua_api_t* api);
