@@ -1,5 +1,5 @@
 /*
- * $Id: send_subscribe.c 3793 2008-02-28 17:12:21Z anca_vamanu $
+ * $Id: send_subscribe.c 4065 2008-04-21 17:06:17Z anca_vamanu $
  *
  * pua module - presence user agent module
  *
@@ -260,9 +260,9 @@ void subs_cback_func(struct cell *t, int cb_type, struct tmcb_params *ps)
 			lock_release(&HashT->p_records[hash_code].lock);
 			goto done;
 		}
+
 		delete_htable(presentity, hash_code);
 		lock_release(&HashT->p_records[hash_code].lock);
-
 		goto done;
 	}
 
@@ -388,6 +388,7 @@ void subs_cback_func(struct cell *t, int cb_type, struct tmcb_params *ps)
 			subs.outbound_proxy= hentity->outbound_proxy;
 			subs.extra_headers= hentity->extra_headers;
 			subs.cb_param= hentity->cb_param;
+		
 			if(send_subscribe(&subs)< 0)
 			{
 				LM_ERR("when trying to send SUBSCRIBE\n");
@@ -417,6 +418,7 @@ void subs_cback_func(struct cell *t, int cb_type, struct tmcb_params *ps)
 		lock_release(&HashT->p_records[hash_code].lock);
 		goto done;
 	}
+
 	lock_release(&HashT->p_records[hash_code].lock);
 
 	/* if a new dialog -> insert */
@@ -820,7 +822,7 @@ insert:
 
 		result= tmb.t_request
 			(&met,						  /* Type of the message */
-			subs->pres_uri,				  /* Request-URI */
+		subs->remote_target?subs->remote_target:subs->pres_uri,/* Request-URI*/
 			subs->pres_uri,				  /* To */
 			subs->watcher_uri,			  /* From */
 			str_hdr,					  /* Optional headers including CRLF */
@@ -839,8 +841,10 @@ insert:
 	else
 	{
 		/* tackle the case in which the desired_expires== 0 and subs->expires< 0*/
-		if(presentity->desired_expires== 0)
+		/*
+        if(presentity->desired_expires== 0)
 		{
+			
 			if(subs->expires< 0)
 			{
 				LM_DBG("Found previous request for unlimited subscribe-"
@@ -853,10 +857,11 @@ insert:
 				goto done;
 			}
 		
+
 			if(subs->event & PWINFO_EVENT)
 			{	
-				if(subs->expires== 0) /* request for unsubscribe*/
-				{
+				if(subs->expires== 0) 
+                {
 					presentity->watcher_count--;
 					if(	presentity->watcher_count> 0)
 					{
@@ -866,16 +871,17 @@ insert:
 				}
 				else
 				{
-					presentity->watcher_count--;
+					presentity->watcher_count++;
 					if(presentity->watcher_count> 1)
 					{
 						lock_release(&HashT->p_records[hash_code].lock);
 						goto done;
 					}
 				}
-			}	
+			}
 			
-		}	
+		}
+        */
 
 		dlg_t* td= NULL;
 		td= pua_build_dlg_t(presentity);
