@@ -1,5 +1,5 @@
 /*
- * $Id: pua_usrloc.h 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: pua_usrloc.h 2589 2007-08-10 12:27:09Z anca_vamanu $
  *
  * pua_urloc module - usrloc pua module
  *
@@ -29,8 +29,9 @@
 send_publish_t pua_send_publish;
 send_subscribe_t pua_send_subscribe;
 void ul_publish(ucontact_t* c, int type, void* param);
+int pua_unset_publish(struct sip_msg* msg , void* param);
 
 extern int pua_ul_publish; 
 extern str default_domain;
-
+extern str pres_prefix;
 #endif
