@@ -1,5 +1,5 @@
 /*
- * $Id: path.h 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: path.h 2757 2007-09-13 15:51:40Z bogdan_iancu $
  *
  * Helper functions for Path support.
  *
@@ -35,9 +35,4 @@
  */
 int build_path_vector(struct sip_msg *_m, str *path, str *received);
 
-/*
- * If Path is available, sets _dst to uri of first element .
- */
-int get_path_dst_uri(str *_p, str *_dst);
-
 #endif /* REG_PATH_H */
