@@ -1,4 +1,4 @@
-/* $Id: seas.c 3102 2007-11-12 14:40:33Z bogdan_iancu $
+/* $Id: seas.c 4020 2008-04-16 15:43:25Z eliasbaixas $
  *
  * Copyright (C) 2006-2007 VozTelecom Sistemas S.L
  *
@@ -232,8 +232,12 @@ static int w_as_relay_t(struct sip_msg *msg, char *entry, char *foo)
    char *buffer,processor_id;
    struct cell *mycel;
    struct as_entry *as;
-   static str msg100={"Your call is important to us",sizeof("Your call is important to us")-1};
-   static str msg500={"Server Internal Error!",sizeof("Server Internal Error!")-1};
+#define SEAS_YOURCALL "Your call is important to us"
+#define SEAS_YOURCALL_L (sizeof(SEAS_YOURCALL)-1)
+#define SEAS_ERROR_MSG "Server Internal Error!"
+#define SEAS_ERROR_MSG_L (sizeof(SEAS_ERROR_MSG)-1)
+   static str msg100={SEAS_YOURCALL ,SEAS_YOURCALL_L};
+   static str msg500={SEAS_ERROR_MSG,SEAS_ERROR_MSG_L};
 
    buffer=(char*)0;
    my_as_ev=(as_msg_p)0;
@@ -458,7 +462,7 @@ char * create_as_event_t(struct cell *t,struct sip_msg *msg,char processor_id,in
 	 goto error;
       }else{
 	 flags|=CANCEL_FOUND;
-	 seas_f.tmb.unref_cell(originalT);
+	 //seas_f.tmb.unref_cell(originalT);
       }
       LM_DBG("Cancelling transaction !!\n");
    }
