@@ -1,5 +1,5 @@
 /* 
- * $Id: siptrace.c 2124 2007-05-03 10:37:47Z miconda $ 
+ * $Id: siptrace.c 3292 2007-12-10 10:01:24Z miconda $ 
  *
  * siptrace module - helper module to trace sip messages
  *
@@ -38,7 +38,7 @@
 #include "../../db/db.h"
 #include "../../parser/parse_content.h"
 #include "../../parser/parse_from.h"
-#include "../../items.h"
+#include "../../pvar.h"
 #include "../tm/tm_load.h"
 #include "../sl/sl_cb.h"
 
@@ -58,8 +58,11 @@ static void trace_onreq_in(struct cell* t, int type, struct tmcb_params *ps);
 static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps);
 static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps);
 static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps);
-static void trace_sl_onreply_out(struct sip_msg* req,
+static void trace_sl_onreply_out(unsigned int types, struct sip_msg* req,
 									struct sl_cb_param *sl_param);
+static void trace_sl_ack_in(unsigned int types, struct sip_msg* req,
+									struct sl_cb_param *sl_param);
+
 static struct mi_root* sip_trace_mi(struct mi_root* cmd, void* param );
 
 char* db_url       = DEFAULT_RODB_URL;
@@ -84,7 +87,7 @@ int   trace_on      = 0;
 str    dup_uri_str      = {0, 0};
 struct sip_uri *dup_uri = 0;
 
-int   *trace_on_flag = NULL;      
+int   *trace_on_flag = NULL;
 
 static unsigned short traced_user_avp_type;
 static int_str traced_user_avp;
@@ -107,9 +110,9 @@ register_slcb_t register_slcb_f=NULL;
  * Exported functions
  */
 static cmd_export_t cmds[] = {
-	{"sip_trace", sip_trace, 0, 0,
+	{"sip_trace", sip_trace, 0, 0, 0,
 		REQUEST_ROUTE|FAILURE_ROUTE|ONREPLY_ROUTE|BRANCH_ROUTE},
-	{0, 0, 0, 0, 0}
+	{0, 0, 0, 0, 0, 0}
 };
 
 
@@ -170,6 +173,7 @@ struct module_exports exports = {
 #endif
 	mi_cmds,    /* exported MI functions */
 	0,          /* exported pseudo-variables */
+	0,          /* extra processes */
 	mod_init,   /* module initialization function */
 	0,          /* response function */
 	destroy,    /* destroy function */
@@ -179,14 +183,14 @@ struct module_exports exports = {
 
 static int mod_init(void)
 {
-	xl_spec_t avp_spec;
+	pv_spec_t avp_spec;
+	str s;
 
-	DBG("siptrace - initializing\n");
+	LM_INFO("initializing...\n");
 	
 	if(siptrace_table==0 || strlen(siptrace_table)<=0)
 	{
-		LOG(L_ERR,
-			"siptrace:mod_init:ERROR: invalid table name\n");
+		LM_ERR("invalid table name\n");
 		return -1;
 	}
 
@@ -196,20 +200,19 @@ static int mod_init(void)
 	/* Find a database module */
 	if (bind_dbmod(db_url, &db_funcs))
 	{
-		LOG(L_ERR, "siptrace:mod_init: Unable to bind database module\n");
+		LM_ERR("unable to bind database module\n");
 		return -1;
 	}
 	if (!DB_CAPABILITY(db_funcs, DB_CAP_INSERT))
 	{
-		LOG(L_ERR, "siptrace:mod_init: Database modules does not "
-			"provide all functions needed by module\n");
+		LM_ERR("database modules does not provide all functions needed by module\n");
 		return -1;
 	}
 
 	trace_on_flag = (int*)shm_malloc(sizeof(int));
 	if(trace_on_flag==NULL)
 	{
-		LOG(L_ERR, "siptrace:mod_init:ERROR: no more shm\n");
+		LM_ERR("no more shm memory left\n");
 		return -1;
 	}
 	
@@ -218,13 +221,13 @@ static int mod_init(void)
 	/* register callbacks to TM */
 	if (load_tm_api(&tmb)!=0)
 	{
-		LOG(L_ERR, "siptrace:mod_init:ERROR: can't load tm api\n");
+		LM_ERR("can't load tm api\n");
 		return -1;
 	}
 
 	if(tmb.register_tmcb( 0, 0, TMCB_REQUEST_IN, trace_onreq_in, 0) <=0)
 	{
-		LOG(L_ERR,"siptrace:mod_init:ERROR: can't register trace_onreq_in\n");
+		LM_ERR("can't register trace_onreq_in\n");
 		return -1;
 	}
 
@@ -232,13 +235,17 @@ static int mod_init(void)
 	register_slcb_f = (register_slcb_t)find_export("register_slcb", 0, 0);
 	if(register_slcb_f==NULL)
 	{
-		LOG(L_ERR, "siptrace:mod_init:ERROR: can't load sl api\n");
+		LM_ERR("can't load sl api\n");
 		return -1;
 	}
-	if(register_slcb_f(trace_sl_onreply_out, NULL)!=0)
+	if(register_slcb_f(SLCB_REPLY_OUT,trace_sl_onreply_out, NULL)!=0)
 	{
-		LOG(L_ERR,
-			"siptrace:mod_init:ERROR: can't register trace_sl_onreply_out\n");
+		LM_ERR("can't register trace_sl_onreply_out\n");
+		return -1;
+	}
+	if(register_slcb_f(SLCB_ACK_IN,trace_sl_ack_in, NULL)!=0)
+	{
+		LM_ERR("can't register trace_sl_ack_in\n");
 		return -1;
 	}
 
@@ -246,31 +253,29 @@ static int mod_init(void)
 		dup_uri_str.len = strlen(dup_uri_str.s);
 		dup_uri = (struct sip_uri *)pkg_malloc(sizeof(struct sip_uri));
 		if(dup_uri==0) {
-			LOG(L_ERR, "siptrace:mod_init:ERROR: no more pkg memory\n");
+			LM_ERR("no more pkg memory left\n");
 			return -1;
 		}
 		memset(dup_uri, 0, sizeof(struct sip_uri));
 		if(parse_uri(dup_uri_str.s, dup_uri_str.len, dup_uri)<0) {
-			LOG(L_ERR, "siptrace:mod_init:ERROR: bad dup uri\n");
+			LM_ERR("bad dup uri\n");
 			return -1;
 		}
 	}
 
 	if(traced_user_avp_str && *traced_user_avp_str)
 	{
-		if (xl_parse_spec(traced_user_avp_str, &avp_spec,
-					XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS)==0
-				|| avp_spec.type!=XL_AVP) {
-			LOG(L_ERR, "ERROR:siptrace:mod_init: malformed or non AVP %s "
-				"AVP definition\n", traced_user_avp_str);
+		s.s = traced_user_avp_str; s.len = strlen(s.s);
+		if (pv_parse_spec(&s, &avp_spec)==0
+				|| avp_spec.type!=PVT_AVP) {
+			LM_ERR("malformed or non AVP %s AVP definition\n", traced_user_avp_str);
 			return -1;
 		}
 
-		if(xl_get_avp_name(0, &avp_spec, &traced_user_avp,
+		if(pv_get_avp_name(0, &avp_spec.pvp, &traced_user_avp,
 					&traced_user_avp_type)!=0)
 		{
-			LOG(L_ERR, "ERROR:siptrace:mod_init: [%s]- invalid "
-				"AVP definition\n", traced_user_avp_str);
+			LM_ERR("[%s] - invalid AVP definition\n", traced_user_avp_str);
 			return -1;
 		}
 	} else {
@@ -279,19 +284,17 @@ static int mod_init(void)
 	}
 	if(trace_table_avp_str && *trace_table_avp_str)
 	{
-		if (xl_parse_spec(trace_table_avp_str, &avp_spec,
-					XL_THROW_ERROR|XL_DISABLE_MULTI|XL_DISABLE_COLORS)==0
-				|| avp_spec.type!=XL_AVP) {
-			LOG(L_ERR, "ERROR:siptrace:mod_init: malformed or non AVP %s "
-				"AVP definition\n", trace_table_avp_str);
+		s.s = trace_table_avp_str; s.len = strlen(s.s);
+		if (pv_parse_spec(&s, &avp_spec)==0
+				|| avp_spec.type!=PVT_AVP) {
+			LM_ERR("malformed or non AVP %s AVP definition\n", trace_table_avp_str);
 			return -1;
 		}
 
-		if(xl_get_avp_name(0, &avp_spec, &trace_table_avp,
+		if(pv_get_avp_name(0, &avp_spec.pvp, &trace_table_avp,
 					&trace_table_avp_type)!=0)
 		{
-			LOG(L_ERR, "ERROR:siptrace:mod_init: [%s]- invalid "
-				"AVP definition\n", trace_table_avp_str);
+			LM_ERR("[%s] - invalid AVP definition\n", trace_table_avp_str);
 			return -1;
 		}
 	} else {
@@ -308,7 +311,7 @@ static int child_init(int rank)
 	db_con = db_funcs.init(db_url);
 	if (!db_con)
 	{
-		LOG(L_ERR, "siptrace:init_child: Unable to connect database\n");
+		LM_ERR("unable to connect database\n");
 		return -1;
 	}
 
@@ -324,7 +327,23 @@ static void destroy(void)
 		shm_free(trace_on_flag);
 }
 
-static inline char* siptrace_get_table()
+static inline int siptrace_copy_proto(int proto, char *buf)
+{
+	if(buf==0)
+		return -1;
+	if(proto==PROTO_TCP) {
+		strcpy(buf, "tcp:");
+	} else if(proto==PROTO_TLS) {
+		strcpy(buf, "tls:");
+	} else if(proto==PROTO_SCTP) {
+		strcpy(buf, "sctp:");
+	} else {
+		strcpy(buf, "udp:");
+	}
+	return 0;
+}
+
+static inline char* siptrace_get_table(void)
 {
 	int_str        avp_value;
 	struct usr_avp *avp;
@@ -354,7 +373,7 @@ static int sip_trace(struct sip_msg *msg, char *s1, char *s2)
 	
 	if(msg==NULL)
 	{
-		DBG("sip_trace: no uas request, local transaction\n");
+		LM_DBG("no uas request, local transaction\n");
 		return -1;
 	}
 
@@ -365,20 +384,20 @@ static int sip_trace(struct sip_msg *msg, char *s1, char *s2)
 
 	if((avp==NULL) && (trace_on_flag==NULL || *trace_on_flag==0))
 	{
-		DBG("sip_trace: trace off...\n");
+		LM_DBG("trace off...\n");
 		return -1;
 	}
 	
 	if(parse_from_header(msg)==-1 || msg->from==NULL || get_from(msg)==NULL)
 	{
-		LOG(L_ERR, "sip_trace: ERROR cannot parse FROM header\n");
+		LM_ERR("cannot parse FROM header\n");
 		goto error;
 	}
 	
 	if(parse_headers(msg, HDR_CALLID_F, 0)!=0 || msg->callid==NULL
 			|| msg->callid->body.s==NULL)
 	{
-		LOG(L_ERR, "sip_trace: ERROR cannot parse call-id\n");
+		LM_ERR("cannot parse call-id\n");
 		goto error;
 	}
 
@@ -421,7 +440,8 @@ static int sip_trace(struct sip_msg *msg, char *s1, char *s2)
 	db_keys[4] = fromip_column;
 	db_vals[4].type = DB_STRING;
 	db_vals[4].nul = 0;
-	strcpy(fromip_buff, ip_addr2a(&msg->rcv.src_ip));
+	siptrace_copy_proto(msg->rcv.proto, fromip_buff);
+	strcat(fromip_buff, ip_addr2a(&msg->rcv.src_ip));
 	strcat(fromip_buff,":");
 	strcat(fromip_buff, int2str(msg->rcv.src_port, NULL));
 	db_vals[4].val.string_val = fromip_buff;
@@ -430,22 +450,16 @@ static int sip_trace(struct sip_msg *msg, char *s1, char *s2)
 	db_vals[5].type = DB_STRING;
 	db_vals[5].nul = 0;
 	// db_vals[5].val.string_val = ip_addr2a(&msg->rcv.dst_ip);;
-	if(msg->rcv.proto==PROTO_TCP) {
-		memcpy(toip_buff, "tcp:", 4);
-	} else if(msg->rcv.proto==PROTO_TLS) {
-		memcpy(toip_buff, "tls:", 4);
-	} else {
-		memcpy(toip_buff, "udp:", 4);
-	}
-	strcpy(toip_buff+4, ip_addr2a(&msg->rcv.dst_ip));
-	strcat(toip_buff+4,":");
-	strcat(toip_buff+4, int2str(msg->rcv.dst_port, NULL));
+	siptrace_copy_proto(msg->rcv.proto, toip_buff);
+	strcat(toip_buff, ip_addr2a(&msg->rcv.dst_ip));
+	strcat(toip_buff,":");
+	strcat(toip_buff, int2str(msg->rcv.dst_port, NULL));
 	db_vals[5].val.string_val = toip_buff;
 	
 	db_keys[6] = date_column;
 	db_vals[6].type = DB_DATETIME;
 	db_vals[6].nul = 0;
-	db_vals[6].val.int_val = time(NULL);
+	db_vals[6].val.time_val = time(NULL);
 	
 	db_keys[7] = direction_column;
 	db_vals[7].type = DB_STRING;
@@ -468,14 +482,19 @@ static int sip_trace(struct sip_msg *msg, char *s1, char *s2)
 		db_vals[9].val.str_val.s   = "";
 		db_vals[9].val.str_val.len = 0;
 	
-		DBG("sip_trace: storing info...\n");
+		LM_DBG("storing info...\n");
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "sip_trace: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
 #ifdef STATISTICS
-		update_stat(siptrace_req, 1);
+		if(msg->first_line.type==SIP_REPLY)
+		{
+			update_stat(siptrace_rpl, 1);
+		} else {
+			update_stat(siptrace_req, 1);
+		}
 #endif
 	}
 	
@@ -488,10 +507,10 @@ static int sip_trace(struct sip_msg *msg, char *s1, char *s2)
 	db_vals[9].val.str_val.s = avp_value.s.s;
 	db_vals[9].val.str_val.len = avp_value.s.len;
 
-	DBG("sip_trace: storing info...\n");
+	LM_DBG("storing info...\n");
 	if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 	{
-		LOG(L_ERR, "sip_trace: error storing trace\n");
+		LM_ERR("error storing trace\n");
 		goto error;
 	}
 
@@ -501,10 +520,10 @@ static int sip_trace(struct sip_msg *msg, char *s1, char *s2)
 		db_vals[9].val.str_val.s = avp_value.s.s;
 		db_vals[9].val.str_val.len = avp_value.s.len;
 
-		DBG("sip_trace: storing info...\n");
+		LM_DBG("storing info...\n");
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "sip_trace: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
 		avp = search_next_avp( avp, &avp_value);
@@ -528,14 +547,14 @@ static void trace_onreq_in(struct cell* t, int type, struct tmcb_params *ps)
 
 	if(t==NULL || ps==NULL)
 	{
-		DBG("trace_onreq_in: no uas request, local transaction\n");
+		LM_DBG("no uas request, local transaction\n");
 		return;
 	}
 	
 	msg = ps->req;
 	if(msg==NULL)
 	{
-		DBG("trace_onreq_in: no uas request, local transaction\n");
+		LM_DBG("no uas request, local transaction\n");
 		return;
 	}
 	
@@ -546,44 +565,42 @@ static void trace_onreq_in(struct cell* t, int type, struct tmcb_params *ps)
 
 	if((avp==NULL) && trace_is_off(msg))
 	{
-		DBG("trace_onreq_in: trace off...\n");
+		LM_DBG("trace off...\n");
 		return;
 	}
 	
 	if(parse_from_header(msg)==-1 || msg->from==NULL || get_from(msg)==NULL)
 	{
-		LOG(L_ERR, "trace_onreq_in: ERROR cannot parse FROM header\n");
+		LM_ERR("cannot parse FROM header\n");
 		return;
 	}
 
 	if(parse_headers(msg, HDR_CALLID_F, 0)!=0)
 	{
-		LOG(L_ERR, "trace_onreq_in: ERROR cannot parse call-id\n");
+		LM_ERR("cannot parse call-id\n");
 		return;
 	}
 
 	if (msg->REQ_METHOD==METHOD_INVITE)
 	{
-		DBG("trace_onreq_in: noisy_timer set for tracing\n");
+		LM_DBG("noisy_timer set for tracing\n");
 		t->flags |= T_NOISY_CTIMER_FLAG;
 	}
 	if(tmb.register_tmcb( 0, t, TMCB_REQUEST_BUILT, trace_onreq_out, 0) <=0)
 	{
-		LOG(L_ERR,"trace_onreq_in:ERROR: can't register trace_onreq_out\n");
+		LM_ERR("can't register trace_onreq_out\n");
 		return;
 	}
 
 	if(tmb.register_tmcb( 0, t, TMCB_RESPONSE_IN, trace_onreply_in, 0) <=0)
 	{
-		LOG(L_ERR,
-			"trace_onreq_in:ERROR: can't register trace_onreply_in\n");
+		LM_ERR("can't register trace_onreply_in\n");
 		return;
 	}
 
 	if(tmb.register_tmcb( 0, t, TMCB_RESPONSE_OUT, trace_onreply_out, 0) <=0)
 	{
-		LOG(L_ERR,
-			"trace_onreq_in:ERROR: can't register trace_onreply_out\n");
+		LM_ERR("can't register trace_onreply_out\n");
 		return;
 	}
 }
@@ -604,13 +621,13 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 	
 	if(t==NULL || ps==NULL)
 	{
-		DBG("trace_onreq_out: no uas request, local transaction\n");
+		LM_DBG("no uas request, local transaction\n");
 		return;
 	}
 	msg=ps->req;
 	if(msg==NULL)
 	{
-		DBG("trace_onreq_out: no uas msg, local transaction\n");
+		LM_DBG("no uas msg, local transaction\n");
 		return;
 	}
 	
@@ -621,19 +638,19 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 
 	if((avp==NULL) && trace_is_off(msg) )
 	{
-		DBG("trace_onreq_out: trace off...\n");
+		LM_DBG("trace off...\n");
 		return;
 	}
 
 	if(parse_from_header(msg)==-1 || msg->from==NULL || get_from(msg)==NULL)
 	{
-		LOG(L_ERR, "trace_onreq_out: ERROR cannot parse FROM header\n");
+		LM_ERR("cannot parse FROM header\n");
 		goto error;
 	}
 
 	if(parse_headers(msg, HDR_CALLID_F, 0)!=0)
 	{
-		LOG(L_ERR, "trace_onreq_out: ERROR cannot parse call-id\n");
+		LM_ERR("cannot parse call-id\n");
 		return;
 	}
 
@@ -653,7 +670,7 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 	/* check Call-ID header */
 	if(msg->callid==NULL || msg->callid->body.s==NULL)
 	{
-		LOG(L_ERR, "trace_onreq_out: ERROR cannot find Call-ID header!\n");
+		LM_ERR("cannot find Call-ID header!\n");
 		goto error;
 	}
 
@@ -682,47 +699,50 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 	db_vals[3].val.str_val.s = "";
 	db_vals[3].val.str_val.len = 0;
 		
+	memset(&to_ip, 0, sizeof(struct ip_addr));
+	dst = (struct dest_info*)ps->extra2;
+
 	db_keys[4] = fromip_column;
 	db_vals[4].type = DB_STRING;
 	db_vals[4].nul = 0;
 	if (trace_local_ip)
 		db_vals[4].val.string_val = trace_local_ip;
 	else {
-		strcpy(fromip_buff, ip_addr2a(&msg->rcv.dst_ip));
-		strcat(fromip_buff,":");
-		strcat(fromip_buff, int2str(msg->rcv.dst_port, NULL));
-		db_vals[4].val.string_val = fromip_buff;
+		if(dst==0 || dst->send_sock==0 || dst->send_sock->sock_str.s==0)
+		{
+			siptrace_copy_proto(msg->rcv.proto, fromip_buff);
+
+			strcat(fromip_buff, ip_addr2a(&msg->rcv.dst_ip));
+			strcat(fromip_buff,":");
+			strcat(fromip_buff, int2str(msg->rcv.dst_port, NULL));
+			db_vals[4].val.string_val = fromip_buff;
+		} else {
+			db_vals[4].type = DB_STR;
+			db_vals[4].val.str_val = dst->send_sock->sock_str;
+		}
 	}
 	
 	db_keys[5] = toip_column;
 	db_vals[5].type = DB_STRING;
 	db_vals[5].nul = 0;
-	memset(&to_ip, 0, sizeof(struct ip_addr));
-	dst = (struct dest_info*)ps->extra2;
 	if(dst==0)
 	{
-		db_vals[5].val.string_val = "255.255.255.255";
+		db_vals[5].val.string_val = "any:255.255.255.255";
 	} else {
 		su2ip_addr(&to_ip, &dst->to);
-		if(dst->proto==PROTO_TCP) {
-			memcpy(toip_buff, "tcp:", 4);
-		} else if(dst->proto==PROTO_TLS) {
-			memcpy(toip_buff, "tls:", 4);
-		} else {
-			memcpy(toip_buff, "udp:", 4);
-		}
-		strcpy(toip_buff+4, ip_addr2a(&to_ip));
-		strcat(toip_buff+4, ":");
-		strcat(toip_buff+4,
+		siptrace_copy_proto(dst->proto, toip_buff);
+		strcat(toip_buff, ip_addr2a(&to_ip));
+		strcat(toip_buff, ":");
+		strcat(toip_buff,
 				int2str((unsigned long)su_getport(&dst->to), &len));
-		DBG("siptrace:trace_onreq_out: dest [%s]\n", toip_buff);
+		LM_DBG("dest [%s]\n", toip_buff);
 		db_vals[5].val.string_val = toip_buff;
 	}
 	
 	db_keys[6] = date_column;
 	db_vals[6].type = DB_DATETIME;
 	db_vals[6].nul = 0;
-	db_vals[6].val.int_val = time(NULL);
+	db_vals[6].val.time_val = time(NULL);
 	
 	db_keys[7] = direction_column;
 	db_vals[7].type = DB_STRING;
@@ -745,10 +765,10 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 		db_vals[9].val.str_val.s   = "";
 		db_vals[9].val.str_val.len = 0;
 	
-		DBG("trace_onreq_out: storing info...\n");
+		LM_DBG("storing info...\n");
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "trace_onreq_out: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
 #ifdef STATISTICS
@@ -765,10 +785,10 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 	db_vals[9].val.str_val.s = avp_value.s.s;
 	db_vals[9].val.str_val.len = avp_value.s.len;
 
-	DBG("trace_onreq_out: storing info...\n");
+	LM_DBG("storing info...\n");
 	if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 	{
-		LOG(L_ERR, "trace_onreq_out: error storing trace\n");
+		LM_ERR("error storing trace\n");
 		goto error;
 	}
 
@@ -778,10 +798,10 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 		db_vals[9].val.str_val.s = avp_value.s.s;
 		db_vals[9].val.str_val.len = avp_value.s.len;
 
-		DBG("trace_onreq_out: storing info...\n");
+		LM_DBG("storing info...\n");
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "trace_onreq_out: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
 		avp = search_next_avp( avp, &avp_value);
@@ -808,7 +828,7 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 	
 	if(t==NULL || t->uas.request==0 || ps==NULL)
 	{
-		DBG("trace_onreply_in: no uas request, local transaction\n");
+		LM_DBG("no uas request, local transaction\n");
 		return;
 	}
 
@@ -816,7 +836,7 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 	msg = ps->rpl;
 	if(msg==NULL || req==NULL)
 	{
-		DBG("trace_onreply_in: no reply\n");
+		LM_DBG("no reply\n");
 		return;
 	}
 	
@@ -827,19 +847,19 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 
 	if((avp==NULL) &&  trace_is_off(req))
 	{
-		DBG("trace_onreply_in: trace off...\n");
+		LM_DBG("trace off...\n");
 		return;
 	}
 
 	if(parse_from_header(msg)==-1 || msg->from==NULL || get_from(msg)==NULL)
 	{
-		LOG(L_ERR, "trace_onreply_in: ERROR cannot parse FROM header\n");
+		LM_ERR("cannot parse FROM header\n");
 		goto error;
 	}
 
 	if(parse_headers(msg, HDR_CALLID_F, 0)!=0)
 	{
-		LOG(L_ERR, "trace_onreply_in: ERROR cannot parse call-id\n");
+		LM_ERR("cannot parse call-id\n");
 		return;
 	}
 
@@ -857,7 +877,7 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 	/* check Call-ID header */
 	if(msg->callid==NULL || msg->callid->body.s==NULL)
 	{
-		LOG(L_ERR, "trace_onreply_in: ERROR cannot find Call-ID header!\n");
+		LM_ERR("cannot find Call-ID header!\n");
 		goto error;
 	}
 
@@ -882,7 +902,8 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 	db_keys[4] = fromip_column;
 	db_vals[4].type = DB_STRING;
 	db_vals[4].nul = 0;
-	strcpy(fromip_buff, ip_addr2a(&msg->rcv.src_ip));
+	siptrace_copy_proto(msg->rcv.proto, fromip_buff);
+	strcat(fromip_buff, ip_addr2a(&msg->rcv.src_ip));
 	strcat(fromip_buff,":");
 	strcat(fromip_buff, int2str(msg->rcv.src_port, NULL));
 	db_vals[4].val.string_val = fromip_buff;
@@ -894,23 +915,17 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 	if(trace_local_ip)
 		db_vals[5].val.string_val = trace_local_ip;
 	else {
-		if(msg->rcv.proto==PROTO_TCP) {
-			memcpy(toip_buff, "tcp:", 4);
-		} else if(msg->rcv.proto==PROTO_TLS) {
-			memcpy(toip_buff, "tls:", 4);
-		} else {
-			memcpy(toip_buff, "udp:", 4);
-		}
-		strcpy(toip_buff+4, ip_addr2a(&msg->rcv.dst_ip));
-		strcat(toip_buff+4,":");
-		strcat(toip_buff+4, int2str(msg->rcv.dst_port, NULL));
+		siptrace_copy_proto(msg->rcv.proto, toip_buff);
+		strcat(toip_buff, ip_addr2a(&msg->rcv.dst_ip));
+		strcat(toip_buff,":");
+		strcat(toip_buff, int2str(msg->rcv.dst_port, NULL));
 		db_vals[5].val.string_val = toip_buff;
 	}
 	
 	db_keys[6] = date_column;
 	db_vals[6].type = DB_DATETIME;
 	db_vals[6].nul = 0;
-	db_vals[6].val.int_val = time(NULL);
+	db_vals[6].val.time_val = time(NULL);
 	
 	db_keys[7] = direction_column;
 	db_vals[7].type = DB_STRING;
@@ -933,12 +948,15 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 		db_vals[9].val.str_val.s   = "";
 		db_vals[9].val.str_val.len = 0;
 	
-		DBG("trace_onreply_in: storing info...\n");
+		LM_DBG("storing info...\n");
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "trace_onreply_out: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
+#ifdef STATISTICS
+		update_stat(siptrace_rpl, 1);
+#endif
 	}
 	
 	if(avp==NULL)
@@ -950,10 +968,10 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 	db_vals[9].val.str_val.s = avp_value.s.s;
 	db_vals[9].val.str_val.len = avp_value.s.len;
 
-	DBG("trace_onreply_in: storing info...\n");
+	LM_DBG("storing info...\n");
 	if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 	{
-		LOG(L_ERR, "trace_onreply_in: error storing trace\n");
+		LM_ERR("error storing trace\n");
 		goto error;
 	}
 
@@ -963,10 +981,10 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 		db_vals[9].val.str_val.s = avp_value.s.s;
 		db_vals[9].val.str_val.len = avp_value.s.len;
 
-		DBG("### - trace_onreply_in: storing info ...\n");
+		LM_DBG("storing info ...\n");
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "trace_onreply_in: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
 		avp = search_next_avp( avp, &avp_value);
@@ -997,7 +1015,7 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 
 	if (t==NULL || t->uas.request==0 || ps==NULL)
 	{
-		DBG("trace_onreply_out: no uas request, local transaction\n");
+		LM_DBG("no uas request, local transaction\n");
 		return;
 	}
 	
@@ -1008,7 +1026,7 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 
 	if((avp==NULL) &&  trace_is_off(t->uas.request))
 	{
-		DBG("trace_onreply_out: trace off...\n");
+		LM_DBG("trace off...\n");
 		return;
 	}
 	
@@ -1022,13 +1040,13 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 
 	if(parse_from_header(msg)==-1 || msg->from==NULL || get_from(msg)==NULL)
 	{
-		LOG(L_ERR, "trace_onreply_out: ERROR cannot parse FROM header\n");
+		LM_ERR("cannot parse FROM header\n");
 		goto error;
 	}
 
 	if(parse_headers(msg, HDR_CALLID_F, 0)!=0)
 	{
-		LOG(L_ERR, "trace_onreply_out: ERROR cannot parse call-id\n");
+		LM_ERR("cannot parse call-id\n");
 		return;
 	}
 
@@ -1067,7 +1085,7 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 	/* check Call-ID header */
 	if(msg->callid==NULL || msg->callid->body.s==NULL)
 	{
-		LOG(L_ERR, "trace_onreply_out: ERROR cannot find Call-ID header!\n");
+		LM_ERR("cannot find Call-ID header!\n");
 		goto error;
 	}
 
@@ -1090,7 +1108,9 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 	if(trace_local_ip)
 		db_vals[4].val.string_val = trace_local_ip;
 	else {
-		strcpy(fromip_buff, ip_addr2a(&req->rcv.dst_ip));
+		siptrace_copy_proto(msg->rcv.proto, fromip_buff);
+
+		strcat(fromip_buff, ip_addr2a(&req->rcv.dst_ip));
 		strcat(fromip_buff,":");
 		strcat(fromip_buff, int2str(req->rcv.dst_port, NULL));
 		db_vals[4].val.string_val = fromip_buff;
@@ -1109,28 +1129,22 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 	dst = (struct dest_info*)ps->extra2;
 	if(dst==0)
 	{
-		db_vals[5].val.string_val = "255.255.255.255";
+		db_vals[5].val.string_val = "any:255.255.255.255";
 	} else {
 		su2ip_addr(&to_ip, &dst->to);
-		if(dst->proto==PROTO_TCP) {
-			memcpy(toip_buff, "tcp:", 4);
-		} else if(dst->proto==PROTO_TLS) {
-			memcpy(toip_buff, "tls:", 4);
-		} else {
-			memcpy(toip_buff, "udp:", 4);
-		}
-		strcpy(toip_buff+4, ip_addr2a(&to_ip));
-		strcat(toip_buff+4, ":");
-		strcat(toip_buff+4,
+		siptrace_copy_proto(dst->proto, toip_buff);
+		strcat(toip_buff, ip_addr2a(&to_ip));
+		strcat(toip_buff, ":");
+		strcat(toip_buff,
 				int2str((unsigned long)su_getport(&dst->to), &len));
-		DBG("siptrace:trace_onreply_out: dest [%s]\n", toip_buff);
+		LM_DBG("dest [%s]\n", toip_buff);
 		db_vals[5].val.string_val = toip_buff;
 	}
 	
 	db_keys[6] = date_column;
 	db_vals[6].type = DB_DATETIME;
 	db_vals[6].nul = 0;
-	db_vals[6].val.int_val = time(NULL);
+	db_vals[6].val.time_val = time(NULL);
 	
 	db_keys[7] = direction_column;
 	db_vals[7].type = DB_STRING;
@@ -1153,12 +1167,15 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 		db_vals[9].val.str_val.s   = "";
 		db_vals[9].val.str_val.len = 0;
 	
-		DBG("trace_onreply_out: storing info...\n");
+		LM_DBG("storing info...\n");
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "trace_onreply_out: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
+#ifdef STATISTICS
+		update_stat(siptrace_rpl, 1);
+#endif
 	}
 	
 	if(avp==NULL)
@@ -1170,10 +1187,10 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 	db_vals[9].val.str_val.s = avp_value.s.s;
 	db_vals[9].val.str_val.len = avp_value.s.len;
 
-	DBG("trace_onreply_out: storing info...\n");
+	LM_DBG("storing info...\n");
 	if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 	{
-		LOG(L_ERR, "trace_onreply_out: error storing trace\n");
+		LM_ERR("error storing trace\n");
 		goto error;
 	}
 
@@ -1183,10 +1200,10 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 		db_vals[9].val.str_val.s = avp_value.s.s;
 		db_vals[9].val.str_val.len = avp_value.s.len;
 
-		DBG("### - trace_onreply_out: storing info (%d) ...\n", faked);
+		LM_DBG("### - storing info (%d) ...\n", faked);
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "trace_onreply_out: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
 		avp = search_next_avp( avp, &avp_value);
@@ -1198,7 +1215,14 @@ error:
 	return;
 }
 
-static void trace_sl_onreply_out(struct sip_msg* req,
+static void trace_sl_ack_in( unsigned int types, struct sip_msg* req,
+									struct sl_cb_param *sl_param)
+{
+	LM_DBG("storing ack...\n");
+	sip_trace(req, 0, 0);
+}
+
+static void trace_sl_onreply_out( unsigned int types, struct sip_msg* req,
 									struct sl_cb_param *sl_param)
 {
 	db_key_t db_keys[NR_KEYS];
@@ -1215,7 +1239,7 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 
 	if(req==NULL || sl_param==NULL)
 	{
-		LOG(L_ERR, "trace_sl_onreply_out: bad parameters\n");
+		LM_ERR("bad parameters\n");
 		goto error;
 	}
 	
@@ -1226,7 +1250,7 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 
 	if((avp==NULL) && trace_is_off(req))
 	{
-		DBG("trace_sl_onreply_out: trace off...\n");
+		LM_DBG("trace off...\n");
 		return;
 	}
 	
@@ -1235,13 +1259,13 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 	
 	if(parse_from_header(msg)==-1 || msg->from==NULL || get_from(msg)==NULL)
 	{
-		LOG(L_ERR, "trace_sl_onreply_out: ERROR cannot parse FROM header\n");
+		LM_ERR("cannot parse FROM header\n");
 		goto error;
 	}
 
 	if(parse_headers(msg, HDR_CALLID_F, 0)!=0)
 	{
-		LOG(L_ERR, "trace_sl_onreply_out: ERROR cannot parse call-id\n");
+		LM_ERR("cannot parse call-id\n");
 		return;
 	}
 
@@ -1254,7 +1278,7 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 	/* check Call-ID header */
 	if(msg->callid==NULL || msg->callid->body.s==NULL)
 	{
-		LOG(L_ERR, "trace_sl_onreply_out: ERROR cannot find Call-ID header!\n");
+		LM_ERR("cannot find Call-ID header!\n");
 		goto error;
 	}
 
@@ -1276,7 +1300,9 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 	if(trace_local_ip)
 		db_vals[4].val.string_val = trace_local_ip;
 	else {
-		strcpy(fromip_buff, ip_addr2a(&req->rcv.dst_ip));
+		siptrace_copy_proto(msg->rcv.proto, fromip_buff);
+
+		strcat(fromip_buff, ip_addr2a(&req->rcv.dst_ip));
 		strcat(fromip_buff,":");
 		strcat(fromip_buff, int2str(req->rcv.dst_port, NULL));
 		db_vals[4].val.string_val = fromip_buff;
@@ -1294,28 +1320,23 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 	memset(&to_ip, 0, sizeof(struct ip_addr));
 	if(sl_param->dst==0)
 	{
-		db_vals[5].val.string_val = "255.255.255.255";
+		db_vals[5].val.string_val = "any:255.255.255.255";
 	} else {
 		su2ip_addr(&to_ip, sl_param->dst);
-		if(req->rcv.proto==PROTO_TCP) {
-			memcpy(toip_buff, "tcp:", 4);
-		} else if(req->rcv.proto==PROTO_TLS) {
-			memcpy(toip_buff, "tls:", 4);
-		} else {
-			memcpy(toip_buff, "udp:", 4);
-		}
-		strcpy(toip_buff+4, ip_addr2a(&to_ip));
-		strcat(toip_buff+4, ":");
-		strcat(toip_buff+4,
+		siptrace_copy_proto(req->rcv.proto, toip_buff);
+
+		strcat(toip_buff, ip_addr2a(&to_ip));
+		strcat(toip_buff, ":");
+		strcat(toip_buff,
 				int2str((unsigned long)su_getport(sl_param->dst), &len));
-		DBG("siptrace:trace_sl_onreply_out: dest [%s]\n", toip_buff);
+		LM_DBG("dest [%s]\n", toip_buff);
 		db_vals[5].val.string_val = toip_buff;
 	}
 	
 	db_keys[6] = date_column;
 	db_vals[6].type = DB_DATETIME;
 	db_vals[6].nul = 0;
-	db_vals[6].val.int_val = time(NULL);
+	db_vals[6].val.time_val = time(NULL);
 	
 	db_keys[7] = direction_column;
 	db_vals[7].type = DB_STRING;
@@ -1338,12 +1359,15 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 		db_vals[9].val.str_val.s   = "";
 		db_vals[9].val.str_val.len = 0;
 	
-		DBG("trace_sl_onreply_out: storing info...\n");
+		LM_DBG("storing info...\n");
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "trace_sl_onreply_out: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
+#ifdef STATISTICS
+		update_stat(siptrace_rpl, 1);
+#endif
 	}
 	
 	if(avp==NULL)
@@ -1355,10 +1379,10 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 	db_vals[9].val.str_val.s = avp_value.s.s;
 	db_vals[9].val.str_val.len = avp_value.s.len;
 
-	DBG("trace_sl_onreply_out: storing info...\n");
+	LM_DBG("storing info...\n");
 	if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 	{
-		LOG(L_ERR, "trace_sl_onreply_out: error storing trace\n");
+		LM_ERR("error storing trace\n");
 		goto error;
 	}
 
@@ -1368,10 +1392,10 @@ static void trace_sl_onreply_out(struct sip_msg* req,
 		db_vals[9].val.str_val.s = avp_value.s.s;
 		db_vals[9].val.str_val.len = avp_value.s.len;
 
-		DBG("### - trace_sl_onreply_out: storing info (%d) ...\n", faked);
+		LM_DBG("### - storing info (%d) ...\n", faked);
 		if(db_funcs.insert(db_con, db_keys, db_vals, NR_KEYS) < 0)
 		{
-			LOG(L_ERR, "trace_sl_onreply_out: error storing trace\n");
+			LM_ERR("error storing trace\n");
 			goto error;
 		}
 		avp = search_next_avp( avp, &avp_value);
@@ -1437,7 +1461,7 @@ static int trace_send_duplicate(char *buf, int len)
 	
 	to=(union sockaddr_union*)pkg_malloc(sizeof(union sockaddr_union));
 	if (to==0){
-		LOG(L_ERR, "trace_send_duplicate:ERROR: out of memory\n");
+		LM_ERR("out of pkg memory\n");
 		return -1;
 	}
 	
@@ -1446,7 +1470,7 @@ static int trace_send_duplicate(char *buf, int len)
 	p=mk_proxy(&dup_uri->host, (dup_uri->port_no)?dup_uri->port_no:SIP_PORT,
 			proto, 0);
 	if (p==0){
-		LOG(L_ERR, "trace_send_duplicate:ERROR:  bad host name in uri\n");
+		LM_ERR("bad host name in uri\n");
 		pkg_free(to);
 		return -1;
 	}
@@ -1459,15 +1483,13 @@ static int trace_send_duplicate(char *buf, int len)
 	do {
 		send_sock=get_send_socket(0, to, proto);
 		if (send_sock==0){
-			LOG(L_ERR,
-				"trace_send_duplicate:ERROR: cannot forward to af %d, proto %d"
-				" no corresponding listening socket\n", to->s.sa_family,proto);
+			LM_ERR("can't forward to af %d, proto %d no corresponding listening socket\n", 
+					to->s.sa_family,proto);
 			continue;
 		}
 
 		if (msg_send(send_sock, proto, to, 0, buf, len)<0){
-			LOG(L_ERR,
-				"trace_send_duplicate:ERROR: cannot send duplicate message\n");
+			LM_ERR("cannot send duplicate message\n");
 			continue;
 		}
 		ret = 0;
