@@ -1,5 +1,5 @@
 /*
- * $Id: sst_handlers.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: sst_handlers.c 3297 2007-12-10 14:14:16Z dan_pascu $
  *
  * Copyright (C) 2006 SOMA Networks, Inc.
  * Written by Ron Winacott (karwin)
@@ -51,7 +51,7 @@
 #include <string.h> /* for memset() */
 #include <stdlib.h> /* For atoi() */
 
-#include "../../items.h"
+#include "../../pvar.h"
 #include "../../parser/parse_sst.h"
 #include "../../parser/parse_supported.h"
 #include "../../mem/mem.h"
@@ -65,18 +65,18 @@
 #include "sst_handlers.h"
 
 /*
- * My own LOG() macros to add the correct message prefix and
+ * My own LM_*() macros to add the correct message prefix and
  * file/function/line number information to all LOG messages.
  */
 
 #define DLOGMSG(msg) {									   \
 		if (msg->first_line.type == SIP_REQUEST) {		   \
-			LOG(L_INFO, "REQUEST: %.*s\n",				   \
+			LM_INFO("REQUEST: %.*s\n",			    	   \
 					msg->first_line.u.request.method.len,  \
 					msg->first_line.u.request.method.s);   \
 		}												   \
 		else {											   \
-			LOG(L_INFO, "RESPONSE: %d %.*s\n",			   \
+			LM_INFO("RESPONSE: %d %.*s\n",		    	   \
 					msg->first_line.u.reply.statuscode,	   \
 					msg->first_line.u.reply.reason.len,	   \
 					msg->first_line.u.reply.reason.s);	   \
@@ -161,7 +161,7 @@ extern struct sl_binds slb;
 /**
  * The dialog modules timeout AVP spac.
  */
-static xl_spec_t *timeout_avp = 0;
+static pv_spec_t *timeout_avp = 0;
 
 /**
  * Our Min-SE: header field value and test.
@@ -193,7 +193,7 @@ static str sst_422_rpl = str_init("Session Timer Too Small");
  * @param minSE - The minimum session expire value allowed by this
  *                PROXY.
  */
-void sst_handler_init(xl_spec_t *timeout_avp_p, unsigned int min_se, 
+void sst_handler_init(pv_spec_t *timeout_avp_p, unsigned int min_se, 
 		int flag, unsigned int reject)
 {
 	timeout_avp = timeout_avp_p;
@@ -255,7 +255,7 @@ void sst_dialog_created_CB(struct dlg_cell *did, int type,
 	 * Only deal with messages flaged as SST interested.
 	 */
 	if ((msg->flags & sst_flag) != sst_flag) {
-		LOG(L_DBG, "SST flag was not set for this request\n");
+		LM_DBG("SST flag was not set for this request\n");
 		return;
 	}
 
@@ -264,7 +264,7 @@ void sst_dialog_created_CB(struct dlg_cell *did, int type,
 	 */
 	if (msg->first_line.type != SIP_REQUEST ||
 			msg->first_line.u.request.method_value != METHOD_INVITE) {
-		LOG(L_WARN, "Dialog create callback called with a non-INVITE request.\n");
+		LM_WARN("dialog create callback called with a non-INVITE request.\n");
 		return;
 	}
 
@@ -272,7 +272,7 @@ void sst_dialog_created_CB(struct dlg_cell *did, int type,
 	 * Gather all he information about SST for this message
 	 */
 	if (parse_msg_for_sst_info(msg, &minfo)) {
-		LOG(L_ERR, "Failed to parse sst information\n");
+		LM_ERR("failed to parse sst information\n");
 		return;
 	}
 
@@ -308,7 +308,7 @@ void sst_dialog_created_CB(struct dlg_cell *did, int type,
 				info->interval = MAX(sst_min_se, minfo.min_se);
 				snprintf(buf, 80, "Min-SE: %d\r\n", info->interval);
 				if (append_header(msg, buf)) {
-					LOG(L_ERR, "Could not append modified Min-SE: header\n");
+					LM_ERR("Could not append modified Min-SE: header\n");
 				}
 			}
 			else if (sst_reject) {
@@ -335,7 +335,7 @@ void sst_dialog_created_CB(struct dlg_cell *did, int type,
 			remove_header(msg, "Min-SE");
 			snprintf(buf, 80, "Min-SE: %d\r\n", info->interval);
 			if (append_header(msg, buf)) {
-				LOG(L_ERR, "Could not append modified Min-SE: header\n");
+				LM_ERR("failed to append modified Min-SE: header\n");
 				/* What to do? Let is slide, we can still work */
 			}
 		}
@@ -343,7 +343,7 @@ void sst_dialog_created_CB(struct dlg_cell *did, int type,
 		info->requester = SST_PXY;
 		snprintf(buf, 80, "Session-Expires: %d\r\n", info->interval);
 		if (append_header(msg, buf)) {
-			LOG(L_ERR, "Error appending Session-Expires header to proxy "
+			LM_ERR("failed to append Session-Expires header to proxy "
 					"requested SST.\n");
 			shm_free(info);
 			return; /* Nothing we can do! */
@@ -361,7 +361,7 @@ void sst_dialog_created_CB(struct dlg_cell *did, int type,
 static void sst_dialog_confirmed_CB(struct dlg_cell *did, int type,
 		struct sip_msg* msg, void** param)
 {
-	LOG(L_DBG, "confirmed dialog CB %p\n", did);
+	LM_DBG("confirmed dialog CB %p\n", did);
 	DLOGMSG(msg);
 }
 #endif /* USE_CONFIRM_CALLBACK */
@@ -382,27 +382,26 @@ static void sst_dialog_terminate_CB(struct dlg_cell* did, int type,
 {
 	switch (type) {
 		case DLGCB_FAILED:
-			LOG(L_ERR, "DID %p failed (canceled). "
+			LM_ERR("DID %p failed (canceled). "
 					"Terminating session.\n", did);
 			break;
 		case DLGCB_EXPIRED:
 			/* In the case of expired, the msg is pointing at a
 			 * FAKED_REPLY (-1)
 			 */
-			LOG(L_ERR, "Terminating session.\n");
+			LM_ERR("Terminating session.\n");
 			break;
 		default: /* Normal termination. */
-			LOG(L_DBG, "Terminating DID %p session\n",
-					did);
+			LM_DBG("Terminating DID %p session\n", did);
 			break;
 	}
 	/*
 	 * Free the param sst_info_t memory
 	 */
 	if (*param) {
-		LOG(L_DBG, "Freeing the sst_info_t from dialog %p\n", did);
+		LM_DBG("freeing the sst_info_t from dialog %p\n", did);
 		shm_free(*param);
-		*param = '\0';
+		*param = NULL;
 	}
 	return;
 }
@@ -434,7 +433,7 @@ static void sst_dialog_request_within_CB(struct dlg_cell* did, int type,
 		if ((msg->first_line.u.request.method_value == METHOD_INVITE ||
 						msg->first_line.u.request.method_value == METHOD_UPDATE)) {
 
-			LOG(L_DBG, "Update by a REQUEST. %.*s\n", 
+			LM_DBG("Update by a REQUEST. %.*s\n", 
 					msg->first_line.u.request.method.len, 
 					msg->first_line.u.request.method.s);
 			if (parse_msg_for_sst_info(msg, &minfo)) {
@@ -457,7 +456,7 @@ static void sst_dialog_request_within_CB(struct dlg_cell* did, int type,
 			 * confiremed callback code should look for the new AVP
 			 * value, which is does not.
 			 */
-			LOG(L_DBG, "PRACK workaround applied!\n");
+			LM_DBG("PRACK workaround applied!\n");
 			set_timeout_avp(msg, info->interval);
 		}
 	}
@@ -468,7 +467,7 @@ static void sst_dialog_request_within_CB(struct dlg_cell* did, int type,
 			 * To spec (RFC) the internal time out value so not be reset
 			 * until here.
 			 */
-			LOG(L_DBG, "Update by a REPLY %d %.*s\n", 
+			LM_DBG("Update by a REPLY %d %.*s\n", 
 					msg->first_line.u.reply.statuscode,
 					msg->first_line.u.reply.reason.len, 
 					msg->first_line.u.reply.reason.s);
@@ -504,7 +503,7 @@ static void sst_dialog_response_fwded_CB(struct dlg_cell* did, int type,
 		sst_msg_info_t minfo = {0,0,0,0};
 		sst_info_t *info = (sst_info_t *)*param;
 
-		LOG(L_DBG, "Dialog seen REPLY %d %.*s\n", 
+		LM_DBG("Dialog seen REPLY %d %.*s\n", 
 				msg->first_line.u.reply.statuscode,
 				msg->first_line.u.reply.reason.len, 
 				msg->first_line.u.reply.reason.s);
@@ -516,7 +515,7 @@ static void sst_dialog_response_fwded_CB(struct dlg_cell* did, int type,
 		 */
 		if (msg->first_line.u.reply.statuscode == 422) {
 			if (parse_msg_for_sst_info(msg, &minfo)) {
-				LOG(L_ERR, "Could not prase sst information for thr 422 reply\n");
+				LM_ERR("failed to prase sst information for thr 422 reply\n");
 				return;
 			}
 			/* Make sure we do not try to use anything smaller */
@@ -529,7 +528,7 @@ static void sst_dialog_response_fwded_CB(struct dlg_cell* did, int type,
 		 * INVITE or reINVTE/UPDATE.
 		 */
 		if (!msg->cseq && ((parse_headers(msg, HDR_CSEQ_F, 0) == -1) || !msg->cseq)) {
-			LOG(L_ERR, "Error while parsing CSeq\n");
+			LM_ERR("failed to parse CSeq\n");
 			return;
 		}
 		
@@ -539,7 +538,7 @@ static void sst_dialog_response_fwded_CB(struct dlg_cell* did, int type,
 				(get_cseq(msg)->method_id == METHOD_INVITE ||
 						get_cseq(msg)->method_id == METHOD_UPDATE)) {
 			if (parse_msg_for_sst_info(msg, &minfo)) {
-				LOG(L_ERR, "Could not parse sst information for the 2XX reply\n");
+				LM_ERR("failed to parse sst information for the 2XX reply\n");
 				return;
 			}
 
@@ -554,7 +553,7 @@ static void sst_dialog_response_fwded_CB(struct dlg_cell* did, int type,
 				if (info->requester == SST_PXY || info->supported == SST_UAC) {
 					char se_buf[80];
 					
-					LOG(L_DBG, "Appending the Session-Expires: header to the 2XX reply."
+					LM_DBG("appending the Session-Expires: header to the 2XX reply."
 							" UAC will deal with it.\n");
 					/*
 					 * GOOD! we can just insert the Session-Expires:
@@ -564,7 +563,7 @@ static void sst_dialog_response_fwded_CB(struct dlg_cell* did, int type,
 					snprintf(se_buf, 80, "Session-Expires: %d;refresher=uac\r\n", 
 							info->interval);
 					if (append_header(msg, se_buf)) {
-						LOG(L_ERR, "Error appending Session-Expires header\n");
+						LM_ERR("failed to append Session-Expires header\n");
 						return;
 					}
 					/* Set the dialog timeout HERE */
@@ -575,7 +574,7 @@ static void sst_dialog_response_fwded_CB(struct dlg_cell* did, int type,
 				else {
 					/* We are sunk, uac did not request it, and it
 					 * does not support it */
-					LOG(L_DBG, "UAC and UAS do not support timers!"
+					LM_DBG("UAC and UAS do not support timers!"
 							" No session timers for this session.\n");
 				}
 			}
@@ -626,12 +625,11 @@ int sst_check_min(struct sip_msg *msg, char *flag, char *str2)
 		 */
 		if ((result = parse_session_expires(msg, &se)) != parse_sst_success) {
 			if (result != parse_sst_header_not_found) {
-				LOG(L_ERR, "error parsing Session-Expires headers.\n");
+				LM_ERR("failed to parse Session-Expires headers.\n");
 				return 0; /* Error drop the message */
 			}
 			/* Session-Expires not supported/stated */
-			LOG(L_DBG, "No Session-Expires header "
-					"found. retuning false (-1)\n");
+			LM_DBG("No Session-Expires header found. retuning false (-1)\n");
 			/*
 			 * NOTE: 0 == drop message, 1 == true, -1 == false
 			 */
@@ -647,19 +645,18 @@ int sst_check_min(struct sip_msg *msg, char *flag, char *str2)
 				 * This is an error. The header was found but could
 				 * not parse it.
 				 */
-				LOG(L_ERR, "Error parsing MIN-SE header.\n");
+				LM_ERR("failed to parse MIN-SE header.\n");
 				return -1; 
 			}
 			/*
 			 * If not stated, use the value from the session-expires
 			 * header
 			 */
-			LOG(L_DBG, "No MIN-SE header found.\n");
+			LM_DBG("No MIN-SE header found.\n");
 			minse = se.interval;
 		}
 		
-		LOG(L_DBG, "Session-Expires: %d; MIN-SE: %d\n",
-				se.interval, minse);
+		LM_DBG("Session-Expires: %d; MIN-SE: %d\n",	se.interval, minse);
 
 		/*
 		 * Now compare our MIN-SE with the messages and see if it is
@@ -677,21 +674,20 @@ int sst_check_min(struct sip_msg *msg, char *flag, char *str2)
 				char *minse_hdr = pkg_malloc(hdr_len+1);
 				memset(minse_hdr, 0, hdr_len+1);
 				snprintf(minse_hdr, hdr_len+1, "%s%d", "MIN-SE:", sst_min_se);
-				LOG(L_DBG, "Sending 422: %.*s\n",
-						hdr_len, minse_hdr);
+				LM_DBG("Sending 422: %.*s\n", hdr_len, minse_hdr);
 				if (send_response(msg, 422, &sst_422_rpl, minse_hdr, hdr_len)){
-					LOG(L_ERR, "Error sending 422 reply.\n");
+					LM_ERR("Error sending 422 reply.\n");
 				}
 				
 				if (minse_hdr) {
 					pkg_free(minse_hdr);
 				}
 			}
-			LOG(L_DBG, "Done returning true (1)\n");
+			LM_DBG("Done returning true (1)\n");
 			return 1; /* return true */
 		}
 	}
-	LOG(L_DBG, "Done returning false (-1)\n");
+	LM_DBG("Done returning false (-1)\n");
 	/*
 	 * All is good.
 	 */
@@ -720,13 +716,13 @@ static int send_response(struct sip_msg *request, int code, str *reason,
 		if ((header) && (header_len)) {
 			if (add_lump_rpl(request, header, header_len, LUMP_RPL_HDR) == 0) {
 				/* An error with adding the lump */
-				LOG(L_ERR, "unable to append header.\n");
+				LM_ERR("unable to append header.\n");
 				return -1;
 			}
 		}
 		/* Now using the sl function, send the reply/response */
 		if (slb.reply(request, code, reason) < 0) {
-			LOG(L_ERR, "Unable to sent reply.\n");
+			LM_ERR("Unable to sent reply.\n");
 			return -1;
 		}
 	}
@@ -750,29 +746,29 @@ static int append_header(struct sip_msg *msg, const char *header)
 	char *s = NULL;
 	int len = 0;
 
-	LOG(L_DBG, "Appending header: %s", header);
+	LM_DBG("Appending header: %s", header);
 
 	if (parse_headers(msg, HDR_EOH_F, 0) == -1) {
-		LOG(L_ERR, "Error while parsing headers in message.\n");
+		LM_ERR("failed to parse headers in message.\n");
 		return(1);
 	}
 
 	if ((anchor = anchor_lump(msg, msg->unparsed - msg->buf, 0, 0)) == 0) {
-		LOG(L_ERR, "Error getting anchor to append header\n");
+		LM_ERR("failed to get anchor to append header\n");
 		return(1);
 	}
 	len = strlen(header);
 	if ((s = (char *)pkg_malloc(len)) == 0) {
-		LOG(L_ERR, "No memory. (size requested = %d)\n", len);
+		LM_ERR("No more pkg memory. (size requested = %d)\n", len);
 		return(1);
 	}
 	memcpy(s, header, len);
 	if (insert_new_lump_before(anchor, s, len, 0) == 0) {
-		LOG(L_ERR, "Can't insert lump\n");
+		LM_ERR("failed to insert lump\n");
 		pkg_free(s);
 		return(1);
 	}
-	LOG(L_DBG, "Done appending header successfully.\n");
+	LM_DBG("Done appending header successfully.\n");
 	return(0);
 }
 
@@ -793,7 +789,7 @@ static int remove_header(struct sip_msg *msg, const char *header)
 	int len = strlen(header);
 
 	if (parse_headers(msg, HDR_EOH_F, 0) == -1) {
-		LOG(L_ERR, "Error while parsing headers in message.\n");
+		LM_ERR("failed to parse headers in message.\n");
 		return(-1);
 	}
 	
@@ -807,7 +803,7 @@ static int remove_header(struct sip_msg *msg, const char *header)
 
 		anchor = del_lump(msg, hf->name.s-msg->buf, hf->len, 0);
 		if (anchor == 0) {
-			LOG(L_ERR, "no memory\n");
+			LM_ERR("no more pkg memory\n");
 			return -1;
 		}
 		cnt++;
@@ -827,22 +823,22 @@ static int remove_header(struct sip_msg *msg, const char *header)
 static int set_timeout_avp(struct sip_msg *msg, unsigned int value)
 {
 	int rtn = -1; /* assume failure */
-	xl_value_t xl_val;
+	pv_value_t pv_val;
 	
 	/* Set the dialog timeout HERE */
 	if (timeout_avp) {
 		int result = 0;
-		if ((result = xl_get_spec_value(msg, timeout_avp, &xl_val, 0)) == 0) {
-			if (xl_val.flags & XL_VAL_INT) {
+		if ((result = pv_get_spec_value(msg, timeout_avp, &pv_val)) == 0) {
+			if (pv_val.flags & PV_VAL_INT) {
 				/* We now hold a reference to the AVP int value */
-				LOG(L_ERR, "Found current timeout value is %d, setting it to %d\n",
-						xl_val.ri, value);
-				xl_val.ri = value;
+				LM_ERR("Found current timeout value is %d, setting it to %d\n",
+						pv_val.ri, value);
+				pv_val.ri = value;
 				rtn = 0;
 			}
 			else {
 				/* Never set so it was never added */
-				if (xl_val.flags == XL_VAL_NULL) {
+				if (pv_val.flags == PV_VAL_NULL) {
 					int_str avp_name;
 					int_str avp_value;
 					unsigned short name_type = 0;
@@ -850,23 +846,24 @@ static int set_timeout_avp(struct sip_msg *msg, unsigned int value)
 					memset(&avp_value, 0, sizeof(int_str));
 					memset(&avp_name, 0, sizeof(int_str));
 
-					xl_get_avp_name(msg, timeout_avp, &avp_name, &name_type);
+					pv_get_avp_name(msg, &timeout_avp->pvp, &avp_name,
+							&name_type);
 					avp_value.n = value;
 					add_avp(name_type, avp_name, avp_value);
-					LOG(L_DBG, "Added the avp and set the value to %d\n", value);
+					LM_DBG("Added the avp and set the value to %d\n", value);
 					rtn = 0;
 				}
 				else {
-					LOG(L_ERR, "AVP wrong type %d. Not an integer.\n", xl_val.flags);
+					LM_ERR("AVP wrong type %d. Not an integer.\n", pv_val.flags);
 				}
 			}
 		}
 		else {
-			LOG(L_ERR, "SST not reset. get avp result is %d\n", result);
+			LM_ERR("SST not reset. get avp result is %d\n", result);
 		}
 	}
 	else {
-		LOG(L_ERR, "SST needs to know the name of the dialog timeout AVP!\n");
+		LM_ERR("SST needs to know the name of the dialog timeout AVP!\n");
 	}
 	return(rtn);
 }
@@ -942,11 +939,11 @@ static int send_reject(struct sip_msg *msg, unsigned int min_se)
 		memset(minse_hdr, 0, hdr_len+1);
 		snprintf(minse_hdr, hdr_len+1, "%s %d", "MIN-SE:", min_se);
 		if (send_response(msg, 422, &sst_422_rpl, minse_hdr, hdr_len)) {
-			LOG(L_ERR, "Error sending 422 reply.\n");
+			LM_ERR("Error sending 422 reply.\n");
 			return(-1);
 		}
 		pkg_free(minse_hdr);
-		LOG(L_DBG, "Send reject reply 422 with Min-SE: %d\n", min_se);
+		LM_DBG("Send reject reply 422 with Min-SE: %d\n", min_se);
 		return(0);
 	}
 	return(-1);
@@ -967,18 +964,17 @@ static void setup_dialog_callbacks(struct dlg_cell *did, sst_info_t *info)
 	 */
 
 #ifdef USE_CONFIRM_CALLBACK
-	LOG(L_DBG, "Adding callback "
-			"DLGCB_CONFIRMED\n");
+	LM_DBG("Adding callback DLGCB_CONFIRMED\n");
 	dlg_binds->register_dlgcb(did,
 			DLGCB_CONFIRMED, sst_dialog_confirmed_CB, info);
 #endif /* USE_CONFIRM_CALLBACK */
 
-	LOG(L_DBG, "Adding callback "
+	LM_DBG("Adding callback "
 			"DLGCB_FAILED|DLGCB_TERMINATED|DLGCB_EXPIRED\n");
 	dlg_binds->register_dlgcb(did,
 			DLGCB_FAILED|DLGCB_TERMINATED|DLGCB_EXPIRED,
 			sst_dialog_terminate_CB, info);
-	LOG(L_DBG, "Adding callback DLGCB_REQ_WITHIN\n");
+	LM_DBG("Adding callback DLGCB_REQ_WITHIN\n");
 	/* This is for the reINVITE/UPDATE requests */
 	dlg_binds->register_dlgcb(did, DLGCB_REQ_WITHIN,
 			sst_dialog_request_within_CB, info);
@@ -988,7 +984,7 @@ static void setup_dialog_callbacks(struct dlg_cell *did, sst_info_t *info)
 	 * immutable! we must do all the real work in the DLGCB_FRD
 	 * callback were we can change the message.
 	 */
-	LOG(L_DBG, "Adding callback DLGCB_RESPONSE_FWDED\n");
+	LM_DBG("Adding callback DLGCB_RESPONSE_FWDED\n");
 	dlg_binds->register_dlgcb(did, DLGCB_RESPONSE_FWDED,
 			sst_dialog_response_fwded_CB, info);
 }
