@@ -1,4 +1,4 @@
-/*$Id: textops.c 1943 2007-04-03 11:06:05Z miconda $
+/*$Id: textops.c 3335 2007-12-12 18:32:52Z bogdan_iancu $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -57,11 +57,13 @@
 #include "../../mem/mem.h"
 #include "../../str.h"
 #include "../../re.h"
+#include "../../mod_fix.h"
 #include "../../parser/parse_uri.h"
 #include "../../parser/parse_hname2.h"
 #include "../../parser/parse_methods.h"
 #include "../../parser/parse_content.h"
 #include "../../parser/parse_privacy.h"
+#include "../../mod_fix.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -93,6 +95,7 @@ static int subst_f(struct sip_msg*, char*, char*);
 static int subst_uri_f(struct sip_msg*, char*, char*);
 static int subst_user_f(struct sip_msg*, char*, char*);
 static int subst_body_f(struct sip_msg*, char*, char*);
+static int filter_body_f(struct sip_msg*, char*, char*);
 static int remove_hf_f(struct sip_msg* msg, char* str_hf, char* foo);
 static int is_present_hf_f(struct sip_msg* msg, char* str_hf, char* foo);
 static int search_append_f(struct sip_msg*, char*, char*);
@@ -108,10 +111,9 @@ static int is_method_f(struct sip_msg* msg, char* , char *);
 static int has_body_f(struct sip_msg *msg, char *type, char *str2 );
 static int is_privacy_f(struct sip_msg *msg, char *privacy, char *str2 );
 
-static int fixup_regex(void**, int);
 static int fixup_substre(void**, int);
-static int str_fixup(void** param, int param_no);
 static int hname_fixup(void** param, int param_no);
+static int free_hname_fixup(void** param, int param_no);
 static int fixup_method(void** param, int param_no);
 static int add_header_fixup(void** param, int param_no);
 static int it_list_fixup(void** param, int param_no);
@@ -122,57 +124,67 @@ static int mod_init(void);
 
 
 static cmd_export_t cmds[]={
-	{"search",           search_f,          1, fixup_regex, 
+	{"search",           search_f,          1, fixup_str2regexp,
+			free_fixup_str2regexp,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"search_body",      search_body_f,     1, fixup_regex, 
+	{"search_body",      search_body_f,     1, fixup_str2regexp,
+			free_fixup_str2regexp,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"search_append",    search_append_f,   2, fixup_regex, 
+	{"search_append",    search_append_f,   2, fixup_str2regexp,
+			free_fixup_str2regexp,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"search_append_body", search_append_body_f,   2, fixup_regex, 
+	{"search_append_body", search_append_body_f,   2, fixup_str2regexp,
+			free_fixup_str2regexp, 
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"replace",          replace_f,         2, fixup_regex, 
+	{"replace",          replace_f,         2, fixup_str2regexp,
+			free_fixup_str2regexp,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"replace_body",     replace_body_f,    2, fixup_regex, 
+	{"replace_body",     replace_body_f,    2, fixup_str2regexp, 
+			free_fixup_str2regexp,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"replace_all",      replace_all_f,     2, fixup_regex, 
+	{"replace_all",      replace_all_f,     2, fixup_str2regexp, 
+			free_fixup_str2regexp,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"replace_body_all", replace_body_all_f,2, fixup_regex, 
+	{"replace_body_all", replace_body_all_f,2, fixup_str2regexp,
+			free_fixup_str2regexp,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"append_to_reply",  append_to_reply_f, 1, it_list_fixup,
+	{"append_to_reply",  append_to_reply_f, 1, it_list_fixup, 0,
 			REQUEST_ROUTE|BRANCH_ROUTE|ERROR_ROUTE},
-	{"append_hf",        append_hf_1,       1, add_header_fixup,
+	{"append_hf",        append_hf_1,       1, add_header_fixup, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"append_hf",        append_hf_2,       2, add_header_fixup,
+	{"append_hf",        append_hf_2,       2, add_header_fixup, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"insert_hf",        insert_hf_1,       1, add_header_fixup,
+	{"insert_hf",        insert_hf_1,       1, add_header_fixup, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"insert_hf",        insert_hf_2,       2, add_header_fixup,
+	{"insert_hf",        insert_hf_2,       2, add_header_fixup, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"append_urihf",     append_urihf,      2, str_fixup,   
+	{"append_urihf",     append_urihf,      2, str_fixup, free_str_fixup,
 			REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"remove_hf",        remove_hf_f,       1, hname_fixup,
+	{"remove_hf",        remove_hf_f,       1, hname_fixup, free_hname_fixup,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"is_present_hf",    is_present_hf_f,   1, hname_fixup,
+	{"is_present_hf",    is_present_hf_f,   1, hname_fixup, free_hname_fixup,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"subst",            subst_f,           1, fixup_substre,
+	{"subst",            subst_f,           1, fixup_substre, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"subst_uri",        subst_uri_f,       1, fixup_substre,
+	{"subst_uri",        subst_uri_f,       1, fixup_substre, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{"subst_user",       subst_user_f,      1, fixup_substre,
+	{"subst_user",       subst_user_f,      1, fixup_substre, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"subst_body",       subst_body_f,      1, fixup_substre,
+	{"subst_body",       subst_body_f,      1, fixup_substre, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"append_time",      append_time_f,     0, 0,
+	{"filter_body",      filter_body_f,     1, str_fixup, 0,
+			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
+	{"append_time",      append_time_f,     0, 0, 0,
 			REQUEST_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE },
-	{"is_method",        is_method_f,       1, fixup_method,
+	{"is_method",        is_method_f,       1, fixup_method, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"has_body",         has_body_f,        0, 0,
+	{"has_body",         has_body_f,        0, 0, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"has_body",         has_body_f,        1, fixup_body_type,
+	{"has_body",         has_body_f,        1, fixup_body_type, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE},
-	{"is_privacy",       is_privacy_f,      1, fixup_privacy,
+	{"is_privacy",       is_privacy_f,      1, fixup_privacy, 0,
 			REQUEST_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE|BRANCH_ROUTE}, 
-	{0,0,0,0,0}
+	{0,0,0,0,0,0}
 };
 
 
@@ -184,6 +196,7 @@ struct module_exports exports= {
 	0,          /* exported statistics */
 	0,          /* exported MI functions */
 	0,          /* exported pseudo-variables */
+	0,          /* extra processes */
 	mod_init,   /* module initialization function */
 	0,          /* response function */
 	0,          /* destroy function */
@@ -193,7 +206,7 @@ struct module_exports exports= {
 
 static int mod_init(void)
 {
-	LOG(L_INFO, "TextOPS - initializing\n");
+	LM_INFO("initializing...\n");
 	return 0;
 }
 
@@ -222,12 +235,12 @@ static int search_body_f(struct sip_msg* msg, char* key, char* str2)
 
 	body.s = get_body(msg);
 	if (body.s==0) {
-		LOG(L_ERR, "ERROR:search_body_f: failed to get the message body\n");
+		LM_ERR("failed to get the message body\n");
 		return -1;
 	}
 	body.len = msg->len -(int)(body.s-msg->buf);
 	if (body.len==0) {
-		DBG("ERROR:search_body_f: message body has zero length\n");
+		LM_DBG("message body has zero length\n");
 		return -1;
 	}
 
@@ -255,12 +268,12 @@ static int search_append_f(struct sip_msg* msg, char* key, char* str2)
 		len=strlen(str2);
 		s=pkg_malloc(len);
 		if (s==0){
-			LOG(L_ERR, "ERROR: search_append_f: mem. allocation failure\n");
+			LM_ERR("memory allocation failure\n");
 			return -1;
 		}
 		memcpy(s, str2, len); 
 		if (insert_new_lump_after(l, s, len, 0)==0){
-			LOG(L_ERR, "ERROR: could not insert new lump\n");
+			LM_ERR("could not insert new lump\n");
 			pkg_free(s);
 			return -1;
 		}
@@ -280,13 +293,12 @@ static int search_append_body_f(struct sip_msg* msg, char* key, char* str2)
 
 	body.s = get_body(msg);
 	if (body.s==0) {
-		LOG(L_ERR,
-				"ERROR:search_append_body_f: failed to get the message body\n");
+		LM_ERR("failed to get the message body\n");
 		return -1;
 	}
 	body.len = msg->len -(int)(body.s-msg->buf);
 	if (body.len==0) {
-		DBG("ERROR:search_append_body_f: message body has zero length\n");
+		LM_DBG("message body has zero length\n");
 		return -1;
 	}
 
@@ -299,12 +311,12 @@ static int search_append_body_f(struct sip_msg* msg, char* key, char* str2)
 		len=strlen(str2);
 		s=pkg_malloc(len);
 		if (s==0){
-			LOG(L_ERR, "ERROR: search_append_f: mem. allocation failure\n");
+			LM_ERR("memory allocation failure\n");
 			return -1;
 		}
 		memcpy(s, str2, len); 
 		if (insert_new_lump_after(l, s, len, 0)==0){
-			LOG(L_ERR, "ERROR: could not insert new lump\n");
+			LM_ERR("could not insert new lump\n");
 			pkg_free(s);
 			return -1;
 		}
@@ -325,7 +337,7 @@ static int replace_all_f(struct sip_msg* msg, char* key, char* str2)
 	int ret;
 	int eflags;
 
-	begin=get_header(msg); /* msg->orig previously .. uri problems */
+	begin = get_header(msg);
 	ret=-1; /* pessimist: we will not find any */
 	len=strlen(str2);
 	eflags=0; /* match ^ at the beginning of the string*/
@@ -333,30 +345,37 @@ static int replace_all_f(struct sip_msg* msg, char* key, char* str2)
 	while (begin<msg->buf+msg->len 
 				&& regexec((regex_t*) key, begin, 1, &pmatch, eflags)==0) {
 		off=begin-msg->buf;
-		/* change eflags, not to match any more at string start */
-		eflags|=REG_NOTBOL;
 		if (pmatch.rm_so==-1){
-			LOG(L_ERR, "ERROR: replace_all_f: offset unknown\n");
+			LM_ERR("offset unknown\n");
+			return -1;
+		}
+		if (pmatch.rm_so==pmatch.rm_eo){
+			LM_ERR("matched string is empty... invalid regexp?\n");
 			return -1;
 		}
 		if ((l=del_lump(msg, pmatch.rm_so+off,
 						pmatch.rm_eo-pmatch.rm_so, 0))==0) {
-			LOG(L_ERR, "ERROR: replace_all_f: del_lump failed\n");
+			LM_ERR("del_lump failed\n");
 			return -1;
 		}
 		s=pkg_malloc(len);
 		if (s==0){
-			LOG(L_ERR, "ERROR: replace_all_f: mem. allocation failure\n");
+			LM_ERR("memory allocation failure\n");
 			return -1;
 		}
 		memcpy(s, str2, len); 
 		if (insert_new_lump_after(l, s, len, 0)==0){
-			LOG(L_ERR, "ERROR: replace_all_f: could not insert new lump\n");
+			LM_ERR("could not insert new lump\n");
 			pkg_free(s);
 			return -1;
 		}
 		/* new cycle */
 		begin=begin+pmatch.rm_eo;
+		/* is it still a string start */
+		if (*(begin-1)=='\n' || *(begin-1)=='\r')
+			eflags&=~REG_NOTBOL;
+		else
+			eflags|=REG_NOTBOL;
 		ret=1;
 	} /* while found ... */
 	return ret;
@@ -376,17 +395,16 @@ static int replace_body_all_f(struct sip_msg* msg, char* key, char* str2)
 
 	body.s = get_body(msg);
 	if (body.s==0) {
-		LOG(L_ERR,
-				"ERROR:replace_body_all_f: failed to get the message body\n");
+		LM_ERR("failed to get the message body\n");
 		return -1;
 	}
 	body.len = msg->len -(int)(body.s-msg->buf);
 	if (body.len==0) {
-		DBG("ERROR:replace_body_all_f: message body has zero length\n");
+		LM_DBG("message body has zero length\n");
 		return -1;
 	}
 
-	begin=body.s; /* msg->orig previously .. uri problems */
+	begin=body.s;
 	ret=-1; /* pessimist: we will not find any */
 	len=strlen(str2);
 	eflags=0; /* match ^ at the beginning of the string*/
@@ -394,31 +412,37 @@ static int replace_body_all_f(struct sip_msg* msg, char* key, char* str2)
 	while (begin<msg->buf+msg->len 
 				&& regexec((regex_t*) key, begin, 1, &pmatch, eflags)==0) {
 		off=begin-msg->buf;
-		/* change eflags, not to match any more at string start */
-		eflags|=REG_NOTBOL;
 		if (pmatch.rm_so==-1){
-			LOG(L_ERR, "ERROR: replace_body_all_f: offset unknown\n");
+			LM_ERR("offset unknown\n");
+			return -1;
+		}
+		if (pmatch.rm_so==pmatch.rm_eo){
+			LM_ERR("matched string is empty... invalid regexp?\n");
 			return -1;
 		}
 		if ((l=del_lump(msg, pmatch.rm_so+off,
 						pmatch.rm_eo-pmatch.rm_so, 0))==0) {
-			LOG(L_ERR, "ERROR: replace_body_all_f: del_lump failed\n");
+			LM_ERR("del_lump failed\n");
 			return -1;
 		}
 		s=pkg_malloc(len);
 		if (s==0){
-			LOG(L_ERR, "ERROR: replace_body_all_f: mem. allocation failure\n");
+			LM_ERR("memory allocation failure\n");
 			return -1;
 		}
 		memcpy(s, str2, len); 
 		if (insert_new_lump_after(l, s, len, 0)==0){
-			LOG(L_ERR,
-				"ERROR: replace_body_all_f: could not insert new lump\n");
+			LM_ERR("could not insert new lump\n");
 			pkg_free(s);
 			return -1;
 		}
 		/* new cycle */
 		begin=begin+pmatch.rm_eo;
+		/* is it still a string start */
+		if (*(begin-1)=='\n' || *(begin-1)=='\r')
+			eflags&=~REG_NOTBOL;
+		else
+			eflags|=REG_NOTBOL;
 		ret=1;
 	} /* while found ... */
 	return ret;
@@ -445,12 +469,12 @@ static int replace_f(struct sip_msg* msg, char* key, char* str2)
 		len=strlen(str2);
 		s=pkg_malloc(len);
 		if (s==0){
-			LOG(L_ERR, "ERROR: replace_f: mem. allocation failure\n");
+			LM_ERR("memory allocation failure\n");
 			return -1;
 		}
 		memcpy(s, str2, len); 
 		if (insert_new_lump_after(l, s, len, 0)==0){
-			LOG(L_ERR, "ERROR: could not insert new lump\n");
+			LM_ERR("could not insert new lump\n");
 			pkg_free(s);
 			return -1;
 		}
@@ -472,13 +496,12 @@ static int replace_body_f(struct sip_msg* msg, char* key, char* str2)
 
 	body.s = get_body(msg);
 	if (body.s==0) {
-		LOG(L_ERR,
-				"ERROR:replace_body_f: failed to get the message body\n");
+		LM_ERR("failed to get the message body\n");
 		return -1;
 	}
 	body.len = msg->len -(int)(body.s-msg->buf);
 	if (body.len==0) {
-		DBG("ERROR:replace_body_f: message body has zero length\n");
+		LM_DBG("message body has zero length\n");
 		return -1;
 	}
 
@@ -494,12 +517,12 @@ static int replace_body_f(struct sip_msg* msg, char* key, char* str2)
 		len=strlen(str2);
 		s=pkg_malloc(len);
 		if (s==0){
-			LOG(L_ERR, "ERROR: replace_body_f: mem. allocation failure\n");
+			LM_ERR("memory allocation failure\n");
 			return -1;
 		}
 		memcpy(s, str2, len); 
 		if (insert_new_lump_after(l, s, len, 0)==0){
-			LOG(L_ERR, "ERROR: replace_body_f: could not insert new lump\n");
+			LM_ERR("could not insert new lump\n");
 			pkg_free(s);
 			return -1;
 		}
@@ -530,7 +553,7 @@ static int subst_f(struct sip_msg* msg, char*  subst, char* ignored)
 	if ((lst=subst_run(se, begin, msg, &nmatches))==0)
 		goto error; /* not found */
 	for (rpl=lst; rpl; rpl=rpl->next){
-		DBG(" %s: subst_f: replacing at offset %d [%.*s] with [%.*s]\n",
+		LM_DBG("%s: replacing at offset %d [%.*s] with [%.*s]\n",
 				exports.name, rpl->offset+off,
 				rpl->size, rpl->offset+off+msg->buf,
 				rpl->rpl.len, rpl->rpl.s);
@@ -539,7 +562,7 @@ static int subst_f(struct sip_msg* msg, char*  subst, char* ignored)
 		/* hack to avoid re-copying rpl, possible because both 
 		 * replace_lst & lumps use pkg_malloc */
 		if (insert_new_lump_after(l, rpl->rpl.s, rpl->rpl.len, 0)==0){
-			LOG(L_ERR, "ERROR: %s: subst_f: could not insert new lump\n",
+			LM_ERR("ERROR: %s: subst_f: could not insert new lump\n",
 					exports.name);
 			goto error;
 		}
@@ -550,10 +573,10 @@ static int subst_f(struct sip_msg* msg, char*  subst, char* ignored)
 	}
 	ret=1;
 error:
-	DBG("subst_f: lst was %p\n", lst);
+	LM_DBG("lst was %p\n", lst);
 	if (lst) replace_lst_free(lst);
 	if (nmatches<0)
-		LOG(L_ERR, "ERROR: %s: subst_run failed\n", exports.name);
+		LM_ERR("ERROR: %s: subst_run failed\n", exports.name);
 	return ret;
 }
 
@@ -586,7 +609,7 @@ static int subst_uri_f(struct sip_msg* msg, char*  subst, char* ignored)
 	result=subst_str(tmp, msg, se, 0); /* pkg malloc'ed result */
 	tmp[len]=c;
 	if (result){
-		DBG("%s: subst_uri_f: match - old uri= [%.*s], new uri= [%.*s]\n",
+		LM_DBG("%s match - old uri= [%.*s], new uri= [%.*s]\n",
 				exports.name, len, tmp,
 				(result->len)?result->len:0,(result->s)?result->s:"");
 		if (msg->new_uri.s) pkg_free(msg->new_uri.s);
@@ -630,7 +653,7 @@ static int subst_user_f(struct sip_msg* msg, char*  subst, char* ignored)
 	if (c)	user.s[user.len]=c;
 	if (result == NULL) {
 		if (nmatches<0)
-			LOG(L_ERR, "subst_user(): subst_str() failed\n");
+			LM_ERR("subst_user(): subst_str() failed\n");
 		return -1;
 	}
 	/* result->s[result->len] = '\0';  --subst_str returns 0-term strings */
@@ -639,6 +662,7 @@ static int subst_user_f(struct sip_msg* msg, char*  subst, char* ignored)
 	act.elem[0].type = STRING_ST;
 	act.elem[0].u.string = result->s;
 	rval = do_action(&act, msg);
+	pkg_free(result->s);
 	pkg_free(result);
 	return rval;
 }
@@ -659,13 +683,12 @@ static int subst_body_f(struct sip_msg* msg, char*  subst, char* ignored)
 
 	body.s = get_body(msg);
 	if (body.s==0) {
-		LOG(L_ERR,
-				"ERROR:subst_body_f: failed to get the message body\n");
+		LM_ERR("failed to get the message body\n");
 		return -1;
 	}
 	body.len = msg->len -(int)(body.s-msg->buf);
 	if (body.len==0) {
-		DBG("ERROR:subst_body_f: message body has zero length\n");
+		LM_DBG("message body has zero length\n");
 		return -1;
 	}
 	
@@ -677,7 +700,7 @@ static int subst_body_f(struct sip_msg* msg, char*  subst, char* ignored)
 	if ((lst=subst_run(se, begin, msg, &nmatches))==0)
 		goto error; /* not found */
 	for (rpl=lst; rpl; rpl=rpl->next){
-		DBG("%s:subst_body_f: replacing at offset %d [%.*s] with [%.*s]\n",
+		LM_DBG("%s replacing at offset %d [%.*s] with [%.*s]\n",
 				exports.name, rpl->offset+off,
 				rpl->size, rpl->offset+off+msg->buf,
 				rpl->rpl.len, rpl->rpl.s);
@@ -686,7 +709,7 @@ static int subst_body_f(struct sip_msg* msg, char*  subst, char* ignored)
 		/* hack to avoid re-copying rpl, possible because both 
 		 * replace_lst & lumps use pkg_malloc */
 		if (insert_new_lump_after(l, rpl->rpl.s, rpl->rpl.len, 0)==0){
-			LOG(L_ERR, "ERROR:%s: subst_body_f: could not insert new lump\n",
+			LM_ERR("%s could not insert new lump\n",
 					exports.name);
 			goto error;
 		}
@@ -697,14 +720,113 @@ static int subst_body_f(struct sip_msg* msg, char*  subst, char* ignored)
 	}
 	ret=1;
 error:
-	DBG("subst_body_f: lst was %p\n", lst);
+	LM_DBG("lst was %p\n", lst);
 	if (lst) replace_lst_free(lst);
 	if (nmatches<0)
-		LOG(L_ERR, "ERROR:%s:subst_body_f: subst_run failed\n", exports.name);
+		LM_ERR("%s subst_run failed\n", exports.name);
 	return ret;
 }
 
 
+static inline int find_line_start(char *text, unsigned int text_len,
+				  char **buf, unsigned int *buf_len)
+{
+    char *ch, *start;
+    unsigned int len;
+
+    start = *buf;
+    len = *buf_len;
+
+    while (text_len <= len) {
+	if (strncmp(text, start, text_len) == 0) {
+	    *buf = start;
+	    *buf_len = len;
+	    return 1;
+	}
+	if ((ch = memchr(start, 13, len - 1))) {
+	    if (*(ch + 1) != 10) {
+		LM_ERR("No LF after CR\n");
+		return 0;
+	    }
+	    len = len - (ch - start + 2);
+	    start = ch + 2;
+	} else {
+	    LM_ERR("No CRLF found\n");
+	    return 0;
+	}
+    }
+    return 0;
+}
+
+
+/* Filters multipart body by leaving out everything else except
+ * first body part of given content type. */
+static int filter_body_f(struct sip_msg* msg, char* _content_type,
+			 char* ignored)
+{
+	char *start;
+	unsigned int len;
+	str *content_type, body;
+
+	body.s = get_body(msg);
+	if (body.s == 0) {
+		LM_ERR("Failed to get the message body\n");
+		return -1;
+	}
+	body.len = msg->len - (int)(body.s - msg->buf);
+	if (body.len == 0) {
+		LM_DBG("Message body has zero length\n");
+		return -1;
+	}
+	
+	content_type = (str *)_content_type;
+	start = body.s;
+	len = body.len;
+	
+	while (find_line_start("Content-Type: ", 14, &start, &len)) {
+	    start = start + 14;
+	    len = len - 14;
+	    if (len > content_type->len + 2) {
+		if (strncasecmp(start, content_type->s, content_type->len)
+		    == 0) {
+		    start = start + content_type->len;
+		    if ((*start != 13) || (*(start + 1) != 10)) {
+			LM_ERR("No CRLF found after content type\n");
+			return -1;
+		    }
+		    start = start + 2;
+		    len = len - content_type->len - 2;
+		    while ((len > 0) && ((*start == 13) || (*start == 10))) {
+			len = len - 1;
+			start = start + 1;
+		    }
+		    if (del_lump(msg, body.s - msg->buf, start - body.s, 0)
+			== 0) {
+			LM_ERR("Deleting lump <%.*s> failed\n",
+			       (int)(start - body.s), body.s);
+			return -1;
+		    }
+		    if (find_line_start("--Boundary", 10, &start, &len)) {
+			if (del_lump(msg, start - msg->buf, len, 0) == 0) {
+			    LM_ERR("Deleting lump <%.*s> failed\n",
+				   len, start);
+			    return -1;
+			} else {
+			    return 1;
+			}
+		    } else {
+			LM_ERR("Boundary not found after content\n");
+			return -1;
+		    }
+		}
+	    } else {
+		return -1;
+	    }
+	}
+	return -1;
+}
+
+
 static int remove_hf_f(struct sip_msg* msg, char* str_hf, char* foo)
 {
 	struct hdr_field *hf;
@@ -715,6 +837,9 @@ static int remove_hf_f(struct sip_msg* msg, char* str_hf, char* foo)
 	/* we need to be sure we have seen all HFs */
 	parse_headers(msg, HDR_EOH_F, 0);
 	for (hf=msg->headers; hf; hf=hf->next) {
+		/* for well known header names str_hf->s will be set to NULL 
+		   during parsing of openser.cfg and str_hf->len contains 
+		   the header type */
 		if(((str*)str_hf)->s==NULL)
 		{
 			if (((str*)str_hf)->len!=hf->type)
@@ -727,7 +852,7 @@ static int remove_hf_f(struct sip_msg* msg, char* str_hf, char* foo)
 		}
 		l=del_lump(msg, hf->name.s-msg->buf, hf->len, 0);
 		if (l==0) {
-			LOG(L_ERR, "ERROR: remove_hf_f: no memory\n");
+			LM_ERR("no memory\n");
 			return -1;
 		}
 		cnt++;
@@ -758,40 +883,18 @@ static int is_present_hf_f(struct sip_msg* msg, char* str_hf, char* foo)
 }
 
 
-
-static int fixup_regex(void** param, int param_no)
-{
-	regex_t* re;
-
-	DBG("module - fixing %s\n", (char*)(*param));
-	if (param_no!=1) return 0;
-	if ((re=pkg_malloc(sizeof(regex_t)))==0) return E_OUT_OF_MEM;
-	if (regcomp(re, *param, REG_EXTENDED|REG_ICASE|REG_NEWLINE) ){
-		pkg_free(re);
-		LOG(L_ERR, "ERROR: %s : bad re %s\n", exports.name, (char*)*param);
-		return E_BAD_RE;
-	}
-	/* free string */
-	pkg_free(*param);
-	/* replace it with the compiled re */
-	*param=re;
-	return 0;
-}
-
-
-
 static int fixup_substre(void** param, int param_no)
 {
 	struct subst_expr* se;
 	str subst;
 
-	DBG("%s module -- fixing %s\n", exports.name, (char*)(*param));
+	LM_DBG("%s module -- fixing %s\n", exports.name, (char*)(*param));
 	if (param_no!=1) return 0;
 	subst.s=*param;
 	subst.len=strlen(*param);
 	se=subst_parser(&subst);
 	if (se==0){
-		LOG(L_ERR, "ERROR: %s: bad subst. re %s\n", exports.name, 
+		LM_ERR("%s: bad subst. re %s\n", exports.name, 
 				(char*)*param);
 		return E_BAD_RE;
 	}
@@ -816,13 +919,13 @@ static int append_time_f(struct sip_msg* msg, char* p1, char *p2)
 
 	bd_time=gmtime(&now);
 	if (bd_time==NULL) {
-		LOG(L_ERR, "ERROR: append_time: gmtime failed\n");
+		LM_ERR("gmtime failed\n");
 		return -1;
 	}
 
 	len=strftime(time_str, MAX_TIME, TIME_FORMAT, bd_time);
 	if (len>MAX_TIME-2 || len==0) {
-		LOG(L_ERR, "ERROR: append_time: unexpected time length\n");
+		LM_ERR("unexpected time length\n");
 		return -1;
 	}
 
@@ -832,7 +935,7 @@ static int append_time_f(struct sip_msg* msg, char* p1, char *p2)
 
 	if (add_lump_rpl(msg, time_str, len+2, LUMP_RPL_HDR)==0)
 	{
-		LOG(L_ERR, "ERROR: append_time: unable to add lump\n");
+		LM_ERR("unable to add lump\n");
 		return -1;
 	}
 
@@ -842,25 +945,25 @@ static int append_time_f(struct sip_msg* msg, char* p1, char *p2)
 
 static int append_to_reply_f(struct sip_msg* msg, char* key, char* str0)
 {
-	xl_elem_t *model;
+	pv_elem_t *model;
 	str s0;
 
 	if(key==NULL)
 	{
-		LOG(L_ERR,"ERROR:append_to_reply: error - bad parameters\n");
+		LM_ERR("bad parameters\n");
 		return -1;
 	}
 
-	model = (xl_elem_t*)key;
-	if (xl_printf_s(msg, model, &s0)<0)
+	model = (pv_elem_t*)key;
+	if (pv_printf_s(msg, model, &s0)<0)
 	{
-		LOG(L_ERR,"textops:add_hf_helper: error - cannot print the format\n");
+		LM_ERR("cannot print the format\n");
 		return -1;
 	}
  
 	if ( add_lump_rpl( msg, s0.s, s0.len, LUMP_RPL_HDR)==0 )
 	{
-		LOG(L_ERR,"ERROR:append_to_reply : unable to add lump_rl\n");
+		LM_ERR("unable to add lump_rl\n");
 		return -1;
 	}
 
@@ -871,7 +974,7 @@ static int append_to_reply_f(struct sip_msg* msg, char* key, char* str0)
 /* add str1 to end of header or str1.r-uri.str2 */
 
 static int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
-		xl_elem_t *model, int mode, str *hfs)
+		pv_elem_t *model, int mode, str *hfs)
 {
 	struct lump* anchor;
 	struct hdr_field *hf;
@@ -880,7 +983,7 @@ static int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
 	str s0;
 
 	if (parse_headers(msg, HDR_EOH_F, 0) == -1) {
-		LOG(L_ERR, "textops:add_hf_helper: Error while parsing message\n");
+		LM_ERR("error while parsing message\n");
 		return -1;
 	}
 	
@@ -916,7 +1019,7 @@ static int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
 	}
 
 	if(anchor == 0) {
-		LOG(L_ERR, "textops:add_hf_helper: Can't get anchor\n");
+		LM_ERR("can't get anchor\n");
 		return -1;
 	}
 
@@ -924,10 +1027,9 @@ static int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
 		s0 = *str1;
 	} else {
 		if(model) {
-			if (xl_printf_s(msg, model, &s0)<0)
+			if (pv_printf_s(msg, model, &s0)<0)
 			{
-				LOG(L_ERR,
-				"textops:add_hf_helper: error - cannot print the format\n");
+				LM_ERR("cannot print the format\n");
 				return -1;
 			}
 		} else {
@@ -941,7 +1043,7 @@ static int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
 
 	s = (char*)pkg_malloc(len);
 	if (!s) {
-		LOG(L_ERR, "textops:add_hf_helper: No memory left\n");
+		LM_ERR("no pkg memory left\n");
 		return -1;
 	}
 
@@ -952,7 +1054,7 @@ static int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
 	}
 
 	if (insert_new_lump_before(anchor, s, len, 0) == 0) {
-		LOG(L_ERR, "textops:add_hf_helper: Can't insert lump\n");
+		LM_ERR("can't insert lump\n");
 		pkg_free(s);
 		return -1;
 	}
@@ -961,23 +1063,23 @@ static int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
 
 static int append_hf_1(struct sip_msg *msg, char *str1, char *str2 )
 {
-	return add_hf_helper(msg, 0, 0, (xl_elem_t*)str1, 0, 0);
+	return add_hf_helper(msg, 0, 0, (pv_elem_t*)str1, 0, 0);
 }
 
 static int append_hf_2(struct sip_msg *msg, char *str1, char *str2 )
 {
-	return add_hf_helper(msg, 0, 0, (xl_elem_t*)str1, 0,
+	return add_hf_helper(msg, 0, 0, (pv_elem_t*)str1, 0,
 			(str*)str2);
 }
 
 static int insert_hf_1(struct sip_msg *msg, char *str1, char *str2 )
 {
-	return add_hf_helper(msg, 0, 0, (xl_elem_t*)str1, 1, 0);
+	return add_hf_helper(msg, 0, 0, (pv_elem_t*)str1, 1, 0);
 }
 
 static int insert_hf_2(struct sip_msg *msg, char *str1, char *str2 )
 {
-	return add_hf_helper(msg, 0, 0, (xl_elem_t*)str1, 1, 
+	return add_hf_helper(msg, 0, 0, (pv_elem_t*)str1, 1, 
 			(str*)str2);
 }
 
@@ -1003,7 +1105,7 @@ static int is_method_f(struct sip_msg *msg, char *meth, char *str2 )
 	}
 	if(parse_headers(msg, HDR_CSEQ_F, 0)!=0 || msg->cseq==NULL)
 	{
-		LOG(L_ERR, "textops:is_method: ERROR - cannot parse cseq header\n");
+		LM_ERR("cannot parse cseq header\n");
 		return -1; /* should it be 0 ?!?! */
 	}
 	if(m->s==0)
@@ -1015,25 +1117,6 @@ static int is_method_f(struct sip_msg *msg, char *meth, char *str2 )
 				m->len)==0))?1:-1;
 }
 
-/*
- * Convert char* parameter to str* parameter
- */
-static int str_fixup(void** param, int param_no)
-{
-	str* s;
-
-	s = (str*)pkg_malloc(sizeof(str));
-	if (!s) {
-		LOG(L_ERR, "str_fixup(): No memory left\n");
-		return E_UNSPEC;
-	}
-
-	s->s = (char*)*param;
-	s->len = strlen(s->s);
-	*param = (void*)s;
-
-	return 0;
-}
 
 /*
  * Convert char* header_name to str* parameter
@@ -1046,7 +1129,7 @@ static int hname_fixup(void** param, int param_no)
 
 	s = (str*)pkg_malloc(sizeof(str));
 	if (!s) {
-		LOG(L_ERR, "textops:hname_fixup: No memory left\n");
+		LM_ERR("no pkg memory left\n");
 		return E_UNSPEC;
 	}
 
@@ -1054,7 +1137,7 @@ static int hname_fixup(void** param, int param_no)
 	s->len = strlen(s->s);
 	if(s->len==0)
 	{
-		LOG(L_ERR,"textops:hname_fixup: empty header name parameter\n");
+		LM_ERR("empty header name parameter\n");
 		pkg_free(s);
 		return E_UNSPEC;
 	}
@@ -1065,7 +1148,7 @@ static int hname_fixup(void** param, int param_no)
 	
 	if (parse_hname2(s->s, s->s + ((s->len<4)?4:s->len), &hdr)==0)
 	{
-		LOG(L_ERR,"textops:hname_fixup: error parsing header name\n");
+		LM_ERR("error parsing header name\n");
 		pkg_free(s);
 		return E_UNSPEC;
 	}
@@ -1075,21 +1158,31 @@ static int hname_fixup(void** param, int param_no)
 
 	if (hdr.type!=HDR_OTHER_T && hdr.type!=HDR_ERROR_T)
 	{
-		LOG(L_INFO,"INFO:textops:hname_fixup: using "
-				"hdr type (%d) instead of <%.*s>\n",
+		LM_INFO("using hdr type (%d) instead of <%.*s>\n",
 				hdr.type, s->len, s->s);
 		pkg_free(s->s);
 		s->s = NULL;
 		s->len = hdr.type;
 	} else {
-		LOG(L_INFO,"INFO:textops:hname_fixup: using "
-				"hdr type name <%.*s>\n", s->len, s->s);
+		LM_INFO("using hdr type name <%.*s>\n", s->len, s->s);
 	}
 	
 	*param = (void*)s;
 	return 0;
 }
 
+static int free_hname_fixup(void** param, int param_no)
+{
+	if(*param)
+	{
+		if(((str*)(*param))->s)
+			pkg_free(((str*)(*param))->s);
+		pkg_free(*param);
+		*param = 0;
+	}
+	return 0;
+}
+
 /*
  * Convert char* method to str* parameter
  */
@@ -1102,7 +1195,7 @@ static int fixup_method(void** param, int param_no)
 	
 	s = (str*)pkg_malloc(sizeof(str));
 	if (!s) {
-		LOG(L_ERR, "textops:fixup_method: No memory left\n");
+		LM_ERR("no pkg memory left\n");
 		return E_UNSPEC;
 	}
 
@@ -1110,7 +1203,7 @@ static int fixup_method(void** param, int param_no)
 	s->len = strlen(s->s);
 	if(s->len==0)
 	{
-		LOG(L_ERR,"textops:fixup_method: empty method name\n");
+		LM_ERR("empty method name\n");
 		pkg_free(s);
 		return E_UNSPEC;
 	}
@@ -1127,7 +1220,7 @@ static int fixup_method(void** param, int param_no)
 	}
 	if(parse_methods(s, &method)!=0)
 	{
-		LOG(L_ERR,"textops:fixup_method: bad method names\n");
+		LM_ERR("bad method names\n");
 		pkg_free(s);
 		return E_UNSPEC;
 	}
@@ -1136,25 +1229,23 @@ static int fixup_method(void** param, int param_no)
 	{
 		if(method==METHOD_UNDEF || method&METHOD_OTHER)
 		{
-			LOG(L_ERR,
-				"textops:fixup_method: unknown method in list [%.*s/%d]"
-				" - must be only defined methods\n",
+			LM_ERR("unknown method in list [%.*s/%d] - must be only defined methods\n",
 				s->len, s->s, method);
 			return E_UNSPEC;
 		}
-		DBG("textops:fixup_method: using id for methods [%.*s/%d]\n",
+		LM_DBG("using id for methods [%.*s/%d]\n",
 				s->len, s->s, method);
 		s->s = 0;
 		s->len = method;
 	} else {
 		if(method!=METHOD_UNDEF && method!=METHOD_OTHER)
 		{
-			DBG("textops:fixup_method: using id for method [%.*s/%d]\n",
+			LM_DBG("using id for method [%.*s/%d]\n",
 				s->len, s->s, method);
 			s->s = 0;
 			s->len = method;
 		} else
-			DBG("textops:fixup_method: name for method [%.*s/%d]\n",
+			LM_DBG("name for method [%.*s/%d]\n",
 				s->len, s->s, method);
 	}
 
@@ -1174,12 +1265,12 @@ static int fixup_privacy(void** param, int param_no)
     p.len = strlen(p.s);
 
     if (p.len == 0) {
-	LOG(L_ERR,"textops:fixup_privacy: empty privacy value\n");
+	LM_ERR("empty privacy value\n");
 	return E_UNSPEC;
     }
 
     if (parse_priv_value(p.s, p.len, &val) != p.len) {
-	LOG(L_ERR,"textops:fixup_privacy: invalid privacy value\n");
+	LM_ERR("invalid privacy value\n");
 	return E_UNSPEC;
     }
     
@@ -1188,16 +1279,18 @@ static int fixup_privacy(void** param, int param_no)
 }
 
 /*
- * Convert char* parameter to xl_elem parameter
+ * Convert char* parameter to pv_elem parameter
  */
 static int it_list_fixup(void** param, int param_no)
 {
-	xl_elem_t *model;
+	pv_elem_t *model;
+	str s;
 	if(*param)
 	{
-		if(xl_parse_format((char*)(*param), &model, XL_DISABLE_COLORS)<0)
+		s.s = (char*)(*param); s.len = strlen(s.s);
+		if(pv_parse_format(&s, &model)<0)
 		{
-			LOG(L_ERR, "ERROR:textops:item_list_fixup: wrong format[%s]\n",
+			LM_ERR("wrong format[%s]\n",
 				(char*)(*param));
 			return E_UNSPEC;
 		}
@@ -1214,8 +1307,7 @@ static int add_header_fixup(void** param, int param_no)
 	} else if(param_no==2) {
 		return hname_fixup(param, param_no);
 	} else {
-		LOG(L_ERR,
-			"ERROR:textops:add_header_fixup: wrong number of parameters\n");
+		LM_ERR("wrong number of parameters\n");
 		return E_UNSPEC;
 	}
 }
@@ -1234,13 +1326,11 @@ static int fixup_body_type(void** param, int param_no)
 		} else {
 			r = decode_mime_type( p, p+strlen(p) , &type);
 			if (r==0) {
-				LOG(L_ERR,"ERROR:textops:fixup_body_type: unsupported "
-					"mime <%s>\n",p);
+				LM_ERR("unsupported mime <%s>\n",p);
 				return E_CFG;
 			}
 			if ( r!=p+strlen(p) ) {
-				LOG(L_ERR,"ERROR:textops:fixup_body_type: multiple mimes not "
-					"supported!\n");
+				LM_ERR("multiple mimes not supported!\n");
 				return E_CFG;
 			}
 		}
@@ -1262,13 +1352,12 @@ static int has_body_f(struct sip_msg *msg, char *type, char *str2 )
 
 	/* all headears are already parsed by "get_body" */
 	if (msg->content_length==0) {
-		LOG (L_ERR, "ERROR:textops:has_body: very bogus message with body "
-			"but no content length hdr\n");
+		LM_ERR("very bogus message with body, but no content length hdr\n");
 		return -1;
 	}
 
 	if (get_content_length (msg)==0) {
-		DBG("DEBUG:textops:has_body: content length is zero\n");
+		LM_DBG("content length is zero\n");
 		/* Nothing to see here, please move on. */
 		return -1;
 	}
@@ -1279,8 +1368,7 @@ static int has_body_f(struct sip_msg *msg, char *type, char *str2 )
 
 	mime = parse_content_type_hdr (msg);
 	if (mime<0) {
-		LOG (L_ERR, "ERROR:textops:has_body: failed to extract "
-			"content type hdr\n");
+		LM_ERR("failed to extract content type hdr\n");
 		return -1;
 	}
 	if (mime==0) {
@@ -1288,7 +1376,7 @@ static int has_body_f(struct sip_msg *msg, char *type, char *str2 )
 		 * assume APPLICATION/SDP  --bogdan */
 		mime = ((TYPE_APPLICATION << 16) + SUBTYPE_SDP);
 	}
-	DBG("DBUG:textops:has_body: Content type is %d\n",mime);
+	LM_DBG("content type is %d\n",mime);
 
 	if ( (unsigned int)mime!=(unsigned int)(unsigned long)type )
 		return -1;
