@@ -1,5 +1,5 @@
 /*
- * $Id: sip_msg.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: sip_msg.c 3148 2007-11-15 20:31:24Z bogdan_iancu $
  * 
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -78,7 +78,7 @@
 
 
 
-inline struct via_body* via_body_cloner( char* new_buf,
+inline static struct via_body* via_body_cloner( char* new_buf,
 					char *org_buf, struct via_body *param_org_via, char **p)
 {
 	struct via_body *new_via;
@@ -148,6 +148,9 @@ inline struct via_body* via_body_cloner( char* new_buf,
 					case PARAM_ALIAS:
 							new_via->alias = new_vp;
 							break;
+					case PARAM_MADDR:
+							new_via->maddr = new_vp;
+							break;
 				}
 
 				if (last_new_vp)
@@ -259,7 +262,7 @@ static inline int clone_authorized_hooks(struct sip_msg* new,
 	while(ptr) {
 		if (ptr == hook1) {
 			if (!new->authorization || !new->authorization->parsed) {
-				LOG(L_CRIT, "BUG: Error in message cloner (authorization)\n");
+				LM_CRIT("message cloner (authorization) failed\n");
 				return -1;
 			}
 			((struct auth_body*)new->authorization->parsed)->authorized =
@@ -269,7 +272,7 @@ static inline int clone_authorized_hooks(struct sip_msg* new,
 		
 		if (ptr == hook2) {
 			if (!new->proxy_auth || !new->proxy_auth->parsed) {
-				LOG(L_CRIT, "BUG: Error in message cloner (proxy_auth)\n");
+				LM_CRIT("message cloner (proxy_auth) failed\n");
 				return -1;
 			}
 			((struct auth_body*)new->proxy_auth->parsed)->authorized =
@@ -378,13 +381,13 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len )
 			case HDR_PPI_T:
 			case HDR_PAI_T:
 			case HDR_PRIVACY_T:
+			case HDR_RETRY_AFTER_T:
 				/* we ignore them for now even if they have something parsed*/
 				break;
 
 			default:
 				if (hdr->parsed) {
-					LOG(L_WARN, "WARNING: sip_msg_cloner: "
-						"header body ignored: %d\n", hdr->type );
+					LM_WARN("header body ignored: %d\n", hdr->type );
 				}
 				break;
 		}/*switch*/
@@ -424,7 +427,7 @@ do { \
 	p=(char *)shm_malloc(len);
 	if (!p)
 	{
-		LOG(L_ERR , "ERROR: sip_msg_cloner: cannot allocate memory\n" );
+		LM_ERR("no more share memory\n" );
 		return 0;
 	}
 	if (sip_msg_len)
