@@ -1,5 +1,5 @@
 /*
- * $Id: sip_msg.h 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: sip_msg.h 3907 2008-03-10 14:12:12Z bogdan_iancu $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -35,4 +35,23 @@
 struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len );
 
 
+static inline void clean_msg_clone(struct sip_msg *msg,void *min, void *max)
+{
+	struct hdr_field *hdr;
+
+	/* free header's parsed structures that were added in pkg mem */
+	for( hdr=msg->headers ; hdr ; hdr=hdr->next ) {
+		if ( hdr->parsed && hdr_allocs_parse(hdr) &&
+		(hdr->parsed<min || hdr->parsed>=max)) {
+			/* header parsed filed doesn't point inside uas.request memory
+			 * chunck -> it was added by failure funcs.-> free it as pkg */
+			LM_DBG("removing hdr->parsed %d\n",	hdr->type);
+			clean_hdr_field(hdr);
+			hdr->parsed = 0;
+		}
+	}
+}
+
+
+
 #endif
