@@ -1,5 +1,5 @@
 /*
- * $Id: t_dlg.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: t_dlg.c 2657 2007-08-28 15:03:09Z henningw $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -31,7 +31,7 @@ int t_newdlg( struct sip_msg *msg )
 	return 0;
 }
 
-dlg_t t_getdlg() {
+dlg_t t_getdlg(void) {
 	return dlg;
 }
 
