@@ -1,5 +1,5 @@
 /*
- * $Id: t_fwd.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: t_fwd.c 3880 2008-03-07 12:27:42Z bogdan_iancu $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -79,7 +79,7 @@ void t_on_branch( unsigned int go_to )
 }
 
 
-unsigned int get_on_branch()
+unsigned int get_on_branch(void)
 {
 	return goto_on_branch;
 }
@@ -96,7 +96,7 @@ static inline int pre_print_uac_request( struct cell *t, int branch,
 	if (!t_calc_branch(t, branch, request->add_to_branch_s,
 			&request->add_to_branch_len ))
 	{
-		LOG(L_ERR, "ERROR:pre_print_uac_request: branch computation failed\n");
+		LM_ERR("branch computation failed\n");
 		goto error;
 	}
 
@@ -109,7 +109,7 @@ static inline int pre_print_uac_request( struct cell *t, int branch,
 		t->uac[branch].path_vec.s =
 			shm_resize(t->uac[branch].path_vec.s, request->path_vec.len+1);
 		if (t->uac[branch].path_vec.s==NULL) {
-			LOG(L_ERR,"ERROR:pre_print_uac_request: shm_resize failed\n");
+			LM_ERR("shm_resize failed\n");
 			goto error;
 		}
 		t->uac[branch].path_vec.len = request->path_vec.len;
@@ -125,7 +125,7 @@ static inline int pre_print_uac_request( struct cell *t, int branch,
 		/* need to pkg_malloc the dst_uri */
 		if ( request->dst_uri.len ) {
 			if ( (p=pkg_malloc(request->dst_uri.len))==0 ) {
-				LOG(L_ERR,"ERROR:tm:pre_print_uac_request: no more pkg mem\n");
+				LM_ERR("no more pkg mem\n");
 				ser_error=E_OUT_OF_MEM;
 				goto error;
 			}
@@ -134,7 +134,7 @@ static inline int pre_print_uac_request( struct cell *t, int branch,
 		}
 		/* need to pkg_malloc the new_uri */
 		if ( (p=pkg_malloc(request->new_uri.len))==0 ) {
-			LOG(L_ERR,"ERROR:tm:pre_print_uac_request: no more pkg mem\n");
+			LM_ERR("no more pkg mem\n");
 			ser_error=E_OUT_OF_MEM;
 			goto error;
 		}
@@ -148,8 +148,8 @@ static inline int pre_print_uac_request( struct cell *t, int branch,
 
 		_tm_branch_index = branch+1;
 		if (run_top_route(branch_rlist[t->on_branch], request)&ACT_FL_DROP) {
-			DBG("DEBUG:tm:pre_print_uac_request: dropping branch <%.*s>\n",
-				request->new_uri.len, request->new_uri.s);
+			LM_DBG("dropping branch <%.*s>\n", request->new_uri.len,
+					request->new_uri.s);
 			_tm_branch_index = 0;
 			goto error;
 		}
@@ -180,7 +180,7 @@ static inline char *print_uac_request(struct sip_msg *i_req, unsigned int *len,
 	/* build the shm buffer now */
 	buf=build_req_buf_from_sip_req( i_req, len, send_sock, proto );
 	if (!buf) {
-		LOG(L_ERR, "ERROR:tm:print_uac_request: no pkg_mem\n"); 
+		LM_ERR("no more pkg_mem\n"); 
 		ser_error=E_OUT_OF_MEM;
 		goto error01;
 	}
@@ -188,7 +188,7 @@ static inline char *print_uac_request(struct sip_msg *i_req, unsigned int *len,
 	shbuf=(char *)shm_malloc(*len);
 	if (!shbuf) {
 		ser_error=E_OUT_OF_MEM;
-		LOG(L_ERR, "ERROR:tm:print_uac_request: no shmem\n");
+		LM_ERR("no more share memory\n");
 		goto error02;
 	}
 	memcpy( shbuf, buf, *len );
@@ -232,7 +232,7 @@ static inline struct proxy_l* shm_clone_proxy(struct proxy_l *sp,
 
 	dp = (struct proxy_l*)shm_malloc(sizeof(struct proxy_l));
 	if (dp==NULL) {
-		LOG(L_ERR,"ERROR:tm:shm_clone_proxy:no more shm memory\n");
+		LM_ERR("no more shm memory\n");
 		return 0;
 	}
 	memset( dp , 0 , sizeof(struct proxy_l));
@@ -275,21 +275,20 @@ error0:
    and FR must be ticking, whereas the request is "forwarded"
    using a non-SIP way and will be replied the same way
 */
-int add_blind_uac( /*struct cell *t*/ )
+int add_blind_uac(void)  /*struct cell *t*/
 {
 	unsigned short branch;
 	struct cell *t;
 
 	t=get_t();
 	if (t==T_UNDEFINED || !t ) {
-		LOG(L_ERR, "ERROR: add_blind_uac: no transaction context\n");
+		LM_ERR("no transaction context\n");
 		return -1;
 	}
 
 	branch=t->nr_of_outgoings;	
 	if (branch==MAX_BRANCHES) {
-		LOG(L_ERR, "ERROR: add_blind_uac: "
-			"maximum number of branches exceeded\n");
+		LM_ERR("maximum number of branches exceeded\n");
 		return -1;
 	}
 	/* make sure it will be replied */
@@ -318,7 +317,7 @@ static inline int update_uac_dst( struct sip_msg *request,
 	send_sock = get_send_socket( request, &uac->request.dst.to ,
 			uac->request.dst.proto );
 	if (send_sock==0) {
-		LOG(L_ERR, "ERROR:tm:update_uac_dst: can't fwd to af %d, proto %d "
+		LM_ERR("failed to fwd to af %d, proto %d "
 			" (no corresponding listening socket)\n",
 			uac->request.dst.to.s.sa_family, uac->request.dst.proto );
 		ser_error=E_NO_SOCKET;
@@ -348,6 +347,28 @@ static inline int update_uac_dst( struct sip_msg *request,
 }
 
 
+static inline unsigned int count_local_rr(struct sip_msg *req)
+{
+	unsigned int cnt = 0;
+	struct lump *r;
+
+	/* we look for the RR anchors only 
+	 * in the main list (no after or before) */
+	for( r=req->add_rm ; r ; r=r->next )
+		if ( r->type==HDR_RECORDROUTE_T && r->op==LUMP_NOP) {
+			if (r->after && r->after->op==LUMP_ADD_OPT) {
+				if (r->after->flags&LUMPFLAG_COND_TRUE) {
+					cnt++;
+				}
+			} else {
+				cnt++;
+			}
+		}
+
+	return cnt;
+}
+
+
 /* introduce a new uac to transaction; returns its branch id (>=0)
    or error (<0); it doesn't send a message yet -- a reply to it
    might interfere with the processes of adding multiple branches
@@ -361,14 +382,14 @@ static int add_uac( struct cell *t, struct sip_msg *request, str *uri,
 
 	branch=t->nr_of_outgoings;
 	if (branch==MAX_BRANCHES) {
-		LOG(L_ERR, "ERROR:tm:add_uac: maximum number of branches exceeded\n");
+		LM_ERR("maximum number of branches exceeded\n");
 		ret=E_CFG;
 		goto error;
 	}
 
 	/* check existing buffer -- rewriting should never occur */
 	if (t->uac[branch].request.buffer.s) {
-		LOG(L_CRIT, "ERROR:tm:add_uac: buffer rewrite attempt\n");
+		LM_CRIT("buffer rewrite attempt\n");
 		ret=ser_error=E_BUG;
 		goto error;
 	}
@@ -420,6 +441,7 @@ static int add_uac( struct cell *t, struct sip_msg *request, str *uri,
 		request->first_line.u.request.method.len+1;
 	t->uac[branch].uri.len=request->new_uri.len;
 	t->uac[branch].br_flags = getb0flags();
+	t->uac[branch].added_rr = count_local_rr( request );
 	t->nr_of_outgoings++;
 
 	/* done! */
@@ -447,7 +469,7 @@ int e2e_cancel_branch( struct sip_msg *cancel_msg, struct cell *t_cancel,
 	str bk_path_vec;
 
 	if (t_cancel->uac[branch].request.buffer.s) {
-		LOG(L_CRIT, "ERROR: e2e_cancel_branch: buffer rewrite attempt\n");
+		LM_CRIT("buffer rewrite attempt\n");
 		ret=ser_error=E_BUG;
 		goto error;
 	}
@@ -478,7 +500,7 @@ int e2e_cancel_branch( struct sip_msg *cancel_msg, struct cell *t_cancel,
 		t_invite->uac[branch].request.dst.send_sock,
 		t_invite->uac[branch].request.dst.proto);
 	if (!shbuf) {
-		LOG(L_ERR, "ERROR: e2e_cancel_branch: printing e2e cancel failed\n");
+		LM_ERR("printing e2e cancel failed\n");
 		ret=ser_error=E_OUT_OF_MEM;
 		goto error01;
 	}
@@ -506,60 +528,26 @@ error:
 
 
 
-void e2e_cancel( struct sip_msg *cancel_msg, 
-	struct cell *t_cancel, struct cell *t_invite )
+void cancel_invite(struct sip_msg *cancel_msg,
+								struct cell *t_cancel, struct cell *t_invite )
 {
-	branch_bm_t cancel_bm;
+	branch_bm_t cancel_bitmap;
 	branch_bm_t dummy_bm;
-	int i;
-	int lowest_error;
-	str backup_uri;
 	str reason;
-	int ret;
+	unsigned int i;
+	int lowest_error;
 
-	cancel_bm=0;
 	lowest_error=0;
+	cancel_bitmap=0;
 
-	/* e2e_cancel_branch() makes no RURI parsing, so no need to 
-	 * save the -> parse_uri_ok */
-	backup_uri = cancel_msg->new_uri;
+	/* send back 200 OK as per RFC3261 */
+	reason.s = CANCELING;
+	reason.len = sizeof(CANCELING)-1;
+	t_reply( t_cancel, cancel_msg, 200, &reason );
 
-	/* determine which branches to cancel ... */
-	which_cancel( t_invite, &cancel_bm );
-	t_cancel->nr_of_outgoings=t_invite->nr_of_outgoings;
-	t_cancel->first_branch=t_invite->first_branch;
-	/* fix label -- it must be same for reply matching */
-	t_cancel->label=t_invite->label;
-	/* ... and install CANCEL UACs */
-	for (i=t_invite->first_branch; i<t_invite->nr_of_outgoings; i++) {
-		if (cancel_bm & (1<<i)) {
-			ret=e2e_cancel_branch(cancel_msg, t_cancel, t_invite, i);
-			if (ret<0) cancel_bm &= ~(1<<i);
-			if (ret<lowest_error) lowest_error=ret;
-		}
-	}
-	/* restore new_uri */
-	cancel_msg->new_uri = backup_uri;
-	cancel_msg->parsed_uri_ok = 0;
-
-	/* set flags */
-	t_cancel->uas.request->flags = cancel_msg->flags;
-
-	/* send them out */
-	for (i=t_cancel->first_branch; i<t_cancel->nr_of_outgoings; i++) {
-		if (cancel_bm & (1<<i)) {
-			if ( has_tran_tmcbs( t_cancel, TMCB_REQUEST_BUILT) ) {
-				set_extra_tmcb_params( &t_cancel->uac[i].request.buffer,
-					&t_cancel->uac[i].request.dst);
-				run_trans_callbacks( TMCB_REQUEST_BUILT, t_cancel, cancel_msg,
-					0, -cancel_msg->REQ_METHOD);
-			}
-			if (SEND_BUFFER( &t_cancel->uac[i].request)==-1) {
-				LOG(L_ERR, "ERROR: e2e_cancel: send failed\n");
-			}
-			start_retr( &t_cancel->uac[i].request );
-		}
-	}
+	/* generate local cancels for all branches */
+	which_cancel(t_invite, &cancel_bitmap );
+	cancel_uacs(t_invite, cancel_bitmap );
 
 	/* internally cancel branches with no received reply */
 	for (i=t_invite->first_branch; i<t_invite->nr_of_outgoings; i++) {
@@ -572,71 +560,10 @@ void e2e_cancel( struct sip_msg *cancel_msg,
 				lowest_error = -1; /* force sending 500 error */
 		}
 	}
-
-	/* do not attmpt to send reply for CANCEL if we already did it once;
-	 * to work arround the race between receiveing reply and generating
-	 * local reply, we better check if we are in failure route (which means that
-	 * the reply to UAC is /to be/ sent) or if was actually sent out */
-	/* calling here t_relay from within failure route will lead to dead lock
-	 * on the transaction's reply lock -bogdan */
-	if (route_type==FAILURE_ROUTE || t_cancel->uas.status>=200)
-		return;
-
-	/* if error occurred, let it know upstream (final reply
-	   will also move the transaction on wait state
-	*/
-	if (lowest_error<0) {
-		LOG(L_ERR, "ERROR: cancel error\n");
-		reason.s = "cancel error";
-		reason.len = 12;
-		t_reply( t_cancel, cancel_msg, 500, &reason);
-	/* if there are pending branches, let upstream know we
-	   are working on it
-	*/
-	} else if (cancel_bm) {
-		DBG("DEBUG: e2e_cancel: e2e cancel proceeding\n");
-		t_cancel->flags |= T_HOPBYHOP_CANCEL_FLAG;
-		reason.s = CANCELING;
-		reason.len = sizeof(CANCELING)-1;
-		t_reply( t_cancel, cancel_msg, 200, &reason );
-	/* if the transaction exists, but there is no more pending
-	   branch, tell upstream we're done
-	*/
-	} else {
-		DBG("DEBUG: e2e_cancel: e2e cancel -- no more pending branches\n");
-		reason.s = CANCEL_DONE;
-		reason.len = sizeof(CANCEL_DONE)-1;
-		t_reply( t_cancel, cancel_msg, 200, &reason );
-	}
-
-#ifdef LOCAL_487
-
-	/* local 487s have been deprecated -- it better handles
-	 * race conditions (UAS sending 200); hopefully there are
-	 * no longer UACs who go crazy waiting for the 487 whose
-	 * forwarding is being blocked by other unresponsive branch
-	 */
-
-	/* we could await downstream UAS's 487 replies; however,
-	   if some of the branches does not do that, we could wait
-	   long time and annoy upstream UAC which wants to see 
-	   a result of CANCEL quickly
-	*/
-	DBG("DEBUG: e2e_cancel: sending 487\n");
-	/* in case that something in the meantime has been sent upstream
-	   (like if FR hit at the same time), don't try to send */
-	if (t_invite->uas.status>=200) return;
-	/* there is still a race-condition -- the FR can hit now; that's
-	   not too bad -- we take care in t_reply's REPLY_LOCK; in
-	   the worst case, both this t_reply and other replier will
-	   try, and the later one will result in error message 
-	   "can't reply twice"
-	*/
-	t_reply(t_invite, t_invite->uas.request, 487, CANCELED );
-#endif
 }
 
 
+
 /* function returns:
  *       1 - forward successful
  *      -1 - error during forward
@@ -668,15 +595,15 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 		t_invite=t_lookupOriginalT(  p_msg );
 		if (t_invite!=T_NULL_CELL) {
 			t_invite->flags |= T_WAS_CANCELLED_FLAG;
-			e2e_cancel( p_msg, t, t_invite );
+			cancel_invite( p_msg, t, t_invite );
 			return 1;
 		}
 	}
 
 	/* do not forward requests which were already cancelled*/
-	if (was_cancelled(t)) {
-		LOG(L_ERR,"ERROR:tm:t_forward_nonack: discarding fwd for "
-				"a cancelled transaction\n");
+	if (was_cancelled(t) || no_new_branches(t)) {
+		LM_ERR("discarding fwd for a cancelled/6xx transaction\n");
+		ser_error = E_NO_DESTINATION;
 		return -1;
 	}
 
@@ -695,7 +622,7 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 	t->first_branch=t->nr_of_outgoings;
 
 	/* on first-time forwarding, use current uri, later only what
-)	   is in additional branches (which may be continuously refilled)
+	   is in additional branches (which may be continuously refilled)
 	*/
 	if (t->first_branch==0) {
 		try_new=1;
@@ -734,15 +661,18 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 	setb0flags(bk_br_flags);
 	/* update on_branch, if modified */
 	t->on_branch = get_on_branch();
+	/* update flags, if changed in branch route */
+	t->uas.request->flags = p_msg->flags;
 
 	/* things went wrong ... no new branch has been fwd-ed at all */
 	if (added_branches==0) {
 		if (try_new==0) {
-			LOG(L_ERR, "ERROR:tm:t_forward_nonack: no branch for "
-				"forwarding\n");
+			ser_error = E_NO_DESTINATION;
+			LM_ERR("no branch for forwarding\n");
 			return -1;
 		}
-		LOG(L_ERR, "ERROR:tm:t_forward_nonack: failure to add branches\n");
+		LM_ERR("failure to add branches\n");
+		ser_error = lowest_ret;
 		return lowest_ret;
 	}
 
@@ -751,24 +681,22 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 	for (i=t->first_branch; i<t->nr_of_outgoings; i++) {
 		if (added_branches & (1<<i)) {
 			do {
-				if (check_blacklists( t->uac[i].proxy->port,
-				t->uac[i].proxy->proto,
+				if (check_blacklists( t->uac[i].request.dst.proto,
 				&t->uac[i].request.dst.to,
 				t->uac[i].request.buffer.s,
 				t->uac[i].request.buffer.len)) {
-					DBG("DEBUG:tm:t_forward_nonack: blocked by blacklists\n");
+					LM_DBG("blocked by blacklists\n");
 					ser_error=E_IP_BLOCKED;
 				} else {
 					if (SEND_BUFFER( &t->uac[i].request)==0) {
 						ser_error = 0;
 						break;
 					}
-					LOG(L_ERR, "ERROR:tm:t_forward_nonack: sending request "
-						"failed\n");
+					LM_ERR("sending request failed\n");
 					ser_error=E_SEND;
 				}
 				/* get next dns entry */
-				if ( t->uac[i].proxy &&
+				if ( t->uac[i].proxy==0 ||
 				get_next_su( t->uac[i].proxy, &t->uac[i].request.dst.to,
 				(ser_error==E_IP_BLOCKED)?0:1)!=0 )
 					break;
@@ -815,12 +743,12 @@ int t_replicate(struct sip_msg *p_msg, str *dst, int flags)
 	*/
 
 	if ( set_dst_uri( p_msg, dst)!=0 ) {
-		LOG(L_ERR,"ERROR:tm:t_replicate: failed to set dst uri\n");
+		LM_ERR("failed to set dst uri\n");
 		return -1;
 	}
 
 	if ( branch_uri2dset( GET_RURI(p_msg) )!=0 ) {
-		LOG(L_ERR,"ERROR:tm:t_replicate: failed to convert uri to dst\n");
+		LM_ERR("failed to convert uri to dst\n");
 		return -1;
 	}
 
