@@ -1,5 +1,5 @@
 /*
- * $Id: t_lookup.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: t_lookup.c 3263 2007-12-05 15:29:03Z henningw $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -148,37 +148,37 @@ static struct cell *cancelled_T;
 static struct cell *e2eack_T;
 
 
-struct cell *get_t() { return T; }
+struct cell *get_t(void) { return T; }
 void set_t(struct cell *t) { T=t; }
-void init_t() {set_t(T_UNDEFINED);}
+void init_t(void) {set_t(T_UNDEFINED);}
 
-struct cell *get_cancelled_t() { return cancelled_T; }
-void reset_cancelled_t() { cancelled_T=T_UNDEFINED; }
+struct cell *get_cancelled_t(void) { return cancelled_T; }
+void reset_cancelled_t(void) { cancelled_T=T_UNDEFINED; }
 
-struct cell *get_e2eack_t() { return e2eack_T; }
-void reset_e2eack_t() { e2eack_T=T_UNDEFINED; }
+struct cell *get_e2eack_t(void) { return e2eack_T; }
+void reset_e2eack_t(void) { e2eack_T=T_UNDEFINED; }
 
 
 
 static inline int parse_dlg( struct sip_msg *msg )
 {
 	if (parse_headers(msg, HDR_FROM_F | HDR_CSEQ_F | HDR_TO_F, 0)==-1) {
-		LOG(L_ERR, "ERROR: parse_dlg: From or Cseq or To invalid\n");
+		LM_ERR("From or Cseq or To invalid\n");
 		return 0;
 	}
 	if ((msg->from==0)||(msg->cseq==0)||(msg->to==0)) {
-		LOG(L_ERR, "ERROR: parse_dlg: missing From or Cseq or To\n");
+		LM_ERR("missing From or Cseq or To\n");
 		return 0;
 	}
 
 	if (parse_from_header(msg)<0) {
-		LOG(L_ERR, "ERROR: parse_dlg: From broken\n");
+		LM_ERR("From broken\n");
 		return 0;
 	}
 	/* To is automatically parsed through HDR_TO in parse bitmap,
 	 * we don't need to worry about it now
 	if (parse_to_header(msg)==-1) {
-		LOG(L_ERR, "ERROR: tid_matching: To broken\n");
+		LM_ERR("To broken\n");
 		return 0;
 	}
 	*/
@@ -196,7 +196,7 @@ static inline int partial_dlg_matching(struct sip_msg *t_msg, struct sip_msg *p_
 		return 0;
 	inv_from=get_from(t_msg);
 	if (!inv_from) {
-		LOG(L_ERR, "ERROR: partial_dlg_matching: INV/From not parsed\n");
+		LM_ERR("INV/From not parsed\n");
 		return 0;
 	}
 	if (inv_from->tag_value.len!=get_from(p_msg)->tag_value.len)
@@ -329,7 +329,7 @@ static int matching_3261( struct sip_msg *p_msg, struct cell **trans,
 			if (!dlg_parsed) {
 				dlg_parsed=1;
 				if (!parse_dlg(p_msg)) {
-					LOG(L_ERR, "ERROR: matching_3261: dlg parsing failed\n");
+					LM_ERR("dlg parsing failed\n");
 					return 0;
 				}
 			}
@@ -348,7 +348,7 @@ static int matching_3261( struct sip_msg *p_msg, struct cell **trans,
 		if (!via_matching(t_msg->via1 /* inv via */, via1 /* ack */ ))
 			continue;
 		/* all matched -- we found the transaction ! */
-		DBG("DEBUG: RFC3261 transaction matched, tid=%.*s\n",
+		LM_DBG("RFC3261 transaction matched, tid=%.*s\n",
 			via1->tid.len, via1->tid.s);
 
 		*trans=p_cell;
@@ -361,7 +361,7 @@ static int matching_3261( struct sip_msg *p_msg, struct cell **trans,
 		*trans=e2e_ack_trans;
 		return ret;
 	}
-	DBG("DEBUG: RFC3261 transaction matching failed\n");
+	LM_DBG("RFC3261 transaction matching failed\n");
 	return 0;
 }
 
@@ -393,7 +393,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 
 	/* parse all*/
 	if (check_transaction_quadruple(p_msg)==0) {
-		LOG(L_ERR, "ERROR: TM module: t_lookup_request: too few headers\n");
+		LM_ERR("too few headers\n");
 		set_t(0);
 		/* stop processing */
 		return 0;
@@ -403,7 +403,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 	if (!p_msg->hash_index)
 		p_msg->hash_index=tm_hash( p_msg->callid->body , 
 			get_cseq(p_msg)->number ) ;
-	DBG("t_lookup_request: start searching: hash=%d, isACK=%d\n",
+	LM_DBG("start searching: hash=%d, isACK=%d\n",
 		p_msg->hash_index,isACK);
 
 
@@ -416,7 +416,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 	 * comparison of many header fields
 	 */
 	if (!p_msg->via1) {
-		LOG(L_ERR, "ERROR: t_lookup_request: no via\n");
+		LM_ERR("no via\n");
 		set_t(0);
 		return 0;
 	}
@@ -439,7 +439,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 	/* ok -- it's ugly old-fashioned transaction matching -- it is
 	 * a bit simplified to be fast -- we don't do all the comparisons
 	 * of parsed uri, which was simply too bloated */
-	DBG("DEBUG: proceeding to pre-RFC3261 transaction matching\n");
+	LM_DBG("proceeding to pre-RFC3261 transaction matching\n");
 
 	/* lock the whole entry*/
 	LOCK_HASH(p_msg->hash_index);
@@ -470,7 +470,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 			if (via1_matching && !EQ_VIA_STR(via1)) continue;
 
 			/* request matched ! */
-			DBG("DEBUG: non-ACK matched\n");
+			LM_DBG("non-ACK matched\n");
 			goto found;
 		} else { /* it's an ACK request*/
 			/* ACK's relate only to INVITEs */
@@ -520,7 +520,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 			if (via1_matching && !EQ_VIA_STR(via1)) continue;
 
 			/* wow -- we survived all the check! we matched! */
-			DBG("DEBUG: non-2xx ACK matched\n");
+			LM_DBG("non-2xx ACK matched\n");
 			goto found;
 		} /* ACK */
 	} /* synonym loop */
@@ -537,7 +537,7 @@ notfound:
 	if (!leave_new_locked || isACK) {
 		UNLOCK_HASH(p_msg->hash_index);
 	}
-	DBG("DEBUG: t_lookup_request: no transaction found\n");
+	LM_DBG("no transaction found\n");
 	return -1;
 
 e2e_ack:
@@ -545,7 +545,7 @@ e2e_ack:
 	UNLOCK_HASH(p_msg->hash_index);
 	e2eack_T = p_cell;
 	set_t(0);
-	DBG("DEBUG: t_lookup_request: e2e proxy ACK found\n");
+	LM_DBG("e2e proxy ACK found\n");
 	return -2;
 
 found:
@@ -553,7 +553,7 @@ found:
 	REF_UNSAFE( T );
 	set_kr(REQ_EXIST);
 	UNLOCK_HASH( p_msg->hash_index );
-	DBG("DEBUG: t_lookup_request: transaction found (T=%p)\n",T);
+	LM_DBG("transaction found (T=%p)\n",T);
 	return 1;
 }
 
@@ -578,7 +578,7 @@ struct cell* t_lookupOriginalT(  struct sip_msg* p_msg )
 
 	/* start searching in the table */
 	hash_index = p_msg->hash_index;
-	DBG("DEBUG: t_lookupOriginalT: searching on hash entry %d\n",hash_index );
+	LM_DBG("searching on hash entry %d\n",hash_index );
 
 
 	/* first of all, look if there is RFC3261 magic cookie in branch; if
@@ -586,7 +586,7 @@ struct cell* t_lookupOriginalT(  struct sip_msg* p_msg )
 	 * comparison of many header fields
 	 */
 	if (!p_msg->via1) {
-		LOG(L_ERR, "ERROR: t_lookupOriginalT: no via\n");
+		LM_ERR("no via\n");
 		cancelled_T = NULL;
 		return 0;
 	}
@@ -669,19 +669,18 @@ struct cell* t_lookupOriginalT(  struct sip_msg* p_msg )
 
 notfound:
 	/* no transaction found */
-	DBG("DEBUG: t_lookupOriginalT: no CANCEL matching found! \n" );
+	LM_DBG("no CANCEL matching found! \n" );
 	UNLOCK_HASH(hash_index);
 	cancelled_T = NULL;
-	DBG("DEBUG: t_lookupOriginalT completed\n");
+	LM_DBG("t_lookupOriginalT completed\n");
 	return 0;
 
 found:
-	DBG("DEBUG: t_lookupOriginalT: canceled transaction"
-		" found (%p)! \n",p_cell );
+	LM_DBG("canceled transaction found (%p)! \n",p_cell );
 	cancelled_T = p_cell;
 	REF_UNSAFE( p_cell );
 	UNLOCK_HASH(hash_index);
-	DBG("DEBUG: t_lookupOriginalT completed\n");
+	LM_DBG("t_lookupOriginalT completed\n");
 	return p_cell;
 }
 
@@ -772,13 +771,12 @@ int t_reply_matching( struct sip_msg *p_msg , int *p_branch )
 		|| (syn_branch ? (entry_label=reverse_hex2int(syni, synl))<0 
 			: loopl!=MD5_LEN )
 	) {
-		DBG("DEBUG: t_reply_matching: poor reply labels %d label %d "
-			"branch %d\n",hash_index, entry_label, branch_id );
+		LM_DBG("poor reply labels %d label %d branch %d\n",
+				hash_index, entry_label, branch_id );
 		goto nomatch2;
 	}
 
-	DBG("DEBUG: t_reply_matching: hash %d label %d branch %d\n",
-		hash_index, entry_label, branch_id );
+	LM_DBG("hash %d label %d branch %d\n",hash_index, entry_label, branch_id);
 
 	cseq = get_cseq(p_msg);
 
@@ -821,7 +819,7 @@ int t_reply_matching( struct sip_msg *p_msg , int *p_branch )
 		*p_branch = branch_id;
 		REF_UNSAFE( T );
 		UNLOCK_HASH(hash_index);
-		DBG("DEBUG: t_reply_matching: reply matched (T=%p)!\n",T);
+		LM_DBG("reply matched (T=%p)!\n",T);
 		/* if this is a 200 for INVITE, we will wish to store to-tags to be
 		 * able to distinguish retransmissions later and not to call
  		 * TMCB_RESPONSE_OUT uselessly; we do it only if callbacks are
@@ -835,7 +833,7 @@ int t_reply_matching( struct sip_msg *p_msg , int *p_branch )
 			|| (is_local(p_cell)&&has_tran_tmcbs(p_cell,TMCB_LOCAL_COMPLETED))
 		)) {
 			if (parse_headers(p_msg, HDR_TO_F, 0)==-1) {
-				LOG(L_ERR, "ERROR: t_reply_matching: to parsing failed\n");
+				LM_ERR("to parsing failed\n");
 			}
 		}
 		if (!is_local(p_cell)) {
@@ -847,10 +845,10 @@ int t_reply_matching( struct sip_msg *p_msg , int *p_branch )
 
 	/* nothing found */
 	UNLOCK_HASH(hash_index);
-	DBG("DEBUG: t_reply_matching: no matching transaction exists\n");
+	LM_DBG("no matching transaction exists\n");
 
 nomatch2:
-	DBG("DEBUG: t_reply_matching: failure to match a transaction\n");
+	LM_DBG("failure to match a transaction\n");
 	*p_branch = -1;
 	set_t(0);
 	return -1;
@@ -870,14 +868,14 @@ int t_check( struct sip_msg* p_msg , int *param_branch )
 	int local_branch;
 
 	/* is T still up-to-date ? */
-	DBG("DEBUG: t_check: start=%p\n", T);
+	LM_DBG("start=%p\n", T);
 	if ( T==T_UNDEFINED )
 	{
 		/* transaction lookup */
 		if ( p_msg->first_line.type==SIP_REQUEST ) {
 			/* force parsing all the needed headers*/
 			if (parse_headers(p_msg, HDR_EOH_F, 0 )==-1) {
-				LOG(L_ERR, "ERROR: t_check: parsing error\n");
+				LM_ERR("parsing error\n");
 				return -1;
 			}
 			/* in case, we act as UAS for INVITE and reply with 200,
@@ -887,7 +885,7 @@ int t_check( struct sip_msg* p_msg , int *param_branch )
 			 */
 			if (p_msg->REQ_METHOD==METHOD_INVITE 
 							&& parse_from_header(p_msg)<0) {
-				LOG(L_ERR, "ERROR: t_check: from parsing failed\n");
+				LM_ERR("from parsing failed\n");
 				return -1;
 			}
 			t_lookup_request( p_msg , 0 /* unlock before returning */ );
@@ -897,7 +895,7 @@ int t_check( struct sip_msg* p_msg , int *param_branch )
 			*/
 			if ( parse_headers(p_msg, HDR_VIA1_F|HDR_CSEQ_F, 0 )==-1
 			|| !p_msg->via1 || !p_msg->cseq ) {
-				LOG(L_ERR, "ERROR: reply cannot be parsed\n");
+				LM_ERR("reply cannot be parsed\n");
 				return -1;
 			}
 
@@ -905,7 +903,7 @@ int t_check( struct sip_msg* p_msg , int *param_branch )
 			   for later ACK matching */
 			if ( get_cseq(p_msg)->method_id==METHOD_INVITE ) {
 					if (parse_headers(p_msg, HDR_TO_F, 0)==-1 || !p_msg->to)  {
-						LOG(L_ERR, "ERROR: INVITE reply cannot be parsed\n");
+						LM_ERR("INVITE reply cannot be parsed\n");
 						return -1;
 					}
 			}
@@ -916,17 +914,17 @@ int t_check( struct sip_msg* p_msg , int *param_branch )
 		}
 #ifdef EXTRA_DEBUG
 		if ( T && T!=T_UNDEFINED && T->damocles) {
-			LOG( L_ERR, "ERROR: transaction %p scheduled for deletion "
+			LM_ERR("transaction %p scheduled for deletion "
 				"and called from t_check\n", T);
 			abort();
 		}
 #endif
-		DBG("DEBUG: t_check: end=%p\n",T);
+		LM_DBG("end=%p\n",T);
 	} else {
 		if (T)
-			DBG("DEBUG: t_check: T already found!\n");
+			LM_DBG("transaction already found!\n");
 		else
-			DBG("DEBUG: t_check: T previously sought and not found\n");
+			LM_DBG("transaction previously sought and not found\n");
 	}
 
 	return T ? (T==T_UNDEFINED ? -1 : 1 ) : 0;
@@ -945,7 +943,7 @@ int init_rb( struct retr_buf *rb, struct sip_msg *msg)
 	} else {
 		/*init retrans buffer*/
 		if (update_sock_struct_from_via( &(rb->dst.to), msg, via )==-1) {
-			LOG(L_ERR, "ERROR: init_rb: cannot lookup reply dst: %.*s\n",
+			LM_ERR("init_rb: cannot lookup reply dst: %.*s\n",
 				via->host.len, via->host.s );
 			ser_error=E_BAD_VIA;
 			return 0;
@@ -989,19 +987,19 @@ static inline int new_t(struct sip_msg *p_msg)
 
 	/* for ACK-dlw-wise matching, we want From-tags */
 	if (p_msg->REQ_METHOD==METHOD_INVITE && parse_from_header(p_msg)<0) {
-			LOG(L_ERR, "ERROR: new_t: no valid From in INVITE\n");
+			LM_ERR("no valid From in INVITE\n");
 			return E_BAD_REQ;
 	}
 	/* make sure uri will be parsed before cloning */
 	if (parse_sip_msg_uri(p_msg)<0) {
-		LOG(L_ERR, "ERROR: new_t: uri invalid\n");
+		LM_ERR("uri invalid\n");
 		return E_BAD_REQ;
 	}
 			
 	/* add new transaction */
 	new_cell = build_cell( p_msg ) ;
 	if  ( !new_cell ){
-		LOG(L_ERR, "ERROR: new_t: out of mem:\n");
+		LM_ERR("out of mem\n");
 		return E_OUT_OF_MEM;
 	} 
 
@@ -1034,11 +1032,10 @@ int t_newtran( struct sip_msg* p_msg )
 	int lret, my_err;
 
 	/* is T still up-to-date ? */
-	DBG("DEBUG: t_newtran:  T on entrance=%p\n",T);
+	LM_DBG("transaction on entrance=%p\n",T);
 
 	if ( T && T!=T_UNDEFINED  ) {
-		LOG(L_ERR, "ERROR: t_newtran: "
-			"transaction already in process %p\n", T );
+		LM_ERR("transaction already in process %p\n", T );
 		return E_SCRIPT;
 	}
 
@@ -1051,11 +1048,11 @@ int t_newtran( struct sip_msg* p_msg )
 	*/
 
 	if (parse_headers(p_msg, HDR_EOH_F, 0 )<0) {
-		LOG(L_ERR, "ERROR: t_newtran: parse_headers failed\n");
+		LM_ERR("parse_headers failed\n");
 		return E_BAD_REQ;
 	}
 	if ((p_msg->parsed_flag & HDR_EOH_F)!=HDR_EOH_F) {
-			LOG(L_ERR, "ERROR: t_newtran: EoH not parsed\n");
+			LM_ERR("EoH not parsed\n");
 			return E_OUT_OF_MEM;
 	}
 	/* t_lookup_requests attempts to find the transaction; 
@@ -1082,7 +1079,7 @@ int t_newtran( struct sip_msg* p_msg )
 	/* from now on, be careful -- hash table is locked */
 
 	if (lret==-2) { /* was it an e2e ACK ? if so, trigger a callback */
-		DBG("DEBUG:tm:t_newtran: building branch for end2end ACK\n");
+		LM_DBG("building branch for end2end ACK\n");
 		/* to ensure unigueness acros time and space, compute the ACK 
 		 * branch in the same maner as for INVITE, but put a t->branch 
 		 * value that cannot exist for that INVITE - as it is compute as 
@@ -1091,7 +1088,7 @@ int t_newtran( struct sip_msg* p_msg )
 		 * corresponding INVITE  --bogdan */
 		if (!t_calc_branch(e2eack_T, e2eack_T->nr_of_outgoings+1,
 		p_msg->add_to_branch_s, &p_msg->add_to_branch_len )) {
-			LOG(L_ERR,"ERROR:tm:t_newtran: ACK branch computation failed\n");
+			LM_ERR("ACK branch computation failed\n");
 		}
 
 		/* no callbacks? complete quickly */
@@ -1117,7 +1114,7 @@ int t_newtran( struct sip_msg* p_msg )
 
 	my_err=new_t(p_msg);
 	if (my_err<0) {
-		LOG(L_ERR, "ERROR: t_newtran: new_t failed\n");
+		LM_ERR("new_t failed\n");
 		goto new_err;
 	}
 
@@ -1133,7 +1130,7 @@ int t_newtran( struct sip_msg* p_msg )
 	   the transaction 
 	*/
 	if (!init_rb( &T->uas.response, p_msg)) {
-		LOG(L_ERR, "ERROR: t_newtran: unresolvable via1\n");
+		LM_ERR("unresolvable via1\n");
 		put_on_wait( T );
 		t_unref(p_msg);
 		return E_BAD_VIA;
@@ -1158,8 +1155,7 @@ int t_unref( struct sip_msg* p_msg  )
 			kr=get_kr();
 			if (kr==0 
 					||(p_msg->REQ_METHOD==METHOD_ACK && !(kr & REQ_RLSD))) {
-				LOG(L_WARN, "WARNING: script writer didn't release "
-					"transaction\n");
+				LM_WARN("script writer didn't release transaction\n");
 				t_release_transaction(T);
 			}
 		}
@@ -1181,12 +1177,12 @@ int t_get_trans_ident(struct sip_msg* p_msg, unsigned int* hash_index,
 {
 	struct cell* t;
 	if(t_check(p_msg,0) != 1){
-		LOG(L_ERR,"ERROR:tm:t_get_trans_ident: no transaction found\n");
+		LM_ERR("no transaction found\n");
 		return -1;
 	}
 	t = get_t();
 	if(!t){
-		LOG(L_ERR,"ERROR:tm:t_get_trans_ident: transaction found is NULL\n");
+		LM_ERR("transaction found is NULL\n");
 		return -1;
 	}
 
@@ -1204,7 +1200,7 @@ int t_lookup_ident(struct cell ** trans, unsigned int hash_index,
 	struct cell* p_cell;
 
 	if(hash_index >= TM_TABLE_ENTRIES){
-		LOG(L_ERR,"ERROR: t_lookup_ident: invalid hash_index=%u\n",hash_index);
+		LM_ERR("invalid hash_index=%u\n",hash_index);
 		return -1;
 	}
 
@@ -1219,7 +1215,7 @@ int t_lookup_ident(struct cell ** trans, unsigned int hash_index,
 			UNLOCK_HASH(hash_index);
 			set_t(p_cell);
 			*trans=p_cell;
-			DBG("DEBUG:tm:t_lookup_ident: transaction found\n");
+			LM_DBG("transaction found\n");
 			return 1;
 		}
 	}
@@ -1228,7 +1224,7 @@ int t_lookup_ident(struct cell ** trans, unsigned int hash_index,
 	set_t(0);
 	*trans=p_cell;
 
-	DBG("DEBUG:tm:t_lookup_ident: transaction not found\n");
+	LM_DBG("transaction not found\n");
 	return -1;
 }
 
@@ -1238,12 +1234,12 @@ int t_is_local(struct sip_msg* p_msg)
 {
 	struct cell* t;
 	if(t_check(p_msg,0) != 1){
-		LOG(L_ERR,"ERROR:tm:t_is_local: no transaction found\n");
+		LM_ERR("no transaction found\n");
 		return -1;
 	}
 	t = get_t();
 	if(!t){
-		LOG(L_ERR,"ERROR:tm:t_is_local: transaction found is NULL\n");
+		LM_ERR("transaction found is NULL\n");
 		return -1;
 	}
 
@@ -1277,18 +1273,18 @@ int t_lookup_callid(struct cell ** trans, str callid, str cseq) {
 	hash_index=tm_hash(callid, cseq);
 
 	if(hash_index >= TM_TABLE_ENTRIES){
-		LOG(L_ERR,"ERROR:tm:t_lookup_callid: invalid hash_index=%u\n",hash_index);
+		LM_ERR("invalid hash_index=%u\n",hash_index);
 		return -1;
 	}
 
 	/* create header fields the same way tm does itself, then compare headers */
 	endpos = print_callid_mini(callid_header, callid);
-	DBG("DEBUG:tm:t_lookup_callid: created comparable call_id header field:"
-		" >%.*s<\n", (int)(endpos - callid_header), callid_header); 
+	LM_DBG("created comparable call_id header field: >%.*s<\n",
+			(int)(endpos - callid_header), callid_header); 
 
 	endpos = print_cseq_mini(cseq_header, &cseq, &invite_method);
-	DBG("DEBUG:tm:t_lookup_callid: created comparable cseq header field: "
-		">%.*s<\n", (int)(endpos - cseq_header), cseq_header); 
+	LM_DBG("created comparable cseq header field: >%.*s<\n",
+			(int)(endpos - cseq_header), cseq_header); 
 
 	LOCK_HASH(hash_index);
 
@@ -1297,27 +1293,27 @@ int t_lookup_callid(struct cell ** trans, str callid, str cseq) {
 	for ( ; p_cell; p_cell = p_cell->next_cell ) {
 
 		/* compare complete header fields, casecmp to make sure invite=INVITE */
-		DBG(" <%.*s>  <%.*s>\n", p_cell->callid.len, p_cell->callid.s,
+		LM_DBG(" <%.*s>  <%.*s>\n", p_cell->callid.len, p_cell->callid.s,
 			p_cell->cseq_n.len,p_cell->cseq_n.s);
 		if ( (strncmp(callid_header, p_cell->callid.s, p_cell->callid.len) == 0)
 			&& (strncasecmp(cseq_header, p_cell->cseq_n.s, p_cell->cseq_n.len) == 0) ) {
-			DBG("DEBUG:tm:t_lookup_callid: we have a match: "
-				"callid=>>%.*s<< cseq=>>%.*s<<\n", p_cell->callid.len, 
-				p_cell->callid.s, p_cell->cseq_n.len, p_cell->cseq_n.s);
+			LM_DBG("we have a match: callid=>>%.*s<< cseq=>>%.*s<<\n",
+				p_cell->callid.len,p_cell->callid.s, p_cell->cseq_n.len,
+				p_cell->cseq_n.s);
 			REF_UNSAFE(p_cell);
 			UNLOCK_HASH(hash_index);
 			set_t(p_cell);
 			*trans=p_cell;
-			DBG("DEBUG:tm:t_lookup_callid: transaction found.\n");
+			LM_DBG("transaction found.\n");
 			return 1;
 		}
-		DBG("DEBUG:tm:t_lookup_callid:: NO match: callid=%.*s cseq=%.*s\n",
+		LM_DBG("NO match: callid=%.*s cseq=%.*s\n",
 			p_cell->callid.len, p_cell->callid.s,
 			p_cell->cseq_n.len, p_cell->cseq_n.s);
 	}
 
 	UNLOCK_HASH(hash_index);
-	DBG("DEBUG:tm:t_lookup_callid: transaction not found.\n");
+	LM_DBG("transaction not found.\n");
 
 	return -1;
 }
