@@ -1,5 +1,5 @@
 /*
- * $Id: tm.c 2246 2007-05-18 10:37:38Z bogdan_iancu $
+ * $Id: tm.c 3383 2007-12-14 12:03:24Z jerocu $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -61,14 +61,13 @@
 #include "../../mi/mi.h"
 #include "../../usr_avp.h"
 #include "../../mem/mem.h"
-#include "../../unixsock_server.h"
-#include "../../items.h"
+#include "../../pvar.h"
+#include "../../mod_fix.h"
 
 #include "sip_msg.h"
 #include "h_table.h"
 #include "ut.h"
 #include "t_reply.h"
-#include "uac_unixsock.h"
 #include "t_fwd.h"
 #include "t_lookup.h"
 #include "callid.h"
@@ -80,15 +79,13 @@
 MODULE_VERSION
 
 /* item functions */
-static int it_get_tm_branch_idx(struct sip_msg *msg, xl_value_t *res,
-		xl_param_t *param, int flags);
-static int it_get_tm_reply_code(struct sip_msg *msg, xl_value_t *res,
-		xl_param_t *param, int flags);
+static int pv_get_tm_branch_idx(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res);
+static int pv_get_tm_reply_code(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res);
 
 /* fixup functions */
 static int fixup_t_send_reply(void** param, int param_no);
-static int fixup_str2int( void** param, int param_no);
-static int fixup_str2regexp(void** param, int param_no);
 static int fixup_local_replied(void** param, int param_no);
 static int fixup_t_relay1(void** param, int param_no);
 static int fixup_t_relay2(void** param, int param_no);
@@ -139,44 +136,44 @@ stat_var *tm_trans_inuse;
 
 static cmd_export_t cmds[]={
 	{"t_newtran",            w_t_newtran,             0, 0,
-			REQUEST_ROUTE},
+			0, REQUEST_ROUTE},
 	{"t_reply",              w_t_reply,               2, fixup_t_send_reply,
-			REQUEST_ROUTE | FAILURE_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"t_release",            w_t_release,             0, 0,
-			REQUEST_ROUTE},
+			0, REQUEST_ROUTE},
 	{"t_replicate",          w_t_replicate,           1, fixup_t_replicate,
-			REQUEST_ROUTE},
+			0, REQUEST_ROUTE},
 	{"t_replicate",          w_t_replicate,           2, fixup_t_replicate,
-			REQUEST_ROUTE},
+			0, REQUEST_ROUTE},
 	{"t_relay",              w_t_relay,               0, 0,
-			REQUEST_ROUTE | FAILURE_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"t_relay",              w_t_relay,               1, fixup_t_relay1,
-			REQUEST_ROUTE | FAILURE_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"t_relay",              w_t_relay,               2, fixup_t_relay2,
-			REQUEST_ROUTE | FAILURE_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE },
 	{"t_on_failure",         w_t_on_negative,         1, fixup_str2int,
-			REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
 	{"t_on_reply",           w_t_on_reply,            1, fixup_str2int,
-			REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
 	{"t_on_branch",          w_t_on_branch,           1, fixup_str2int,
-			REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
 	{"t_check_status",       t_check_status,          1, fixup_str2regexp,
-			REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
 	{"t_write_req",         t_write_req,              2, fixup_t_write,
-			REQUEST_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE },
 	{"t_write_unix",        t_write_unix,             2, fixup_t_write,
-			REQUEST_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE | BRANCH_ROUTE },
 	{"t_flush_flags",       t_flush_flags,            0, 0,
-			REQUEST_ROUTE | BRANCH_ROUTE  },
+			0, REQUEST_ROUTE | BRANCH_ROUTE  },
 	{"t_local_replied",     t_local_replied,          1, fixup_local_replied,
-			REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
+			0, REQUEST_ROUTE | FAILURE_ROUTE | ONREPLY_ROUTE | BRANCH_ROUTE },
 	{"t_check_trans",       t_check_trans,            0, 0,
-			REQUEST_ROUTE | BRANCH_ROUTE },
+			0, REQUEST_ROUTE | BRANCH_ROUTE },
 	{"t_was_cancelled",     t_was_cancelled,          0, 0,
-			FAILURE_ROUTE | ONREPLY_ROUTE },
+			0, FAILURE_ROUTE | ONREPLY_ROUTE },
 	{"load_tm",             (cmd_function)load_tm,    0, 0,
-			0},
-	{0,0,0,0,0}
+			0, 0},
+	{0,0,0,0,0,0}
 };
 
 
@@ -239,14 +236,13 @@ static stat_export_t mod_stats[] = {
 
 /**
  * pseudo-variables exported by TM module
- *   { name, function, type, xl_param_t}
- *   type: can be 0 if you do not want to use it in identifying the PV
- *   xl_param_t: is given as parameter to 'function'
  */
-static item_export_t mod_items[] = {
-	{ "T_branch_idx", it_get_tm_branch_idx, 100, {{0, 0}, 0, 0} },
-	{ "T_reply_code", it_get_tm_reply_code, 100, {{0, 0}, 0, 0} },
-	{ 0, 0, 0, {{0, 0}, 0, 0} }
+static pv_export_t mod_items[] = {
+	{ {"T_branch_idx", sizeof("T_branch_idx")-1}, 900, pv_get_tm_branch_idx, 0,
+		 0, 0, 0, 0 },
+	{ {"T_reply_code", sizeof("T_reply_code")-1}, 901, pv_get_tm_reply_code, 0,
+		 0, 0, 0, 0 },
+	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
 };
 
 
@@ -271,6 +267,7 @@ struct module_exports exports= {
 	mod_stats, /* exported statistics */
 	mi_cmds,   /* exported MI functions */
 	mod_items, /* exported pseudo-variables */
+	0,         /* extra processes */
 	mod_init,  /* module initialization function */
 	(response_function) reply_received,
 	(destroy_function) tm_shutdown,
@@ -288,6 +285,7 @@ static int flag_fixup(void** param, int param_no)
 	if (param_no == 1) {
 		s.s = (char*)*param;
 		s.len = strlen(s.s);
+		flags = 0;
 		if ( strno2int(&s, &flags )<0 ) {
 			return -1;
 		}
@@ -306,7 +304,7 @@ static int fixup_t_replicate(void** param, int param_no)
 		/* string */
 		s = (str*)pkg_malloc( sizeof(str) );
 		if (s==0) {
-			LOG(L_ERR,"ERROR:tm:fixup_str: no more pkg mem\n");
+			LM_ERR("no more pkg mem\n");
 			return E_OUT_OF_MEM;
 		}
 		s->s = (char*)*param;
@@ -315,29 +313,7 @@ static int fixup_t_replicate(void** param, int param_no)
 	} else {
 		/* flags */
 		if (flag_fixup( param, 1)!=0) {
-			LOG(L_ERR, "ERROR:TM:fixup_t_relay2: bad flags <%s>\n",
-				(char *)(*param));
-			return E_CFG;
-		}
-	}
-	return 0;
-}
-
-
-static int fixup_str2int( void** param, int param_no)
-{
-	unsigned long go_to;
-	int err;
-
-	if (param_no==1) {
-		go_to=str2s(*param, strlen(*param), &err );
-		if (err==0) {
-			pkg_free(*param);
-			*param=(void *)go_to;
-			return 0;
-		} else {
-			LOG(L_ERR, "ERROR:tm:fixup_str2int: bad number <%s>\n",
-				(char *)(*param));
+			LM_ERR("bad flags <%s>\n", (char *)(*param));
 			return E_CFG;
 		}
 	}
@@ -354,27 +330,24 @@ static int fixup_phostport2proxy(void** param, int param_no)
 	str host;
 
 	if (param_no!=1) {
-		LOG(L_CRIT,"BUG:tm:fixup_phostport2proxy: called with more than "
-			" one parameter\n");
+		LM_CRIT("called with more than  one parameter\n");
 		return E_BUG;
 	}
 
 	s = (char *) (*param);
 	if (s==0 || *s==0) {
-		LOG(L_CRIT,"ERROR:tm:fixup_phostport2proxy: empty parameter\n");
+		LM_CRIT("empty parameter\n");
 		return E_UNSPEC;
 	}
 
 	if (parse_phostport( s, strlen(s), &host.s, &host.len, &port, &proto)!=0){
-		LOG(L_CRIT,"ERROR:tm:fixup_phostport2proxy: invalid parameter "
-			"<%s>\n",s);
+		LM_CRIT("invalid parameter <%s>\n",s);
 		return E_UNSPEC;
 	}
 
 	proxy = mk_proxy( &host, port, proto, 0);
 	if (proxy==0) {
-		LOG(L_ERR, "ERROR:tm:fixup_phostport2proxy: failed to resolve "
-			"<%.*s>\n", host.len, host.s );
+		LM_ERR("failed to resolve <%.*s>\n", host.len, host.s );
 		return ser_error;
 	}
 	*(param)=proxy;
@@ -393,8 +366,7 @@ static int fixup_t_relay1(void** param, int param_no)
 		/* param is OBP -> nothing else to do */
 		return 0;
 	} else {
-		LOG(L_ERR,"ERROR:TM:fixup_t_relay1: param is neither flag, nor OBP "
-			"<%s>\n",(char *)(*param));
+		LM_ERR("param is neither flag, nor OBP <%s>\n",(char *)(*param));
 		return E_CFG;
 	}
 }
@@ -406,8 +378,7 @@ static int fixup_t_relay2(void** param, int param_no)
 		return fixup_phostport2proxy( param, param_no);
 	} else if (param_no==2) {
 		if (flag_fixup( param, 1)!=0) {
-			LOG(L_ERR, "ERROR:TM:fixup_t_relay2: bad flags <%s>\n",
-				(char *)(*param));
+			LM_ERR("bad flags <%s>\n", (char *)(*param));
 			return E_CFG;
 		}
 	}
@@ -429,14 +400,13 @@ static int fixup_t_send_reply(void** param, int param_no)
 			*param=(void*)code;
 			return 0;
 		}else{
-			LOG(L_ERR, "ERROR:tm:fixup_t_send_reply: bad  number <%s>\n",
-					(char*)(*param));
+			LM_ERR("bad  number <%s>\n", (char*)(*param));
 			return E_UNSPEC;
 		}
 	} else if (param_no==2) {
 		s = (str*)pkg_malloc(sizeof(str));
 		if (s==0) {
-			LOG(L_ERR, "ERROR:sl:fixup_t_send_reply: no more pkg mem\n");
+			LM_ERR("no more pkg mem\n");
 			return E_OUT_OF_MEM;
 		}
 		s->s = (char*)*param;
@@ -448,30 +418,6 @@ static int fixup_t_send_reply(void** param, int param_no)
 }
 
 
-static int fixup_str2regexp(void** param, int param_no)
-{
-	regex_t* re;
-
-	if (param_no==1) {
-		if ((re=pkg_malloc(sizeof(regex_t)))==0)
-			return E_OUT_OF_MEM;
-		if (regcomp(re, *param, REG_EXTENDED|REG_ICASE|REG_NEWLINE) ) {
-			pkg_free(re);
-			LOG(L_ERR,"ERROR: %s : bad re %s\n", exports.name, (char*)*param);
-			return E_BAD_RE;
-		}
-		/* free string */
-		pkg_free(*param);
-		/* replace it with the compiled re */
-		*param=re;
-	} else {
-		LOG(L_ERR,"ERROR: fixup_str2regexp called with parameter != 1\n");
-		return E_BUG;
-	}
-	return 0;
-}
-
-
 static int fixup_local_replied(void** param, int param_no)
 {
 	char *val;
@@ -486,8 +432,7 @@ static int fixup_local_replied(void** param, int param_no)
 		} else if (strcasecmp(val,"last")==0) {
 			n = 2;
 		} else {
-			LOG(L_ERR,"ERROR:tm:fixup_local_replied: invalid param \"%s\"\n",
-				val);
+			LM_ERR("invalid param \"%s\"\n", val);
 			return E_CFG;
 		}
 		/* free string */
@@ -495,7 +440,7 @@ static int fixup_local_replied(void** param, int param_no)
 		/* replace it with the compiled re */
 		*param=(void*)(long)n;
 	} else {
-		LOG(L_ERR,"ERROR:fixup_local_replied: called with parameter != 1\n");
+		LM_ERR("called with parameter != 1\n");
 		return E_BUG;
 	}
 	return 0;
@@ -583,12 +528,12 @@ static int script_init( struct sip_msg *foo, void *bar)
 
 static int mod_init(void)
 {
-	LOG(L_INFO,"TM - initializing...\n");
+	LM_INFO("TM - initializing...\n");
 
 	/* checking if we have sufficient bitmap capacity for given
 	   maximum number of  branches */
 	if (MAX_BRANCHES+1>31) {
-		LOG(L_CRIT, "Too many max UACs for UAC branch_bm_t bitmap: %d\n",
+		LM_CRIT("Too many max UACs for UAC branch_bm_t bitmap: %d\n",
 			MAX_BRANCHES );
 		return -1;
 	}
@@ -602,33 +547,13 @@ static int mod_init(void)
 #endif
 
 	if (init_callid() < 0) {
-		LOG(L_CRIT, "Error while initializing Call-ID generator\n");
-		return -1;
-	}
-
-	if (unixsock_register_cmd("t_uac_dlg", unixsock_uac) < 0) {
-		LOG(L_CRIT, "cannot register t_uac with the unix server\n");
-		return -1;
-	}
-
-	if (unixsock_register_cmd("t_uac_cancel", unixsock_uac_cancel) < 0) {
-		LOG(L_CRIT, "cannot register t_uac_cancel with the unix server\n");
-		return -1;
-	}
-
-	if (unixsock_register_cmd("t_hash", unixsock_hash) < 0) {
-		LOG(L_CRIT, "cannot register t_hash with the unix server\n");
-		return -1;
-	}
-
-	if (unixsock_register_cmd("t_reply", unixsock_t_reply) < 0) {
-		LOG(L_CRIT, "cannot register t_reply with the unix server\n");
+		LM_CRIT("Error while initializing Call-ID generator\n");
 		return -1;
 	}
 
 	/* building the hash table*/
 	if (!init_hash_table()) {
-		LOG(L_ERR, "ERROR:tm:mod_init: initializing hash_table failed\n");
+		LM_ERR("initializing hash_table failed\n");
 		return -1;
 	}
 
@@ -636,51 +561,49 @@ static int mod_init(void)
 	init_t();
 
 	if (!tm_init_timers()) {
-		LOG(L_ERR, "ERROR:tm:mod_init: timer init failed\n");
+		LM_ERR("timer init failed\n");
 		return -1;
 	}
 
 	/* register the timer functions */
 	if (register_timer( timer_routine , 0, 1 )<0) {
-		LOG(L_ERR, "ERROR:tm:mod_init: failed to register timer\n");
+		LM_ERR("failed to register timer\n");
 		return -1;
 	}
 	if (register_utimer( utimer_routine , 0, 100*1000 )<0) {
-		LOG(L_ERR, "ERROR:tm:mod_init: failed to register utimer\n");
+		LM_ERR("failed to register utimer\n");
 		return -1;
 	}
 
 	if (uac_init()==-1) {
-		LOG(L_ERR, "ERROR:tm:mod_init: uac_init failed\n");
+		LM_ERR("uac_init failed\n");
 		return -1;
 	}
 
 	if (init_tmcb_lists()!=1) {
-		LOG(L_CRIT, "ERROR:tm:mod_init: failed to init tmcb lists\n");
+		LM_CRIT("failed to init tmcb lists\n");
 		return -1;
 	}
 
 	tm_init_tags();
 	init_twrite_lines();
 	if (init_twrite_sock() < 0) {
-		LOG(L_ERR, "ERROR:tm:mod_init: Unable to create socket\n");
+		LM_ERR("failed to create socket\n");
 		return -1;
 	}
 
 	/* register post-script clean-up function */
 	if (register_script_cb( do_t_unref, POST_SCRIPT_CB|REQ_TYPE_CB, 0)<0 ) {
-		LOG(L_ERR,"ERROR:tm:mod_init: failed to register POST request "
-			"callback\n");
+		LM_ERR("failed to register POST request callback\n");
 		return -1;
 	}
 	if (register_script_cb( script_init, PRE_SCRIPT_CB|REQ_TYPE_CB , 0)<0 ) {
-		LOG(L_ERR,"ERROR:tm:mod_init: failed to register PRE request "
-			"callback\n");
+		LM_ERR("failed to register PRE request callback\n");
 		return -1;
 	}
 
 	if ( init_avp_params( fr_timer_param, fr_inv_timer_param)<0 ){
-		LOG(L_ERR,"ERROR:tm:mod_init: failed to process timer AVPs\n");
+		LM_ERR("ERROR:tm:mod_init: failed to process timer AVPs\n");
 		return -1;
 	}
 
@@ -691,8 +614,7 @@ static int mod_init(void)
 static int child_init(int rank)
 {
 	if (child_init_callid(rank) < 0) {
-		LOG(L_ERR, "ERROR:tm:child_init: Error while initializing "
-			"Call-ID generator\n");
+		LM_ERR("failed to initialize Call-ID generator\n");
 		return -2;
 	}
 
@@ -715,8 +637,8 @@ static int t_check_status(struct sip_msg* msg, char *regexp, char *foo)
 	/* first get the transaction */
 	t = get_t();
 	if ( t==0 || t==T_UNDEFINED ) {
-		LOG(L_ERR, "ERROR: t_check_status: cannot check status for a reply "
-			"which has no T-state established\n");
+		LM_ERR("cannot check status for a reply which"
+				" has no transaction-state established\n");
 		return -1;
 	}
 	backup = 0;
@@ -735,19 +657,18 @@ static int t_check_status(struct sip_msg* msg, char *regexp, char *foo)
 		case FAILURE_ROUTE:
 			/* use the status of the winning reply */
 			if ( (branch=t_get_picked_branch())<0 ) {
-				LOG(L_CRIT,"BUG:t_check_status: no picked branch (%d) for"
-					" a final response in MODE_ONFAILURE\n", branch);
+				LM_CRIT("no picked branch (%d) for a final response"
+						" in MODE_ONFAILURE\n", branch);
 				return -1;
 			}
 			status = int2str( t->uac[branch].last_received , 0);
 			break;
 		default:
-			LOG(L_ERR,"ERROR:t_check_status: unsupported route_type %d\n",
-					route_type);
+			LM_ERR("unsupported route_type %d\n", route_type);
 			return -1;
 	}
 
-	DBG("DEBUG:t_check_status: checked status is <%s>\n",status);
+	LM_DBG("checked status is <%s>\n",status);
 	/* do the checking */
 	n = regexec((regex_t*)regexp, status, 1, &pmatch, 0);
 
@@ -764,7 +685,7 @@ inline static int t_check_trans(struct sip_msg* msg, char *foo, char *bar)
 	if (msg->REQ_METHOD==METHOD_CANCEL) {
 		/* parse needed hdrs*/
 		if (check_transaction_quadruple(msg)==0) {
-			LOG(L_ERR, "ERROR:tm:t_check_trans: too few headers\n");
+			LM_ERR("too few headers\n");
 			return 0; /*drop request!*/
 		}
 		if (!msg->hash_index)
@@ -807,8 +728,8 @@ static int t_flush_flags(struct sip_msg* msg, char *foo, char *bar)
 	/* first get the transaction */
 	t = get_t();
 	if ( t==0 || t==T_UNDEFINED) {
-		LOG(L_ERR, "ERROR: t_flush_flags: cannot flush flags for a message "
-			"which has no T-state established\n");
+		LM_ERR("failed to flush flags for a message which has"
+				" no transaction-state established\n");
 		return -1;
 	}
 
@@ -826,7 +747,7 @@ inline static int t_local_replied(struct sip_msg* msg, char *type, char *bar)
 
 	t = get_t();
 	if (t==0 || t==T_UNDEFINED) {
-		LOG(L_ERR,"ERROR:t_local_replied: no trasaction created\n");
+		LM_ERR("no trasaction created\n");
 		return -1;
 	}
 
@@ -841,9 +762,9 @@ inline static int t_local_replied(struct sip_msg* msg, char *type, char *bar)
 		/* check branch */
 		case 1:
 			if (route_type==FAILURE_ROUTE) {
-				/* use the the winning reply */
+				/* use the winning reply */
 				if ( (branch=t_get_picked_branch())<0 ) {
-					LOG(L_CRIT,"BUG:t_local_replied: no picked branch (%d) for"
+					LM_CRIT("no picked branch (%d) for"
 						" a final response in MODE_ONFAILURE\n", branch);
 					return -1;
 				}
@@ -855,9 +776,9 @@ inline static int t_local_replied(struct sip_msg* msg, char *type, char *bar)
 		/* check last */
 		case 2:
 			if (route_type==FAILURE_ROUTE) {
-				/* use the the winning reply */
+				/* use the winning reply */
 				if ( (branch=t_get_picked_branch())<0 ) {
-					LOG(L_CRIT,"BUG:t_local_replied: no picked branch (%d) for"
+					LM_CRIT("no picked branch (%d) for"
 						" a final response in MODE_ONFAILURE\n", branch);
 					return -1;
 				}
@@ -879,11 +800,11 @@ static int t_was_cancelled(struct sip_msg* msg, char *foo, char *bar)
 	/* first get the transaction */
 	t = get_t();
 	if ( t==0 || t==T_UNDEFINED ) {
-		LOG(L_ERR, "ERROR:tm:t_was_cancelled: cannot check cancel flag for "
-			"a reply without a transaction\n");
+		LM_ERR("failed to check cancel flag for a reply"
+				" without a transaction\n");
 		return -1;
 	}
-	return (t->flags&T_WAS_CANCELLED_FLAG)?1:-1;
+	return was_cancelled(t)?1:-1;
 }
 
 
@@ -892,13 +813,13 @@ inline static int w_t_reply(struct sip_msg* msg, char* str1, char* str2)
 	struct cell *t;
 
 	if (msg->REQ_METHOD==METHOD_ACK) {
-		LOG(L_WARN, "WARNING: t_reply: ACKs are not replied\n");
+		LM_WARN("ACKs are not replied\n");
 		return -1;
 	}
 	t=get_t();
 	if ( t==0 || t==T_UNDEFINED ) {
-		LOG(L_ERR, "ERROR: t_reply: cannot send a t_reply to a message "
-			"for which no T-state has been established\n");
+		LM_ERR("failed to send a t_reply to a message for which no transaction-"
+				"state has been established\n");
 		return -1;
 	}
 	/* if called from reply_route, make sure that unsafe version
@@ -907,13 +828,12 @@ inline static int w_t_reply(struct sip_msg* msg, char* str1, char* str2)
 	 */
 	switch (route_type) {
 		case FAILURE_ROUTE:
-			DBG("DEBUG: t_reply_unsafe called from w_t_reply\n");
+			LM_DBG("t_reply_unsafe called from w_t_reply\n");
 			return t_reply_unsafe(t, msg, (unsigned int)(long)str1,(str*)str2);
 		case REQUEST_ROUTE:
 			return t_reply( t, msg, (unsigned int)(long) str1, (str*)str2);
 		default:
-			LOG(L_CRIT, "BUG:tm:w_t_reply: unsupported route_type (%d)\n",
-				route_type);
+			LM_CRIT("unsupported route_type (%d)\n", route_type);
 			return -1;
 	}
 }
@@ -964,6 +884,34 @@ inline static int w_t_replicate(struct sip_msg *p_msg, char *dst, char *flags)
 	return t_replicate( p_msg, (str*)dst, (int)(long)flags);
 }
 
+static inline int t_relay_inerr2scripterr(void)
+{
+	switch (ser_error) {
+		case E_BAD_URI:
+		case E_BAD_REQ:
+		case E_BAD_TO:
+		case E_INVALID_PARAMS:
+			/* bad message */
+			return -2;
+		case E_NO_DESTINATION:
+			/* no available destination */
+			return -3;
+		case E_BAD_ADDRESS:
+			/* bad destination */
+			return -4;
+		case E_IP_BLOCKED:
+			/* destination filtered */
+			return -5;
+		case E_NO_SOCKET:
+		case E_SEND:
+			/* send failed */
+			return -6;
+		default:
+			/* generic internal error */
+			return -1;
+	}
+}
+
 
 inline static int w_t_relay( struct sip_msg  *p_msg , char *proxy, char *flags)
 {
@@ -975,10 +923,14 @@ inline static int w_t_relay( struct sip_msg  *p_msg , char *proxy, char *flags)
 	if (!t || t==T_UNDEFINED) {
 		/* no transaction yet */
 		if (route_type==FAILURE_ROUTE) {
-			LOG(L_CRIT, "BUG:tm:w_t_relay: undefined T\n");
+			LM_CRIT(" BUG - undefined transaction in failure route\n");
 			return -1;
 		}
-		return t_relay_to( p_msg, (struct proxy_l *)proxy, (int)(long)flags );
+		ret = t_relay_to( p_msg, (struct proxy_l *)proxy, (int)(long)flags );
+		if (ret<0) {
+			return t_relay_inerr2scripterr();
+		}
+		return ret;
 	} else {
 		/* transaction already created */
 
@@ -995,22 +947,23 @@ inline static int w_t_relay( struct sip_msg  *p_msg , char *proxy, char *flags)
 			t->flags|=T_NO_DNS_FAILOVER_FLAG;
 
 		ret = t_forward_nonack( t, p_msg, (struct proxy_l *)proxy);
-		if (ret<=0 )
-			LOG(L_ERR, "ERROR:tm:w_t_relay: t_forward_nonack failed\n");
+		if (ret<=0 ) {
+			LM_ERR("t_forward_nonack failed\n");
+			return t_relay_inerr2scripterr();
+		}
 		return ret;
 	}
 
 route_err:
-	LOG(L_CRIT, "ERROR:tm:w_t_relay: unsupported route type: %d\n",
-		route_type);
+	LM_CRIT("unsupported route type: %d\n", route_type);
 	return 0;
 }
 
 
 
 /* item functions */
-static int it_get_tm_branch_idx(struct sip_msg *msg, xl_value_t *res,
-		xl_param_t *param, int flags)
+static int pv_get_tm_branch_idx(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res)
 {
 	extern int _tm_branch_index;
 	int l = 0;
@@ -1025,13 +978,13 @@ static int it_get_tm_branch_idx(struct sip_msg *msg, xl_value_t *res,
 	res->rs.len = l;
 
 	res->ri = _tm_branch_index;
-	res->flags = XL_VAL_STR|XL_VAL_INT|XL_TYPE_INT;
+	res->flags = PV_VAL_STR|PV_VAL_INT|PV_TYPE_INT;
 
 	return 0;
 }
 
-static int it_get_tm_reply_code(struct sip_msg *msg, xl_value_t *res,
-		xl_param_t *param, int flags)
+static int pv_get_tm_reply_code(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res)
 {
 	struct cell *t;
 	int code;
@@ -1058,27 +1011,25 @@ static int it_get_tm_reply_code(struct sip_msg *msg, xl_value_t *res,
 			case FAILURE_ROUTE:
 				/* use the status of the winning reply */
 				if ( (branch=t_get_picked_branch())<0 ) {
-					LOG(L_CRIT,"BUG:tm:it_get_tm_reply_code: no picked "
-						"branch (%d) for a final response in MODE_ONFAILURE\n",
-						branch);
+					LM_CRIT("no picked branch (%d) for a final response"
+							" in MODE_ONFAILURE\n", branch);
 					code = 0;
 				} else {
 					code = t->uac[branch].last_received;
 				}
 				break;
 			default:
-				LOG(L_ERR,"ERROR:tm:it_get_tm_reply_code: unsupported "
-					"route_type %d\n", route_type);
+				LM_ERR("unsupported route_type %d\n", route_type);
 				code = 0;
 		}
 	}
 
-	DBG("DEBUG:it_get_tm_reply_code: reply code is <%d>\n",code);
+	LM_DBG("reply code is <%d>\n",code);
 
 	res->rs.s = int2str( code, &res->rs.len);
 
 	res->ri = code;
-	res->flags = XL_VAL_STR|XL_VAL_INT|XL_TYPE_INT;
+	res->flags = PV_VAL_STR|PV_VAL_INT|PV_TYPE_INT;
 	return 0;
 }
 
