@@ -1,5 +1,5 @@
 /*
- * $Id: rd_funcs.h 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: rd_funcs.h 2951 2007-10-20 16:28:01Z bogdan_iancu $
  *
  * Copyright (C) 2005 Voice Sistem SRL
  *
@@ -33,6 +33,7 @@
 #include "../../sr_module.h"
 #include "../../str.h"
 #include "../tm/tm_load.h"
+#include "../acc/acc_logic.h"
 
 typedef int (*tm_get_trans_f)( struct sip_msg*, struct cell**);
 
@@ -41,7 +42,8 @@ extern cmd_function   rd_acc_fct;
 
 extern char *acc_db_table;
 
-int get_redirect( struct sip_msg *msg , int maxt, int maxb, str *reason);
+int get_redirect( struct sip_msg *msg , int maxt, int maxb,
+		struct acc_param *reason);
 
 #endif
 
