@@ -1,5 +1,5 @@
 /* 
- * $Id: db_con.c 1827 2007-03-12 15:22:53Z bogdan_iancu $ 
+ * $Id: db_con.c 2943 2007-10-19 15:15:01Z anca_vamanu $ 
  *
  * UNIXODBC connection related functions
  *
@@ -41,7 +41,7 @@
 int use_table(db_con_t* _h, const char* _t)
 {
 	if ((!_h) || (!_t)) {
-		LOG(L_ERR, "use_table: Invalid parameter value\n");
+		LM_ERR("invalid parameter value\n");
 		return -1;
 	}
 
