@@ -1,5 +1,5 @@
 /* 
- * $Id: res.c 3348 2007-12-13 10:27:30Z bogdan_iancu $
+ * $Id: res.c 4193 2008-05-15 11:52:40Z henningw $
  *
  * UNIXODBC module result related functions
  *
@@ -123,6 +123,8 @@ static inline int get_columns(db_con_t* _h, db_res_t* _r)
 			case SQL_VARBINARY:
 			case SQL_LONGVARBINARY:
 			case SQL_BIT:
+			case SQL_LONGVARCHAR:
+			case SQL_WLONGVARCHAR:
 				RES_TYPES(_r)[i] = DB_BLOB;
 				break;
 
