@@ -1,5 +1,5 @@
 /* 
- * $Id: res.h 1827 2007-03-12 15:22:53Z bogdan_iancu $ 
+ * $Id: res.h 2871 2007-10-05 14:32:16Z henningw $ 
  *
  * UNIXODBC module result related functions
  *
@@ -34,12 +34,6 @@
 #include "../../db/db_con.h"
 
 
-/*
- * Create a new result structure and initialize it
- */
-db_res_t* new_result(void);
-
-
 /*
  * Fill the structure with data from database
  */
