@@ -1,5 +1,5 @@
 /* 
- * $Id: row.h 1827 2007-03-12 15:22:53Z bogdan_iancu $ 
+ * $Id: row.h 2185 2007-05-10 12:37:19Z henningw $ 
  *
  * UNIXODBC module row related functions
  *
@@ -41,11 +41,4 @@
 int convert_row(db_con_t* _h, db_res_t* _res, db_row_t* _r,
 		unsigned long* lengths);
 
-
-/*
- * Release memory used by row
- */
-int free_row(db_row_t* _r);
-
-
 #endif /* ROW_H */
