@@ -1,5 +1,5 @@
 /* 
- * $Id: val.h 1827 2007-03-12 15:22:53Z bogdan_iancu $ 
+ * $Id: val.h 2353 2007-06-11 13:57:32Z henningw $ 
  *
  * UNIXODBC module
  *
@@ -36,6 +36,7 @@
 #include <sql.h>
 #include <sqlext.h>
 #include "../../db/db_val.h"
+#include "../../db/db.h"
 
 
 /*
@@ -47,7 +48,7 @@ int str2val(db_type_t _t, db_val_t* _v, const char* _s, int _l);
 /*
  * Used when converting result from a query
  */
-int val2str(SQLHDBC* _c, db_val_t* _v, char* _s, int* _len);
+int val2str(db_con_t* _c, db_val_t* _v, char* _s, int* _len);
 
 
 #endif /* VAL_H */
