@@ -1,5 +1,5 @@
 /* 
- * $Id: ucontact.c 1866 2007-03-16 15:13:58Z miconda $ 
+ * $Id: ucontact.c 3830 2008-03-03 15:22:11Z bogdan_iancu $ 
  *
  * Usrloc contact structure
  *
@@ -50,35 +50,24 @@
 ucontact_t* new_ucontact(str* _dom, str* _aor, str* _contact,
 														ucontact_info_t* _ci)
 {
-#define str_dup(_new,_org) \
-	do{ \
-		(_new)->s = (char*)shm_malloc((_org)->len); \
-		if ( (_new)->s == 0) { \
-			LOG(L_ERR, "ERROR:usrloc:new_contact: no more shm memory\n"); \
-			goto error;\
-		}\
-		memcpy((_new)->s, (_org)->s, (_org)->len);\
-		(_new)->len = (_org)->len; \
-	}while(0)
-
 	ucontact_t *c;
 
 	c = (ucontact_t*)shm_malloc(sizeof(ucontact_t));
 	if (!c) {
-		LOG(L_ERR, "ERROR:usrloc:new_ucontact: no more shm memory\n");
+		LM_ERR("no more shm memory\n");
 		return 0;
 	}
 	memset(c, 0, sizeof(ucontact_t));
 
-	str_dup( &c->c, _contact);
-	str_dup( &c->callid, _ci->callid);
-	str_dup( &c->user_agent, _ci->user_agent);
+	if (shm_str_dup( &c->c, _contact) < 0) goto error;
+	if (shm_str_dup( &c->callid, _ci->callid) < 0) goto error;
+	if (shm_str_dup( &c->user_agent, _ci->user_agent) < 0) goto error;
 
 	if (_ci->received.s && _ci->received.len) {
-		str_dup( &c->received, &_ci->received);
+		if (shm_str_dup( &c->received, &_ci->received) < 0) goto error;
 	}
 	if (_ci->path && _ci->path->len) {
-		str_dup( &c->path, _ci->path);
+		if (shm_str_dup( &c->path, _ci->path) < 0) goto error;
 	}
 
 	c->domain = _dom;
@@ -95,6 +84,7 @@ ucontact_t* new_ucontact(str* _dom, str* _aor, str* _contact,
 
 	return c;
 error:
+	LM_ERR("no more shm memory\n");
 	if (c->path.s) shm_free(c->path.s);
 	if (c->received.s) shm_free(c->received.s);
 	if (c->user_agent.s) shm_free(c->user_agent.s);
@@ -185,8 +175,7 @@ int mem_update_ucontact(ucontact_t* _c, ucontact_info_t* _ci)
 		if ((_old)->len < (_new)->len) { \
 			ptr = (char*)shm_malloc((_new)->len); \
 			if (ptr == 0) { \
-				LOG(L_ERR, "ERROR:usrloc:update_ucontact: no "\
-					"more shm memory\n"); \
+				LM_ERR("no more shm memory\n"); \
 				return -1; \
 			}\
 			memcpy(ptr, (_new)->s, (_new)->len);\
@@ -244,28 +233,28 @@ void st_update_ucontact(ucontact_t* _c)
 {
 	switch(_c->state) {
 	case CS_NEW:
-		     /* Contact is new and is not in the database yet,
-		      * we remain in the same state here because the
-		      * contact must be inserted later in the timer
-		      */
+			 /* Contact is new and is not in the database yet,
+			  * we remain in the same state here because the
+			  * contact must be inserted later in the timer
+			  */
 		break;
 
 	case CS_SYNC:
-		     /* For db mode 2 a modified contact needs to be 
+			 /* For db mode 1 & 2 a modified contact needs to be 
 			  * updated also in the database, so transit into 
 			  * CS_DIRTY and let the timer to do the update 
-			  * again. For db mode 1 the db update is already
-			  * done and we don't have to change the state.
-		      */
-		if (db_mode == WRITE_BACK) {
+			  * again. For db mode 1 we try to update right
+			  * now and if fails, let the timer to do the job
+			  */
+		if (db_mode == WRITE_BACK || db_mode == WRITE_THROUGH) {
 			_c->state = CS_DIRTY;
 		}
 		break;
 
 	case CS_DIRTY:
-		     /* Modification of dirty contact results in
-		      * dirty contact again, don't change anything
-		      */
+			 /* Modification of dirty contact results in
+			  * dirty contact again, don't change anything
+			  */
 		break;
 	}
 }
@@ -503,12 +492,12 @@ int db_insert_ucontact(ucontact_t* _c)
 	}
 
 	if (ul_dbf.use_table(ul_dbh, _c->domain->s) < 0) {
-		LOG(L_ERR, "db_insert_ucontact(): Error in use_table\n");
+		LM_ERR("sql use_table failed\n");
 		return -1;
 	}
 
 	if (ul_dbf.insert(ul_dbh, keys, vals, (use_domain) ? (15) : (14)) < 0) {
-		LOG(L_ERR, "db_insert_ucontact(): Error while inserting contact\n");
+		LM_ERR("inserting contact in db failed\n");
 		return -1;
 	}
 
@@ -631,13 +620,13 @@ int db_update_ucontact(ucontact_t* _c)
 	}
 
 	if (ul_dbf.use_table(ul_dbh, _c->domain->s) < 0) {
-		LOG(L_ERR, "db_upd_ucontact(): Error in use_table\n");
+		LM_ERR("sql use_table failed\n");
 		return -1;
 	}
 
 	if (ul_dbf.update(ul_dbh, keys1, 0, vals1, keys2, vals2, 
 	(use_domain) ? (4) : (3), 11) < 0) {
-		LOG(L_ERR, "db_upd_ucontact(): Error while updating database\n");
+		LM_ERR("updating database failed\n");
 		return -1;
 	}
 
@@ -686,12 +675,12 @@ int db_delete_ucontact(ucontact_t* _c)
 	}
 
 	if (ul_dbf.use_table(ul_dbh, _c->domain->s) < 0) {
-		LOG(L_ERR, "db_del_ucontact: Error in use_table\n");
+		LM_ERR("sql use_table failed\n");
 		return -1;
 	}
 
 	if (ul_dbf.delete(ul_dbh, keys, 0, vals, (use_domain) ? (4) : (3)) < 0) {
-		LOG(L_ERR, "db_del_ucontact(): Error while deleting from database\n");
+		LM_ERR("deleting from database failed\n");
 		return -1;
 	}
 
@@ -769,12 +758,14 @@ int update_ucontact(struct urecord* _r, ucontact_t* _c, ucontact_info_t* _ci)
 	/* we have to update memory in any case, but database directly
 	 * only in db_mode 1 */
 	if (mem_update_ucontact( _c, _ci) < 0) {
-		LOG(L_ERR, "ERROR:usrloc:update_ucontact: failed to update memory\n");
+		LM_ERR("failed to update memory\n");
 		return -1;
 	}
 
 	/* run callbacks for UPDATE event */
-	if (exists_ulcb_type(UL_CONTACT_UPDATE)) {
+	if (exists_ulcb_type(UL_CONTACT_UPDATE))
+	{
+		LM_DBG("exists callback for type= UL_CONTACT_UPDATE\n");
 		run_ul_callbacks( UL_CONTACT_UPDATE, _c);
 	}
 
@@ -785,8 +776,9 @@ int update_ucontact(struct urecord* _r, ucontact_t* _c, ucontact_info_t* _ci)
 
 	if (db_mode == WRITE_THROUGH || db_mode==DB_ONLY) {
 		if (db_update_ucontact(_c) < 0) {
-			LOG(L_ERR, "ERROR:usrloc:update_ucontact: failed to update "
-				"database\n");
+			LM_ERR("failed to update database\n");
+		} else {
+			_c->state = CS_SYNC;
 		}
 	}
 	return 0;
