@@ -1,5 +1,5 @@
 /**
- * $Id: xl_lib.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: xl_lib.c 2725 2007-09-09 20:55:56Z miconda $
  *
  * XLOG module
  *
@@ -34,9 +34,9 @@
 
 #include "xl_lib.h"
 
-int xl_print_log(struct sip_msg* msg, xl_elem_p list, char *buf, int *len)
+int xl_print_log(struct sip_msg* msg, pv_elem_p list, char *buf, int *len)
 {
-	return xl_printf(msg, list, buf, len);
+	return pv_printf(msg, list, buf, len);
 }
 
 
