@@ -1,5 +1,5 @@
 /* 
- * $Id: msg_translator.c 1827 2007-03-12 15:22:53Z bogdan_iancu $
+ * $Id: msg_translator.c 3259 2007-12-05 09:14:39Z bogdan_iancu $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  * Copyright (C) 2006 Andreas Granig <agranig@linguin.org>
@@ -162,8 +162,7 @@ static int check_via_address(struct ip_addr* ip, str *name,
 	/* maybe we are lucky and name it's an ip */
 	s=ip_addr2a(ip);
 	if (s){
-		DBG("check_via_address(%s, %.*s, %d)\n", 
-			s, name->len, name->s, resolver);
+		LM_DBG("params %s, %.*s, %d\n", s, name->len, name->s, resolver);
 		len=strlen(s);
 
 	#ifdef USE_IPV6
@@ -183,13 +182,13 @@ static int check_via_address(struct ip_addr* ip, str *name,
 			if (strncmp(name->s, s, name->len)==0) 
 				return 0;
 	}else{
-		LOG(L_CRIT, "check_via_address: BUG: could not convert ip address\n");
+		LM_CRIT("could not convert ip address\n");
 		return -1;
 	}
 
 	if (port==0) port=SIP_PORT;
 	if (resolver&DO_DNS){
-		DBG("check_via_address: doing dns lookup\n");
+		LM_DBG("doing dns lookup\n");
 		/* try all names ips */
 		he=sip_resolvehost(name, &port, &proto, 0, 0);
 		if (he && (int)ip->af==he->h_addrtype){
@@ -200,7 +199,7 @@ static int check_via_address(struct ip_addr* ip, str *name,
 		}
 	}
 	if (resolver&DO_REV_DNS){
-		DBG("check_via_address: doing rev. dns lookup\n");
+		LM_DBG("doing rev. dns lookup\n");
 		/* try reverse dns */
 		he=rev_resolvehost(ip);
 		if (he && (strncmp(he->h_name, name->s, name->len)==0))
@@ -218,10 +217,19 @@ static int check_via_address(struct ip_addr* ip, str *name,
 int received_test( struct sip_msg *msg )
 {
 	int rcvd;
+	
+	if(msg->via1->received !=NULL)
+		return 1;
 
-	rcvd = msg->via1->received || check_via_address(&msg->rcv.src_ip,
+	if(msg->via1->maddr){
+		rcvd = check_via_address(&msg->rcv.src_ip, &msg->via1->maddr->value, 
+			msg->via1->port, msg->via1->proto, received_dns);
+	} else {
+		rcvd = check_via_address(&msg->rcv.src_ip,
 			&msg->via1->host, msg->via1->port, msg->via1->proto, received_dns);
-	return rcvd;
+	}
+
+    return rcvd; 
 }
 
 
@@ -295,7 +303,7 @@ static char * warning_builder( struct sip_msg *msg, unsigned int *returned_len)
 	*returned_len=clen;
 	return buf;
 error_overflow:
-	LOG(L_ERR, "ERROR: warning_builder: buffer size exceeded\n");
+	LM_ERR("buffer size exceeded\n");
 	*returned_len=0;
 	return 0;
 }
@@ -316,7 +324,7 @@ char* received_builder(struct sip_msg *msg, unsigned int *received_len)
 	buf=pkg_malloc(sizeof(char)*MAX_RECEIVED_SIZE);
 	if (buf==0){
 		ser_error=E_OUT_OF_MEM;
-		LOG(L_ERR, "ERROR: received_builder: out of memory\n");
+		LM_ERR("out of pkg memory\n");
 		return 0;
 	}
 	memcpy(buf, RECEIVED, RECEIVED_LEN);
@@ -347,7 +355,7 @@ char* rport_builder(struct sip_msg *msg, unsigned int *rport_len)
 	buf=pkg_malloc(sizeof(char)*(len+1));/* space for null term */
 	if (buf==0){
 		ser_error=E_OUT_OF_MEM;
-		LOG(L_ERR, "ERROR: rport_builder: out of memory\n");
+		LM_ERR("out of pkg memory\n");
 		return 0;
 	}
 	memcpy(buf, RPORT, RPORT_LEN);
@@ -371,7 +379,7 @@ char* id_builder(struct sip_msg* msg, unsigned int *id_len)
 	size=sizeof(int)*2;
 	p=&revhex[0];
 	if (int2reverse_hex(&p, &size, msg->rcv.proto_reserved1)==-1){
-		LOG(L_CRIT, "BUG: id_builder: not enough space for id\n");
+		LM_CRIT("not enough space for id\n");
 		return 0;
 	}
 	value_len=p-&revhex[0];
@@ -379,7 +387,7 @@ char* id_builder(struct sip_msg* msg, unsigned int *id_len)
 	buf=pkg_malloc(sizeof(char)*(len+1));/* place for ending \0 */
 	if (buf==0){
 		ser_error=E_OUT_OF_MEM;
-		LOG(L_ERR, "ERROR: rport_builder: out of memory\n");
+		LM_ERR("out of pkg memory\n");
 		return 0;
 	}
 	memcpy(buf, ID_PARAM, ID_PARAM_LEN);
@@ -404,19 +412,18 @@ char* clen_builder(struct sip_msg* msg, int *clen_len, int diff)
 	body=get_body(msg);
 	if (body==0){
 		ser_error=E_BAD_REQ;
-		LOG(L_ERR, "ERROR: clen_builder: no message body found"
-					" (missing crlf?)");
+		LM_ERR("no message body found (missing crlf?)");
 		return 0;
 	}
 	value=msg->len-(int)(body-msg->buf)+diff;
 	value_s=int2str(value, &value_len);
-	DBG("clen_builder: content-length: %d (%s)\n", value, value_s);
+	LM_DBG("content-length: %d (%s)\n", value, value_s);
 		
 	len=CONTENT_LENGTH_LEN+value_len+CRLF_LEN;
 	buf=pkg_malloc(sizeof(char)*(len+1));
 	if (buf==0){
 		ser_error=E_OUT_OF_MEM;
-		LOG(L_ERR, "ERROR: clen_builder: out of memory\n");
+		LM_ERR("out of pkg memory\n");
 		return 0;
 	}
 	memcpy(buf, CONTENT_LENGTH, CONTENT_LENGTH_LEN);
@@ -431,7 +438,7 @@ char* clen_builder(struct sip_msg* msg, int *clen_len, int diff)
 
 /* checks if a lump opt condition 
  * returns 1 if cond is true, 0 if false */
-static inline int lump_check_opt(	enum lump_conditions cond,
+static inline int lump_check_opt(	struct lump *l,
 									struct sip_msg* msg,
 									struct socket_info* snd_s
 									)
@@ -442,7 +449,7 @@ static inline int lump_check_opt(	enum lump_conditions cond,
 
 #define get_ip_port_proto \
 			if (snd_s==0){ \
-				LOG(L_CRIT, "ERROR: lump_check_opt: null send socket\n"); \
+				LM_CRIT("null send socket\n"); \
 				return 1; /* we presume they are different :-) */ \
 			} \
 			if (msg->rcv.bind_address){ \
@@ -455,10 +462,11 @@ static inline int lump_check_opt(	enum lump_conditions cond,
 				proto=msg->rcv.proto; \
 			} \
 			
-	switch(cond){
+	switch(l->u.cond){
 		case COND_FALSE:
 			return 0;
 		case COND_TRUE:
+			l->flags |= LUMPFLAG_COND_TRUE;
 			return 1;
 		case COND_IF_DIFF_REALMS:
 			get_ip_port_proto;
@@ -466,28 +474,32 @@ static inline int lump_check_opt(	enum lump_conditions cond,
 			if ((port==snd_s->port_no)&&(proto==snd_s->proto)&&
 				(ip_addr_cmp(ip, &snd_s->address)))
 				return 0;
-			else return 1;
+			l->flags |= LUMPFLAG_COND_TRUE;
+			return 1;
 		case COND_IF_DIFF_AF:
 			get_ip_port_proto;
-			if (ip->af!=snd_s->address.af) return 1;
-			else return 0;
+			if (ip->af==snd_s->address.af) return 0;
+			l->flags |= LUMPFLAG_COND_TRUE;
+			return 1;
 		case COND_IF_DIFF_PROTO:
 			get_ip_port_proto;
-			if (proto!=snd_s->proto) return 1;
-			else return 0;
+			if (proto==snd_s->proto) return 0;
+			l->flags |= LUMPFLAG_COND_TRUE;
+			return 1;
 		case COND_IF_DIFF_PORT:
 			get_ip_port_proto;
-			if (port!=snd_s->port_no) return 1;
-			else return 0;
+			if (port==snd_s->port_no) return 0;
+			l->flags |= LUMPFLAG_COND_TRUE;
+			return 1;
 		case COND_IF_DIFF_IP:
 			get_ip_port_proto;
 			if (ip_addr_cmp(ip, &snd_s->address)) return 0;
-			else return 1;
+			l->flags |= LUMPFLAG_COND_TRUE;
+			return 1;
 		case COND_IF_RAND:
 			return (rand()>=RAND_MAX/2);
 		default:
-			LOG(L_CRIT, "BUG: lump_check_opt: unknown lump condition %d\n",
-					cond);
+			LM_CRIT("unknown lump condition %d\n", l->u.cond);
 	}
 	return 0; /* false */
 }
@@ -515,7 +527,7 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 						new_len+=2; \
 				}else{ \
 					/* FIXME */ \
-					LOG(L_CRIT, "FIXME: null bind_address\n"); \
+					LM_CRIT("fixme:null bind_address\n"); \
 				}; \
 				break; \
 			case SUBST_RCV_PORT: \
@@ -523,7 +535,7 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 					new_len+=msg->rcv.bind_address->port_no_str.len; \
 				}else{ \
 					/* FIXME */ \
-					LOG(L_CRIT, "FIXME: null bind_address\n"); \
+					LM_CRIT("fixme: null bind_address\n"); \
 				}; \
 				break; \
 			case SUBST_RCV_PROTO: \
@@ -539,12 +551,12 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 								new_len+=4; \
 								break; \
 						default: \
-						LOG(L_CRIT, "BUG: lumps_len: unknown proto %d\n", \
+						LM_CRIT("unknown proto %d\n", \
 								msg->rcv.bind_address->proto); \
 					}\
 				}else{ \
 					/* FIXME */ \
-					LOG(L_CRIT, "FIXME: null bind_address\n"); \
+					LM_CRIT("fixme: null bind_address\n"); \
 				}; \
 				break; \
 			case SUBST_RCV_ALL: \
@@ -569,12 +581,12 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 								new_len+=TRANSPORT_PARAM_LEN+4; \
 								break; \
 						default: \
-						LOG(L_CRIT, "BUG: lumps_len: unknown proto %d\n", \
+						LM_CRIT("unknown proto %d\n", \
 								msg->rcv.bind_address->proto); \
 					}\
 				}else{ \
 					/* FIXME */ \
-					LOG(L_CRIT, "FIXME: null bind_address\n"); \
+					LM_CRIT("fixme: null bind_address\n"); \
 				}; \
 				break; \
 			case SUBST_SND_IP: \
@@ -584,7 +596,7 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 							send_address_str==&(send_sock->address_str)) \
 						new_len+=2; \
 				}else{ \
-					LOG(L_CRIT, "FIXME: lumps_len called with" \
+					LM_CRIT("fixme: lumps_len called with" \
 							" null send_sock\n"); \
 				}; \
 				break; \
@@ -592,7 +604,7 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 				if (send_sock){ \
 					new_len+=send_port_str->len; \
 				}else{ \
-					LOG(L_CRIT, "FIXME: lumps_len called with" \
+					LM_CRIT("lumps_len called with" \
 							" null send_sock\n"); \
 				}; \
 				break; \
@@ -609,11 +621,11 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 								new_len+=4; \
 								break; \
 						default: \
-						LOG(L_CRIT, "BUG: lumps_len: unknown proto %d\n", \
+						LM_CRIT("unknown proto %d\n", \
 								send_sock->proto); \
 					}\
 				}else{ \
-					LOG(L_CRIT, "FIXME: lumps_len called with" \
+					LM_CRIT("lumps_len called with" \
 							" null send_sock\n"); \
 				}; \
 				break; \
@@ -641,19 +653,19 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 								new_len+=TRANSPORT_PARAM_LEN+4; \
 								break; \
 						default: \
-						LOG(L_CRIT, "BUG: lumps_len: unknown proto %d\n", \
+						LM_CRIT("unknown proto %d\n", \
 								send_sock->proto); \
 					}\
 				}else{ \
 					/* FIXME */ \
-					LOG(L_CRIT, "FIXME: lumps_len called with" \
+					LM_CRIT("lumps_len called with" \
 							" null send_sock\n"); \
 				}; \
 				break; \
 			case SUBST_NOP: /* do nothing */ \
 				break; \
 			default: \
-				LOG(L_CRIT, "BUG: unknown subst type %d\n", \
+				LM_CRIT("unknown subst type %d\n", \
 						(subst_l)->u.subst); \
 		}
 	
@@ -665,14 +677,14 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 	else
 		send_address_str=&(send_sock->address_str);
 	if (msg->set_global_port.len)
-		send_port_str=&(msg->set_global_address);
+		send_port_str=&(msg->set_global_port);
 	else
 		send_port_str=&(send_sock->port_no_str);
 	
 	
 	for(t=lumps;t;t=t->next){
 		/* skip if this is an OPT lump and the condition is not satisfied */
-		if ((t->op==LUMP_ADD_OPT) && !lump_check_opt(t->u.cond, msg, send_sock))
+		if ((t->op==LUMP_ADD_OPT) && !lump_check_opt(t, msg, send_sock))
 			continue;
 		for(r=t->before;r;r=r->before){
 			switch(r->op){
@@ -685,13 +697,12 @@ static inline int lumps_len(struct sip_msg* msg, struct lump* lumps,
 				case LUMP_ADD_OPT:
 					/* skip if this is an OPT lump and the condition is 
 					 * not satisfied */
-					if (!lump_check_opt(r->u.cond, msg, send_sock))
+					if (!lump_check_opt(r, msg, send_sock))
 						goto skip_before;
 					break;
 				default:
 					/* only ADD allowed for before/after */
-						LOG(L_CRIT, "BUG: lumps_len: invalid op "
-							"for data lump (%x)\n", r->op);
+						LM_CRIT("invalid op for data lump (%x)\n", r->op);
 			}
 		}
 skip_before:
@@ -727,8 +738,7 @@ skip_before:
 				/* do nothing */
 				break;
 			default:
-				LOG(L_CRIT,"BUG:lumps_len: invalid"
-							" op for data lump (%x)\n", r->op);
+				LM_CRIT("op for data lump (%x)\n", r->op);
 		}
 		for (r=t->after;r;r=r->after){
 			switch(r->op){
@@ -741,13 +751,12 @@ skip_before:
 				case LUMP_ADD_OPT:
 					/* skip if this is an OPT lump and the condition is 
 					 * not satisfied */
-					if (!lump_check_opt(r->u.cond, msg, send_sock))
+					if (!lump_check_opt(r, msg, send_sock))
 						goto skip_after;
 					break;
 				default:
 					/* only ADD allowed for before/after */
-					LOG(L_CRIT, "BUG:lumps_len: invalid"
-								" op for data lump (%x)\n", r->op);
+					LM_CRIT("invalid op for data lump (%x)\n", r->op);
 			}
 		}
 skip_after:
@@ -792,7 +801,7 @@ static inline void process_lumps(	struct sip_msg* msg,
 				}\
 			}else{  \
 				/*FIXME*/ \
-				LOG(L_CRIT, "FIXME: process_lumps: null bind_address\n"); \
+				LM_CRIT("null bind_address\n"); \
 			}; \
 			break; \
 		case SUBST_RCV_PORT: \
@@ -802,7 +811,7 @@ static inline void process_lumps(	struct sip_msg* msg,
 				offset+=msg->rcv.bind_address->port_no_str.len; \
 			}else{  \
 				/*FIXME*/ \
-				LOG(L_CRIT, "FIXME: process_lumps: null bind_address\n"); \
+				LM_CRIT("null bind_address\n"); \
 			}; \
 			break; \
 		case SUBST_RCV_ALL: \
@@ -851,12 +860,12 @@ static inline void process_lumps(	struct sip_msg* msg,
 						offset+=4; \
 						break; \
 					default: \
-						LOG(L_CRIT, "BUG: process_lumps: unknown proto %d\n", \
+						LM_CRIT("unknown proto %d\n", \
 								msg->rcv.bind_address->proto); \
 				} \
 			}else{  \
 				/*FIXME*/ \
-				LOG(L_CRIT, "FIXME: process_lumps: null bind_address\n"); \
+				LM_CRIT("null bind_address\n"); \
 			}; \
 			break; \
 		case SUBST_SND_IP: \
@@ -874,8 +883,7 @@ static inline void process_lumps(	struct sip_msg* msg,
 				}\
 			}else{  \
 				/*FIXME*/ \
-				LOG(L_CRIT, "FIXME: process_lumps: called with" \
-							" null send_sock\n"); \
+				LM_CRIT("called with null send_sock\n"); \
 			}; \
 			break; \
 		case SUBST_SND_PORT: \
@@ -885,8 +893,7 @@ static inline void process_lumps(	struct sip_msg* msg,
 				offset+=send_port_str->len; \
 			}else{  \
 				/*FIXME*/ \
-				LOG(L_CRIT, "FIXME: process_lumps: called with" \
-						" null send_sock\n"); \
+				LM_CRIT("called with null send_sock\n"); \
 			}; \
 			break; \
 		case SUBST_SND_ALL: \
@@ -937,12 +944,12 @@ static inline void process_lumps(	struct sip_msg* msg,
 						offset+=4; \
 						break; \
 					default: \
-						LOG(L_CRIT, "BUG: process_lumps: unknown proto %d\n", \
+						LM_CRIT("unknown proto %d\n", \
 								send_sock->proto); \
 				} \
 			}else{  \
 				/*FIXME*/ \
-				LOG(L_CRIT, "FIXME: process_lumps: null bind_address\n"); \
+				LM_CRIT("null bind_address\n"); \
 			}; \
 			break; \
 		case SUBST_RCV_PROTO: \
@@ -966,13 +973,12 @@ static inline void process_lumps(	struct sip_msg* msg,
 						offset+=4; \
 						break; \
 					default: \
-						LOG(L_CRIT, "BUG: process_lumps: unknown proto %d\n", \
+						LM_CRIT("unknown proto %d\n", \
 								msg->rcv.bind_address->proto); \
 				} \
 			}else{  \
 				/*FIXME*/ \
-				LOG(L_CRIT, "FIXME: process_lumps: called with null" \
-							" send_sock \n"); \
+				LM_CRIT("called with null send_sock \n"); \
 			}; \
 			break; \
 		case  SUBST_SND_PROTO: \
@@ -996,17 +1002,16 @@ static inline void process_lumps(	struct sip_msg* msg,
 						offset+=4; \
 						break; \
 					default: \
-						LOG(L_CRIT, "BUG: process_lumps: unknown proto %d\n", \
+						LM_CRIT("unknown proto %d\n", \
 								send_sock->proto); \
 				} \
 			}else{  \
 				/*FIXME*/ \
-				LOG(L_CRIT, "FIXME: process_lumps: called with null" \
-							" send_sock \n"); \
+				LM_CRIT("called with null send_sock \n"); \
 			}; \
 			break; \
 		default: \
-					LOG(L_CRIT, "BUG: process_lumps: unknown subst type %d\n", \
+					LM_CRIT("unknown subst type %d\n", \
 							(subst_l)->u.subst); \
 	} \
  \
@@ -1017,7 +1022,7 @@ static inline void process_lumps(	struct sip_msg* msg,
 	else
 		send_address_str=&(send_sock->address_str);
 	if (msg->set_global_port.len)
-		send_port_str=&(msg->set_global_address);
+		send_port_str=&(msg->set_global_port);
 	else
 		send_port_str=&(send_sock->port_no_str);
 	
@@ -1034,7 +1039,7 @@ static inline void process_lumps(	struct sip_msg* msg,
 				/* skip if this is an OPT lump and the condition is 
 				 * not satisfied */
 				if ((t->op==LUMP_ADD_OPT) &&
-						(!lump_check_opt(t->u.cond, msg, send_sock))) 
+						(!lump_check_opt(t, msg, send_sock))) 
 					continue;
 				/* just add it here! */
 				/* process before  */
@@ -1051,13 +1056,12 @@ static inline void process_lumps(	struct sip_msg* msg,
 						case LUMP_ADD_OPT:
 							/* skip if this is an OPT lump and the condition is
 					 		* not satisfied */
-							if (!lump_check_opt(r->u.cond, msg, send_sock))
+							if (!lump_check_opt(r, msg, send_sock))
 								goto skip_before;
 							break;
 						default:
 							/* only ADD allowed for before/after */
-							LOG(L_CRIT, "BUG:process_lumps: "
-									"invalid op for data lump (%x)\n", r->op);
+							LM_CRIT("invalid op for data lump (%x)\n", r->op);
 					}
 				}
 skip_before:
@@ -1075,8 +1079,7 @@ skip_before:
 						break;
 					default: 
 						/* should not ever get here */
-						LOG(L_CRIT, "BUG: process_lumps: unhandled data lump "
-								" op %d\n", t->op);
+						LM_CRIT("unhandled data lump op %d\n", t->op);
 				}
 				/* process after */
 				for(r=t->after;r;r=r->after){
@@ -1092,13 +1095,12 @@ skip_before:
 						case LUMP_ADD_OPT:
 							/* skip if this is an OPT lump and the condition is 
 					 		* not satisfied */
-							if (!lump_check_opt(r->u.cond, msg, send_sock))
+							if (!lump_check_opt(r, msg, send_sock))
 								goto skip_after;
 							break;
 						default:
 							/* only ADD allowed for before/after */
-							LOG(L_CRIT, "BUG:process_lumps: "
-									"invalid op for data lump (%x)\n", r->op);
+							LM_CRIT("invalid op for data lump (%x)\n", r->op);
 					}
 				}
 skip_after:
@@ -1107,7 +1109,7 @@ skip_after:
 			case LUMP_DEL:
 				/* copy till offset */
 				if (s_offset>t->u.offset){
-					DBG("Warning: (%d) overlapped lumps offsets,"
+					LM_WARN("(%d) overlapped lumps offsets,"
 						" ignoring(%x, %x)\n", t->op, s_offset,t->u.offset);
 					/* this should've been fixed above (when computing len) */
 					/* just ignore it*/
@@ -1133,13 +1135,12 @@ skip_after:
 						case LUMP_ADD_OPT:
 							/* skip if this is an OPT lump and the condition is 
 					 		* not satisfied */
-							if (!lump_check_opt(r->u.cond, msg, send_sock))
+							if (!lump_check_opt(r, msg, send_sock))
 								goto skip_nop_before;
 							break;
 						default:
 							/* only ADD allowed for before/after */
-							LOG(L_CRIT, "BUG:process_lumps: "
-									"invalid op for data lump (%x)\n",r->op);
+							LM_CRIT("invalid op for data lump (%x)\n",r->op);
 					}
 				}
 skip_nop_before:
@@ -1162,20 +1163,18 @@ skip_nop_before:
 						case LUMP_ADD_OPT:
 							/* skip if this is an OPT lump and the condition is 
 					 		* not satisfied */
-							if (!lump_check_opt(r->u.cond, msg, send_sock)) 
+							if (!lump_check_opt(r, msg, send_sock)) 
 								goto skip_nop_after;
 							break;
 						default:
 							/* only ADD allowed for before/after */
-							LOG(L_CRIT, "BUG:process_lumps: "
-									"invalid op for data lump (%x)\n", r->op);
+							LM_CRIT("invalid op for data lump (%x)\n", r->op);
 					}
 				}
 skip_nop_after:
 				break;
 			default:
-					LOG(L_CRIT, "BUG: process_lumps: "
-							"unknown op (%x)\n", t->op);
+					LM_CRIT("unknown op (%x)\n", t->op);
 		}
 	}
 	*new_buf_offs=offset;
@@ -1208,7 +1207,7 @@ static inline int adjust_clen(struct sip_msg* msg, int body_delta, int proto)
 #endif
 	    ) {
 		if (parse_headers(msg, HDR_CONTENTLENGTH_F, 0)==-1){
-			LOG(L_ERR, "adjust_clen: error parsing content-length\n");
+			LM_ERR("parsing content-length\n");
 			goto error;
 		}
 		if (msg->content_length==0){
@@ -1219,7 +1218,7 @@ static inline int adjust_clen(struct sip_msg* msg, int body_delta, int proto)
 			anchor=anchor_lump(msg, msg->unparsed-msg->buf, 0,
 												HDR_CONTENTLENGTH_T);
 			if (anchor==0){
-				LOG(L_ERR, "adjust_clen: cannot set clen anchor\n");
+				LM_ERR("cannot set clen anchor\n");
 				goto error;
 			}
 		}
@@ -1229,7 +1228,7 @@ static inline int adjust_clen(struct sip_msg* msg, int body_delta, int proto)
 	
 	if ((anchor==0) && body_delta){
 		if (parse_headers(msg, HDR_CONTENTLENGTH_F, 0) == -1) {
-			LOG(L_ERR, "adjust_clen: Error parsing Content-Length\n");
+			LM_ERR("parsing Content-Length\n");
 			goto error;
 		}
 		
@@ -1247,19 +1246,18 @@ static inline int adjust_clen(struct sip_msg* msg, int body_delta, int proto)
 				anchor=anchor_lump(msg, msg->unparsed-msg->buf, 0,
 													HDR_CONTENTLENGTH_T);
 				if (anchor==0){
-					LOG(L_ERR, "adjust_clen: cannot set clen anchor\n");
+					LM_ERR("cannot set clen anchor\n");
 					goto error;
 				}
 			}else{
-				DBG("add_clen: UDP packet with no clen => not adding one \n");
+				LM_DBG("the UDP packet has no clen => not adding one \n");
 			}
 		}else{
 			/* Content-Length has been found, remove it */
 			anchor = del_lump(	msg, msg->content_length->name.s - msg->buf,
 								msg->content_length->len, HDR_CONTENTLENGTH_T);
 			if (anchor==0) {
-				LOG(L_ERR, "adjust_clen: Can't remove original"
-							" Content-Length\n");
+				LM_ERR("can't remove original Content-Length\n");
 				goto error;
 			}
 		}
@@ -1319,13 +1317,13 @@ static inline int insert_path_as_route(struct sip_msg* msg, str* path)
 	}
 
 	if (anchor == 0) {
-		LOG(L_ERR, "ERROR: insert_path_as_route(): Failed to get anchor\n");
+		LM_ERR("failed to get anchor\n");
 		return -1;
 	}
 
 	route = pkg_malloc(ROUTE_LEN + path->len + CRLF_LEN);
 	if (!route) {
-		LOG(L_ERR, "ERROR: insert_path_as_route(): Out of memory\n");
+		LM_ERR("out of pkg memory\n");
 		return -1;
 	}
 	memcpy(route, ROUTE_STR, ROUTE_LEN);
@@ -1333,7 +1331,7 @@ static inline int insert_path_as_route(struct sip_msg* msg, str* path)
 	memcpy(route + ROUTE_LEN + path->len, CRLF, CRLF_LEN);
 
 	if (insert_new_lump_before(anchor, route, ROUTE_LEN + path->len + CRLF_LEN, 0) == 0) {
-		LOG(L_ERR, "ERROR: insert_path_as_route(): Failed to insert lump\n");
+		LM_ERR("failed to insert lump\n");
 		return -1;
 	}
 
@@ -1384,12 +1382,11 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 #endif
 	){
 		if  ((id_buf=id_builder(msg, &id_len))==0){
-			LOG(L_ERR, "ERROR: build_req_buf_from_sip_req:"
-							" id_builder failed\n");
+			LM_ERR("id_builder failed\n");
 			goto error; /* we don't need to free anything,
 			                 nothing alloc'ed yet*/
 		}
-		DBG("build_req_from_req: id added: <%.*s>, rcv proto=%d\n",
+		LM_DBG("id added: <%.*s>, rcv proto=%d\n",
 				(int)id_len, id_buf, msg->rcv.proto);
 		extra_params.s=id_buf;
 		extra_params.len=id_len;
@@ -1398,8 +1395,7 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 
 	if (msg->path_vec.len) {
 		if (insert_path_as_route(msg, &msg->path_vec) < 0) {
-			LOG(L_ERR,"ERROR: build_req_buf_from_sip_req: "
-				"adding path lumps failed\n");
+			LM_ERR("adding path lumps failed\n");
 			goto error;
 		}
 	}
@@ -1413,8 +1409,7 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 										will be null terminated */
 		extra_params.s = (char*)pkg_malloc(extra_params.len+1);
 		if(extra_params.s==0) {
-			LOG(L_ERR, "ERROR: build_req_buf_from_sip_req:"
-							" extra params building failed\n");
+			LM_ERR("extra params building failed\n");
 			if (id_buf) pkg_free(id_buf);
 			goto error;
 		}
@@ -1425,8 +1420,7 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 		}
 		memcpy(extra_params.s+id_len, RPORT, RPORT_LEN-1);
 		extra_params.s[extra_params.len]='\0';
-		DBG("build_req_from_req: extra param added: <%.*s>\n",
-				extra_params.len, extra_params.s);
+		LM_DBG("extra param added: <%.*s>\n",extra_params.len, extra_params.s);
 	}
 	
 	/* Calculate message body difference and adjust
@@ -1434,8 +1428,7 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 	 */
 	body_delta = lumps_len(msg, msg->body_lumps, send_sock);
 	if (adjust_clen(msg, body_delta, proto) < 0) {
-		LOG(L_ERR, "ERROR: build_req_buf_from_sip_req: Error while adjusting"
-				" Content-Length\n");
+		LM_ERR("failed to adjust Content-Length\n");
 		goto error00;
 	}
 
@@ -1445,14 +1438,13 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 	line_buf = via_builder( &via_len, send_sock, &branch,
 							extra_params.len?&extra_params:0, proto, &hp);
 	if (!line_buf){
-		LOG(L_ERR,"ERROR: build_req_buf_from_sip_req: no via received!\n");
+		LM_ERR("no via received!\n");
 		goto error00;
 	}
 	/* check if received needs to be added */
 	if ( received_test(msg) ) { 
 		if ((received_buf=received_builder(msg,&received_len))==0){
-			LOG(L_ERR, "ERROR: build_req_buf_from_sip_req:"
-							" received_builder failed\n");
+			LM_ERR("received_builder failed\n");
 			goto error01;  /* free also line_buf */
 		}
 	}
@@ -1465,8 +1457,7 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 	if ((msg->msg_flags&FL_FORCE_RPORT)||
 			(msg->via1->rport /*&& msg->via1->rport->value.s==0*/)){
 		if ((rport_buf=rport_builder(msg, &rport_len))==0){
-			LOG(L_ERR, "ERROR: build_req_buf_from_sip_req:"
-							" rport_builder failed\n");
+			LM_ERR("rport_builder failed\n");
 			goto error01; /* free everything */
 		}
 	}
@@ -1525,7 +1516,7 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 	/* compute new msg len and fix overlapping zones*/
 	new_len=len+body_delta+lumps_len(msg, msg->add_rm, send_sock);
 #ifdef XL_DEBUG
-	LOG(L_ERR, "DEBUG: new_len(%d)=len(%d)+lumps_len\n", new_len, len);
+	LM_DBG("new_len(%d)=len(%d)+lumps_len\n", new_len, len);
 #endif
 
 	if (msg->new_uri.s){
@@ -1535,7 +1526,7 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 	new_buf=(char*)pkg_malloc(new_len+1);
 	if (new_buf==0){
 		ser_error=E_OUT_OF_MEM;
-		LOG(L_ERR, "ERROR: build_req_buf_from_sip_req: out of memory\n");
+		LM_ERR("out of pkg memory\n");
 		goto error00;
 	}
 
@@ -1559,13 +1550,6 @@ char * build_req_buf_from_sip_req( struct sip_msg* msg,
 	memcpy(new_buf+offset, buf+s_offset, len-s_offset);
 	new_buf[new_len]=0;
 
-#ifdef DBG_MSG_QA
-	if (new_buf[new_len-1]==0) {
-		LOG(L_ERR, "ERROR: build_req_buf_from_sip_req: 0 in the end\n");
-		abort();
-	}
-#endif
-
 	*returned_len=new_len;
 	/* cleanup */
 	if (extra_params.s) pkg_free(extra_params.s);
@@ -1607,32 +1591,30 @@ char * build_res_buf_from_sip_res( struct sip_msg* msg,
 		via_offset=msg->h_via1->name.s-buf;
 	}
 
-	     /* Calculate message body difference and adjust
-	      * Content-Length
-	      */
+	/* Calculate message body difference and adjust
+	 * Content-Length
+	 */
 	body_delta = lumps_len(msg, msg->body_lumps, 0);
 	if (adjust_clen(msg, body_delta, (msg->via2? msg->via2->proto:PROTO_UDP))
 			< 0) {
-		LOG(L_ERR, "ERROR: build_req_buf_from_sip_req: Error while adjusting"
-				" Content-Length\n");
+		LM_ERR("failed to adjust Content-Length\n");
 		goto error;
 	}
 
 	/* remove the first via*/
 	if (del_lump( msg, via_offset, via_len, HDR_VIA_T)==0){
-		LOG(L_ERR, "build_res_buf_from_sip_res: error trying to remove first"
-					"via\n");
+		LM_ERR("failed to remove first via\n");
 		goto error;
 	}
 
 	new_len=len+body_delta+lumps_len(msg, msg->add_rm, 0); /*FIXME: we don't
 														know the send sock */
 	
-	DBG(" old size: %d, new size: %d\n", len, new_len);
+	LM_DBG(" old size: %d, new size: %d\n", len, new_len);
 	new_buf=(char*)pkg_malloc(new_len+1); /* +1 is for debugging 
 											 (\0 to print it )*/
 	if (new_buf==0){
-		LOG(L_ERR, "ERROR: build_res_buf_from_sip_res: out of mem\n");
+		LM_ERR("out of pkg mem\n");
 		goto error;
 	}
 	new_buf[new_len]=0; /* debug: print the message */
@@ -1644,8 +1626,11 @@ char * build_res_buf_from_sip_res( struct sip_msg* msg,
 	memcpy(new_buf+offset,
 		buf+s_offset, 
 		len-s_offset);
-	 /* send it! */
-	DBG("build_res_from_sip_res: copied size: orig:%d, new: %d, rest: %d"
+	/* as it is a relaied reply, if 503, make it 500 (just reply code) */
+	if ( msg->first_line.u.reply.statuscode==503 )
+		new_buf[(int)(msg->first_line.u.reply.status.s-msg->buf)+2] = '0'; 
+	/* send it! */
+	LM_DBG("copied size: orig:%d, new: %d, rest: %d"
 			" msg=\n%s\n", s_offset, offset, len-s_offset, new_buf);
 
 	*returned_len=new_len;
@@ -1689,8 +1674,7 @@ char * build_res_buf_from_sip_req( unsigned int code, str *text ,str *new_tag,
 	end of header (non-block Vias are a really poor property
 	of SIP :( ) */
 	if (parse_headers( msg, HDR_EOH_F, 0 )==-1) {
-		LOG(L_ERR, "ERROR: build_res_buf_from_sip_req: "
-			"alas, parse_headers failed\n");
+		LM_ERR("parse_headers failed\n");
 		goto error00;
 	}
 
@@ -1700,8 +1684,7 @@ char * build_res_buf_from_sip_req( unsigned int code, str *text ,str *new_tag,
 	/* check if received needs to be added */
 	if (received_test(msg)) {
 		if ((received_buf=received_builder(msg,&received_len))==0) {
-			LOG(L_ERR, "ERROR: build_res_buf_from_sip_req: "
-				"alas, received_builder failed\n");
+			LM_ERR("received_builder failed\n");
 			goto error00;
 		}
 	}
@@ -1709,8 +1692,7 @@ char * build_res_buf_from_sip_req( unsigned int code, str *text ,str *new_tag,
 	if ( (msg->msg_flags&FL_FORCE_RPORT)||
 		(msg->via1->rport /*&& msg->via1->rport->value.s==0*/)){
 		if ((rport_buf=rport_builder(msg, &rport_len))==0){
-			LOG(L_ERR, "ERROR: build_res_buf_from_sip_req:"
-							" rport_builder failed\n");
+			LM_ERR("rport_builder failed\n");
 			goto error01; /* free everything */
 		}
 		if (msg->via1->rport) 
@@ -1766,7 +1748,7 @@ char * build_res_buf_from_sip_req( unsigned int code, str *text ,str *new_tag,
 	if (sip_warning) {
 		warning_buf = warning_builder(msg,&warning_len);
 		if (warning_buf) len += warning_len + CRLF_LEN;
-		else LOG(L_WARN, "WARNING: warning skipped -- too big\n");
+		else LM_WARN("warning skipped -- too big\n");
 	}
 	/* content length hdr */
 	if (body) {
@@ -1782,8 +1764,7 @@ char * build_res_buf_from_sip_req( unsigned int code, str *text ,str *new_tag,
 	buf = (char*) pkg_malloc( len+1 );
 	if (!buf)
 	{
-		LOG(L_ERR, "ERROR: build_res_buf_from_sip_req: out of memory "
-			" ; needs %d\n",len);
+		LM_ERR("out of pkg memory  ; needs %d\n",len);
 		goto error01;
 	}
 
@@ -1922,8 +1903,7 @@ char * build_res_buf_from_sip_req( unsigned int code, str *text ,str *new_tag,
 	}
 
 	if (len!=(unsigned long)(p-buf))
-		LOG(L_CRIT,"BUG:build_res_from_sip_req: diff len=%d p-buf=%d\n",
-					len, (int)(p-buf));
+		LM_CRIT("diff len=%d p-buf=%d\n", len, (int)(p-buf));
 
 	*(p) = 0;
 	*returned_len = len;
@@ -2004,6 +1984,7 @@ char* via_builder( unsigned int *len,
 	unsigned int  via_len, extra_len;
 	char               *line_buf;
 	int max_len;
+	int local_via_len=MY_VIA_LEN;
 	str* address_str; /* address displayed in via */
 	str* port_str; /* port no displayed in via */
 	
@@ -2017,7 +1998,7 @@ char* via_builder( unsigned int *len,
 	else
 		port_str=&(send_sock->port_no_str);
 
-	max_len=MY_VIA_LEN+address_str->len /* space in MY_VIA */
+	max_len=local_via_len+address_str->len /* space in MY_VIA */
 		+2 /* just in case it is a v6 address ... [ ] */
 		+1 /*':'*/+port_str->len
 		+(branch?(MY_BRANCH_LEN+branch->len):0)
@@ -2026,43 +2007,46 @@ char* via_builder( unsigned int *len,
 	line_buf=pkg_malloc( max_len );
 	if (line_buf==0){
 		ser_error=E_OUT_OF_MEM;
-		LOG(L_ERR, "ERROR: via_builder: out of memory\n");
+		LM_ERR("out of pkg memory\n");
 		return 0;
 	}
 
 	extra_len=0;
 
-	via_len=MY_VIA_LEN+address_str->len; /*space included in MY_VIA*/
-
-	memcpy(line_buf, MY_VIA, MY_VIA_LEN); 
+	memcpy(line_buf, MY_VIA, local_via_len); 
 	if (proto==PROTO_UDP){
 		/* do nothing */
 	}else if (proto==PROTO_TCP){
-		memcpy(line_buf+MY_VIA_LEN-4, "TCP ", 4);
+		memcpy(line_buf+local_via_len-4, "TCP ", 4);
 	}else if (proto==PROTO_TLS){
-		memcpy(line_buf+MY_VIA_LEN-4, "TLS ", 4);
+		memcpy(line_buf+local_via_len-4, "TLS ", 4);
+	}else if(proto==PROTO_SCTP){
+		memcpy(line_buf+local_via_len-4, "SCTP ", 5);
+		local_via_len++;
 	}else{
-		LOG(L_CRIT, "BUG: via_builder: unknown proto %d\n", proto);
+		LM_CRIT("unknown proto %d\n", proto);
 		return 0;
 	}
+	
+	via_len=local_via_len+address_str->len; /*space included in MY_VIA*/
+	
 #	ifdef USE_IPV6
 	/* add [] only if ipv6 and outbound socket address is used;
 	 * if using pre-set no check is made */
 	if ((send_sock->address.af==AF_INET6) &&
 		(address_str==&(send_sock->address_str))) {
-		line_buf[MY_VIA_LEN]='[';
-		line_buf[MY_VIA_LEN+1+address_str->len]=']';
+		line_buf[local_via_len]='[';
+		line_buf[local_via_len+1+address_str->len]=']';
 		extra_len=1;
 		via_len+=2; /* [ ]*/
 	}
 #	endif
-	memcpy(line_buf+MY_VIA_LEN+extra_len, address_str->s, address_str->len);
+	memcpy(line_buf+local_via_len+extra_len, address_str->s, address_str->len);
 	if ((send_sock->port_no!=SIP_PORT) || (port_str!=&send_sock->port_no_str)){
 		line_buf[via_len]=':'; via_len++;
 		memcpy(line_buf+via_len, port_str->s, port_str->len);
 		via_len+=port_str->len;
 	}
-
 	/* branch parameter */
 	if (branch){
 		memcpy(line_buf+via_len, MY_BRANCH, MY_BRANCH_LEN );
