@@ -1,5 +1,5 @@
 /*
- * $Id: parse_via.c 1827 2007-03-12 15:22:53Z bogdan_iancu $ 
+ * $Id: parse_via.c 2802 2007-09-21 14:11:40Z bogdan_iancu $ 
  *
  * via parsing automaton
  * 
@@ -82,6 +82,7 @@ enum {
 	UDP1, UDP2, FIN_UDP,
 	TCP_TLS1, TCP2, FIN_TCP,
 	          TLS2, FIN_TLS,
+	SCTP1, SCTP2, SCTP3, FIN_SCTP,
 	L_PROTO, F_PROTO
 };
 
@@ -124,6 +125,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 	unsigned char saved_state;
 
 	state=*pstate;
+
 	saved_state=*psaved_state;
 	param->type=PARAM_ERROR;
 
@@ -263,8 +265,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 					case F_PARAM:
 					case FIN_HIDDEN:
 					case FIN_ALIAS:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c> in"
-								" state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 					case F_CR:
 					case F_LF:
@@ -293,8 +294,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 					case FIN_TTL:
 					case FIN_RECEIVED:
 					case FIN_I:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c> in"
-								" state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 					case F_CR:
 					case F_LF:
@@ -323,9 +323,8 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 					case FIN_TTL:
 					case FIN_RECEIVED:
 					case FIN_I:
-						LOG(L_ERR, "ERROR: parse_via_param: new via found" 
-								"(',') when '=' expected (state %d=)\n",
-								state);
+						LM_ERR("new via found (',') when '=' expected"
+								"(state %d=)\n", state);
 						goto error; /* or we could ignore this bad param*/
 					case F_CR:
 					case F_LF:
@@ -763,8 +762,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 						state=saved_state;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
-								" in state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -790,8 +788,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 						state=F_CRLF;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
-								" in state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -815,8 +812,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 						state=END_OF_HEADER;
 						goto end_via;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
-								" in state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -834,8 +830,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 						state=END_OF_HEADER;
 						goto end_via;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
-								" in state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -865,8 +860,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 						};
 						/* no break */
 					default:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
-								" in state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -891,8 +885,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 						};
 						/* no break */
 					default:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
-								" in state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 				}
 				break; /* what to do? */
@@ -912,8 +905,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 						state=END_OF_HEADER;
 						goto end_via;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
-								" in state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -932,8 +924,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 						state=END_OF_HEADER;
 						goto end_via;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: invalid char <%c>"
-								" in state %d\n", *tmp, state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp, state);
 						goto error;
 				}
 		}
@@ -950,7 +941,7 @@ static /*inline*/ char* parse_via_param(char* p, char* end,
 normal_exit:
 	*pstate=state;
 	*psaved_state=saved_state;
-	DBG("Found param type %d, <%.*s> = <%.*s>; state=%d\n", param->type,
+	LM_DBG("found param type %d, <%.*s> = <%.*s>; state=%d\n", param->type,
 			param->name.len, ZSW(param->name.s), 
 			(param->value.len?param->value.len:3),
 			(param->value.len?param->value.s:"n/a"), state);
@@ -969,11 +960,11 @@ normal_exit:
 	}
 	*pstate=state;
 	*psaved_state=saved_state;
-	DBG("Error on  param type %d, <%.*s>, state=%d, saved_state=%d\n",
+	LM_DBG("error on  param type %d, <%.*s>, state=%d, saved_state=%d\n",
 		param->type, param->name.len, ZSW(param->name.s), state, saved_state);
 
  error:
-	LOG(L_ERR, "error: parse_via_param\n");
+	LM_ERR("parse_via_param\n");
 	param->type=PARAM_ERROR;
 	*pstate=PARAM_ERROR;
 	*psaved_state=state;
@@ -1034,6 +1025,12 @@ parse_again:
 						vb->proto=PROTO_TLS;
 						state=F_HOST; /* start looking for host*/
 						goto main_via;
+					case FIN_SCTP:
+						/* finished proto parsing */
+						vb->transport.len=tmp-vb->transport.s;
+						vb->proto=PROTO_SCTP;
+						state=F_HOST; /* start looking for host*/
+						goto main_via;
 					case FIN_SIP:
 						vb->name.len=tmp-vb->name.s;
 						state=L_VER;
@@ -1048,8 +1045,7 @@ parse_again:
 						state=saved_state;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1099,8 +1095,7 @@ parse_again:
 						state=saved_state;
 						goto endofheader;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1148,8 +1143,7 @@ parse_again:
 						state=saved_state;
 						goto endofheader;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1171,8 +1165,7 @@ parse_again:
 						state=F_PROTO;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1187,9 +1180,12 @@ parse_again:
 					case TLS2:
 						state=FIN_TLS;
 						break;
+					case F_PROTO:
+						state=SCTP1;
+						vb->transport.s=tmp;
+						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1200,8 +1196,7 @@ parse_again:
 						state=SIP2;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1218,9 +1213,11 @@ parse_again:
 					case TCP2:
 						state=FIN_TCP;
 						break;
+					case SCTP3:
+						state=FIN_SCTP;
+						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1232,8 +1229,7 @@ parse_again:
 						vb->transport.s=tmp;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1244,8 +1240,7 @@ parse_again:
 						state=UDP2;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1256,9 +1251,11 @@ parse_again:
 						state=TCP_TLS1;
 						vb->transport.s=tmp;
 						break;
+					case SCTP2:
+						state=SCTP3;
+						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1268,9 +1265,11 @@ parse_again:
 					case TCP_TLS1:
 						state=TCP2;
 						break;
+					case SCTP1:
+						state=SCTP2;
+						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1281,8 +1280,7 @@ parse_again:
 						state=TLS2;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1294,8 +1292,7 @@ parse_again:
 						vb->version.s=tmp;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1305,8 +1302,7 @@ parse_again:
 						state=VER2;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				 break;
@@ -1316,23 +1312,20 @@ parse_again:
 						state=FIN_VER;
 						break;
 					default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
 			
 			default:
-						LOG(L_ERR, "ERROR: parse_via: bad char <%c> on"
-								" state %d\n", *tmp, state);
+						LM_ERR("bad char <%c> on state %d\n", *tmp, state);
 						goto error;
 				break;
 		}
 	} /* for tmp*/
 
 	/* we should not be here! if everything is ok > main_via*/
-	LOG(L_ERR, "ERROR: parse_via: bad via: end of packet on state=%d\n",
-			state);
+	LM_ERR("bad via: end of packet on state=%d\n", state);
 	goto error;
 
  main_via:
@@ -1375,7 +1368,7 @@ parse_again:
 						break;
 					case F_IP6HOST: /*no spaces allowed*/
 					case P_IP6HOST:
-						LOG(L_ERR, "ERROR:parse_via: bad ipv6 reference\n");
+						LM_ERR("bad ipv6 reference\n");
 						goto error;
 					case F_CRLF:
 					case F_LF:
@@ -1384,8 +1377,7 @@ parse_again:
 						state=saved_state;
 						break;
 					default:
-						LOG(L_CRIT,"BUG: parse_via"
-							" on <%c>, state=%d\n",*tmp, state);
+						LM_CRIT("on <%c>, state=%d\n",*tmp, state);
 						goto  error;
 				}
 			break;
@@ -1430,8 +1422,7 @@ parse_again:
 						state=saved_state;
 						goto endofheader;
 					default:
-						LOG(L_CRIT,"BUG: parse_via"
-							" on <%c>\n",*tmp);
+						LM_CRIT("BUG on <%c>\n",*tmp);
 						goto  error;
 				}
 			break;
@@ -1474,8 +1465,7 @@ parse_again:
 						state=saved_state;
 						goto endofheader;
 					default:
-						LOG(L_CRIT,"BUG: parse_via"
-							" on <%c>\n",*tmp);
+						LM_CRIT("on <%c>\n",*tmp);
 						goto  error;
 				}
 			break;
@@ -1497,20 +1487,17 @@ parse_again:
 						state=F_PORT;
 						break;
 					case P_PORT:
-						LOG(L_ERR, "ERROR:parse_via:"
-							" bad port\n");
+						LM_ERR("bad port\n");
 						goto error;
 					case L_PARAM:
 					case F_PARAM:
 					case P_PARAM:
-						LOG(L_ERR, "ERROR:parse_via:"
-						" bad char <%c> in state %d\n",
+						LM_ERR("bad char <%c> in state %d\n",
 							*tmp,state);
 						goto error;
 					case L_VIA:
 					case F_VIA:
-						LOG(L_ERR, "ERROR:parse_via:"
-						" bad char in compact via\n");
+						LM_ERR("bad char in compact via\n");
 						goto error;
 					case F_CRLF:
 					case F_LF:
@@ -1524,9 +1511,7 @@ parse_again:
 					case P_COMMENT: /*everything is allowed in a comment*/
 						break;
 					default:
-						LOG(L_CRIT,"BUG: parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_CRIT("on <%c> state %d\n", *tmp, state);
 						goto error;
 				}
 				break;
@@ -1534,11 +1519,10 @@ parse_again:
 				switch(state){
 					case F_HOST:
 					case F_IP6HOST:
-						LOG(L_ERR,"ERROR:parse_via:"
-							" no host found\n");
+						LM_ERR(" no host found\n");
 						goto error;
 					case P_IP6HOST:
-						LOG(L_ERR, "ERROR:parse_via: bad ipv6 reference\n");
+						LM_ERR(" bad ipv6 reference\n");
 						goto error;
 					case P_HOST:
 						vb->host.len=tmp-vb->host.s;
@@ -1554,13 +1538,11 @@ parse_again:
 						param_start=tmp+1;
 						break;
 					case F_PORT:
-						LOG(L_ERR, "ERROR:parse_via:"
-						" bad char <%c> in state %d\n",
+						LM_ERR(" bad char <%c> in state %d\n",
 							*tmp,state);
 						goto error;
 					case F_PARAM:
-						LOG(L_ERR,  "ERROR:parse_via:"
-							" null param?\n");
+						LM_ERR("null param?\n");
 						goto error;
 					case P_PARAM:
 						/*hmm next, param?*/
@@ -1569,9 +1551,7 @@ parse_again:
 						break;
 					case L_VIA:
 					case F_VIA:
-						LOG(L_ERR, "ERROR:parse_via:"
-						" bad char <%c> in next via\n",
-							*tmp);
+						LM_ERR("bad char <%c> in next via\n", *tmp);
 						goto error;
 					case F_CRLF:
 					case F_LF:
@@ -1586,9 +1566,7 @@ parse_again:
 						break;
 					
 					default:
-						LOG(L_CRIT,"BUG: parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_CRIT("on <%c> state %d\n", *tmp, state);
 						goto  error;
 				}
 			break;
@@ -1596,11 +1574,10 @@ parse_again:
 				switch(state){
 					case F_HOST:
 					case F_IP6HOST:
-						LOG(L_ERR,"ERROR:parse_via:"
-							" no host found\n");
+						LM_ERR("no host found\n");
 						goto error;
 					case P_IP6HOST:
-						LOG(L_ERR, "ERROR:parse_via: bad ipv6 reference\n");
+						LM_ERR(" bad ipv6 reference\n");
 						goto error;
 					case P_HOST:
 						/*mark the end*/
@@ -1620,9 +1597,7 @@ parse_again:
 						break;
 					case F_PORT:
 					case F_PARAM:
-						LOG(L_ERR, "ERROR:parse_via:"
-						" invalid char <%c> in state"
-						" %d\n", *tmp,state);
+						LM_ERR("invalid char <%c> in state %d\n", *tmp,state);
 						goto error;
 					case F_VIA:
 						/* do  nothing,  eat ","*/
@@ -1639,9 +1614,7 @@ parse_again:
 					case P_COMMENT: /*everything is allowed in a comment*/
 						break;
 					default:
-						LOG(L_CRIT,"BUG: parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_CRIT("on <%c> state %d\n",*tmp, state);
 						goto  error;
 				}
 			break;
@@ -1653,9 +1626,7 @@ parse_again:
 					case F_VIA:
 					case F_IP6HOST:
 					case P_IP6HOST: /*must be terminated in ']'*/
-						LOG(L_ERR,"ERROR:parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_ERR(" on <%c> state %d\n", *tmp, state);
 						goto  error;
 					case P_HOST:
 						/*mark the end*/
@@ -1692,9 +1663,7 @@ parse_again:
 						/*previous=crlf and now !=' '*/
 						goto endofheader;
 					default:
-						LOG(L_CRIT,"BUG: parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_CRIT("on <%c> state %d\n", *tmp, state);
 						goto  error;
 				}
 			break;
@@ -1710,11 +1679,7 @@ parse_again:
 								break;
 							}
 						}else{
-							LOG(L_ERR,"ERROR:"
-							    "parse_via: "
-							    "missing '(' - "
-							    "nesting = %d\n",
-							    c_nest);
+							LM_ERR(" missing '(' - nesting= %d\n", c_nest);
 							 goto error;
 						}
 						break;
@@ -1730,9 +1695,7 @@ parse_again:
 					case L_VIA:
 					case F_IP6HOST:
 					case P_IP6HOST:
-						LOG(L_ERR,"ERROR:parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_ERR(" on <%c> state %d\n",*tmp, state);
 						goto  error;
 					case F_CRLF:
 					case F_LF:
@@ -1740,9 +1703,7 @@ parse_again:
 						/*previous=crlf and now !=' '*/
 						goto endofheader;
 					default:
-						LOG(L_CRIT,"BUG: parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_CRIT("on <%c> state %d\n", *tmp, state);
 						goto  error;
 				}
 				break;
@@ -1764,9 +1725,7 @@ parse_again:
 						/*previous=crlf and now !=' '*/
 						goto endofheader;
 					default:
-						LOG(L_ERR,"ERROR:parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_ERR("on <%c> state %d\n",*tmp, state);
 						goto  error;
 				}
 				break;
@@ -1789,9 +1748,7 @@ parse_again:
 					case P_COMMENT:
 						break;
 					default:
-						LOG(L_ERR,"ERROR:parse_via"
-							" on <%c> state %d\n",
-							*tmp, state);
+						LM_ERR("on <%c> state %d\n",*tmp, state);
 						goto  error;
 				}
 				break;
@@ -1816,8 +1773,7 @@ parse_again:
 						if(vb->params.s==0) vb->params.s=param_start;
 						param=pkg_malloc(sizeof(struct via_param));
 						if (param==0){
-							LOG(L_ERR, "ERROR:parse_via: mem. allocation"
-									" error\n");
+							LM_ERR("no pkg memory left\n");
 							goto error;
 						}
 						memset(param,0, sizeof(struct via_param));
@@ -1845,10 +1801,8 @@ parse_again:
 								goto error;
 							default:
 								pkg_free(param);
-								LOG(L_ERR, "ERROR: parse_via after"
-										" parse_via_param: invalid"
-										" char <%c> on state %d\n",
-										*tmp, state);
+								LM_ERR(" after parse_via_param: invalid"
+										" char <%c> on state %d\n",*tmp, state);
 								goto error;
 						}
 						/*add param to the list*/
@@ -1872,6 +1826,9 @@ parse_again:
 							case PARAM_ALIAS:
 								vb->alias=param;
 								break;
+							case PARAM_MADDR:
+								vb->maddr=param;
+								break;
 						}
 						
 						if (state==END_OF_HEADER){
@@ -1887,9 +1844,7 @@ parse_again:
 					case L_PORT:
 					case L_PARAM:
 					case L_VIA:
-						LOG(L_ERR,"ERROR:parse_via"
-							" on <%c> state %d (default)\n",
-							*tmp, state);
+						LM_ERR("on <%c> state %d (default)\n",*tmp, state);
 						goto  error;
 					case F_COMMENT:
 						state=P_COMMENT;
@@ -1908,10 +1863,7 @@ parse_again:
 						/*previous=crlf and now !=' '*/
 						goto endofheader;
 					default:
-						LOG(L_ERR, "BUG:parse_via:"
-							" invalid char <%c>"
-							" in state %d\n",
-							*tmp, state);
+						LM_CRIT("invalid char <%c> in state %d\n",*tmp, state);
 						goto error;
 				}
 
@@ -1919,12 +1871,12 @@ parse_again:
 		}
 	}
 
-	DBG("end of packet reached, state=%d\n", state);
+	LM_DBG("end of packet reached, state=%d\n", state);
 	goto endofpacket; /*end of packet, probably should be goto error*/
 	
 endofheader:
 	state=saved_state;
-	DBG("end of header reached, state=%d\n", state);
+	LM_DBG("end of header reached, state=%d\n", state);
 endofpacket:
 	/* check if error*/
 	switch(state){
@@ -1939,8 +1891,7 @@ endofpacket:
 		case L_VIA:
 			break;
 		default:
-			LOG(L_ERR, "ERROR: parse_via: invalid via - end of header in"
-					" state %d\n", state);
+			LM_ERR(" invalid via - end of header in state %d\n", state);
 			goto error;
 	}
 
@@ -1953,34 +1904,34 @@ endofpacket:
 	if (comment) printf("comment= <%s>\n", comment);
 	if(next_via) printf("next_via= <%s>\n", next_via);
 	*/
-	/*DBG("parse_via: rest=<%s>\n", tmp);*/
+	/*LM_DBG("rest=<%s>\n", tmp);*/
 
 	vb->error=PARSE_OK;
 	vb->bsize=tmp-buffer;
 	if (vb->port_str.s){
 		vb->port=str2s(vb->port_str.s, vb->port_str.len, &err);
 		if (err){
-					LOG(L_ERR, "ERROR: parse_via: invalid port number <%.*s>\n",
+					LM_ERR(" invalid port number <%.*s>\n",
 						vb->port_str.len, ZSW(vb->port_str.s));
 					goto error;
 		}
 	}
 	return tmp;
 nextvia:
-	DBG("parse_via: next_via\n");
+	LM_DBG("next_via\n");
 	vb->error=PARSE_OK;
 	vb->bsize=tmp-buffer;
 	if (vb->port_str.s){
 		vb->port=str2s(vb->port_str.s, vb->port_str.len, &err);
 		if (err){
-					LOG(L_ERR, "ERROR: parse_via: invalid port number <%.*s>\n",
+					LM_ERR(" invalid port number <%.*s>\n",
 						vb->port_str.len, ZSW(vb->port_str.s));
 					goto error;
 		}
 	}
 	vb->next=pkg_malloc(sizeof(struct via_body));
 	if (vb->next==0){
-		LOG(L_ERR, "ERROR: parse_via: out of memory\n");
+		LM_ERR(" out of pkg memory\n");
 		goto error;
 	}
 	vb=vb->next;
@@ -1990,13 +1941,13 @@ nextvia:
 
 error:
 	if (end>buffer){
-		LOG(L_ERR, "ERROR: parse_via on: <%.*s>\n", (int)(end-buffer), ZSW(buffer));
+		LM_ERR(" <%.*s>\n", (int)(end-buffer), ZSW(buffer));
 	}
 	if ((tmp>buffer)&&(tmp<end)){
-		LOG(L_ERR, "ERROR: parse_via parse error, parsed so far:<%.*s>\n",
+		LM_ERR("parsed so far:<%.*s>\n",
 				(int)(tmp-buffer), ZSW(buffer) );
 	}else{
-		LOG(L_ERR, "ERROR: parse_via: via parse error\n");
+		LM_ERR("via parse failed\n");
 	}
 	vb->error=PARSE_ERROR;
 	vbody->error=PARSE_ERROR; /* make sure the first via body is marked
