@@ -1,5 +1,5 @@
 /*
- * $Id: route.c 2247 2007-05-18 10:53:27Z bogdan_iancu $
+ * $Id: route.c 2785 2007-09-17 17:07:01Z bogdan_iancu $
  *
  * SIP routing engine
  *
@@ -86,7 +86,7 @@ extern int return_code;
 /*
  *
  */
-void init_route_lists()
+void init_route_lists(void)
 {
 	memset(rlist, 0, sizeof(rlist));
 	memset(onreply_rlist, 0, sizeof(onreply_rlist));
@@ -104,7 +104,7 @@ static int fix_expr(struct expr* exp)
 	
 	ret=E_BUG;
 	if (exp==0){
-		LOG(L_CRIT, "BUG: fix_expr: null pointer\n");
+		LM_CRIT("null pointer\n");
 		return E_BUG;
 	}
 	if (exp->type==EXP_T){
@@ -120,22 +120,19 @@ static int fix_expr(struct expr* exp)
 						ret=fix_expr(exp->left.v.expr);
 						break;
 			default:
-						LOG(L_CRIT, "BUG: fix_expr: unknown op %d\n",
-								exp->op);
+						LM_CRIT("unknown op %d\n", exp->op);
 		}
 	}else if (exp->type==ELEM_T){
 			if (exp->op==MATCH_OP || exp->op==NOTMATCH_OP){
 				if (exp->right.type==STRING_ST){
 					re=(regex_t*)pkg_malloc(sizeof(regex_t));
 					if (re==0){
-						LOG(L_CRIT, "ERROR: fix_expr: memory allocation"
-								" failure\n");
+						LM_CRIT("out of pkg memory\n");
 						return E_OUT_OF_MEM;
 					}
 					if (regcomp(re, (char*) exp->right.v.data,
 								REG_EXTENDED|REG_NOSUB|REG_ICASE) ){
-						LOG(L_CRIT, "ERROR: fix_expr : bad re \"%s\"\n",
-									(char*) exp->right.v.data);
+						LM_CRIT("bad re \"%s\"\n", (char*) exp->right.v.data);
 						pkg_free(re);
 						return E_BAD_RE;
 					}
@@ -145,28 +142,28 @@ static int fix_expr(struct expr* exp)
 					exp->right.type=RE_ST;
 				}else if (exp->right.type!=RE_ST
 						&& exp->right.type!=SCRIPTVAR_ST){
-					LOG(L_CRIT, "BUG: fix_expr : invalid type for match\n");
+					LM_CRIT("invalid type for match\n");
 					return E_BUG;
 				}
 			}
 			if (exp->left.type==ACTION_O){
 				ret=fix_actions((struct action*)exp->right.v.data);
 				if (ret!=0){
-					LOG(L_CRIT, "ERROR: fix_expr : fix_actions error\n");
+					LM_CRIT("fix_actions error\n");
 					return ret;
 				}
 			}
 			if (exp->left.type==EXPR_O){
 				ret=fix_expr(exp->left.v.expr);
 				if (ret!=0){
-					LOG(L_CRIT, "ERROR: fix_expr : fix left exp error\n");
+					LM_CRIT("fix left exp error\n");
 					return ret;
 				}
 			}
 			if (exp->right.type==EXPR_ST){
 				ret=fix_expr(exp->right.v.expr);
 				if (ret!=0){
-					LOG(L_CRIT, "ERROR: fix_expr : fix rigth exp error\n");
+					LM_CRIT("fix rigth exp error\n");
 					return ret;
 				}
 			}
@@ -188,12 +185,12 @@ static int fix_actions(struct action* a)
 	struct ip_addr ip;
 	struct socket_info* si;
 	str host;
-	int proto, port;
+	int proto=PROTO_NONE, port;
 	struct proxy_l *p;
 	struct bl_head *blh;
 
 	if (a==0){
-		LOG(L_CRIT,"BUG: fix_actions: null pointer\n");
+		LM_CRIT("null pointer\n");
 		return E_BUG;
 	}
 	for(t=a; t!=0; t=t->next){
@@ -203,22 +200,20 @@ static int fix_actions(struct action* a)
 					break;
 			case SEND_T:
 				if (t->elem[0].type!=STRING_ST) {
-					LOG(L_CRIT, "BUG: fix_actions: invalid type"
-						"%d (should be string)\n", t->type);
+					LM_CRIT("invalid type %d (should be string)\n", t->type);
 					return E_BUG;
 				}
 				ret = parse_phostport( t->elem[0].u.string,
 						strlen(t->elem[0].u.string),
 						&host.s, &host.len, &port, &proto);
 				if (ret!=0) {
-					LOG(L_ERR,"ERROR:fix_actions: FORWARD/SEND bad "
+					LM_ERR("ERROR:fix_actions: FORWARD/SEND bad "
 						"argument\n");
 					return E_CFG;
 				}
 				p = add_proxy( &host,(unsigned short)port, proto);
 				if (p==0) {
-					LOG(L_ERR,"ERROR:fix_actions: FORWARD/SEND failed to "
-						"add proxy");
+					LM_ERR("forward/send failed to add proxy");
 					return E_CFG;
 				}
 				t->elem[0].type = PROXY_ST;
@@ -226,21 +221,18 @@ static int fix_actions(struct action* a)
 				break;
 			case IF_T:
 				if (t->elem[0].type!=EXPR_ST){
-					LOG(L_CRIT, "BUG: fix_actions: invalid subtype"
-								"%d for if (should be expr)\n",
+					LM_CRIT("invalid subtype %d for if (should be expr)\n",
 								t->elem[0].type);
 					return E_BUG;
 				}else if( (t->elem[1].type!=ACTIONS_ST)
 						&&(t->elem[1].type!=NOSUBTYPE) ){
-					LOG(L_CRIT, "BUG: fix_actions: invalid subtype"
-								"%d for if() {...} (should be action)\n",
-								t->elem[1].type);
+					LM_CRIT("invalid subtype %d for if() {...} (should be"
+								"action)\n", t->elem[1].type);
 					return E_BUG;
 				}else if( (t->elem[2].type!=ACTIONS_ST)
 						&&(t->elem[2].type!=NOSUBTYPE) ){
-					LOG(L_CRIT, "BUG: fix_actions: invalid subtype"
-								"%d for if() {} else{...}(should be action)\n",
-								t->elem[2].type);
+					LM_CRIT("invalid subtype %d for if() {} else{...}(should"
+							"be action)\n", t->elem[2].type);
 					return E_BUG;
 				}
 				if (t->elem[0].u.data){
@@ -256,6 +248,26 @@ static int fix_actions(struct action* a)
 						return ret;
 				}
 				break;
+			case WHILE_T:
+				if (t->elem[0].type!=EXPR_ST){
+					LM_CRIT("invalid subtype %d for while (should be expr)\n",
+								t->elem[0].type);
+					return E_BUG;
+				}else if( (t->elem[1].type!=ACTIONS_ST)
+						&&(t->elem[1].type!=NOSUBTYPE) ){
+					LM_CRIT("invalid subtype %d for while() {...} (should be"
+								"action)\n", t->elem[1].type);
+					return E_BUG;
+				}
+				if (t->elem[0].u.data){
+					if ((ret=fix_expr((struct expr*)t->elem[0].u.data))<0)
+						return ret;
+				}
+				if ( (t->elem[1].type==ACTIONS_ST)&&(t->elem[1].u.data) ){
+					if ((ret=fix_actions((struct action*)t->elem[1].u.data))<0)
+						return ret;
+				}
+				break;
 			case SWITCH_T:
 				if ( (t->elem[1].type==ACTIONS_ST)&&(t->elem[1].u.data) ){
 					if ((ret=fix_actions((struct action*)t->elem[1].u.data))<0)
@@ -276,7 +288,7 @@ static int fix_actions(struct action* a)
 				break;
 			case MODULE_T:
 				cmd = (cmd_export_t*)t->elem[0].u.data;
-				DBG("fixing %s, line %d\n", cmd->name, t->line);
+				LM_DBG("fixing %s, line %d\n", cmd->name, t->line);
 				if (cmd->fixup){
 					if (cmd->param_no>0){
 						ret=cmd->fixup(&t->elem[1].u.data, 1);
@@ -296,15 +308,13 @@ static int fix_actions(struct action* a)
 				break;
 			case FORCE_SEND_SOCKET_T:
 				if (t->elem[0].type!=SOCKID_ST){
-					LOG(L_CRIT, "BUG: fix_actions: invalid subtype"
-								"%d for force_send_socket\n",
+					LM_CRIT("invalid subtype %d for force_send_socket\n",
 								t->elem[0].type);
 					return E_BUG;
 				}
 				he=resolvehost(((struct socket_id*)t->elem[0].u.data)->name,0);
 				if (he==0){
-					LOG(L_ERR, "ERROR: fix_actions: force_send_socket:"
-								" could not resolve %s\n",
+					LM_ERR(" could not resolve %s\n",
 								((struct socket_id*)t->elem[0].u.data)->name);
 					ret = E_BAD_ADDRESS;
 					goto error;
@@ -313,7 +323,7 @@ static int fix_actions(struct action* a)
 				si=find_si(&ip, ((struct socket_id*)t->elem[0].u.data)->port,
 								((struct socket_id*)t->elem[0].u.data)->proto);
 				if (si==0){
-					LOG(L_ERR, "ERROR: fix_actions: bad force_send_socket"
+					LM_ERR("bad force_send_socket"
 							" argument: %s:%d (ser doesn't listen on it)\n",
 							((struct socket_id*)t->elem[0].u.data)->name,
 							((struct socket_id*)t->elem[0].u.data)->port);
@@ -323,12 +333,26 @@ static int fix_actions(struct action* a)
 				t->elem[0].u.data=si;
 				t->elem[0].type=SOCKETINFO_ST;
 				break;
+			case SET_DEBUG_T:
+				if (t->elem[0].type==NOSUBTYPE)
+					break;
+				if (t->elem[0].type!=NUMBER_ST) {
+					LM_CRIT("fix_actions: BUG in setdebug() type %d\n",
+						t->elem[0].type );
+					ret=E_BUG;
+					goto error;
+				}
+				/* normalize the value */
+				if (t->elem[0].u.number>L_DBG)
+					t->elem[0].u.number = L_DBG;
+				else if (t->elem[0].u.number<L_ALERT)
+					t->elem[0].u.number = L_ALERT;
+				break;
 			case SETFLAG_T:
 			case RESETFLAG_T:
 			case ISFLAGSET_T:
 				if (t->elem[0].type!=NUMBER_ST) {
-					LOG(L_CRIT, "BUG: fix_actions: bad xxxflag() type %d\n",
-						t->elem[0].type );
+					LM_CRIT("bad xxxflag() type %d\n", t->elem[0].type );
 					ret=E_BUG;
 					goto error;
 				}
@@ -341,8 +365,7 @@ static int fix_actions(struct action* a)
 			case RESETSFLAG_T:
 			case ISSFLAGSET_T:
 				if (t->elem[0].type!=NUMBER_ST) {
-					LOG(L_CRIT, "BUG: fix_actions: bad xxxsflag() type %d\n",
-						t->elem[0].type );
+					LM_CRIT("bad xxxsflag() type %d\n", t->elem[0].type );
 					ret=E_BUG;
 					goto error;
 				}
@@ -356,7 +379,7 @@ static int fix_actions(struct action* a)
 			case RESETBFLAG_T:
 			case ISBFLAGSET_T:
 				if (t->elem[0].type!=NUMBER_ST || t->elem[1].type!=NUMBER_ST) {
-					LOG(L_CRIT, "BUG: fix_actions: bad xxxbflag() type "
+					LM_CRIT("bad xxxbflag() type "
 						"%d,%d\n", t->elem[0].type, t->elem[0].type);
 					ret=E_BUG;
 					goto error;
@@ -368,6 +391,7 @@ static int fix_actions(struct action* a)
 				}
 				break;
 			case EQ_T:
+			case COLONEQ_T:
 			case PLUSEQ_T:
 			case MINUSEQ_T:
 			case DIVEQ_T:
@@ -383,8 +407,7 @@ static int fix_actions(struct action* a)
 				break;
 			case USE_BLACKLIST_T:
 				if (t->elem[0].type!=STRING_ST) {
-					LOG(L_CRIT, "BUG: fix_actions: bad USE_BLACKLIST type "
-						"%d\n", t->elem[0].type);
+					LM_CRIT("bad USE_BLACKLIST type %d\n", t->elem[0].type);
 					ret=E_BUG;
 					goto error;
 				}
@@ -392,7 +415,7 @@ static int fix_actions(struct action* a)
 				host.len = strlen(host.s);
 				blh = get_bl_head_by_name(&host);
 				if (blh==NULL) {
-					LOG(L_ERR, "ERROR: fix_actions: USE_BLACKLIST - list "
+					LM_ERR("USE_BLACKLIST - list "
 						"%s not configured\n", t->elem[0].u.string);
 					ret=E_CFG;
 					goto error;
@@ -404,8 +427,7 @@ static int fix_actions(struct action* a)
 	}
 	return 0;
 error:
-	LOG(L_ERR,"ERROR: fix_actions: fixing failed (code=%d) at cfg line %d\n",
-		ret, t->line);
+	LM_ERR("fixing failed (code=%d) at cfg line %d\n", ret, t->line);
 	return ret;
 }
 
@@ -414,7 +436,7 @@ inline static int comp_no( int port, void *param, int op, int subtype )
 {
 	
 	if (subtype!=NUMBER_ST) {
-		LOG(L_CRIT, "BUG: comp_no: number expected: %d\n", subtype );
+		LM_CRIT("number expected: %d\n", subtype );
 		return E_BUG;
 	}
 	switch (op){
@@ -431,7 +453,7 @@ inline static int comp_no( int port, void *param, int op, int subtype )
 		case LTE_OP:
 			return port<=(long)param;
 		default:
-		LOG(L_CRIT, "BUG: comp_no: unknown operator: %d\n", op );
+		LM_CRIT("unknown operator: %d\n", op );
 		return E_BUG;
 	}
 }
@@ -445,17 +467,20 @@ inline static int comp_strval(struct sip_msg *msg, int op, str* ival,
 	char backup;
 	char backup2;
 	str res;
-	xl_value_t value;
+	pv_value_t value;
+			
+	if(ival==NULL || ival->s==NULL)
+		goto error;
 	
 	res.s = 0; res.len = 0;
 	if(opd->type == SCRIPTVAR_ST)
 	{
-		if(xl_get_spec_value(msg, opd->v.spec, &value, 0)!=0)
+		if(pv_get_spec_value(msg, opd->v.spec, &value)!=0)
 		{
-			LOG(L_CRIT, "comp_strval: cannot get var value\n");
+			LM_CRIT("cannot get var value\n");
 			goto error;
 		}
-		if(value.flags&XL_VAL_STR)
+		if(value.flags&PV_VAL_STR)
 		{
 			res = value.rs;
 		} else {
@@ -468,8 +493,7 @@ inline static int comp_strval(struct sip_msg *msg, int op, str* ival,
 	} else {
 		if((op!=MATCH_OP && op!=NOTMATCH_OP) || opd->type != RE_ST)
 		{
-			LOG(L_CRIT, "comp_strval: invalid operation %d/%d\n", op,
-					opd->type);
+			LM_CRIT("invalid operation %d/%d\n", op, opd->type);
 			goto error;
 		}
 	}
@@ -492,8 +516,7 @@ inline static int comp_strval(struct sip_msg *msg, int op, str* ival,
 			if(opd->type == SCRIPTVAR_ST) {
 				re=(regex_t*)pkg_malloc(sizeof(regex_t));
 				if (re==0){
-					LOG(L_CRIT, "ERROR: comp_strval: memory allocation"
-					    " failure\n");
+					LM_CRIT("pkg memory allocation failure\n");
 					ival->s[ival->len]=backup;
 					goto error;
 				}
@@ -517,7 +540,7 @@ inline static int comp_strval(struct sip_msg *msg, int op, str* ival,
 				ret = !ret;
 			break;
 		default:
-			LOG(L_CRIT, "BUG: comp_strval: unknown op %d\n", op);
+			LM_CRIT("unknown op %d\n", op);
 			goto error;
 	}
 	return ret;
@@ -535,38 +558,34 @@ inline static int comp_str(char* str, void* param, int op, int subtype)
 	switch(op){
 		case EQUAL_OP:
 			if (subtype!=STRING_ST){
-				LOG(L_CRIT, "BUG: comp_str: bad type %d, "
-						"string expected\n", subtype);
+				LM_CRIT("bad type %d, string expected\n", subtype);
 				goto error;
 			}
 			ret=(strcasecmp(str, (char*)param)==0);
 			break;
 		case DIFF_OP:
 			if (subtype!=STRING_ST){
-				LOG(L_CRIT, "BUG: comp_str: bad type %d, "
-						"string expected\n", subtype);
+				LM_CRIT("bad type %d, string expected\n", subtype);
 				goto error;
 			}
 			ret=(strcasecmp(str, (char*)param)!=0);
 			break;
 		case MATCH_OP:
 			if (subtype!=RE_ST){
-				LOG(L_CRIT, "BUG: comp_str: bad type %d, "
-						" RE expected\n", subtype);
+				LM_CRIT("bad type %d, RE expected\n", subtype);
 				goto error;
 			}
 			ret=(regexec((regex_t*)param, str, 0, 0, 0)==0);
 			break;
 		case NOTMATCH_OP:
 			if (subtype!=RE_ST){
-				LOG(L_CRIT, "BUG: comp_str: bad type %d, "
-						" RE expected!\n", subtype);
+				LM_CRIT("bad type %d, RE expected!\n", subtype);
 				goto error;
 			}
 			ret=(regexec((regex_t*)param, str, 0, 0, 0)!=0);
 			break;
 		default:
-			LOG(L_CRIT, "BUG: comp_str: unknown op %d\n", op);
+			LM_CRIT("unknown op %d\n", op);
 			goto error;
 	}
 	return ret;
@@ -589,7 +608,7 @@ inline static int check_self_op(int op, str* s, unsigned short p)
 			if (ret>=0) ret=!ret;
 			break;
 		default:
-			LOG(L_CRIT, "BUG: check_self_op: invalid operator %d\n", op);
+			LM_CRIT("invalid operator %d\n", op);
 			ret=-1;
 	}
 	return ret;
@@ -631,8 +650,7 @@ inline static int comp_ip(struct sip_msg *msg, int op, struct ip_addr* ip,
 					if (opd->type==STRING_ST){
 						he=resolvehost((char*)opd->v.data,0);
 						if (he==0){
-							DBG("comp_ip: could not resolve %s\n",
-									(char*)opd->v.data);
+							LM_DBG("could not resolve %s\n",(char*)opd->v.data);
 						}else if (he->h_addrtype==(int)ip->af){
 							for(h=he->h_addr_list;(ret!=1)&& (*h); h++){
 								ret=(memcmp(ip->u.addr, *h, ip->len)==0);
@@ -671,13 +689,12 @@ inline static int comp_ip(struct sip_msg *msg, int op, struct ip_addr* ip,
 			ret=check_self_op(op, &tmp, 0);
 			break;
 		default:
-			LOG(L_CRIT, "BUG: comp_ip: invalid type for "
-						" src_ip or dst_ip (%d)\n", opd->type);
+			LM_CRIT("invalid type for src_ip or dst_ip (%d)\n", opd->type);
 			ret=-1;
 	}
 	return ret;
 error_op:
-	LOG(L_CRIT, "BUG: comp_ip: invalid operator %d\n", op);
+	LM_CRIT("invalid operator %d\n", op);
 	return -1;
 	
 }
@@ -755,7 +772,7 @@ inline static int comp_s2s(int op, str *s1, str *s2)
 		case NOTMATCHD_OP:
 			re=(regex_t*)pkg_malloc(sizeof(regex_t));
 			if (re==0) {
-				LOG(L_CRIT, "ERROR: comp_strval: memory allocation failure\n");
+				LM_CRIT("pkg memory allocation failure\n");
 				return -1;
 			}
 
@@ -778,7 +795,7 @@ inline static int comp_s2s(int op, str *s1, str *s2)
 			s1->s[s1->len] = backup;
 			break;
 		default:
-			LOG(L_CRIT, "BUG: comp_s2s: unknown op %d\n", op);
+			LM_CRIT("unknown op %d\n", op);
 	}
 	return ret;
 }
@@ -816,7 +833,7 @@ inline static int comp_n2n(int op, int n1, int n2)
 				return 1;
 			return 0;
 		default:
-			LOG(L_CRIT, "BUG: comp_n2n: unknown op %d\n", op);
+			LM_CRIT("unknown op %d\n", op);
 	}
 	return -1;
 }
@@ -829,27 +846,27 @@ inline static int comp_scriptvar(struct sip_msg *msg, int op, operand_t *left,
 	str rstr;
 	int ln;
 	int rn;
-	xl_value_t lvalue;
-	xl_value_t rvalue;
+	pv_value_t lvalue;
+	pv_value_t rvalue;
 	int type;
 	
 	lstr.s = 0; lstr.len = 0;
 	rstr.s = 0; rstr.len = 0;
 	ln = 0; rn =0;
-	if(xl_get_spec_value(msg, left->v.spec, &lvalue, 0)!=0)
+	if(pv_get_spec_value(msg, left->v.spec, &lvalue)!=0)
 	{
-		LOG(L_CRIT, "comp_scriptvar: cannot get left var value\n");
+		LM_CRIT("cannot get left var value\n");
 		goto error;
 	}
 	if(right->type==NULLV_ST)
 	{
 		if(op==EQUAL_OP)
 		{
-			if(lvalue.flags&XL_VAL_NULL)
+			if(lvalue.flags&PV_VAL_NULL)
 				return 1;
 			return 0;
 		} else {
-			if(lvalue.flags&XL_VAL_NULL)
+			if(lvalue.flags&PV_VAL_NULL)
 				return 0;
 			return 1;
 		}
@@ -861,23 +878,22 @@ inline static int comp_scriptvar(struct sip_msg *msg, int op, operand_t *left,
 
 	if(right->type == SCRIPTVAR_ST)
 	{
-		if(xl_get_spec_value(msg, right->v.spec, &rvalue, 0)!=0)
+		if(pv_get_spec_value(msg, right->v.spec, &rvalue)!=0)
 		{
-			LOG(L_CRIT, "comp_scriptvar: cannot get right var value\n");
+			LM_CRIT("cannot get right var value\n");
 			goto error;
 		}
-		if(rvalue.flags&XL_VAL_NULL || lvalue.flags&XL_VAL_NULL ) {
-			if (rvalue.flags&XL_VAL_NULL && lvalue.flags&XL_VAL_NULL )
+		if(rvalue.flags&PV_VAL_NULL || lvalue.flags&PV_VAL_NULL ) {
+			if (rvalue.flags&PV_VAL_NULL && lvalue.flags&PV_VAL_NULL )
 				return (op==EQUAL_OP)?1:0;
 			return (op==DIFF_OP)?1:0;
 		}
 		
 		if(op==MATCH_OP||op==NOTMATCH_OP)
 		{
-			if(!((rvalue.flags&XL_VAL_STR) && (lvalue.flags&XL_VAL_STR)))
+			if(!((rvalue.flags&PV_VAL_STR) && (lvalue.flags&PV_VAL_STR)))
 			{
-				LOG(L_CRIT, "comp_scriptvar: invalid operation %d/%d\n", op,
-					right->type);
+				LM_CRIT("invalid operation %d/%d\n", op, right->type);
 				goto error;
 			}
 			if(op==MATCH_OP)
@@ -886,39 +902,38 @@ inline static int comp_scriptvar(struct sip_msg *msg, int op, operand_t *left,
 				return comp_s2s(NOTMATCHD_OP, &lstr, &rvalue.rs);
 		}
 
-		if((rvalue.flags&XL_VAL_INT) && (lvalue.flags&XL_VAL_INT)) {
+		if((rvalue.flags&PV_VAL_INT) && (lvalue.flags&PV_VAL_INT)) {
 			/* comparing int */
 			rn = rvalue.ri;
 			type =2;
-		} else if((rvalue.flags&XL_VAL_STR) && (lvalue.flags&XL_VAL_STR)) {
+		} else if((rvalue.flags&PV_VAL_STR) && (lvalue.flags&PV_VAL_STR)) {
 			/* comparing string */
 			rstr = rvalue.rs;
 			type =1;
 		} else {
-			LOG(L_CRIT, "comp_scriptvar: invalid operation %d/%d!\n", op,
+			LM_CRIT("invalid operation %d/%d!\n", op,
 					right->type);
 			goto error;
 		}
 	} else {
 		/* null against a not-null constant */
-		if(lvalue.flags&XL_VAL_NULL)
+		if(lvalue.flags&PV_VAL_NULL)
 			return (op==DIFF_OP || op==NOTMATCH_OP || op==NOTMATCHD_OP)?1:0;
 
 		if(right->type == NUMBER_ST) {
-			if(!(lvalue.flags&XL_VAL_INT))
+			if(!(lvalue.flags&PV_VAL_INT))
 			{
-				LOG(L_CRIT,"comp_scriptvar: invalid operation %d/%d/%d!!\n", op,
-					right->type, lvalue.flags);
+				LM_CRIT("invalid operation %d/%d/%d!!\n", op, 
+						right->type, lvalue.flags);
 				goto error;
 			}
 			/* comparing int */
 			type =2;
 			rn = right->v.n;
 		} else if(right->type == STRING_ST) {
-			if(!(lvalue.flags&XL_VAL_STR))
+			if(!(lvalue.flags&PV_VAL_STR))
 			{
-				LOG(L_CRIT, "comp_scriptvar: invalid operation %d/%d!!!\n", op,
-					right->type);
+				LM_CRIT("invalid operation %d/%d!!!\n", op,	right->type);
 				goto error;
 			}
 			/* comparing string */
@@ -927,10 +942,9 @@ inline static int comp_scriptvar(struct sip_msg *msg, int op, operand_t *left,
 		} else {
 			if(op==MATCH_OP || op==NOTMATCH_OP)
 			{
-				if(!(lvalue.flags&XL_VAL_STR) || right->type != RE_ST)
+				if(!(lvalue.flags&PV_VAL_STR) || right->type != RE_ST)
 				{
-					LOG(L_CRIT, "comp_scriptvar: invalid operation %d/%d\n", op,
-						right->type);
+					LM_CRIT("invalid operation %d/%d\n", op, right->type);
 					goto error;
 				}
 				return comp_s2s(op, &lstr, (str*)right->v.expr);
@@ -941,14 +955,13 @@ inline static int comp_scriptvar(struct sip_msg *msg, int op, operand_t *left,
 	}
 
 	if(type==1) { /* compare str */
-		DBG("comp_scriptvar: str %d : %.*s\n", op, lstr.len, ZSW(lstr.s)); 
+		LM_DBG("str %d : %.*s\n", op, lstr.len, ZSW(lstr.s)); 
 		return comp_s2s(op, &lstr, &rstr);
 	} else if(type==2) {
-		DBG("comp_scriptvar: int %d : %d / %d\n", op, ln, rn); 
+		LM_DBG("int %d : %d / %d\n", op, ln, rn); 
 		return comp_n2n(op, ln, rn);
 	} else {
-		LOG(L_CRIT, "comp_scriptvar: invalid operation %d/%d\n", op,
-			right->type);
+		LM_CRIT("invalid operation %d/%d\n", op, right->type);
 	}
 	
 error:
@@ -957,7 +970,7 @@ error:
 
 
 /* returns: 0/1 (false/true) or -1 on error, -127 EXPR_DROP */
-static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
+static int eval_elem(struct expr* e, struct sip_msg* msg, pv_value_t *val)
 {
 
 	struct sip_uri uri;
@@ -965,17 +978,17 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 	int retl;
 	int retr;
 	int ival;
-	xl_value_t lval;
-	xl_value_t rval;
+	pv_value_t lval;
+	pv_value_t rval;
 	char *p;
 	
 	ret=E_BUG;
 	if (e->type!=ELEM_T){
-		LOG(L_CRIT," BUG: eval_elem: invalid type\n");
+		LM_CRIT("invalid type\n");
 		goto error;
 	}
 	
-	if(val) memset(val, 0, sizeof(xl_value_t));
+	if(val) memset(val, 0, sizeof(pv_value_t));
 
 	switch(e->left.type){
 		case METHOD_O:
@@ -1007,14 +1020,13 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 				break;
 		case FROM_URI_O:
 				if (parse_from_header(msg)<0){
-					LOG(L_ERR, "ERROR: eval_elem: bad or missing"
-								" From: header\n");
+					LM_ERR("bad or missing From: header\n");
 					goto error;
 				}
 				if (e->right.type==MYSELF_ST){
 					if (parse_uri(get_from(msg)->uri.s, get_from(msg)->uri.len,
 									&uri) < 0){
-						LOG(L_ERR, "ERROR: eval_elem: bad uri in From:\n");
+						LM_ERR("bad uri in From:\n");
 						goto error;
 					}
 					ret=check_self_op(e->op, &uri.host,
@@ -1027,15 +1039,14 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 		case TO_URI_O:
 				if ((msg->to==0) && ((parse_headers(msg, HDR_TO_F, 0)==-1) ||
 							(msg->to==0))){
-					LOG(L_ERR, "ERROR: eval_elem: bad or missing"
-								" To: header\n");
+					LM_ERR("bad or missing To: header\n");
 					goto error;
 				}
 				/* to content is parsed automatically */
 				if (e->right.type==MYSELF_ST){
 					if (parse_uri(get_to(msg)->uri.s, get_to(msg)->uri.len,
 									&uri) < 0){
-						LOG(L_ERR, "ERROR: eval_elem: bad uri in To:\n");
+						LM_ERR("bad uri in To:\n");
 						goto error;
 					}
 					ret=check_self_op(e->op, &uri.host,
@@ -1058,7 +1069,7 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 				ret=run_action_list( (struct action*)e->right.v.data, msg);
 				if(val)
 				{
-					val->flags = XL_TYPE_INT|XL_VAL_INT;
+					val->flags = PV_TYPE_INT|PV_VAL_INT;
 					val->ri = ret;
 				}
 				if (ret<=0) ret=(ret==0)?EXPR_DROP:0;
@@ -1066,48 +1077,48 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 				return ret;
 		case EXPR_O:
 				retl = retr = 0;
-				memset(&lval, 0, sizeof(xl_value_t));
-				memset(&rval, 0, sizeof(xl_value_t));
+				memset(&lval, 0, sizeof(pv_value_t));
+				memset(&rval, 0, sizeof(pv_value_t));
 				if(e->left.v.data)
 					retl=eval_expr((struct expr*)e->left.v.data,msg,&lval);
-				if(lval.flags == XL_VAL_NONE)
+				if(lval.flags == PV_VAL_NONE)
 				{
-					xl_value_destroy(&lval);
-					xl_value_destroy(&rval);
+					pv_value_destroy(&lval);
+					pv_value_destroy(&rval);
 					return 0;
 				}
 				if(e->op == BNOT_OP)
 				{
-					if(lval.flags&XL_VAL_INT)
+					if(lval.flags&PV_VAL_INT)
 					{
 						if(val!=NULL)
 						{
-							val->flags = XL_TYPE_INT|XL_VAL_INT;
+							val->flags = PV_TYPE_INT|PV_VAL_INT;
 							val->ri = ~lval.ri;
 						}
-						xl_value_destroy(&lval);
-						xl_value_destroy(&rval);
+						pv_value_destroy(&lval);
+						pv_value_destroy(&rval);
 						return (val->ri)?1:0;
 					}
-					LOG(L_ERR, "eval_elem: binary NOT on non-numeric value\n");
-					xl_value_destroy(&lval);
-					xl_value_destroy(&rval);
+					LM_ERR("binary NOT on non-numeric value\n");
+					pv_value_destroy(&lval);
+					pv_value_destroy(&rval);
 					return 0;
 				}
 				if(e->right.v.data)
 					retr=eval_expr((struct expr*)e->right.v.data,msg,&rval);
 			
-				if(lval.flags&XL_TYPE_INT)
+				if(lval.flags&PV_TYPE_INT)
 				{
-					if(!(rval.flags&XL_VAL_INT))
+					if(!(rval.flags&PV_VAL_INT))
 					{
-						LOG(L_ERR, "eval_elem: invalid numeric operands\n");
-						xl_value_destroy(&lval);
-						xl_value_destroy(&rval);
+						LM_ERR("invalid numeric operands\n");
+						pv_value_destroy(&lval);
+						pv_value_destroy(&rval);
 						return 0;
 					}
 					if(val!=NULL)
-						val->flags = XL_TYPE_INT|XL_VAL_INT;
+						val->flags = PV_TYPE_INT|PV_VAL_INT;
 
 					ival = 0;
 					switch(e->op) {
@@ -1120,10 +1131,9 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 						case DIV_OP:
 							if(rval.ri==0)
 							{
-								LOG(L_ERR,
-									"eval_elem: divide by 0\n");
-								xl_value_destroy(&lval);
-								xl_value_destroy(&rval);
+								LM_ERR("divide by 0\n");
+								pv_value_destroy(&lval);
+								pv_value_destroy(&rval);
 								return 0;
 							} else 
 								ival = lval.ri / rval.ri;
@@ -1134,10 +1144,9 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 						case MODULO_OP:
 							if(rval.ri==0)
 							{
-								LOG(L_ERR,
-									"eval_elem: divide by 0\n");
-								xl_value_destroy(&lval);
-								xl_value_destroy(&rval);
+								LM_ERR("divide by 0\n");
+								pv_value_destroy(&lval);
+								pv_value_destroy(&rval);
 								return 0;
 							} else 
 								ival = lval.ri % rval.ri;
@@ -1152,58 +1161,54 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 							ival = lval.ri ^ rval.ri;
 							break;
 						default:
-							LOG(L_ERR,
-									"eval_elem: invalid int op %d\n", e->op);
+							LM_ERR("invalid int op %d\n", e->op);
 								val->ri = 0;
-							xl_value_destroy(&lval);
-							xl_value_destroy(&rval);
+							pv_value_destroy(&lval);
+							pv_value_destroy(&rval);
 							return 0;
 					}
-					xl_value_destroy(&lval);
-					xl_value_destroy(&rval);
+					pv_value_destroy(&lval);
+					pv_value_destroy(&rval);
 					if(val!=NULL) val->ri = ival;
 					return (ival)?1:0;
 				} else {
-					if(!(rval.flags&XL_VAL_STR))
+					if(!(rval.flags&PV_VAL_STR))
 					{
-						LOG(L_ERR,
-								"eval_elem: invalid string operands\n");
-						xl_value_destroy(&lval);
-						xl_value_destroy(&rval);
+						LM_ERR("invalid string operands\n");
+						pv_value_destroy(&lval);
+						pv_value_destroy(&rval);
 						return 0;
 					}
 					if(e->op != PLUS_OP)
 					{
-						LOG(L_ERR,
-								"eval_elem: invalid string operator %d\n",
-								e->op);
-						xl_value_destroy(&lval);
-						xl_value_destroy(&rval);
+						LM_ERR("invalid string operator %d\n", e->op);
+						pv_value_destroy(&lval);
+						pv_value_destroy(&rval);
 						return 0;
 					}
 					if(val==NULL)
 					{
 						ret = (lval.rs.len>0 || rval.rs.len>0);
-						xl_value_destroy(&lval);
-						xl_value_destroy(&rval);
+						pv_value_destroy(&lval);
+						pv_value_destroy(&rval);
 						return ret;
 					}
 					val->rs.s=(char*)pkg_malloc((lval.rs.len+rval.rs.len+1)
 							*sizeof(char));
 					if(val->rs.s==0)
 					{
-						LOG(L_ERR, "eval_elem: no more memory\n");
-						xl_value_destroy(&lval);
-						xl_value_destroy(&rval);
+						LM_ERR("no more memory\n");
+						pv_value_destroy(&lval);
+						pv_value_destroy(&rval);
 						return 0;
 					}
-					val->flags = XL_VAL_PKG|XL_VAL_STR;
+					val->flags = PV_VAL_PKG|PV_VAL_STR;
 					memcpy(val->rs.s, lval.rs.s, lval.rs.len);
 					memcpy(val->rs.s+lval.rs.len, rval.rs.s, rval.rs.len);
 					val->rs.len = lval.rs.len + rval.rs.len;
 					val->rs.s[val->rs.len] = '\0';
-					xl_value_destroy(&lval);
-					xl_value_destroy(&rval);
+					pv_value_destroy(&lval);
+					pv_value_destroy(&rval);
 					return 1;
 				}
 				break;
@@ -1235,14 +1240,14 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 				break;
 		case STRINGV_O:
 				if(val) {
-					val->flags = XL_VAL_STR;
+					val->flags = PV_VAL_STR;
 					val->rs = e->left.v.s;
 				}
 				/* optimization for no dup ?!?! */
 				return (e->left.v.s.len>0)?1:0;
 		case NUMBERV_O:
 				if(val) {
-					val->flags = XL_TYPE_INT|XL_VAL_INT;
+					val->flags = PV_TYPE_INT|PV_VAL_INT;
 					val->ri = e->left.v.n;
 				}
 				ret=!(!e->left.v.n); /* !! to transform it in {0,1} */
@@ -1250,40 +1255,40 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 		case SCRIPTVAR_O:
 				if(e->op==NO_OP)
 				{
-					memset(&rval, 0, sizeof(xl_value_t));
-					if(xl_get_spec_value(msg, e->right.v.spec, &rval, 0)==0)
+					memset(&rval, 0, sizeof(pv_value_t));
+					if(pv_get_spec_value(msg, e->right.v.spec, &rval)==0)
 					{
-						if(rval.flags==XL_VAL_NONE || (rval.flags&XL_VAL_NULL)
-								|| (rval.flags&XL_VAL_EMPTY)
-								|| ((rval.flags&XL_TYPE_INT)&&rval.ri==0))
+						if(rval.flags==PV_VAL_NONE || (rval.flags&PV_VAL_NULL)
+								|| (rval.flags&PV_VAL_EMPTY)
+								|| ((rval.flags&PV_TYPE_INT)&&rval.ri==0))
 						{
-							xl_value_destroy(&rval);
+							pv_value_destroy(&rval);
 							return 0;
 						}
-						if(rval.flags&XL_TYPE_INT)
+						if(rval.flags&PV_TYPE_INT)
 						{
-							xl_value_destroy(&rval);
+							pv_value_destroy(&rval);
 							return 1;
 						}
 						if(rval.rs.len!=0)
 						{
-							xl_value_destroy(&rval);
+							pv_value_destroy(&rval);
 							return 1;
 						}
-						xl_value_destroy(&rval);
+						pv_value_destroy(&rval);
 					}
 					return 0;
 				}
 				if(e->op==VALUE_OP)
 				{
-					if(xl_get_spec_value(msg, e->left.v.spec, &lval, 0)==0)
+					if(pv_get_spec_value(msg, e->left.v.spec, &lval)==0)
 					{
 						if(val!=NULL)
-							memcpy(val, &lval, sizeof(xl_value_t));
-						if(lval.flags&XL_VAL_STR)
+							memcpy(val, &lval, sizeof(pv_value_t));
+						if(lval.flags&PV_VAL_STR)
 						{
-							if(!((lval.flags&XL_VAL_PKG) 
-									|| (lval.flags&XL_VAL_SHM)))
+							if(!((lval.flags&PV_VAL_PKG) 
+									|| (lval.flags&PV_VAL_SHM)))
 							{
 								if(val!=NULL)
 								{
@@ -1292,24 +1297,23 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 											*sizeof(char));
 									if(p==0)
 									{
-										LOG(L_ERR,
-												"eval_elem: no more memory\n");
-										memset(val, 0, sizeof(xl_value_t));
+										LM_ERR("no more pkg memory\n");
+										memset(val, 0, sizeof(pv_value_t));
 										return 0;
 									}
 									memcpy(p, val->rs.s, val->rs.len);
 									p[val->rs.len] = 0;
 									val->rs.s = p;
-									val->flags|= XL_VAL_PKG;
+									val->flags|= PV_VAL_PKG;
 								}
 							}
 							return 1;
 						}
-						if(lval.flags==XL_VAL_NONE 
-								|| (lval.flags & XL_VAL_NULL)
-								|| (lval.flags & XL_VAL_EMPTY))
+						if(lval.flags==PV_VAL_NONE 
+								|| (lval.flags & PV_VAL_NULL)
+								|| (lval.flags & PV_VAL_EMPTY))
 							return 0;
-						if(lval.flags&XL_TYPE_INT)
+						if(lval.flags&PV_TYPE_INT)
 							return (lval.ri!=0);
 						else
 							return (lval.rs.len>0);
@@ -1320,19 +1324,18 @@ static int eval_elem(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 				ret=comp_scriptvar(msg, e->op, &e->left, &e->right);
 				break;
 		default:
-				LOG(L_CRIT, "BUG: eval_elem: invalid operand %d\n",
-							e->left.type);
+				LM_CRIT("invalid operand %d\n", e->left.type);
 	}
 	if(val)
 	{
-		val->flags = XL_TYPE_INT|XL_VAL_INT;
+		val->flags = PV_TYPE_INT|PV_VAL_INT;
 		val->ri = ret;
 	}
 	return ret;
 error:
 	if(val)
 	{
-		val->flags = XL_TYPE_INT|XL_VAL_INT;
+		val->flags = PV_TYPE_INT|PV_VAL_INT;
 		val->ri = -1;
 	}
 	return -1;
@@ -1341,15 +1344,14 @@ error:
 
 
 /* ret= 0/1 (true/false) ,  -1 on error or EXPR_DROP (-127)  */
-int eval_expr(struct expr* e, struct sip_msg* msg, xl_value_t *val)
+int eval_expr(struct expr* e, struct sip_msg* msg, pv_value_t *val)
 {
 	static int rec_lev=0;
 	int ret;
 	
 	rec_lev++;
 	if (rec_lev>MAX_REC_LEV){
-		LOG(L_CRIT, "ERROR: eval_expr: too many expressions (%d)\n",
-				rec_lev);
+		LM_CRIT("too many expressions (%d)\n", rec_lev);
 		ret=-1;
 		goto skip;
 	}
@@ -1379,11 +1381,11 @@ int eval_expr(struct expr* e, struct sip_msg* msg, xl_value_t *val)
 				ret=eval_expr(e->left.v.expr, msg, val);
 				break;
 			default:
-				LOG(L_CRIT, "BUG: eval_expr: unknown op %d\n", e->op);
+				LM_CRIT("unknown op %d\n", e->op);
 				ret=-1;
 		}
 	}else{
-		LOG(L_CRIT, "BUG: eval_expr: unknown type %d\n", e->type);
+		LM_CRIT("unknown type %d\n", e->type);
 		ret=-1;
 	}
 
@@ -1410,7 +1412,7 @@ int add_actions(struct action* a, struct action** head)
 {
 	int ret;
 
-	LOG(L_DBG, "add_actions: fixing actions...\n");
+	LM_DBG("fixing actions...\n");
 	if ((ret=fix_actions(a))!=0) goto error;
 	push(a,head);
 	return 0;
@@ -1423,7 +1425,7 @@ error:
 
 /* fixes all action tables */
 /* returns 0 if ok , <0 on error */
-int fix_rls()
+int fix_rls(void)
 {
 	int i,ret;
 	for(i=0;i<RT_NO;i++){
@@ -1484,8 +1486,7 @@ static int check_actions(struct action *a, int r_type)
 				if (n!=rcheck_stack_p)
 					break;
 				if (++rcheck_stack_p==RT_NO) {
-					LOG(L_CRIT,"BUG:check_actions: stack overflow (%d)\n",
-						rcheck_stack_p);
+					LM_CRIT("stack overflow (%d)\n", rcheck_stack_p);
 					goto error;
 				}
 				rcheck_stack[rcheck_stack_p] = a->elem[0].u.number;
@@ -1499,6 +1500,10 @@ static int check_actions(struct action *a, int r_type)
 				if (check_actions((struct action*)a->elem[2].u.data, r_type)!=0)
 					goto error;
 				break;
+			case WHILE_T:
+				if (check_actions((struct action*)a->elem[1].u.data, r_type)!=0)
+					goto error;
+				break;
 			case SWITCH_T:
 				aitem = (struct action*)a->elem[1].u.data;
 				for( ; aitem ; aitem=aitem->next ) {
@@ -1512,12 +1517,11 @@ static int check_actions(struct action *a, int r_type)
 				fct = (cmd_export_t*)(a->elem[0].u.data);
 				if ( (fct->flags&r_type)!=r_type ) {
 					rcheck_status = -1;
-					LOG(L_ERR,"ERROR:check_actions: script function "
+					LM_ERR("script function "
 						"\"%s\" (types=%d) does not support route type "
 						"(%d)\n",fct->name, fct->flags, r_type);
 					for( n=rcheck_stack_p-1; n>=0 ; n-- ) {
-						LOG(L_ERR,"ERROR:check_actions: route "
-							"stack[%d]=%d\n",n,rcheck_stack[n]);
+						LM_ERR("route stack[%d]=%d\n",n,rcheck_stack[n]);
 					}
 				}
 				break;
@@ -1535,7 +1539,7 @@ error:
 /* check all routing tables for compatiblity between
  * route types and called module functions;
  * returns 0 if ok , <0 on error */
-int check_rls()
+int check_rls(void)
 {
 	int i,ret;
 
@@ -1543,16 +1547,14 @@ int check_rls()
 
 	if(rlist[0]){
 		if ((ret=check_actions(rlist[0],REQUEST_ROUTE))!=0){
-			LOG(L_ERR,"ERROR:check_rls: check failed for main "
-				"request route\n");
+			LM_ERR("check failed for main request route\n");
 			return ret;
 		}
 	}
 	for(i=0;i<ONREPLY_RT_NO;i++){
 		if(onreply_rlist[i]){
 			if ((ret=check_actions(onreply_rlist[i],ONREPLY_ROUTE))!=0){
-				LOG(L_ERR,"ERROR:check_rls: check failed for "
-					"onreply_route[%d]\n",i);
+				LM_ERR("check failed for onreply_route[%d]\n",i);
 				return ret;
 			}
 		}
@@ -1560,8 +1562,7 @@ int check_rls()
 	for(i=0;i<FAILURE_RT_NO;i++){
 		if(failure_rlist[i]){
 			if ((ret=check_actions(failure_rlist[i],FAILURE_ROUTE))!=0){
-				LOG(L_ERR,"ERROR:check_rls: check failed for "
-					"failure_route[%d]\n",i);
+				LM_ERR("check failed for failure_route[%d]\n",i);
 				return ret;
 			}
 		}
@@ -1569,16 +1570,14 @@ int check_rls()
 	for(i=0;i<BRANCH_RT_NO;i++){
 		if(branch_rlist[i]){
 			if ((ret=check_actions(branch_rlist[i],BRANCH_ROUTE))!=0){
-				LOG(L_ERR,"ERROR:check_rls: check failed for "
-					"branch_route[%d]\n",i);
+				LM_ERR("check failed for branch_route[%d]\n",i);
 				return ret;
 			}
 		}
 	}
 	if(error_rlist){
 		if ((ret=check_actions(error_rlist,ERROR_ROUTE))!=0){
-			LOG(L_ERR,"ERROR:check_rls: check failed for "
-				"error_route\n");
+			LM_ERR("check failed for error_route\n");
 			return ret;
 		}
 	}
@@ -1589,42 +1588,42 @@ int check_rls()
 
 
 /* debug function, prints main routing table */
-void print_rl()
+void print_rl(void)
 {
 	int j;
 
 	for(j=0; j<RT_NO; j++){
 		if (rlist[j]==0){
-			if (j==0) DBG("WARNING: the main routing table is empty\n");
+			if (j==0) LM_DBG("WARNING: the main routing table is empty\n");
 			continue;
 		}
-		DBG("routing table %d:\n",j);
+		LM_DBG("routing table %d:\n",j);
 		print_actions(rlist[j]);
-		DBG("\n");
+		LM_DBG("\n");
 	}
 	for(j=0; j<ONREPLY_RT_NO; j++){
 		if (onreply_rlist[j]==0){
 			continue;
 		}
-		DBG("onreply routing table %d:\n",j);
+		LM_DBG("onreply routing table %d:\n",j);
 		print_actions(onreply_rlist[j]);
-		DBG("\n");
+		LM_DBG("\n");
 	}
 	for(j=0; j<FAILURE_RT_NO; j++){
 		if (failure_rlist[j]==0){
 			continue;
 		}
-		DBG("failure routing table %d:\n",j);
+		LM_DBG("failure routing table %d:\n",j);
 		print_actions(failure_rlist[j]);
-		DBG("\n");
+		LM_DBG("\n");
 	}
 	for(j=0; j<BRANCH_RT_NO; j++){
 		if (branch_rlist[j]==0){
 			continue;
 		}
-		DBG("T-branch routing table %d:\n",j);
+		LM_DBG("T-branch routing table %d:\n",j);
 		print_actions(branch_rlist[j]);
-		DBG("\n");
+		LM_DBG("\n");
 	}
 }
 
