@@ -1,4 +1,4 @@
-/* $Id: socket_info.h 2785 2007-09-17 17:07:01Z bogdan_iancu $
+/* $Id: socket_info.h 4115 2008-05-05 16:32:04Z miconda $
  *
  * find & manage listen addresses 
  *
@@ -41,15 +41,15 @@
 #include "ut.h"
 /* struct socket_info is defined in ip_addr.h */
 
-struct socket_info* udp_listen;
+extern struct socket_info* udp_listen;
 #ifdef USE_TCP
-struct socket_info* tcp_listen;
+extern struct socket_info* tcp_listen;
 #endif
 #ifdef USE_TLS
-struct socket_info* tls_listen;
+extern struct socket_info* tls_listen;
 #endif
 #ifdef USE_SCTP
-struct socket_info* sctp_listen;
+extern struct socket_info* sctp_listen;
 #endif
 
 
