@@ -1,5 +1,5 @@
 /*
- * $Id: strcommon.c 3436 2007-12-19 11:42:46Z dan_pascu $
+ * $Id: strcommon.c 4130 2008-05-08 07:41:29Z dan_pascu $
  *
  * Copyright (C) 2007 voice-system.ro
  *
@@ -42,6 +42,10 @@ int escape_common(char *dst, char *src, int src_len)
 				dst[j++] = '\\';
 				dst[j++] = src[i];
 				break;
+			case '"':
+				dst[j++] = '\\';
+				dst[j++] = src[i];
+				break;
 			case '\\':
 				dst[j++] = '\\';
 				dst[j++] = src[i];
@@ -77,11 +81,15 @@ int unescape_common(char *dst, char *src, int src_len)
 					dst[j++] = '\'';
 					i++;
 					break;
+				case '"':
+					dst[j++] = '"';
+					i++;
+					break;
 				case '\\':
 					dst[j++] = '\\';
 					i++;
 					break;
-				case '\0':
+				case '0':
 					dst[j++] = '\0';
 					i++;
 					break;
