@@ -1,5 +1,5 @@
 /*
- * $Id: transformations.c 2017 2007-04-17 14:00:20Z miconda $
+ * $Id: transformations.c 3428 2007-12-18 16:49:54Z dan_pascu $
  *
  * Copyright (C) 2007 voice-system.ro
  *
@@ -45,18 +45,22 @@
 
 static char _tr_buffer[TR_BUFFER_SIZE];
 
-int run_transformations(struct sip_msg *msg, trans_t *tr, xl_value_t *val)
+int run_transformations(struct sip_msg *msg, trans_t *tr, pv_value_t *val)
 {
 	trans_t *it;
 	int ret = 0;
 
-	if(tr==NULL || val==NULL)
+	if(tr==NULL || val==NULL){
+		
+		LM_DBG("null pointer\n");
 		return -1;
+	}
 	
 	it = tr;
 	while(it)
 	{
 		ret = (*it->trf)(msg, it->params, it->subtype, val);
+		LM_DBG("return val is %i\n", ret);
 		if(ret!=0)
 			return ret;
 		it = it->next;
@@ -65,50 +69,51 @@ int run_transformations(struct sip_msg *msg, trans_t *tr, xl_value_t *val)
 }
 
 int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
-		xl_value_t *val)
+		pv_value_t *val)
 {
 	int i, j;
 	char *p, *s;
 	str st;
-	xl_value_t v;
-	if(val==NULL)
+	pv_value_t v;
+
+	if(val==NULL || val->flags&PV_VAL_NULL)
 		return -1;
 
 	switch(subtype)
 	{
 		case TR_S_LEN:
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 
-			val->flags = XL_TYPE_INT|XL_VAL_INT|XL_VAL_STR;
+			val->flags = PV_TYPE_INT|PV_VAL_INT|PV_VAL_STR;
 			val->ri = val->rs.len;
 			val->rs.s = int2str(val->ri, &val->rs.len);
 			break;
 		case TR_S_INT:
-			if(!(val->flags&XL_VAL_INT))
+			if(!(val->flags&PV_VAL_INT))
 			{
 				if(str2sint(&val->rs, &val->ri)!=0)
 					return -1;
 			} else { 
-				if(!(val->flags&XL_VAL_STR))
+				if(!(val->flags&PV_VAL_STR))
 					val->rs.s = int2str(val->ri, &val->rs.len);
 			}
 
-			val->flags = XL_TYPE_INT|XL_VAL_INT|XL_VAL_STR;
+			val->flags = PV_TYPE_INT|PV_VAL_INT|PV_VAL_STR;
 			break;
 		case TR_S_MD5:
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 
 			compute_md5(_tr_buffer, val->rs.s, val->rs.len);
 			_tr_buffer[MD5_LEN] = '\0';
-			val->flags = XL_VAL_STR;
+			val->flags = PV_VAL_STR;
 			val->ri = 0;
 			val->rs.s = _tr_buffer;
 			val->rs.len = MD5_LEN;
 			break;
 		case TR_S_ENCODEHEXA:
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 			if(val->rs.len>TR_BUFFER_SIZE/2-1)
 				return -1;
@@ -119,13 +124,13 @@ int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
 				_tr_buffer[j++] = fourbits2char[val->rs.s[i] & 0xf];
 			}
 			_tr_buffer[j] = '\0';
-			memset(val, 0, sizeof(xl_value_t));
-			val->flags = XL_VAL_STR;
+			memset(val, 0, sizeof(pv_value_t));
+			val->flags = PV_VAL_STR;
 			val->rs.s = _tr_buffer;
 			val->rs.len = j;
 			break;
 		case TR_S_DECODEHEXA:
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 			if(val->rs.len>TR_BUFFER_SIZE*2-1)
 				return -1;
@@ -148,75 +153,77 @@ int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
 				else return -1;
 			}
 			_tr_buffer[i] = '\0';
-			memset(val, 0, sizeof(xl_value_t));
-			val->flags = XL_VAL_STR;
+			memset(val, 0, sizeof(pv_value_t));
+			val->flags = PV_VAL_STR;
 			val->rs.s = _tr_buffer;
 			val->rs.len = i;
 			break;
 		case TR_S_ESCAPECOMMON:
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 			if(val->rs.len>TR_BUFFER_SIZE/2-1)
 				return -1;
 			i = escape_common(_tr_buffer, val->rs.s, val->rs.len);
 			_tr_buffer[i] = '\0';
-			memset(val, 0, sizeof(xl_value_t));
-			val->flags = XL_VAL_STR;
+			memset(val, 0, sizeof(pv_value_t));
+			val->flags = PV_VAL_STR;
 			val->rs.s = _tr_buffer;
 			val->rs.len = i;
 			break;
 		case TR_S_UNESCAPECOMMON:
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 			if(val->rs.len>TR_BUFFER_SIZE-1)
 				return -1;
 			i = unescape_common(_tr_buffer, val->rs.s, val->rs.len);
 			_tr_buffer[i] = '\0';
-			memset(val, 0, sizeof(xl_value_t));
-			val->flags = XL_VAL_STR;
+			memset(val, 0, sizeof(pv_value_t));
+			val->flags = PV_VAL_STR;
 			val->rs.s = _tr_buffer;
 			val->rs.len = i;
 			break;
 		case TR_S_ESCAPEUSER:
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 			if(val->rs.len>TR_BUFFER_SIZE/2-1)
 				return -1;
 			st.s = _tr_buffer;
 			st.len = TR_BUFFER_SIZE;
-			escape_user(&val->rs, &st);
-			memset(val, 0, sizeof(xl_value_t));
-			val->flags = XL_VAL_STR;
+			if (escape_user(&val->rs, &st) < 0)
+				return -1;
+			memset(val, 0, sizeof(pv_value_t));
+			val->flags = PV_VAL_STR;
 			val->rs = st;
 			break;
 		case TR_S_UNESCAPEUSER:
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 			if(val->rs.len>TR_BUFFER_SIZE-1)
 				return -1;
 			st.s = _tr_buffer;
 			st.len = TR_BUFFER_SIZE;
-			unescape_user(&val->rs, &st);
-			memset(val, 0, sizeof(xl_value_t));
-			val->flags = XL_VAL_STR;
+			if (unescape_user(&val->rs, &st) < 0)
+				return -1;
+			memset(val, 0, sizeof(pv_value_t));
+			val->flags = PV_VAL_STR;
 			val->rs = st;
 			break;
 		case TR_S_SUBSTR:
 			if(tp==NULL || tp->next==NULL)
 			{
-				LOG(L_ERR, "tr_eval_string: substr invalid parameters\n");
+				LM_ERR("substr invalid parameters\n");
 				return -1;
 			}
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 			if(tp->type==TR_PARAM_NUMBER)
 			{
 				i = tp->v.n;
 			} else {
-				if(xl_get_spec_value(msg, (xl_spec_p)tp->v.data, &v, 0)!=0
-						|| (!(v.flags&XL_VAL_INT)))
+				if(pv_get_spec_value(msg, (pv_spec_p)tp->v.data, &v)!=0
+						|| (!(v.flags&PV_VAL_INT)))
 				{
-					LOG(L_ERR, "tr_eval_string: substr cannot get p1\n");
+					LM_ERR("substr cannot get p1\n");
 					return -1;
 				}
 				i = v.ri;
@@ -225,10 +232,10 @@ int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			{
 				j = tp->next->v.n;
 			} else {
-				if(xl_get_spec_value(msg, (xl_spec_p)tp->next->v.data, &v, 0)!=0
-						|| (!(v.flags&XL_VAL_INT)))
+				if(pv_get_spec_value(msg, (pv_spec_p)tp->next->v.data, &v)!=0
+						|| (!(v.flags&PV_VAL_INT)))
 				{
-					LOG(L_ERR, "tr_eval_string: substr cannot get p2\n");
+					LM_ERR("substr cannot get p2\n");
 					return -1;
 				}
 				j = v.ri;
@@ -236,16 +243,16 @@ int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			DBG("tr_eval_string: i=%d j=%d\n", i, j);
 			if(j<0)
 			{
-				LOG(L_ERR, "tr_eval_string: substr negative offset\n");
+				LM_ERR("substr negative offset\n");
 				return -1;
 			}
-			val->flags = XL_VAL_STR;
+			val->flags = PV_VAL_STR;
 			val->ri = 0;
 			if(i>=0)
 			{
 				if(i>=val->rs.len)
 				{
-					LOG(L_ERR, "tr_eval_string: substr out of range\n");
+					LM_ERR("substr out of range\n");
 					return -1;
 				}
 				if(i+j>=val->rs.len) j=0;
@@ -262,7 +269,7 @@ int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			i = -i;
 			if(i>val->rs.len)
 			{
-				LOG(L_ERR, "tr_eval_string: substr out of range\n");
+				LM_ERR("substr out of range\n");
 				return -1;
 			}
 			if(i<j) j=0;
@@ -279,24 +286,24 @@ int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
 		case TR_S_SELECT:
 			if(tp==NULL || tp->next==NULL)
 			{
-				LOG(L_ERR, "tr_eval_string: select invalid parameters\n");
+				LM_ERR("select invalid parameters\n");
 				return -1;
 			}
-			if(!(val->flags&XL_VAL_STR))
+			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
 			if(tp->type==TR_PARAM_NUMBER)
 			{
 				i = tp->v.n;
 			} else {
-				if(xl_get_spec_value(msg, (xl_spec_p)tp->v.data, &v, 0)!=0
-						|| (!(v.flags&XL_VAL_INT)))
+				if(pv_get_spec_value(msg, (pv_spec_p)tp->v.data, &v)!=0
+						|| (!(v.flags&PV_VAL_INT)))
 				{
-					LOG(L_ERR, "tr_eval_string: select cannot get p1\n");
+					LM_ERR("select cannot get p1\n");
 					return -1;
 				}
 				i = v.ri;
 			}
-			val->flags = XL_VAL_STR;
+			val->flags = PV_VAL_STR;
 			val->ri = 0;
 			if(i<0)
 			{
@@ -349,7 +356,7 @@ int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			break;
 
 		default:
-			LOG(L_ERR, "tr_eval_string: unknown subtype %d\n",
+			LM_ERR("unknown subtype %d\n",
 					subtype);
 			return -1;
 	}
@@ -362,14 +369,14 @@ static struct sip_uri _tr_parsed_uri;
 static param_t* _tr_uri_params = NULL;
 
 int tr_eval_uri(struct sip_msg *msg, tr_param_t *tp, int subtype,
-		xl_value_t *val)
+		pv_value_t *val)
 {
-	xl_value_t v;
+	pv_value_t v;
 	str sv;
 	param_hooks_t phooks;
 	param_t *pit=NULL;
 
-	if(val==NULL || (!(val->flags&XL_VAL_STR)) || val->rs.len<=0)
+	if(val==NULL || (!(val->flags&PV_VAL_STR)) || val->rs.len<=0)
 		return -1;
 
 	if(_tr_uri.len==0 || _tr_uri.len!=val->rs.len ||
@@ -381,14 +388,14 @@ int tr_eval_uri(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			_tr_uri.s = (char*)pkg_malloc((val->rs.len+1)*sizeof(char));
 			if(_tr_uri.s==NULL)
 			{
-				LOG(L_ERR, "tr_eval_uri: no more memory\n");
-				memset(&_tr_uri, 0, sizeof(str));
-				memset(&_tr_parsed_uri, 0, sizeof(struct sip_uri));
+				LM_ERR("no more private memory\n");
 				if(_tr_uri_params != NULL)
 				{
 					free_params(_tr_uri_params);
 					_tr_uri_params = 0;
 				}
+				memset(&_tr_uri, 0, sizeof(str));
+				memset(&_tr_parsed_uri, 0, sizeof(struct sip_uri));
 				return -1;
 			}
 		}
@@ -403,23 +410,23 @@ int tr_eval_uri(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			_tr_uri_params = 0;
 		}
 		/* parse uri -- params only when requested */
-		if(parse_uri(val->rs.s, val->rs.len, &_tr_parsed_uri)!=0)
+		if(parse_uri(_tr_uri.s, _tr_uri.len, &_tr_parsed_uri)!=0)
 		{
-			pkg_free(_tr_uri.s);
-			LOG(L_ERR, "tr_eval_uri: invalid uri [%.*s]\n", val->rs.len,
+			LM_ERR("invalid uri [%.*s]\n", val->rs.len,
 					val->rs.s);
-			memset(&_tr_uri, 0, sizeof(str));
-			memset(&_tr_parsed_uri, 0, sizeof(struct sip_uri));
 			if(_tr_uri_params != NULL)
 			{
 				free_params(_tr_uri_params);
 				_tr_uri_params = 0;
 			}
+			pkg_free(_tr_uri.s);
+			memset(&_tr_uri, 0, sizeof(str));
+			memset(&_tr_parsed_uri, 0, sizeof(struct sip_uri));
 			return -1;
 		}
 	}
-	memset(val, 0, sizeof(xl_value_t));
-	val->flags = XL_VAL_STR;
+	memset(val, 0, sizeof(pv_value_t));
+	val->flags = PV_VAL_STR;
 
 	switch(subtype)
 	{
@@ -433,7 +440,7 @@ int tr_eval_uri(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			val->rs = (_tr_parsed_uri.passwd.s)?_tr_parsed_uri.passwd:_tr_empty;
 			break;
 		case TR_URI_PORT:
-			val->flags |= XL_TYPE_INT|XL_VAL_INT;
+			val->flags |= PV_TYPE_INT|PV_VAL_INT;
 			val->rs = (_tr_parsed_uri.port.s)?_tr_parsed_uri.port:_tr_empty;
 			val->ri = _tr_parsed_uri.port_no;
 			break;
@@ -443,29 +450,31 @@ int tr_eval_uri(struct sip_msg *msg, tr_param_t *tp, int subtype,
 		case TR_URI_PARAM:
 			if(tp==NULL)
 			{
-				LOG(L_ERR, "tr_eval_uri: param invalid parameters\n");
+				LM_ERR("param invalid parameters\n");
 				return -1;
 			}
 			if(_tr_parsed_uri.params.len<=0)
 			{
 				val->rs = _tr_empty;
-				val->flags = XL_VAL_STR;
+				val->flags = PV_VAL_STR;
 				val->ri = 0;
 				break;
 			}
 
 			if(_tr_uri_params == NULL)
-				if (parse_params(&_tr_parsed_uri.params, CLASS_ANY, &phooks,
-							&_tr_uri_params)<0)
+			{
+				sv = _tr_parsed_uri.params;
+				if (parse_params(&sv, CLASS_ANY, &phooks, &_tr_uri_params)<0)
 					return -1;
+			}
 			if(tp->type==TR_PARAM_STRING)
 			{
 				sv = tp->v.s;
 			} else {
-				if(xl_get_spec_value(msg, (xl_spec_p)tp->v.data, &v, 0)!=0
-						|| (!(v.flags&XL_VAL_STR)) || v.rs.len<=0)
+				if(pv_get_spec_value(msg, (pv_spec_p)tp->v.data, &v)!=0
+						|| (!(v.flags&PV_VAL_STR)) || v.rs.len<=0)
 				{
-					LOG(L_ERR, "tr_eval_uri: param cannot get p1\n");
+					LM_ERR("param cannot get p1\n");
 					return -1;
 				}
 				sv = v.rs;
@@ -514,7 +523,7 @@ int tr_eval_uri(struct sip_msg *msg, tr_param_t *tp, int subtype,
 				_tr_parsed_uri.r2_val:_tr_empty;
 			break;
 		default:
-			LOG(L_ERR, "tr_eval_uri: unknown subtype %d\n",
+			LM_ERR("unknown subtype %d\n",
 					subtype);
 			return -1;
 	}
@@ -526,15 +535,15 @@ static str _tr_params_str = {0, 0};
 static param_t* _tr_params_list = NULL;
 
 int tr_eval_paramlist(struct sip_msg *msg, tr_param_t *tp, int subtype,
-		xl_value_t *val)
+		pv_value_t *val)
 {
-	xl_value_t v;
+	pv_value_t v;
 	str sv;
 	int n, i;
 	param_hooks_t phooks;
 	param_t *pit=NULL;
 
-	if(val==NULL || (!(val->flags&XL_VAL_STR)) || val->rs.len<=0)
+	if(val==NULL || (!(val->flags&PV_VAL_STR)) || val->rs.len<=0)
 		return -1;
 
 	if(_tr_params_str.len==0 || _tr_params_str.len!=val->rs.len ||
@@ -546,7 +555,7 @@ int tr_eval_paramlist(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			_tr_params_str.s = (char*)pkg_malloc((val->rs.len+1)*sizeof(char));
 			if(_tr_params_str.s==NULL)
 			{
-				LOG(L_ERR, "tr_eval_paramlist: no more memory\n");
+				LM_ERR("no more private memory\n");
 				memset(&_tr_params_str, 0, sizeof(str));
 				if(_tr_params_list != NULL)
 				{
@@ -568,23 +577,23 @@ int tr_eval_paramlist(struct sip_msg *msg, tr_param_t *tp, int subtype,
 		}
 		
 		/* parse params */
-		if (parse_params(&_tr_params_str, CLASS_ANY, &phooks,
-				&_tr_params_list)<0)
+		sv = _tr_params_str;
+		if (parse_params(&sv, CLASS_ANY, &phooks, &_tr_params_list)<0)
 			return -1;
 	}
 	
 	if(_tr_params_list==NULL)
 		return -1;
 
-	memset(val, 0, sizeof(xl_value_t));
-	val->flags = XL_VAL_STR;
+	memset(val, 0, sizeof(pv_value_t));
+	val->flags = PV_VAL_STR;
 
 	switch(subtype)
 	{
 		case TR_PL_VALUE:
 			if(tp==NULL)
 			{
-				LOG(L_ERR, "tr_eval_paramlist: value invalid parameters\n");
+				LM_ERR("value invalid parameters\n");
 				return -1;
 			}
 
@@ -592,10 +601,10 @@ int tr_eval_paramlist(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			{
 				sv = tp->v.s;
 			} else {
-				if(xl_get_spec_value(msg, (xl_spec_p)tp->v.data, &v, 0)!=0
-						|| (!(v.flags&XL_VAL_STR)) || v.rs.len<=0)
+				if(pv_get_spec_value(msg, (pv_spec_p)tp->v.data, &v)!=0
+						|| (!(v.flags&PV_VAL_STR)) || v.rs.len<=0)
 				{
-					LOG(L_ERR, "tr_eval_paramlist: value cannot get p1\n");
+					LM_ERR("value cannot get p1\n");
 					return -1;
 				}
 				sv = v.rs;
@@ -615,7 +624,7 @@ int tr_eval_paramlist(struct sip_msg *msg, tr_param_t *tp, int subtype,
 		case TR_PL_NAME:
 			if(tp==NULL)
 			{
-				LOG(L_ERR, "tr_eval_paramlist: name invalid parameters\n");
+				LM_ERR("name invalid parameters\n");
 				return -1;
 			}
 
@@ -623,10 +632,10 @@ int tr_eval_paramlist(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			{
 				n = tp->v.n;
 			} else {
-				if(xl_get_spec_value(msg, (xl_spec_p)tp->v.data, &v, 0)!=0
-						|| (!(v.flags&XL_VAL_INT)))
+				if(pv_get_spec_value(msg, (pv_spec_p)tp->v.data, &v)!=0
+						|| (!(v.flags&PV_VAL_INT)))
 				{
-					LOG(L_ERR, "tr_eval_paramlist: name cannot get p1\n");
+					LM_ERR("name cannot get p1\n");
 					return -1;
 				}
 				n = v.ri;
@@ -667,7 +676,7 @@ int tr_eval_paramlist(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			val->rs = _tr_empty;
 			break;
 		default:
-			LOG(L_ERR, "tr_eval_paramlist: unknown subtype %d\n",
+			LM_ERR("unknown subtype %d\n",
 					subtype);
 			return -1;
 	}
@@ -676,21 +685,23 @@ done:
 }
 
 
+#define is_in_str(p, in) (p<in->s+in->len && *p)
 
-char* parse_transformation(char *s, trans_t **tr)
+char* parse_transformation(str *in, trans_t **tr)
 {
 	char *p;
 	char *p0;
 	str tclass;
 	trans_t *t = NULL;
 	trans_t *t0 = NULL;
+	str s;
 
-	if(s==NULL || tr==NULL)
+	if(in==NULL || in->s==NULL || tr==NULL)
 		return NULL;
 	
-	p = s;
+	p = in->s;
 	do {
-		while(*p && (*p==' ' || *p=='\t' || *p=='\n')) p++;
+		while(is_in_str(p, in) && (*p==' ' || *p=='\t' || *p=='\n')) p++;
 		if(*p != TR_LBRACKET)
 			break;
 		p++;
@@ -698,23 +709,22 @@ char* parse_transformation(char *s, trans_t **tr)
 		t = (trans_t*)pkg_malloc(sizeof(trans_t));
 		if(t == NULL)
 		{
-			LOG(L_ERR, "parse_transformation: no more memory\n");
+			LM_ERR("no more private memory\n");
 			return NULL;
 		}
 		memset(t, 0, sizeof(trans_t));
 		if(t0==NULL)
 			*tr = t;
 		else
-			t0->next = t;;
+			t0->next = t;
 		t0 = t;
 
 		/* find transformation class */
 		tclass.s = p;
-		while(*p && *p!=TR_CLASS_MARKER) p++;
+		while(is_in_str(p, in) && *p!=TR_CLASS_MARKER) p++;
 		if(*p!=TR_CLASS_MARKER || tclass.s == p)
 		{
-			LOG(L_ERR,
-				"parse_transformation: invalid transformation: %s!\n", s);
+			LM_ERR("invalid transformation: %.*s (%c)!\n", in->len, in->s, *p);
 			goto error;
 		}
 		tclass.len = p - tclass.s;
@@ -724,44 +734,47 @@ char* parse_transformation(char *s, trans_t **tr)
 		{
 			t->type = TR_STRING;
 			t->trf = tr_eval_string;
-			p0 = tr_parse_string(p, t);
+			s.s = p; s.len = in->s + in->len - p;
+			p0 = tr_parse_string(&s, t);
 			if(p0==NULL)
 				goto error;
 			p = p0;
 		} else if(tclass.len==3 && strncasecmp(tclass.s, "uri", 3)==0) {
 			t->type = TR_URI;
 			t->trf = tr_eval_uri;
-			p0 = tr_parse_uri(p, t);
+			s.s = p; s.len = in->s + in->len - p;
+			p0 = tr_parse_uri(&s, t);
 			if(p0==NULL)
 				goto error;
 			p = p0;
 		} else if(tclass.len==5 && strncasecmp(tclass.s, "param", 5)==0) {
 			t->type = TR_PARAMLIST;
 			t->trf = tr_eval_paramlist;
-			p0 = tr_parse_paramlist(p, t);
+			s.s = p; s.len = in->s + in->len - p;
+			p0 = tr_parse_paramlist(&s, t);
 			if(p0==NULL)
 				goto error;
 			p = p0;
 		} else {
-			LOG(L_ERR,
-			"parse_transformation: unknown transformation: [%.*s] in [%s]\n",
-				tclass.len, tclass.s, s);
+			LM_ERR("unknown transformation: [%.*s] in [%.*s]\n",
+				tclass.len, tclass.s, in->len, in->s);
 			goto error;
 		}
 
 		if(*p != TR_RBRACKET)
 		{
-			LOG(L_ERR,
-				"parse_transformation: invalid transformation: %s!!\n", s);
+			LM_ERR("invalid transformation: %.*s | %c !!\n", in->len, in->s, *p);
 			goto error;
 		}
 
 		p++;
+		if(!is_in_str(p, in))
+			break;
 	} while(1);
 
 	return p;
 error:
-	LOG(L_ERR, "parse_transformation: error parsing [%s]\n", s);
+	LM_ERR("error parsing [%.*s]\n", in->len, in->s);
 	t = *tr;
 	while(t)
 	{
@@ -773,29 +786,29 @@ error:
 	return NULL;
 }
 
-#define _tr_parse_nparam(_p, _p0, _tp, _spec, _n, _sign, _s) \
-	while(*_p && (*_p==' ' || *_p=='\t' || *_p=='\n')) _p++; \
-	if(*_p==ITEM_MARKER) \
+#define _tr_parse_nparam(_p, _p0, _tp, _spec, _n, _sign, _in, _s) \
+	while(is_in_str(_p, _in) && (*_p==' ' || *_p=='\t' || *_p=='\n')) _p++; \
+	if(*_p==PV_MARKER) \
 	{ /* pseudo-variable */ \
-		_spec = (xl_spec_t*)pkg_malloc(sizeof(xl_spec_t)); \
+		_spec = (pv_spec_t*)pkg_malloc(sizeof(pv_spec_t)); \
 		if(_spec==NULL) \
 		{ \
-			LOG(L_ERR, "tr_parse_nparam: no more memory!\n"); \
+			LM_ERR("no more private memory!\n"); \
 			goto error; \
 		} \
-		_p0 = xl_parse_spec(_p, _spec, 0); \
+		_s.s = _p; _s.len = _in->s + _in->len - _p; \
+		_p0 = pv_parse_spec(&_s, _spec); \
 		if(_p0==NULL) \
 		{ \
-			LOG(L_ERR, \
-			"tr_parse_nparam: invalid spec in substr transformation: %s!\n", \
-				_s); \
+			LM_ERR("invalid spec in substr transformation: %.*s!\n", \
+				_in->len, _in->s); \
 			goto error; \
 		} \
 		_p = _p0; \
 		_tp = (tr_param_t*)pkg_malloc(sizeof(tr_param_t)); \
 		if(_tp==NULL) \
 		{ \
-			LOG(L_ERR, "tr_nparam_string: no more memory!\n"); \
+			LM_ERR("no more private memory!\n"); \
 			goto error; \
 		} \
 		memset(_tp, 0, sizeof(tr_param_t)); \
@@ -810,8 +823,9 @@ error:
 				_sign = -1; \
 			} else if(*_p=='+') _p++; \
 			_n = 0; \
-			while(*_p && (*_p==' ' || *_p=='\t' || *_p=='\n')) _p++; \
-			while(*_p>='0' && *_p<='9') \
+			while(is_in_str(_p, _in) && (*_p==' ' || *_p=='\t' || *_p=='\n')) \
+					_p++; \
+			while(is_in_str(_p, _in) && *_p>='0' && *_p<='9') \
 			{ \
 				_n = _n*10 + *_p - '0'; \
 				_p++; \
@@ -819,65 +833,64 @@ error:
 			_tp = (tr_param_t*)pkg_malloc(sizeof(tr_param_t)); \
 			if(_tp==NULL) \
 			{ \
-				LOG(L_ERR, "tr_parse_nparam: no more memory!\n"); \
+				LM_ERR("no more private memory!\n"); \
 				goto error; \
 			} \
 			memset(_tp, 0, sizeof(tr_param_t)); \
 			_tp->type = TR_PARAM_NUMBER; \
 			_tp->v.n = sign*n; \
 		} else { \
-			LOG(L_ERR, \
-				"tr_parse_nparam: invalid param in transformation: %s!!\n", \
-				_s); \
+			LM_ERR("tinvalid param in transformation: %.*s!!\n", \
+				_in->len, _in->s); \
 			goto error; \
 		} \
 	}
 
-#define _tr_parse_sparam(_p, _p0, _tp, _spec, _ps, _s) \
-	while(*_p && (*_p==' ' || *_p=='\t' || *_p=='\n')) _p++; \
-	if(*_p==ITEM_MARKER) \
+#define _tr_parse_sparam(_p, _p0, _tp, _spec, _ps, _in, _s) \
+	while(is_in_str(_p, _in) && (*_p==' ' || *_p=='\t' || *_p=='\n')) _p++; \
+	if(*_p==PV_MARKER) \
 	{ /* pseudo-variable */ \
-		_spec = (xl_spec_t*)pkg_malloc(sizeof(xl_spec_t)); \
+		_spec = (pv_spec_t*)pkg_malloc(sizeof(pv_spec_t)); \
 		if(_spec==NULL) \
 		{ \
-			LOG(L_ERR, "tr_parse_sparam: no more memory!\n"); \
+			LM_ERR("no more private memory!\n"); \
 			goto error; \
 		} \
-		_p0 = xl_parse_spec(_p, _spec, 0); \
+		_s.s = _p; _s.len = _in->s + _in->len - _p; \
+		_p0 = pv_parse_spec(&_s, _spec); \
 		if(_p0==NULL) \
 		{ \
-			LOG(L_ERR, \
-			"tr_parse_sparam: invalid spec in substr transformation: %s!\n", \
-				_s); \
+			LM_ERR("invalid spec in substr transformation: %.*s!\n", \
+				_in->len, _in->s); \
 			goto error; \
 		} \
 		_p = _p0; \
 		_tp = (tr_param_t*)pkg_malloc(sizeof(tr_param_t)); \
 		if(_tp==NULL) \
 		{ \
-			LOG(L_ERR, "tr_sparam_string: no more memory!\n"); \
+			LM_ERR("no more private memory!\n"); \
 			goto error; \
 		} \
 		memset(_tp, 0, sizeof(tr_param_t)); \
 		_tp->type = TR_PARAM_SPEC; \
 		_tp->v.data = (void*)_spec; \
 	} else { /* string */ \
-		while(*_p && (*_p==' ' || *_p=='\t' || *_p=='\n')) _p++; \
+		while(is_in_str(_p, _in) && (*_p==' ' || *_p=='\t' || *_p=='\n')) \
+				_p++; \
 		_ps = _p; \
-		while(*_p!='\0' && *_p!=' ' && *_p!='\t' && *_p!='\n' \
+		while(is_in_str(_p, _in) && *_p!=' ' && *_p!='\t' && *_p!='\n' \
 				&& *_p!=TR_PARAM_MARKER && *_p!=TR_RBRACKET) \
 				_p++; \
 		if(*_p=='\0') \
 		{ \
-			LOG(L_ERR, \
-				"tr_parse_sparam: invalid param in transformation: %s!!\n", \
-				_s); \
+			LM_ERR("invalid param in transformation: %.*s!!\n", \
+				_in->len, _in->s); \
 			goto error; \
 		} \
 		_tp = (tr_param_t*)pkg_malloc(sizeof(tr_param_t)); \
 		if(_tp==NULL) \
 		{ \
-			LOG(L_ERR, "tr_parse_sparam: no more memory!\n"); \
+			LM_ERR("no more private memory!\n"); \
 			goto error; \
 		} \
 		memset(_tp, 0, sizeof(tr_param_t)); \
@@ -887,27 +900,29 @@ error:
 	}
 
 
-char* tr_parse_string(char* s, trans_t *t)
+char* tr_parse_string(str* in, trans_t *t)
 {
 	char *p;
 	char *p0;
 	str name;
-	xl_spec_t *spec = NULL;
+	str s;
+	pv_spec_t *spec = NULL;
 	int n;
 	int sign;
 	tr_param_t *tp = NULL;
 
-	if(s==NULL || t==NULL)
+	if(in==NULL || t==NULL)
 		return NULL;
 
-	p = s;
-	name.s = s;
+	p = in->s;
+	name.s = in->s;
 
 	/* find next token */
-	while(*p && *p!=TR_PARAM_MARKER && *p!=TR_RBRACKET) p++;
+	while(is_in_str(p, in) && *p!=TR_PARAM_MARKER && *p!=TR_RBRACKET) p++;
 	if(*p=='\0')
 	{
-		LOG(L_ERR, "tr_parse_string: invalid transformation: %s\n", s);
+		LM_ERR("invalid transformation: %.*s\n",
+				in->len, in->s);
 		goto error;
 	}
 	name.len = p - name.s;
@@ -945,37 +960,34 @@ char* tr_parse_string(char* s, trans_t *t)
 		t->subtype = TR_S_SUBSTR;
 		if(*p!=TR_PARAM_MARKER)
 		{
-			LOG(L_ERR, "tr_parse_string: invalid substr transformation: %s!\n",
-					s);
+			LM_ERR("invalid substr transformation: %.*s!\n", in->len, in->s);
 			goto error;
 		}
 		p++;
-		_tr_parse_nparam(p, p0, tp, spec, n, sign, s);
+		_tr_parse_nparam(p, p0, tp, spec, n, sign, in, s);
 		t->params = tp;
 		tp = 0;
 		while(*p && (*p==' ' || *p=='\t' || *p=='\n')) p++;
 		if(*p!=TR_PARAM_MARKER)
 		{
-			LOG(L_ERR,
-				"tr_parse_string: invalid substr transformation: %s!\n",
-				s);
+			LM_ERR("invalid substr transformation: %.*s!\n",
+				in->len, in->s);
 			goto error;
 		}
 		p++;
-		_tr_parse_nparam(p, p0, tp, spec, n, sign, s);
+		_tr_parse_nparam(p, p0, tp, spec, n, sign, in, s);
 		if(tp->type==TR_PARAM_NUMBER && tp->v.n<0)
 		{
-			LOG(L_ERR, "tr_parse_string: substr negative offset\n");
+			LM_ERR("substr negative offset\n");
 			goto error;
 		}
 		t->params->next = tp;
 		tp = 0;
-		while(*p && (*p==' ' || *p=='\t' || *p=='\n')) p++;
+		while(is_in_str(p, in) && (*p==' ' || *p=='\t' || *p=='\n')) p++;
 		if(*p!=TR_RBRACKET)
 		{
-			LOG(L_ERR,
-				"tr_parse_string: invalid substr transformation: %s!!\n",
-				s);
+			LM_ERR("invalid substr transformation: %.*s!!\n",
+				in->len, in->s);
 			goto error;
 		}
 		return p;
@@ -983,27 +995,25 @@ char* tr_parse_string(char* s, trans_t *t)
 		t->subtype = TR_S_SELECT;
 		if(*p!=TR_PARAM_MARKER)
 		{
-			LOG(L_ERR, "tr_parse_string: invalid select transformation: %s!\n",
-					s);
+			LM_ERR("invalid select transformation: %.*s!\n",
+					in->len, in->s);
 			goto error;
 		}
 		p++;
-		_tr_parse_nparam(p, p0, tp, spec, n, sign, s);
+		_tr_parse_nparam(p, p0, tp, spec, n, sign, in, s);
 		t->params = tp;
 		tp = 0;
 		while(*p && (*p==' ' || *p=='\t' || *p=='\n')) p++;
 		if(*p!=TR_PARAM_MARKER || *(p+1)=='\0')
 		{
-			LOG(L_ERR,
-				"tr_parse_string: invalid select transformation: %s!\n",
-				s);
+			LM_ERR("invalid select transformation: %.*s!\n", in->len, in->s);
 			goto error;
 		}
 		p++;
 		tp = (tr_param_t*)pkg_malloc(sizeof(tr_param_t));
 		if(tp==NULL)
 		{
-			LOG(L_ERR, "tr_parse_string: no more memory!\n");
+			LM_ERR("no more private memory!\n");
 			goto error;
 		}
 		memset(tp, 0, sizeof(tr_param_t));
@@ -1016,45 +1026,44 @@ char* tr_parse_string(char* s, trans_t *t)
 		while(*p && (*p==' ' || *p=='\t' || *p=='\n')) p++;
 		if(*p!=TR_RBRACKET)
 		{
-			LOG(L_ERR,
-				"tr_parse_string: invalid select transformation: %s!!\n",
-				s);
+			LM_ERR("invalid select transformation: %.*s!!\n",
+				in->len, in->s);
 			goto error;
 		}
 		return p;
 	}
 
 
-	LOG(L_ERR, "tr_parse_string: unknown transformation: %s/%s/%d!\n", s,
-			name.s, name.len);
+	LM_ERR("unknown transformation: %.*s/%.*s/%d!\n", in->len, in->s,
+			name.len, name.s, name.len);
 error:
 	if(tp)
 		free_tr_param(tp);
 	if(spec)
-		xl_spec_free(spec);
+		pv_spec_free(spec);
 	return NULL;
 }
 
-char* tr_parse_uri(char* s, trans_t *t)
+char* tr_parse_uri(str* in, trans_t *t)
 {
 	char *p;
 	char *p0;
 	char *ps;
 	str name;
-	xl_spec_t *spec = NULL;
+	str s;
+	pv_spec_t *spec = NULL;
 	tr_param_t *tp = NULL;
 
-	if(s==NULL || t==NULL)
+	if(in==NULL || in->s==NULL || t==NULL)
 		return NULL;
-
-	p = s;
-	name.s = s;
+	p = in->s;
+	name.s = in->s;
 
 	/* find next token */
 	while(*p && *p!=TR_PARAM_MARKER && *p!=TR_RBRACKET) p++;
 	if(*p=='\0')
 	{
-		LOG(L_ERR, "tr_parse_uri: invalid transformation: %s\n", s);
+		LM_ERR("invalid transformation: %.*s\n", in->len, in->s);
 		goto error;
 	}
 	name.len = p - name.s;
@@ -1081,17 +1090,17 @@ char* tr_parse_uri(char* s, trans_t *t)
 		t->subtype = TR_URI_PARAM;
 		if(*p!=TR_PARAM_MARKER)
 		{
-			LOG(L_ERR, "tr_parse_uri: invalid param transformation: %s\n", s);
+			LM_ERR("invalid param transformation: %.*s\n", in->len, in->s);
 			goto error;
 		}
 		p++;
-		_tr_parse_sparam(p, p0, tp, spec, ps, s);
+		_tr_parse_sparam(p, p0, tp, spec, ps, in, s);
 		t->params = tp;
 		tp = 0;
 		while(*p && (*p==' ' || *p=='\t' || *p=='\n')) p++;
 		if(*p!=TR_RBRACKET)
 		{
-			LOG(L_ERR, "tr_parse_uri: invalid param transformation: %s!\n", s);
+			LM_ERR("invalid param transformation: %.*s!\n", in->len, in->s);
 			goto error;
 		}
 		return p;
@@ -1119,38 +1128,40 @@ char* tr_parse_uri(char* s, trans_t *t)
 	}
 
 
-	LOG(L_ERR, "tr_parse_uri: unknown transformation: %s/%.*s!\n", s,
-			name.len, name.s);
+	LM_ERR("unknown transformation: %.*s/%.*s!\n", in->len,
+			in->s, name.len, name.s);
 error:
 	if(tp)
 		free_tr_param(tp);
 	if(spec)
-		xl_spec_free(spec);
+		pv_spec_free(spec);
 	return NULL;
 }
 
-char* tr_parse_paramlist(char* s, trans_t *t)
+char* tr_parse_paramlist(str* in, trans_t *t)
 {
 	char *p;
 	char *p0;
 	char *ps;
+	str s;
 	str name;
 	int n;
 	int sign;
-	xl_spec_t *spec = NULL;
+	pv_spec_t *spec = NULL;
 	tr_param_t *tp = NULL;
 
-	if(s==NULL || t==NULL)
+	if(in==NULL || in->s==NULL || t==NULL)
 		return NULL;
 
-	p = s;
-	name.s = s;
+	p = in->s;
+	name.s = in->s;
 
 	/* find next token */
-	while(*p && *p!=TR_PARAM_MARKER && *p!=TR_RBRACKET) p++;
+	while(is_in_str(p, in) && *p!=TR_PARAM_MARKER && *p!=TR_RBRACKET) p++;
 	if(*p=='\0')
 	{
-		LOG(L_ERR, "tr_parse_param: invalid transformation: %s\n", s);
+		LM_ERR("invalid transformation: %.*s\n",
+				in->len, in->s);
 		goto error;
 	}
 	name.len = p - name.s;
@@ -1161,18 +1172,19 @@ char* tr_parse_paramlist(char* s, trans_t *t)
 		t->subtype = TR_PL_VALUE;
 		if(*p!=TR_PARAM_MARKER)
 		{
-			LOG(L_ERR, "tr_parse_param: invalid value transformation: %s\n", s);
+			LM_ERR("invalid value transformation: %.*s\n",
+					in->len, in->s);
 			goto error;
 		}
 		p++;
-		_tr_parse_sparam(p, p0, tp, spec, ps, s);
+		_tr_parse_sparam(p, p0, tp, spec, ps, in, s);
 		t->params = tp;
 		tp = 0;
 		while(*p && (*p==' ' || *p=='\t' || *p=='\n')) p++;
 		if(*p!=TR_RBRACKET)
 		{
-			LOG(L_ERR, "tr_parse_param: invalid value transformation: %s!\n",
-					s);
+			LM_ERR("invalid value transformation: %.*s!\n",
+					in->len, in->s);
 			goto error;
 		}
 		return p;
@@ -1180,30 +1192,31 @@ char* tr_parse_paramlist(char* s, trans_t *t)
 		t->subtype = TR_PL_NAME;
 		if(*p!=TR_PARAM_MARKER)
 		{
-			LOG(L_ERR, "tr_parse_param: invalid name transformation: %s\n", s);
+			LM_ERR("invalid name transformation: %.*s\n",
+					in->len, in->s);
 			goto error;
 		}
 		p++;
-		_tr_parse_nparam(p, p0, tp, spec, n, sign, s)
+		_tr_parse_nparam(p, p0, tp, spec, n, sign, in, s)
 		t->params = tp;
 		tp = 0;
-		while(*p && (*p==' ' || *p=='\t' || *p=='\n')) p++;
+		while(is_in_str(p, in) && (*p==' ' || *p=='\t' || *p=='\n')) p++;
 		if(*p!=TR_RBRACKET)
 		{
-			LOG(L_ERR, "tr_parse_param: invalid name transformation: %s!\n",
-					s);
+			LM_ERR("invalid name transformation: %.*s!\n",
+					in->len, in->s);
 			goto error;
 		}
 		return p;
 	}
 
-	LOG(L_ERR, "tr_parse_param: unknown transformation: %s/%.*s!\n", s,
-			name.len, name.s);
+	LM_ERR("unknown transformation: %.*s/%.*s!\n",
+			in->len, in->s, name.len, name.s);
 error:
 	if(tp)
 		free_tr_param(tp);
 	if(spec)
-		xl_spec_free(spec);
+		pv_spec_free(spec);
 	return NULL;
 }
 
@@ -1248,7 +1261,7 @@ void free_tr_param(tr_param_t *tp)
 		tp0 = tp;
 		tp = tp->next;
 		if(tp0->type==TR_PARAM_SPEC)
-			xl_spec_free((xl_spec_t*)tp0->v.data);
+			pv_spec_free((pv_spec_t*)tp0->v.data);
 		pkg_free(tp0);
 	}
 }
